@@ -12,6 +12,7 @@
 
 #include "stylesheet.h"
 #include "lex/lex.h"
+#include "parse/font_face.h"
 #include "parse/important.h"
 #include "parse/language.h"
 #include "parse/parse.h"
@@ -120,9 +121,7 @@ css_error css__language_create(css_stylesheet *sheet, css_parser *parser,
 	css_language *c;
 	css_parser_optparams params;
 	parserutils_error perror;
-	lwc_error lerror;
 	css_error error;
-	int i;
 
 	if (sheet == NULL || parser == NULL || alloc == NULL || 
 			language == NULL)
@@ -140,18 +139,6 @@ css_error css__language_create(css_stylesheet *sheet, css_parser *parser,
 		return css_error_from_parserutils_error(perror);
 	}
 
-	/* Intern all known strings */
-	for (i = 0; i < LAST_KNOWN; i++) {
-		lerror = lwc_intern_string(stringmap[i].data,
-					    stringmap[i].len,
-					    &(c->strings[i]));
-		if (lerror != lwc_error_ok) {
-			parserutils_stack_destroy(c->context);
-			alloc(c, 0, pw);
-			return CSS_NOMEM;
-		}
-	}
-
 	params.event_handler.handler = language_handle_event;
 	params.event_handler.pw = c;
 	error = css__parser_setopt(parser, CSS_PARSER_EVENT_HANDLER, &params);
@@ -166,6 +153,7 @@ css_error css__language_create(css_stylesheet *sheet, css_parser *parser,
 	c->default_namespace = NULL;
 	c->namespaces = NULL;
 	c->num_namespaces = 0;
+	c->strings = sheet->propstrings;
 	c->alloc = alloc;
 	c->pw = pw;
 
@@ -201,10 +189,6 @@ css_error css__language_destroy(css_language *language)
 
 	parserutils_stack_destroy(language->context);
 	
-	for (i = 0; i < LAST_KNOWN; ++i) {
-		lwc_string_unref(language->strings[i]);
-	}
-	
 	language->alloc(language, 0, language->pw);
 
 	return CSS_OK;
@@ -571,6 +555,26 @@ css_error handleStartAtRule(css_language *c, const parserutils_vector *vector)
 			return error;
 		}
 
+		/* Rule is now owned by the sheet, 
+		 * so no need to destroy it */
+
+		c->state = HAD_RULE;
+	} else if (lwc_string_caseless_isequal(atkeyword->idata, 
+			c->strings[FONT_FACE], &match) == lwc_error_ok && 
+			match) {
+		error = css__stylesheet_rule_create(c->sheet,
+				CSS_RULE_FONT_FACE, &rule);
+		if (error != CSS_OK)
+			return error;
+		
+		consumeWhitespace(vector, &ctx);
+
+		error = css__stylesheet_add_rule(c->sheet, rule, NULL);
+		if (error != CSS_OK) {
+			css__stylesheet_rule_destroy(c->sheet, rule);
+			return error;
+		}
+
 		/* Rule is now owned by the sheet, 
 		 * so no need to destroy it */
 
@@ -710,9 +714,9 @@ css_error handleBlockContent(css_language *c, const parserutils_vector *vector)
 	context_entry *entry;
 	css_rule *rule;
 
-	/* In CSS 2.1, block content comprises either declarations (if the 
-	 * current block is associated with @page or a selector), or rulesets 
-	 * (if the current block is associated with @media). */
+	/* Block content comprises either declarations (if the current block is
+	 * associated with @page, @font-face or a selector), or rulesets (if the
+	 * current block is associated with @media). */
 
 	entry = parserutils_stack_get_current(c->context);
 	if (entry == NULL || entry->data == NULL)
@@ -721,7 +725,8 @@ css_error handleBlockContent(css_language *c, const parserutils_vector *vector)
 	rule = entry->data;
 	if (rule == NULL || (rule->type != CSS_RULE_SELECTOR && 
 			rule->type != CSS_RULE_PAGE &&
-			rule->type != CSS_RULE_MEDIA))
+			rule->type != CSS_RULE_MEDIA && 
+			rule->type != CSS_RULE_FONT_FACE))
 		return CSS_INVALID;
 
 	if (rule->type == CSS_RULE_MEDIA) {
@@ -746,6 +751,7 @@ css_error handleDeclaration(css_language *c, const parserutils_vector *vector)
 	/* Locations where declarations are permitted:
 	 *
 	 * + In @page
+	 * + In @font-face
 	 * + In ruleset
 	 */
 	entry = parserutils_stack_get_current(c->context);
@@ -754,7 +760,8 @@ css_error handleDeclaration(css_language *c, const parserutils_vector *vector)
 
 	rule = entry->data;
 	if (rule == NULL || (rule->type != CSS_RULE_SELECTOR && 
-			rule->type != CSS_RULE_PAGE))
+				rule->type != CSS_RULE_PAGE && 
+				rule->type != CSS_RULE_FONT_FACE))
 		return CSS_INVALID;
 
 	/* Strip any leading whitespace (can happen if in nested block) */
@@ -776,7 +783,13 @@ css_error handleDeclaration(css_language *c, const parserutils_vector *vector)
 
 	consumeWhitespace(vector, &ctx);
 
-	error = parseProperty(c, ident, vector, &ctx, rule);
+	if (rule->type == CSS_RULE_FONT_FACE) {
+		css_rule_font_face * ff_rule = (css_rule_font_face *) rule;
+		error = css__parse_font_descriptor(
+				c, ident, vector, &ctx, ff_rule);
+	} else {
+		error = parseProperty(c, ident, vector, &ctx, rule);
+	}
 	if (error != CSS_OK)
 		return error;
 
