@@ -6,6 +6,21 @@
  */
 
 #include "parse/propstrings.h"
+#include "stylesheet.h"
+
+#include <assert.h>
+
+typedef struct stringmap_entry {
+	const char *data;
+	size_t len;
+} stringmap_entry;
+
+typedef struct css__propstrings_ctx {
+	uint32_t count;
+	lwc_string *strings[LAST_KNOWN];
+} css__propstrings_ctx;
+
+static css__propstrings_ctx css__propstrings;
 
 /* Must be synchronised with enum in propstrings.h */
 const stringmap_entry stringmap[LAST_KNOWN] = {
@@ -15,6 +30,7 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "import", SLEN("import") },
 	{ "media", SLEN("media") },
 	{ "namespace", SLEN("namespace") },
+	{ "font-face", SLEN("font-face") },
 	{ "page", SLEN("page") },
 
 	{ "aural", SLEN("aural") },
@@ -89,10 +105,23 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "border-top-width", SLEN("border-top-width") },
 	{ "border-width", SLEN("border-width") },
 	{ "bottom", SLEN("bottom") },
+	{ "break-after", SLEN("break-after") },
+	{ "break-before", SLEN("break-before") },
+	{ "break-inside", SLEN("break-inside") },
 	{ "caption-side", SLEN("caption-side") },
 	{ "clear", SLEN("clear") },
 	{ "clip", SLEN("clip") },
 	{ "color", SLEN("color") },
+	{ "columns", SLEN("columns") },
+	{ "column-count", SLEN("column-count") },
+	{ "column-fill", SLEN("column-fill") },
+	{ "column-gap", SLEN("column-gap") },
+	{ "column-rule", SLEN("column-rule") },
+	{ "column-rule-color", SLEN("column-rule-color") },
+	{ "column-rule-style", SLEN("column-rule-style") },
+	{ "column-rule-width", SLEN("column-rule_width") },
+	{ "column-span", SLEN("column-span") },
+	{ "column-width", SLEN("column-width") },
 	{ "content", SLEN("content") },
 	{ "counter-increment", SLEN("counter-increment") },
 	{ "counter-reset", SLEN("counter-reset") },
@@ -357,6 +386,19 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 	{ "currentColor", SLEN("currentColor") },
 	{ "odd", SLEN("odd") },
 	{ "even", SLEN("even") },
+	{ "src", SLEN("src") },
+	{ "local", SLEN("local") },
+	{ "initial", SLEN("initial") },
+	{ "format", SLEN("format") },
+	{ "woff", SLEN("woff") },
+	{ "truetype", SLEN("truetype") },
+	{ "opentype", SLEN("opentype") },
+	{ "embedded-opentype", SLEN("embedded-opentype") },
+	{ "svg", SLEN("svg") },
+	{ "column", SLEN("column") },
+	{ "avoid-page", SLEN("avoid_page") },
+	{ "avoid-column", SLEN("avoid-column") },
+	{ "balance", SLEN("balance") },
 
 	{ "aliceblue", SLEN("aliceblue") },
 	{ "antiquewhite", SLEN("antiquewhite") },
@@ -511,3 +553,57 @@ const stringmap_entry stringmap[LAST_KNOWN] = {
 };
 
 
+/**
+ * Obtain pointer to interned propstring list
+ *
+ * \param sheet	    Returns pointer to propstring table
+ * \return CSS_OK on success,
+ *	   CSS_NOMEM on memory exhaustion
+ *
+ * The propstring list is generated with the first call to this function and
+ * destroyed when it has no more users.  Call css__propstrings_unref() when
+ * finished with the propstring list.
+ */
+css_error css__propstrings_get(lwc_string ***strings)
+{
+	if (css__propstrings.count > 0) {
+		css__propstrings.count++;
+	} else {
+		int i;
+		lwc_error lerror;
+
+		/* Intern all known strings */
+		for (i = 0; i < LAST_KNOWN; i++) {
+			lerror = lwc_intern_string(stringmap[i].data,
+					stringmap[i].len,
+					&css__propstrings.strings[i]);
+
+			if (lerror != lwc_error_ok)
+				return CSS_NOMEM;
+		}
+		css__propstrings.count++;
+	}
+
+	*strings = css__propstrings.strings;
+
+	return CSS_OK;
+}
+
+/**
+ * Reduce reference count for propstring list by one.
+ *
+ * When count hits zero, the list is destroyed.
+ */
+void css__propstrings_unref(void)
+{
+	css__propstrings.count--;
+
+	if (css__propstrings.count == 0) {
+		int i;
+
+		for (i = 0; i < LAST_KNOWN; i++)
+			lwc_string_unref(css__propstrings.strings[i]);
+	}
+}
+
+
