@@ -15,6 +15,7 @@
 #include "utils/parserutilserror.h"
 #include "utils/utils.h"
 #include "select/dispatch.h"
+#include "select/font_face.h"
 
 static css_error _add_selectors(css_stylesheet *sheet, css_rule *rule);
 static css_error _remove_selectors(css_stylesheet *sheet, css_rule *rule);
@@ -140,6 +141,12 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 		return CSS_NOMEM;
 
 	memset(sheet, 0, sizeof(css_stylesheet));
+
+	error = css__propstrings_get(&sheet->propstrings);
+	if (error != CSS_OK) {
+		alloc(sheet, 0, alloc_pw);
+		return error;
+	}
 	
 	sheet->inline_style = params->inline_style;
 
@@ -156,6 +163,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 	}
 
 	if (error != CSS_OK) {
+		css__propstrings_unref();
 		alloc(sheet, 0, alloc_pw);
 		return error;
 	}
@@ -169,6 +177,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 				&optparams);
 		if (error != CSS_OK) {
 			css__parser_destroy(sheet->parser);
+			css__propstrings_unref();
 			alloc(sheet, 0, alloc_pw);
 			return error;
 		}
@@ -179,6 +188,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 			&sheet->parser_frontend);
 	if (error != CSS_OK) {
 		css__parser_destroy(sheet->parser);
+		css__propstrings_unref();
 		alloc(sheet, 0, alloc_pw);
 		return error;
 	}
@@ -188,6 +198,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 	if (error != CSS_OK) {
 		css__language_destroy(sheet->parser_frontend);
 		css__parser_destroy(sheet->parser);
+		css__propstrings_unref();
 		alloc(sheet, 0, alloc_pw);
 		return error;
 	}
@@ -198,6 +209,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 		css__selector_hash_destroy(sheet->selectors);
 		css__language_destroy(sheet->parser_frontend);
 		css__parser_destroy(sheet->parser);
+		css__propstrings_unref();
 		alloc(sheet, 0, alloc_pw);
 		return CSS_NOMEM;
 	}
@@ -211,6 +223,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
 			css__selector_hash_destroy(sheet->selectors);
 			css__language_destroy(sheet->parser_frontend);
 			css__parser_destroy(sheet->parser);
+			css__propstrings_unref();
 			alloc(sheet, 0, alloc_pw);
 			return CSS_NOMEM;
 		}
@@ -249,7 +262,7 @@ css_error css_stylesheet_create(const css_stylesheet_params *params,
  */
 css_error css_stylesheet_destroy(css_stylesheet *sheet)
 {
-	uint32_t string_index;
+	uint32_t index;
 	css_rule *r, *s;
 
 	if (sheet == NULL)
@@ -284,15 +297,17 @@ css_error css_stylesheet_destroy(css_stylesheet *sheet)
 		css__stylesheet_style_destroy(sheet->cached_style);
 
 	/* destroy string vector */
-	for (string_index = 0;
-	     string_index < sheet->string_vector_c;
-	     string_index++) {
-		lwc_string_unref(sheet->string_vector[string_index]);		
+	for (index = 0;
+	     index < sheet->string_vector_c;
+	     index++) {
+		lwc_string_unref(sheet->string_vector[index]);		
 	}
 
 	if (sheet->string_vector != NULL)
 		sheet->alloc(sheet->string_vector, 0, sheet->pw);
 
+	css__propstrings_unref();
+	
 	sheet->alloc(sheet, 0, sheet->pw);
 
 	return CSS_OK;
@@ -611,7 +626,7 @@ css_error css_stylesheet_size(css_stylesheet *sheet, size_t *size)
 
 		bytes += hash_size;
 	}
-
+	
 	*size = bytes;
 
 	return CSS_OK;
@@ -1180,10 +1195,10 @@ css_error css__stylesheet_rule_destroy(css_stylesheet *sheet, css_rule *rule)
 		break;
 	case CSS_RULE_FONT_FACE:
 	{
-		css_rule_font_face *font_face = (css_rule_font_face *) rule;
+		css_rule_font_face *font_face_r = (css_rule_font_face *) rule;
 
-		if (font_face->style != NULL)
-			css__stylesheet_style_destroy(font_face->style);
+		if (font_face_r->font_face != NULL)
+			css__font_face_destroy(font_face_r->font_face);
 	}
 		break;
 	case CSS_RULE_PAGE:
@@ -1691,8 +1706,8 @@ size_t _rule_size(const css_rule *r)
 
 		bytes += sizeof(css_rule_font_face);
 
-		if (rf->style != NULL)
-			bytes += (rf->style->used * sizeof(css_code_t));
+		if (rf->font_face != NULL)
+			bytes += sizeof(css_font_face);
 	} else if (r->type == CSS_RULE_PAGE) {
 		const css_rule_page *rp = (const css_rule_page *) r;
 		const css_selector *s = rp->selector;
@@ -1717,4 +1732,3 @@ size_t _rule_size(const css_rule *r)
 
 	return bytes;
 }
-
