@@ -8,6 +8,7 @@
 #include "stylesheet.h"
 #include "bytecode/bytecode.h"
 #include "bytecode/opcodes.h"
+#include "select/font_face.h"
 
 #include "testutils.h"
 
@@ -18,11 +19,14 @@ static void dump_rule_charset(css_rule_charset *s, char **buf, size_t *buflen);
 static void dump_rule_import(css_rule_import *s, char **buf, size_t *buflen);
 static void dump_rule_media(css_rule_media *s, char **buf, size_t *buflen);
 static void dump_rule_page(css_rule_page *s, char **buf, size_t *buflen);
+static void dump_rule_font_face(css_rule_font_face *s, 
+		char **buf, size_t *buflen);
 static void dump_selector_list(css_selector *list, char **ptr);
 static void dump_selector(css_selector *selector, char **ptr);
 static void dump_selector_detail(css_selector_detail *detail, char **ptr);
 static void dump_bytecode(css_style *style, char **ptr, uint32_t depth);
 static void dump_string(lwc_string *string, char **ptr);
+static void dump_font_face(css_font_face *font_face, char**ptr);
 
 void dump_sheet(css_stylesheet *sheet, char *buf, size_t *buflen)
 {
@@ -50,6 +54,10 @@ void dump_sheet(css_stylesheet *sheet, char *buf, size_t *buflen)
 			dump_rule_page((css_rule_page *) rule,
 				&buf, buflen);
 			break;
+		case CSS_RULE_FONT_FACE:
+			dump_rule_font_face((css_rule_font_face *) rule,
+				&buf, buflen);
+			break;
 		default:
 		{
 			int written = sprintf(buf, "Unhandled rule type %d\n",
@@ -157,6 +165,22 @@ void dump_rule_page(css_rule_page *s, char **buf, size_t *buflen)
 	*buf = ptr;
 }
 
+void dump_rule_font_face(css_rule_font_face *s, char **buf, size_t *buflen)
+{
+	char *ptr = *buf;
+
+	ptr += sprintf(ptr, "| @font-face ");
+
+	if (s->font_face != NULL) {
+		dump_font_face(s->font_face, &ptr);
+	}
+	
+	*ptr++ = '\n';
+
+	*buflen -= ptr - *buf;
+	*buf = ptr;
+}
+
 void dump_selector_list(css_selector *list, char **ptr)
 {
 	if (list->combinator != NULL) {
@@ -441,6 +465,17 @@ static const char *opcode_names[] = {
 	"word-spacing",
 	"z-index",
 	"opacity",
+	"break-after",
+	"break-before",
+	"break-inside",
+	"column-count",
+	"column-fill",
+	"column-gap",
+	"column-rule-color",
+	"column-rule-style",
+	"column-rule-width",
+	"column-span",
+	"column-width",
 };
 
 static void dump_css_fixed(css_fixed f, char **ptr)
@@ -777,6 +812,7 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 			case CSS_PROP_BORDER_BOTTOM_COLOR:
 			case CSS_PROP_BORDER_LEFT_COLOR:
 			case CSS_PROP_BACKGROUND_COLOR:
+			case CSS_PROP_COLUMN_RULE_COLOR:
 				assert(BACKGROUND_COLOR_TRANSPARENT ==
 						(enum op_background_color)
 						BORDER_COLOR_TRANSPARENT);
@@ -833,7 +869,8 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 					break;
 				case BACKGROUND_IMAGE_URI:
 				{
-					uint32_t snum = *((uint32_t *) bytecode);					lwc_string *he;
+					uint32_t snum = *((uint32_t *) bytecode);
+					lwc_string *he;
 					css__stylesheet_string_get(style->sheet, 
 								  snum, 
 								  &he);
@@ -940,37 +977,68 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 			case CSS_PROP_BORDER_RIGHT_STYLE:
 			case CSS_PROP_BORDER_BOTTOM_STYLE:
 			case CSS_PROP_BORDER_LEFT_STYLE:
+			case CSS_PROP_COLUMN_RULE_STYLE:
 			case CSS_PROP_OUTLINE_STYLE:
 				assert(BORDER_STYLE_NONE ==
 						(enum op_border_style)
 						OUTLINE_STYLE_NONE);
+				assert(BORDER_STYLE_NONE ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_NONE);
 				assert(BORDER_STYLE_HIDDEN ==
 						(enum op_border_style)
 						OUTLINE_STYLE_HIDDEN);
+				assert(BORDER_STYLE_HIDDEN ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_HIDDEN);
 				assert(BORDER_STYLE_DOTTED ==
 						(enum op_border_style)
 						OUTLINE_STYLE_DOTTED);
+				assert(BORDER_STYLE_DOTTED ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_DOTTED);
 				assert(BORDER_STYLE_DASHED ==
 						(enum op_border_style)
 						OUTLINE_STYLE_DASHED);
+				assert(BORDER_STYLE_DASHED ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_DASHED);
 				assert(BORDER_STYLE_SOLID ==
 						(enum op_border_style)
 						OUTLINE_STYLE_SOLID);
+				assert(BORDER_STYLE_SOLID ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_SOLID);
 				assert(BORDER_STYLE_DOUBLE ==
 						(enum op_border_style)
 						OUTLINE_STYLE_DOUBLE);
+				assert(BORDER_STYLE_DOUBLE ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_DOUBLE);
 				assert(BORDER_STYLE_GROOVE ==
 						(enum op_border_style)
 						OUTLINE_STYLE_GROOVE);
+				assert(BORDER_STYLE_GROOVE ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_GROOVE);
 				assert(BORDER_STYLE_RIDGE ==
 						(enum op_border_style)
 						OUTLINE_STYLE_RIDGE);
+				assert(BORDER_STYLE_RIDGE ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_RIDGE);
 				assert(BORDER_STYLE_INSET ==
 						(enum op_border_style)
 						OUTLINE_STYLE_INSET);
+				assert(BORDER_STYLE_INSET ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_INSET);
 				assert(BORDER_STYLE_OUTSET ==
 						(enum op_border_style)
 						OUTLINE_STYLE_OUTSET);
+				assert(BORDER_STYLE_OUTSET ==
+						(enum op_border_style)
+						COLUMN_RULE_STYLE_OUTSET);
 
 				switch (value) {
 				case BORDER_STYLE_NONE:
@@ -1009,6 +1077,7 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 			case CSS_PROP_BORDER_RIGHT_WIDTH:
 			case CSS_PROP_BORDER_BOTTOM_WIDTH:
 			case CSS_PROP_BORDER_LEFT_WIDTH:
+			case CSS_PROP_COLUMN_RULE_WIDTH:
 			case CSS_PROP_OUTLINE_WIDTH:
 				assert(BORDER_WIDTH_SET ==
 						(enum op_border_width)
@@ -1055,6 +1124,7 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 			case CSS_PROP_TOP:
 			case CSS_PROP_HEIGHT:
 			case CSS_PROP_WIDTH:
+			case CSS_PROP_COLUMN_WIDTH:
 				assert(BOTTOM_SET ==
 						(enum op_bottom) LEFT_SET);
 				assert(BOTTOM_AUTO ==
@@ -1079,6 +1149,12 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 						(enum op_bottom) WIDTH_SET);
 				assert(BOTTOM_AUTO ==
 						(enum op_bottom) WIDTH_AUTO);
+				assert(BOTTOM_SET ==
+						(enum op_bottom)
+							COLUMN_WIDTH_SET);
+				assert(BOTTOM_AUTO ==
+						(enum op_bottom)
+							COLUMN_WIDTH_AUTO);
 
 				switch (value) {
 				case BOTTOM_SET:
@@ -1096,6 +1172,82 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 					break;
 				}
 				break;
+			case CSS_PROP_BREAK_AFTER:
+			case CSS_PROP_BREAK_BEFORE:
+				assert(BREAK_AFTER_AUTO ==
+						(enum op_break_after)
+						BREAK_BEFORE_AUTO);
+				assert(BREAK_AFTER_ALWAYS ==
+						(enum op_break_after)
+						BREAK_BEFORE_ALWAYS);
+				assert(BREAK_AFTER_AVOID ==
+						(enum op_break_after)
+						BREAK_BEFORE_AVOID);
+				assert(BREAK_AFTER_LEFT ==
+						(enum op_break_after)
+						BREAK_BEFORE_LEFT);
+				assert(BREAK_AFTER_RIGHT ==
+						(enum op_break_after)
+						BREAK_BEFORE_RIGHT);
+				assert(BREAK_AFTER_PAGE ==
+						(enum op_break_after)
+						BREAK_BEFORE_PAGE);
+				assert(BREAK_AFTER_COLUMN ==
+						(enum op_break_after)
+						BREAK_BEFORE_COLUMN);
+				assert(BREAK_AFTER_AVOID_PAGE ==
+						(enum op_break_after)
+						BREAK_BEFORE_AVOID_PAGE);
+				assert(BREAK_AFTER_AVOID_COLUMN ==
+						(enum op_break_after)
+						BREAK_BEFORE_AVOID_COLUMN);
+
+				switch (value) {
+				case BREAK_AFTER_AUTO:
+					*ptr += sprintf(*ptr, "auto");
+					break;
+				case BREAK_AFTER_ALWAYS:
+					*ptr += sprintf(*ptr, "always");
+					break;
+				case BREAK_AFTER_AVOID:
+					*ptr += sprintf(*ptr, "avoid");
+					break;
+				case BREAK_AFTER_LEFT:
+					*ptr += sprintf(*ptr, "left");
+					break;
+				case BREAK_AFTER_RIGHT:
+					*ptr += sprintf(*ptr, "right");
+					break;
+				case BREAK_AFTER_PAGE:
+					*ptr += sprintf(*ptr, "page");
+					break;
+				case BREAK_AFTER_COLUMN:
+					*ptr += sprintf(*ptr, "column");
+					break;
+				case BREAK_AFTER_AVOID_PAGE:
+					*ptr += sprintf(*ptr, "avoid-page");
+					break;
+				case BREAK_AFTER_AVOID_COLUMN:
+					*ptr += sprintf(*ptr, "avoid-column");
+					break;
+				}
+				break;
+			case CSS_PROP_BREAK_INSIDE:
+				switch (value) {
+				case BREAK_INSIDE_AUTO:
+					*ptr += sprintf(*ptr, "auto");
+					break;
+				case BREAK_INSIDE_AVOID:
+					*ptr += sprintf(*ptr, "avoid");
+					break;
+				case BREAK_INSIDE_AVOID_PAGE:
+					*ptr += sprintf(*ptr, "avoid-page");
+					break;
+				case BREAK_INSIDE_AVOID_COLUMN:
+					*ptr += sprintf(*ptr, "avoid-column");
+					break;
+				}
+				break;
 			case CSS_PROP_CAPTION_SIDE:
 				switch (value) {
 				case CAPTION_SIDE_TOP:
@@ -1195,6 +1347,57 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
 					break;
 				}
 				break;
+			case CSS_PROP_COLUMN_COUNT:
+				switch (value) {
+				case COLUMN_COUNT_SET:
+				{
+					css_fixed val = *((css_fixed *) bytecode);
+					ADVANCE(sizeof(val));
+					dump_number(val, ptr);
+				}
+					break;
+				case COLUMN_COUNT_AUTO:
+					*ptr += sprintf(*ptr, "auto");
+					break;
+				}
+				break;
+			case CSS_PROP_COLUMN_FILL:
+				switch (value) {
+				case COLUMN_FILL_BALANCE:
+					*ptr += sprintf(*ptr, "balance");
+					break;
+				case COLUMN_FILL_AUTO:
+					*ptr += sprintf(*ptr, "auto");
+					break;
+				}
+				break;
+			case CSS_PROP_COLUMN_GAP:
+				switch (value) {
+				case COLUMN_GAP_SET:
+				{
+					uint32_t unit;
+					css_fixed val = *((css_fixed *) bytecode);
+					ADVANCE(sizeof(val));
+					unit = *((uint32_t *) bytecode);
+					ADVANCE(sizeof(unit));
+					dump_unit(val, unit, ptr);
+				}
+					break;
+				case COLUMN_GAP_NORMAL:
+					*ptr += sprintf(*ptr, "normal");
+					break;
+				}
+				break;
+			case CSS_PROP_COLUMN_SPAN:
+				switch (value) {
+				case COLUMN_SPAN_NONE:
+					*ptr += sprintf(*ptr, "none");
+					break;
+				case COLUMN_SPAN_ALL:
+					*ptr += sprintf(*ptr, "all");
+					break;
+				}
+				break;
 			case CSS_PROP_CONTENT:
 				if (value == CONTENT_NORMAL) {
 					*ptr += sprintf(*ptr, "normal");
@@ -2017,9 +2220,7 @@ void dump_bytecode(css_style *style, char **ptr, uint32_t depth)
                                                                 (int) lwc_string_length(he), 
                                                                 lwc_string_data(he));
 
-						he = 
-						*((lwc_string **) 
-						bytecode);
+						css__stylesheet_string_get(style->sheet, snum, &he);
 						ADVANCE(sizeof(he));
 						*ptr += sprintf(*ptr, " '%.*s' ", 
                                                                 (int) lwc_string_length(he), 
@@ -2365,4 +2566,137 @@ void dump_string(lwc_string *string, char **ptr)
                         lwc_string_data(string));
 }
 
+void dump_font_face(css_font_face *font_face, char **ptr)
+{
+	uint8_t style, weight;
+
+	if (font_face->font_family != NULL) {
+		*(*ptr)++ = '\n';
+		*ptr += sprintf(*ptr, "|  font-family: %.*s",
+				(int) lwc_string_length(font_face->font_family),
+				lwc_string_data(font_face->font_family));
+	}
+	
+	*ptr += sprintf(*ptr, "\n|  font-style: ");
+	style = css_font_face_font_style(font_face);
+	switch (style) {
+	case CSS_FONT_STYLE_INHERIT:
+		*ptr += sprintf(*ptr, "unspecified");
+		break;
+	case CSS_FONT_STYLE_NORMAL:
+		*ptr += sprintf(*ptr, "normal");
+		break;
+	case CSS_FONT_STYLE_ITALIC:
+		*ptr += sprintf(*ptr, "italic");
+		break;
+	case CSS_FONT_STYLE_OBLIQUE:
+		*ptr += sprintf(*ptr, "oblique");
+		break;
+	}
+
+	*ptr += sprintf(*ptr, "\n|  font-weight: ");
+	weight = css_font_face_font_weight(font_face);
+	switch (weight) {
+	case CSS_FONT_WEIGHT_INHERIT:
+		*ptr += sprintf(*ptr, "unspecified");
+		break;
+	case CSS_FONT_WEIGHT_NORMAL:
+		*ptr += sprintf(*ptr, "normal");
+		break;
+	case CSS_FONT_WEIGHT_BOLD:
+		*ptr += sprintf(*ptr, "bold");
+		break;
+	case CSS_FONT_WEIGHT_100:
+		*ptr += sprintf(*ptr, "100");
+		break;
+	case CSS_FONT_WEIGHT_200:
+		*ptr += sprintf(*ptr, "200");
+		break;
+	case CSS_FONT_WEIGHT_300:
+		*ptr += sprintf(*ptr, "300");
+		break;
+	case CSS_FONT_WEIGHT_400:
+		*ptr += sprintf(*ptr, "400");
+		break;
+	case CSS_FONT_WEIGHT_500:
+		*ptr += sprintf(*ptr, "500");
+		break;
+	case CSS_FONT_WEIGHT_600:
+		*ptr += sprintf(*ptr, "600");
+		break;
+	case CSS_FONT_WEIGHT_700:
+		*ptr += sprintf(*ptr, "700");
+		break;
+	case CSS_FONT_WEIGHT_800:
+		*ptr += sprintf(*ptr, "800");
+		break;
+	case CSS_FONT_WEIGHT_900:
+		*ptr += sprintf(*ptr, "900");
+		break;
+	default:
+		*ptr += sprintf(*ptr, "Unhandled weight %d\n", (int)weight);
+		break;
+	}
+
+	
+	if (font_face->srcs != NULL) {
+		uint32_t i;
+		css_font_face_src *srcs = font_face->srcs;
+		for (i = 0; i < font_face->n_srcs; ++i) {
+			*ptr += sprintf(*ptr, "\n|  src: ");
+			
+			css_font_face_format format = 
+					css_font_face_src_format(&srcs[i]);
+			
+			*ptr += sprintf(*ptr, "\n|   format: ");
+			
+			switch (format) {
+			case CSS_FONT_FACE_FORMAT_UNSPECIFIED:
+				*ptr += sprintf(*ptr, "unspecified");
+				break;
+			case CSS_FONT_FACE_FORMAT_WOFF:
+				*ptr += sprintf(*ptr, "WOFF");
+				break;
+			case CSS_FONT_FACE_FORMAT_OPENTYPE:
+				*ptr += sprintf(*ptr, "OTF");
+				break;
+			case CSS_FONT_FACE_FORMAT_EMBEDDED_OPENTYPE:
+				*ptr += sprintf(*ptr, "EOTF");
+				break;
+			case CSS_FONT_FACE_FORMAT_SVG:
+				*ptr += sprintf(*ptr, "SVG");
+				break;
+			case CSS_FONT_FACE_FORMAT_UNKNOWN:
+				*ptr += sprintf(*ptr, "unknown");
+				break;
+			default:
+				*ptr += sprintf(*ptr, "UNEXPECTED");
+				break;
+			}
+
+			if (srcs[i].location != NULL) {
+				*ptr += sprintf(*ptr, "\n|   location: ");
+
+				switch (css_font_face_src_location_type(
+						&srcs[i])) {
+				case CSS_FONT_FACE_LOCATION_TYPE_LOCAL:
+					*ptr += sprintf(*ptr, "local");
+					break;
+				case CSS_FONT_FACE_LOCATION_TYPE_URI:
+					*ptr += sprintf(*ptr, "url");
+					break;
+				default:
+					*ptr += sprintf(*ptr, "UNKNOWN");
+					break;
+				}
+				
+				*ptr += sprintf(*ptr, "(%.*s)",
+					(int) lwc_string_length(
+							srcs[i].location), 
+					lwc_string_data(srcs[i].location));
+			}
+		}
+	}
+}
+
 #endif
