@@ -13,18 +13,18 @@
 #include "libnsfb_plot.h"
 #include "libnsfb_event.h"
 #include "nsfb.h"
-#include "frontend.h"
+#include "surface.h"
 
 #define UNUSED(x) ((x) = (x))
 
-static int able_set_geometry(nsfb_t *nsfb, int width, int height, int bpp)
+static int able_set_geometry(nsfb_t *nsfb, int width, int height, enum nsfb_format_e format)
 {
-    if (nsfb->frontend_priv != NULL)
+    if (nsfb->surface_priv != NULL)
         return -1; /* if were already initialised fail */
 
     nsfb->width = width;
     nsfb->height = height;
-    nsfb->bpp = bpp;
+    nsfb->format = format;
 
     return 0;
 }
@@ -49,11 +49,11 @@ static bool able_input(nsfb_t *nsfb, nsfb_event_t *event, int timeout)
     return false;
 }
 
-const nsfb_frontend_rtns_t able_rtns = {
+const nsfb_surface_rtns_t able_rtns = {
     .initialise = able_initialise,
     .finalise = able_finalise,
     .input = able_input,
     .geometry = able_set_geometry,
 };
 
-NSFB_FRONTEND_DEF(able, NSFB_FRONTEND_ABLE, &able_rtns)
+NSFB_SURFACE_DEF(able, NSFB_SURFACE_ABLE, &able_rtns)
