@@ -37,8 +37,10 @@ struct bitmap {
 	int nativebmheight;
 	char *url;   /* temporary storage space */
 	char *title; /* temporary storage space */
+	ULONG *icondata; /* for appicons */
 };
 
 struct BitMap *ami_getcachenativebm(struct bitmap *bitmap,int width,int height,struct BitMap *friendbm);
 Object *ami_datatype_object_from_bitmap(struct bitmap *bitmap);
+struct bitmap *ami_bitmap_from_datatype(char *filename);
 #endif
