@@ -1,5 +1,5 @@
 /*
- * Copyright 2008-2011 Chris Young <chris@unsatisfactorysoftware.co.uk>
+ * Copyright 2008-2012 Chris Young <chris@unsatisfactorysoftware.co.uk>
  *
  * This file is part of NetSurf, http://www.netsurf-browser.org/
  *
@@ -111,6 +111,7 @@ void gui_paste_from_clipboard(struct gui_window *g, int x, int y)
 	struct ContextNode *cn;
 	ULONG rlen=0,error;
 	struct CSet cset;
+	LONG codeset = 0;
 	char *clip;
 	STRPTR readbuf = AllocVec(1024,MEMF_PRIVATE | MEMF_CLEAR);
 
@@ -130,14 +131,16 @@ void gui_paste_from_clipboard(struct gui_window *g, int x, int y)
 
 		if((cn)&&(cn->cn_Type == ID_FTXT)&&(cn->cn_ID == ID_CSET))
 		{
-			rlen = ReadChunkBytes(iffh,&cset,24);
+			rlen = ReadChunkBytes(iffh,&cset,32);
+			if(cset.CodeSet == 1) codeset = 106;
+				else codeset = cset.CodeSet;
 		}
 
 		if((cn)&&(cn->cn_Type == ID_FTXT)&&(cn->cn_ID == ID_CHRS))
 		{
 			while((rlen = ReadChunkBytes(iffh,readbuf,1024)) > 0)
 			{
-				if(cset.CodeSet == 0)
+				if(codeset == 0)
 				{
 					utf8_from_local_encoding(readbuf,rlen,&clip);
 				}
@@ -145,7 +148,7 @@ void gui_paste_from_clipboard(struct gui_window *g, int x, int y)
 				{
 					utf8_from_enc(readbuf,
 						(const char *)ObtainCharsetInfo(DFCS_NUMBER,
-										cset.CodeSet, DFCS_MIMENAME),
+										codeset, DFCS_MIMENAME),
 						rlen, &clip);
 				}
 
@@ -169,10 +172,10 @@ bool gui_empty_clipboard(void)
 		{
 			if(option_utf8_clipboard || ami_utf8_clipboard)
 			{
-				if(!(PushChunk(iffh,0,ID_CSET,24)))
+				if(!(PushChunk(iffh,0,ID_CSET,32)))
 				{
 					cset.CodeSet = 106; // UTF-8
-					WriteChunkBytes(iffh,&cset,24);
+					WriteChunkBytes(iffh,&cset,32);
 					PopChunk(iffh);
 				}
 			}
@@ -283,15 +286,17 @@ bool ami_clipboard_copy(const char *text, size_t length, struct box *box,
 
 bool gui_copy_to_clipboard(struct selection *s)
 {
+	bool success;
+
+	if(s->defined == false) return false;
 	if(!gui_empty_clipboard()) return false;
 
-	if (s->defined && selection_traverse(s, ami_clipboard_copy, NULL))
-	{
-		gui_commit_clipboard();
-		return true;
-	}
+	success = selection_traverse(s, ami_clipboard_copy, NULL);
 
-	return false;
+	/* commit regardless, otherwise we leave the clipboard in an unusable state */
+	gui_commit_clipboard();
+
+	return success;
 }
 
 struct ami_text_selection *ami_selection_to_text(struct gui_window_2 *gwin)
@@ -334,7 +339,7 @@ void ami_drag_selection(struct selection *s)
 	char *utf8text;
 	struct ami_text_selection *sel;
 	struct IFFHandle *old_iffh = iffh;
-	struct gui_window_2 *gwin = ami_window_at_pointer();
+	struct gui_window_2 *gwin = ami_window_at_pointer(AMINS_WINDOW);
 
 	if(!gwin)
 	{
@@ -426,7 +431,7 @@ bool ami_easy_clipboard_svg(struct hlcache_handle *c)
 
 	if(!(OpenIFF(iffh,IFFF_WRITE)))
 	{
-		ami_svg_to_dr2d(iffh, source_data, source_size, content_get_url(c));
+		ami_svg_to_dr2d(iffh, source_data, source_size, nsurl_access(hlcache_handle_get_url(c)));
 		CloseIFF(iffh);
 	}
 
