@@ -35,5 +35,5 @@ struct dlnode
 void ami_download_window_abort(struct gui_download_window *dw);
 BOOL ami_download_window_event(struct gui_download_window *dw);
 void ami_free_download_list(struct List *dllist);
-BOOL ami_download_check_overwrite(const char *file, struct Window *win);
+BOOL ami_download_check_overwrite(const char *file, struct Window *win, ULONG size);
 #endif
