@@ -31,19 +31,16 @@
 #endif
 #include <workbench/icon.h>
 
+#include "amiga/bitmap.h"
 #include "amiga/clipboard.h"
 #include "amiga/download.h"
 #include "amiga/drag.h"
+#include "amiga/file.h"
 #include "amiga/filetype.h"
 #include "amiga/options.h"
-#include "amiga/icon.h"
-#include "amiga/iff_dr2d.h"
 #include "amiga/theme.h"
 
 #include "desktop/selection.h"
-#include "desktop/save_complete.h"
-
-#include "amiga/bitmap.h"
 
 #include "utils/errors.h"
 #include "utils/log.h"
@@ -156,25 +153,13 @@ void ami_drag_save(struct Window *win)
 			BPTR fh = 0;
 			AddPart(path, content_get_title(c), 1024);
 
-			if(!ami_download_check_overwrite(path, win, 0))
-				break;
-
-			if(fh = FOpen(path,MODE_NEWFILE,0))
-			{
-				if((source_data = content_get_source_data(c, &source_size)))
-					FWrite(fh, source_data, 1, source_size);
-
-				FClose(fh);
-				SetComment(path, content_get_url(c));
-			}
+			ami_file_save(AMINS_SAVE_SOURCE, path, win, c, NULL, NULL);
 		}
 		break;
 
 		case GUI_SAVE_TEXT_SELECTION: // selection
-			AddPart(path,"netsurf_text_file",1024);
-			if(!ami_download_check_overwrite(path, win, 0))
-				break;
-			selection_save_text((struct selection *)drag_save_data,path);
+			AddPart(path,"netsurf_text_selection",1024);
+			ami_file_save(AMINS_SAVE_SELECTION, path, win, NULL, NULL, drag_save_data);
 		break;
 
 		case GUI_SAVE_COMPLETE:
@@ -183,40 +168,17 @@ void ami_drag_save(struct Window *win)
 			BPTR lock = 0;
 
 			AddPart(path, content_get_title(c), 1024);
-			if(!ami_download_check_overwrite(path, win, 0))
-				break;
 
-			if(lock = CreateDir(path))
-			{
-				UnLock(lock);
-				save_complete(c,path);
-				SetComment(path, content_get_url(c));
-			}
-			ami_superimpose_favicon(path,
-				drag_save_gui->favicon, NULL);
+			ami_file_save(AMINS_SAVE_COMPLETE, path, win, c, drag_save_gui->favicon, NULL);
 		}
 		break;
 
 		case GUI_SAVE_OBJECT_NATIVE:
 		{
 			hlcache_handle *c = drag_save_data;
-			struct bitmap *bm;
-
 			AddPart(path, content_get_title(c), 1024);
-			if(bm = content_get_bitmap(c))
-			{
-				bm->url = (char *)content_get_url(c);
-				bm->title = (char *)content_get_title(c);
-				if(bitmap_save(bm, path, 0))
-					SetComment(path, content_get_url(c));
-			}
-#ifdef WITH_NS_SVG
-			else if(ami_mime_compare(c, "svg") == true)
-			{
-				if(ami_save_svg(c, path))
-					SetComment(path, content_get_url(c));
-			}
-#endif
+
+			ami_file_save(AMINS_SAVE_IFF, path, win, c, NULL, NULL);
 		}
 		break;
 
@@ -314,7 +276,7 @@ BOOL ami_drag_in_progress(void)
 	return drag_in_progress;
 }
 
-struct gui_window_2 *ami_find_gwin_by_id(struct Window *win)
+void *ami_find_gwin_by_id(struct Window *win, int type)
 {
 	struct nsObject *node, *nnode;
 	struct gui_window_2 *gwin;
@@ -327,7 +289,7 @@ struct gui_window_2 *ami_find_gwin_by_id(struct Window *win)
 		{
 			nnode=(struct nsObject *)GetSucc((struct Node *)node);
 
-			if(node->Type == AMINS_WINDOW) // or frame?
+			if(node->Type == type)
 			{
 				gwin = node->objstruct;
 				if(win == gwin->win) return gwin;
@@ -337,7 +299,7 @@ struct gui_window_2 *ami_find_gwin_by_id(struct Window *win)
 	return NULL;
 }
 
-struct gui_window_2 *ami_window_at_pointer(void)
+void *ami_window_at_pointer(int type)
 {
 	struct Layer *layer;
 
@@ -347,6 +309,6 @@ struct gui_window_2 *ami_window_at_pointer(void)
 
 	UnlockLayerInfo(&scrn->LayerInfo);
 
-	if(layer) return ami_find_gwin_by_id(layer->Window);
+	if(layer) return ami_find_gwin_by_id(layer->Window, type);
 		else return NULL;
 }
