@@ -32,5 +32,5 @@ void ami_drag_icon_close(struct Window *win);
 void ami_drag_icon_move(void);
 BOOL ami_drag_in_progress(void);
 
-struct gui_window_2 *ami_window_at_pointer(void);
+void *ami_window_at_pointer(int type);
 #endif
