@@ -52,6 +52,8 @@
 #define NSA_ITALIC 1
 #define NSA_BOLD 2
 #define NSA_BOLDITALIC 3
+#define NSA_OBLIQUE 4
+#define NSA_BOLDOBLIQUE 6
 
 #define NSA_VALUE_BOLDX (1 << 12)
 #define NSA_VALUE_BOLDY 0
@@ -70,6 +72,7 @@ struct ami_font_node
 };
 
 struct MinList *ami_font_list = NULL;
+struct List ami_diskfontlib_list;
 ULONG ami_devicedpi;
 ULONG ami_xdpi;
 
@@ -331,18 +334,13 @@ struct ami_font_node *ami_font_open(const char *font)
 	LOG(("Font cache miss: %s", font));
 
 	nodedata = AllocVec(sizeof(struct ami_font_node), MEMF_PRIVATE | MEMF_CLEAR);
+	nodedata->font = OpenOutlineFont(font, &ami_diskfontlib_list, OFF_OPEN);
 
-	node = AddObject(ami_font_list, AMINS_FONT);
-	if(!node) return NULL;
-
-	node->objstruct = nodedata;
-	node->dtz_Node.ln_Name = strdup(font);
-
-	nodedata->font = OpenOutlineFont(font, NULL, OFF_OPEN);
 	if(!nodedata->font)
 	{
 		LOG(("Requested font not found: %s", font));
 		warn_user("CompError", font);
+		FreeVec(nodedata);
 		return NULL;
 	}
 
@@ -366,6 +364,13 @@ struct ami_font_node *ami_font_open(const char *font)
 
 	GetSysTime(&nodedata->lastused);
 
+	node = AddObject(ami_font_list, AMINS_FONT);
+	if(node)
+	{
+		node->objstruct = nodedata;
+		node->dtz_Node.ln_Name = strdup(font);
+	}
+
 	return nodedata;
 }
 
@@ -418,8 +423,11 @@ struct OutlineFont *ami_open_outline_font(const plot_font_style_t *fstyle, BOOL
 	node = ami_font_open(fontname);
 	if(!node) return NULL;
 
-	if ((fstyle->flags & FONTF_ITALIC) || (fstyle->flags & FONTF_OBLIQUE))
-		tstyle += NSA_ITALIC;
+	if (fstyle->flags & FONTF_OBLIQUE)
+		tstyle = NSA_OBLIQUE;
+
+	if (fstyle->flags & FONTF_ITALIC)
+		tstyle = NSA_ITALIC;
 
 	if (fstyle->weight >= 700)
 		tstyle += NSA_BOLD;
@@ -439,6 +447,11 @@ struct OutlineFont *ami_open_outline_font(const plot_font_style_t *fstyle, BOOL
 			}
 		break;
 
+		case NSA_OBLIQUE:
+			shearsin = NSA_VALUE_SHEARSIN;
+			shearcos = NSA_VALUE_SHEARCOS;
+		break;
+
 		case NSA_BOLD:
 			if(node->bold)
 			{
@@ -452,6 +465,22 @@ struct OutlineFont *ami_open_outline_font(const plot_font_style_t *fstyle, BOOL
 			}
 		break;
 
+		case NSA_BOLDOBLIQUE:
+			shearsin = NSA_VALUE_SHEARSIN;
+			shearcos = NSA_VALUE_SHEARCOS;
+
+			if(node->bold)
+			{
+				node = ami_font_open(node->bold);
+				if(!node) return NULL;
+			}
+			else
+			{
+				emboldenx = NSA_VALUE_BOLDX;
+				emboldeny = NSA_VALUE_BOLDY;
+			}
+		break;
+
 		case NSA_BOLDITALIC:
 			if(node->bolditalic)
 			{
@@ -539,9 +568,91 @@ int32 ami_font_plot_glyph(struct OutlineFont *ofont, struct RastPort *rp,
 	return char_advance;
 }
 
+uint16 ami_font_translate_smallcaps(uint16 utf16char)
+{
+	uint16 *p;
+	uint16 sc_table[] = {
+		0x0061, 0x1D00, /* a */
+		0x0062, 0x0299, /* b */
+		0x0063, 0x1D04, /* c */
+		0x0064, 0x1D05, /* d */
+		0x0065, 0x1D07, /* e */
+		0x0066, 0xA730, /* f */
+		0x0067, 0x0262, /* g */
+		0x0068, 0x029C, /* h */
+		0x0069, 0x026A, /* i */
+		0x006A, 0x1D0A, /* j */
+		0x006B, 0x1D0B, /* k */
+		0x006C, 0x029F, /* l */
+		0x006D, 0x1D0D, /* m */
+		0x006E, 0x0274, /* n */
+		0x006F, 0x1D0F, /* o */
+		0x0070, 0x1D18, /* p */
+		0x0071, 0xA7EE, /* q (proposed) (Adobe codepoint 0xF771) */
+		0x0072, 0x0280, /* r */
+		0x0073, 0xA731, /* s */
+		0x0074, 0x1D1B, /* t */
+		0x0075, 0x1D1C, /* u */
+		0x0076, 0x1D20, /* v */
+		0x0077, 0x1D21, /* w */
+		0x0078, 0xA7EF, /* x (proposed) (Adobe codepoint 0xF778) */
+		0x0079, 0x028F, /* y */
+		0x007A, 0x1D22, /* z */
+
+		0x00C6, 0x1D01, /* ae */
+		0x0153, 0x0276, /* oe */
+
+#if 0
+/* TODO: fill in the non-small caps character ids for these */
+		0x0000, 0x1D03, /* barred b */
+		0x0000, 0x0281, /* inverted r */
+		0x0000, 0x1D19, /* reversed r */
+		0x0000, 0x1D1A, /* turned r */
+		0x0000, 0x029B, /* g with hook */
+		0x0000, 0x1D06, /* eth  */
+		0x0000, 0x1D0C, /* l with stroke */
+		0x0000, 0xA7FA, /* turned m */
+		0x0000, 0x1D0E, /* reversed n */
+		0x0000, 0x1D10, /* open o */
+		0x0000, 0x1D15, /* ou */
+		0x0000, 0x1D23, /* ezh */
+		0x0000, 0x1D26, /* gamma */
+		0x0000, 0x1D27, /* lamda */
+		0x0000, 0x1D28, /* pi */
+		0x0000, 0x1D29, /* rho */
+		0x0000, 0x1D2A, /* psi */
+		0x0000, 0x1D2B, /* el */
+		0x0000, 0xA776, /* rum */
+
+		0x0000, 0x1DDB, /* combining g */
+		0x0000, 0x1DDE, /* combining l */
+		0x0000, 0x1DDF, /* combining m */
+		0x0000, 0x1DE1, /* combining n */
+		0x0000, 0x1DE2, /* combining r */
+
+		0x0000, 0x1DA6, /* modifier i */
+		0x0000, 0x1DA7, /* modifier i with stroke */
+		0x0000, 0x1DAB, /* modifier l */
+		0x0000, 0x1DB0, /* modifier n */
+		0x0000, 0x1DB8, /* modifier u */
+#endif
+		0, 0};
+
+	p = &sc_table[0];
+
+	while (*p != 0)
+	{
+		if(*p == utf16char) return p[1];
+		p++;
+	}
+
+	return utf16char;
+}
+
 ULONG ami_unicode_text(struct RastPort *rp,const char *string,ULONG length,const plot_font_style_t *fstyle,ULONG dx, ULONG dy)
 {
 	uint16 *utf16 = NULL, *outf16 = NULL;
+	uint16 utf16charsc = 0, utf16nextsc = 0;
 	uint16 utf16next = 0;
 	int utf16charlen;
 	struct OutlineFont *ofont, *ufont = NULL;
@@ -570,7 +681,17 @@ ULONG ami_unicode_text(struct RastPort *rp,const char *string,ULONG length,const
 
 		utf16next = utf16[utf16charlen];
 
-		tempx = ami_font_plot_glyph(ofont, rp, *utf16, utf16next, dx + x, dy, emwidth);
+		if(fstyle->flags & FONTF_SMALLCAPS)
+		{
+			utf16charsc = ami_font_translate_smallcaps(*utf16);
+			utf16nextsc = ami_font_translate_smallcaps(utf16next);
+
+			tempx = ami_font_plot_glyph(ofont, rp, utf16charsc, utf16nextsc, dx + x, dy, emwidth);
+		}
+		else tempx = 0;
+
+		if(tempx == 0)
+			tempx = ami_font_plot_glyph(ofont, rp, *utf16, utf16next, dx + x, dy, emwidth);
 
 		if(tempx == 0)
 		{
@@ -605,6 +726,7 @@ ULONG ami_unicode_text(struct RastPort *rp,const char *string,ULONG length,const
 void ami_init_fonts(void)
 {
 	ami_font_list = NewObjList();
+	NewList(&ami_diskfontlib_list);
 
 	/* run first cleanup in ten minutes */
 	schedule(60000, (schedule_callback_fn)ami_font_cleanup, ami_font_list);
@@ -621,7 +743,7 @@ void ami_font_close(struct ami_font_node *node)
 {
 	/* Called from FreeObjList if node type is AMINS_FONT */
 
-	CloseOutlineFont(node->font, NULL);
+	CloseOutlineFont(node->font, &ami_diskfontlib_list);
 }
 
 static void ami_font_cleanup(struct MinList *ami_font_list)
@@ -662,7 +784,7 @@ void ami_font_setdevicedpi(int id)
 
 	nscss_screen_dpi = INTTOFIX(option_amiga_ydpi);
 
-	if(id)
+	if(id && (option_monitor_aspect_x != 0) && (option_monitor_aspect_y != 0))
 	{
 		if(dih = FindDisplayInfo(id))
 		{
@@ -672,12 +794,12 @@ void ami_font_setdevicedpi(int id)
 				int xres = dinfo.Resolution.x;
 				int yres = dinfo.Resolution.y;
 
-				if(option_widescreen)
+				if((option_monitor_aspect_x != 4) || (option_monitor_aspect_y != 3))
 				{
 					/* AmigaOS sees 4:3 modes as square in the DisplayInfo database,
-					 * so we correct 16:10 modes to square for widescreen displays. */
-					xres = (xres * 16) / 4;
-					yres = (yres * 10) / 3;
+					 * so we correct other modes to "4:3 equiv" here. */
+					xres = (xres * option_monitor_aspect_x) / 4;
+					yres = (yres * option_monitor_aspect_y) / 3;
 				}
 
 				xdpi = (yres * ydpi) / xres;
