@@ -20,16 +20,19 @@
 #define AMIGA_FONT_H
 
 #include "desktop/plotters.h"
+#include <graphics/rastport.h>
 #include <graphics/text.h>
 
-#define NSA_NORMAL 0
-#define NSA_ITALIC 1
-#define NSA_BOLD 2
-#define NSA_BOLDITALIC 3
-
-void ami_close_font(struct TextFont *tfont);
-ULONG ami_unicode_text(struct RastPort *rp,const char *string,ULONG length,const plot_font_style_t *fstyle,ULONG x,ULONG y);
+struct ami_font_node;
 
+ULONG ami_unicode_text(struct RastPort *rp, const char *string,
+	ULONG length, const plot_font_style_t *fstyle, ULONG x, ULONG y);
+void ami_font_setdevicedpi(int id);
 void ami_init_fonts(void);
 void ami_close_fonts(void);
+void ami_font_close(struct ami_font_node *node);
+
+/* Simple diskfont functions for graphics.library use (not page rendering) */
+struct TextFont *ami_font_open_disk_font(struct TextAttr *tattr);
+void ami_font_close_disk_font(struct TextFont *tfont);
 #endif
