@@ -58,10 +58,6 @@ enum
 	GID_PASS,
 	GID_LOGIN,
 	GID_CANCEL,
-	GID_OPEN,
-	GID_NEWF,
-	GID_NEWB,
-	GID_DEL,
 	GID_NEXT,
 	GID_PREV,
 	GID_SEARCHSTRING,
@@ -110,6 +106,8 @@ struct gui_window_2 {
 	struct AppIcon *appicon; /* iconify appicon */
 	struct DiskObject *dobj; /* iconify appicon */
 	struct Hook search_ico_hook;
+	gui_drag_type drag_op;
+	struct IBox *ptr_lock;
 };
 
 struct gui_window
@@ -148,8 +146,6 @@ struct TextFont *origrpfont;
 struct MinList *window_list;
 struct Screen *scrn;
 STRPTR nsscreentitle;
-struct FileRequester *filereq;
-struct FileRequester *savereq;
 struct MsgPort *sport;
 bool win_destroyed;
 struct browser_window *curbw;
