@@ -42,35 +42,28 @@
 #include "amiga/bitmap.h"
 #include "amiga/clipboard.h"
 #include "amiga/cookies.h"
-#include "amiga/download.h"
+#include "amiga/file.h"
 #include "amiga/filetype.h"
 #include "amiga/gui.h"
 #include "amiga/gui_options.h"
 #include "amiga/history.h"
 #include "amiga/history_local.h"
 #include "amiga/hotlist.h"
-#include "amiga/icon.h"
-#include "amiga/iff_dr2d.h"
 #include "amiga/menu.h"
 #include "amiga/options.h"
 #include "amiga/print.h"
-#include "amiga/save_pdf.h"
 #include "amiga/search.h"
 #include "amiga/theme.h"
 #include "amiga/tree.h"
 #include "amiga/utf8.h"
-#include "content/fetch.h"
 #include "desktop/tree_url_node.h"
 #include "desktop/hotlist.h"
 #include "desktop/browser.h"
 #include "desktop/gui.h"
-#include "desktop/save_text.h"
-#include "desktop/save_pdf/pdf_plotters.h"
-#include "desktop/save_complete.h"
 #include "desktop/selection.h"
 #include "desktop/textinput.h"
 #include "utils/messages.h"
-#include "utils/url.h"
+#include "utils/schedule.h"
 
 #define IMAGE_MENU_ITEM(n, i, t) \
 	gwin->menulab[n] = LabelObject, \
@@ -86,13 +79,11 @@ BOOL menualreadyinit;
 const char * const netsurf_version;
 const char * const versvn;
 const char * const verdate;
-static struct Hook aslhookfunc;
 
 ULONG ami_menu_scan(struct tree *tree, bool count, struct gui_window_2 *gwin);
 void ami_menu_scan_2(struct tree *tree, struct node *root, WORD *gen,
 		ULONG *item, bool count, struct gui_window_2 *gwin);
 void ami_menu_arexx_scan(struct gui_window_2 *gwin);
-static const ULONG ami_asl_mime_hook(struct Hook *mh,struct FileRequester *fr,struct AnchorPathOld *ap);
 
 
 void ami_free_menulabs(struct gui_window_2 *gwin)
@@ -264,7 +255,7 @@ void ami_init_menulabs(struct gui_window_2 *gwin)
 	gwin->menutype[AMI_MENU_AREXX_MAX] = NM_END;
 }
 
-/* Menu refresh for hotlist - disabled, see below.
+/* Menu refresh for hotlist */
 void ami_menu_refresh(struct gui_window_2 *gwin)
 {
 	SetAttrs(gwin->objects[OID_MAIN],
@@ -277,10 +268,7 @@ void ami_menu_refresh(struct gui_window_2 *gwin)
 	SetAttrs(gwin->objects[OID_MAIN],
 			WINDOW_NewMenu, gwin->menu,
 			TAG_DONE);
-
-	schedule(6000,(void *)ami_menu_refresh,gwin);
 }
-*/
 
 struct NewMenu *ami_create_menu(struct gui_window_2 *gwin)
 {
@@ -302,29 +290,18 @@ struct NewMenu *ami_create_menu(struct gui_window_2 *gwin)
 	gwin->menu[12].nm_Flags = 0;
 	gwin->menu[13].nm_Flags = 0;
 
-	gwin->menu[15].nm_Flags = NM_ITEMDISABLED;
-
 #ifndef WITH_PDF_EXPORT
 	gwin->menu[9].nm_Flags = NM_ITEMDISABLED;
 #endif
 
+	gwin->menu[15].nm_Flags = NM_ITEMDISABLED;
+
 	ami_menu_scan(ami_tree_get_tree(hotlist_window), false, gwin);
 	ami_menu_arexx_scan(gwin);
 
-	if(!menualreadyinit)
-	{
-		aslhookfunc.h_Entry = (void *)&ami_asl_mime_hook;
-		aslhookfunc.h_SubEntry = NULL;
-		aslhookfunc.h_Data = NULL;
-
-		menualreadyinit = TRUE;
-	}
-
-/*	Set up scheduler to refresh the hotlist menu
-	Disabled as it causes everything to slow down to a halt after
-	several iterations
-	schedule(6000,(void *)ami_menu_refresh,gwin);
-*/
+/*	Set up scheduler to refresh the hotlist menu */
+	if(option_menu_refresh > 0)
+		schedule(option_menu_refresh, (void *)ami_menu_refresh, gwin);
 
 	return(gwin->menu);
 }
@@ -471,11 +448,8 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 	itemnum = ITEMNUM(code);
 	subnum = SUBNUM(code);
 	char *temp, *temp2;
-	BPTR lock = 0;
-	const char *source_data;
-	ULONG source_size;
-	struct bitmap *bm;
 	int sel = 0;
+	struct bitmap *bm = NULL;
 
 	switch(menunum)
 	{
@@ -491,154 +465,35 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 				break;
 
 				case 3: // open local file
-					if(AslRequestTags(filereq,
-						ASLFR_TitleText,messages_get("NetSurf"),
-						ASLFR_Screen,scrn,
-						ASLFR_DoSaveMode,FALSE,
-						ASLFR_RejectIcons,TRUE,
-						ASLFR_FilterFunc,&aslhookfunc,
-//						ASLFR_DoPatterns,TRUE,
-//						ASLFR_InitialPattern,"~(#?.info)",
-						TAG_DONE))
-					{
-						if(temp = AllocVec(1024,MEMF_PRIVATE | MEMF_CLEAR))
-						{
-							char *temp2;
-							strlcpy(temp,filereq->fr_Drawer,1024);
-							AddPart(temp,filereq->fr_File,1024);
-							temp2 = path_to_url(temp);
-							browser_window_go(gwin->bw,temp2,NULL, true);
-							free(temp2);
-							FreeVec(temp);
-						}
-					}
+					ami_file_open(gwin);
 				break;
 
 				case 4: // save
 					switch(subnum)
 					{
-						BPTR fh=0;
-						char fname[1024];
-
 						case 0:
-							if(AslRequestTags(savereq,
-								ASLFR_TitleText,messages_get("NetSurf"),
-								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
-								TAG_DONE))
-							{
-								strlcpy(fname,savereq->fr_Drawer,1024);
-								AddPart(fname,savereq->fr_File,1024);
-								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-
-								if(ami_download_check_overwrite(fname, gwin->win, 0))
-								{
-									if(fh = FOpen(fname,MODE_NEWFILE,0))
-									{
-										if((source_data =
-											content_get_source_data(gwin->bw->current_content, &source_size)))
-												FWrite(fh,source_data, 1, source_size);
-										FClose(fh);
-										SetComment(fname, content_get_url(gwin->bw->current_content));
-									}
-								}
-								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
-							}
+							ami_file_save_req(AMINS_SAVE_SOURCE, gwin,
+								gwin->bw->current_content, NULL);
 						break;
 
 						case 1:
-							if(AslRequestTags(savereq,
-								ASLFR_TitleText,messages_get("NetSurf"),
-								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
-								TAG_DONE))
-							{
-								strlcpy(fname,savereq->fr_Drawer,1024);
-								AddPart(fname,savereq->fr_File,1024);
-								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-
-								if(ami_download_check_overwrite(fname, gwin->win, 0))
-								{
-									save_as_text(gwin->bw->current_content,fname);
-									SetComment(fname,content_get_url(gwin->bw->current_content));
-								}
-								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
-							}
+							ami_file_save_req(AMINS_SAVE_TEXT, gwin,
+								gwin->bw->current_content, NULL);
 						break;
 
 						case 2:
-							if(AslRequestTags(savereq,
-								ASLFR_TitleText,messages_get("NetSurf"),
-								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
-								TAG_DONE))
-							{
-								strlcpy(fname,savereq->fr_Drawer,1024);
-								AddPart(fname,savereq->fr_File,1024);
-								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-								if(ami_download_check_overwrite(fname, gwin->win, 0))
-								{
-									if(lock = CreateDir(fname))
-									{
-										UnLock(lock);
-										save_complete(gwin->bw->current_content,fname);
-										SetComment(fname,content_get_url(gwin->bw->current_content));
-										ami_superimpose_favicon(fname,
-											gwin->bw->window->favicon, NULL);
-									}
-								}
-								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
-							}
+							ami_file_save_req(AMINS_SAVE_COMPLETE, gwin,
+								gwin->bw->current_content, NULL);
 						break;
 
 						case 3:
-#ifdef WITH_PDF_EXPORT
-							if(AslRequestTags(savereq,
-								ASLFR_TitleText,messages_get("NetSurf"),
-								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
-								TAG_DONE))
-							{
-								strlcpy(&fname,savereq->fr_Drawer,1024);
-								AddPart(fname,savereq->fr_File,1024);
-								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-								if(save_as_pdf(gwin->bw->current_content,fname))
-								{
-									SetComment(fname, content_get_url(gwin->bw->current_content));
-									ami_superimpose_favicon(fname,
-										gwin->bw->window->favicon, "pdf");
-								}
-								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
-							}
-#endif
+							ami_file_save_req(AMINS_SAVE_PDF, gwin,
+								gwin->bw->current_content, NULL);
 						break;
 
 						case 4: // iff
-							if(AslRequestTags(savereq,
-								ASLFR_TitleText,messages_get("NetSurf"),
-								ASLFR_Screen,scrn,
-								ASLFR_InitialFile,FilePart(content_get_url(gwin->bw->current_content)),
-								TAG_DONE))
-							{
-								strlcpy(fname,savereq->fr_Drawer,1024);
-								AddPart(fname,savereq->fr_File,1024);
-								ami_update_pointer(gwin->win,GUI_POINTER_WAIT);
-								if((bm = content_get_bitmap(gwin->bw->current_content)))
-								{
-									bm->url = (char *)content_get_url(gwin->bw->current_content);
-									bm->title = (char *)content_get_title(gwin->bw->current_content);
-									if(bitmap_save(bm, fname, 0))
-										SetComment(fname, content_get_url(gwin->bw->current_content));
-								}
-#ifdef WITH_NS_SVG
-								else if(ami_mime_compare(gwin->bw->current_content, "svg") == true)
-								{
-									if(ami_save_svg(gwin->bw->current_content,fname))
-										SetComment(fname, content_get_url(gwin->bw->current_content));
-								}
-#endif
-								ami_update_pointer(gwin->win,GUI_POINTER_DEFAULT);
-							}
+							ami_file_save_req(AMINS_SAVE_IFF, gwin,
+								gwin->bw->current_content, NULL);
 						break;
 					}
 				break;
@@ -718,7 +573,7 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 					}
 					else if(bm = content_get_bitmap(gwin->bw->current_content))
 					{
-						bm->url = (char *)content_get_url(gwin->bw->current_content);
+						bm->url = (char *)nsurl_access(hlcache_handle_get_url(gwin->bw->current_content));
 						bm->title = (char *)content_get_title(gwin->bw->current_content);
 						ami_easy_clipboard_bitmap(bm);
 					}
@@ -800,10 +655,10 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 
 					if (bw == NULL ||
 						bw->current_content == NULL ||
-						content_get_url(bw->current_content) == NULL)
+						nsurl_access(hlcache_handle_get_url(bw->current_content)) == NULL)
 					break;
 
-					hotlist_add_page(content_get_url(bw->current_content));
+					hotlist_add_page(nsurl_access(hlcache_handle_get_url(bw->current_content)));
 				break;
 
 				case 1: // show
@@ -876,36 +731,6 @@ void ami_menupick(ULONG code,struct gui_window_2 *gwin,struct MenuItem *item)
 	}
 }
 
-static const ULONG ami_asl_mime_hook(struct Hook *mh,struct FileRequester *fr,struct AnchorPathOld *ap)
-{
-	BPTR file = 0;
-	char buffer[10];
-	char fname[1024];
-	BOOL ret = FALSE;
-	char *mt = NULL;
-	lwc_string *lwc_mt = NULL;
-	lwc_error lerror;
-	content_type ct;
-
-	if(ap->ap_Info.fib_DirEntryType > 0) return(TRUE);
-
-	strcpy(fname,fr->fr_Drawer);
-	AddPart(fname,ap->ap_Info.fib_FileName,1024);
-
-  	mt = fetch_mimetype(fname);
-	lerror = lwc_intern_string(mt, strlen(mt), &lwc_mt);
-	if (lerror != lwc_error_ok)
-		return FALSE;
-
-	ct = content_factory_type_from_mime_type(lwc_mt);
-	lwc_string_unref(lwc_mt);
-
-	if(ct != CONTENT_NONE) ret = TRUE;
-
-	free(mt);
-	return ret;
-}
-
 void ami_menu_update_disabled(struct gui_window *g, hlcache_handle *c)
 {
 	struct Window *win = g->shared->win;
