@@ -26,7 +26,6 @@ enum
 	AMINS_UNKNOWN,
 	AMINS_CALLBACK,
 	AMINS_WINDOW,
-	AMINS_FRAME,
 	AMINS_DLWINDOW,
 	AMINS_LOGINWINDOW,
 	AMINS_TVWINDOW,
@@ -34,19 +33,20 @@ enum
 	AMINS_HISTORYWINDOW,
 	AMINS_GUIOPTSWINDOW,
 	AMINS_PRINTWINDOW,
-	AMINS_FETCHER,
+	AMINS_FONT,
+	AMINS_MIME
 };
 
 struct nsObject
 {
-	struct MinNode dtz_Node;
+	struct Node dtz_Node;
 	ULONG Type;
 	void *objstruct;
 	ULONG objstruct_size;
 };
 
 struct MinList *NewObjList(void);
-struct nsObject *AddObject(struct MinList *objlist,ULONG otype);
+struct nsObject *AddObject(struct MinList *objlist, ULONG otype);
 void DelObject(struct nsObject *dtzo);
 void DelObjectNoFree(struct nsObject *dtzo);
 void FreeObjList(struct MinList *objlist);
