@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 - 2011 Chris Young <chris@unsatisfactorysoftware.co.uk>
+ * Copyright 2008 - 2012 Chris Young <chris@unsatisfactorysoftware.co.uk>
  *
  * This file is part of NetSurf, http://www.netsurf-browser.org/
  *
@@ -24,6 +24,7 @@ extern char *option_url_file;
 extern char *option_hotlist_file;
 extern char *option_use_pubscreen;
 extern char *option_modeid;
+extern int option_screen_compositing;
 extern int option_cache_bitmaps;
 extern char *option_theme;
 extern bool option_utf8_clipboard;
@@ -34,7 +35,6 @@ extern bool option_new_tab_active;
 extern bool option_new_tab_last;
 extern bool option_kiosk_mode;
 extern char *option_search_engines_file;
-extern char *option_search_ico_file;
 extern char *option_arexx_dir;
 extern char *option_arexx_startup;
 extern char *option_arexx_shutdown;
@@ -65,14 +65,19 @@ extern int option_cookies_window_ysize;
 extern int option_cairo_renderer;
 extern bool option_direct_render;
 extern int option_amiga_ydpi;
-extern bool option_widescreen;
+extern int option_redraw_tile_size_x;
+extern int option_redraw_tile_size_y;
+extern int option_monitor_aspect_x;
+extern int option_monitor_aspect_y;
 extern bool option_accept_lang_locale;
+extern int option_menu_refresh;
 
 #define EXTRA_OPTION_DEFINE \
 char *option_url_file = 0; \
 char *option_hotlist_file = 0; \
 char *option_use_pubscreen = 0; \
 char *option_modeid = 0; \
+int option_screen_compositing = -1; \
 int option_cache_bitmaps = 0; \
 char *option_theme = 0; \
 bool option_utf8_clipboard = false; \
@@ -83,7 +88,6 @@ bool option_new_tab_active = false; \
 bool option_new_tab_last = false; \
 bool option_kiosk_mode = false; \
 char *option_search_engines_file = 0; \
-char *option_search_ico_file = 0; \
 char *option_arexx_dir = 0; \
 char *option_arexx_startup = 0; \
 char *option_arexx_shutdown = 0; \
@@ -114,14 +118,19 @@ int option_cookies_window_ysize = 0; \
 int option_cairo_renderer = 1; \
 bool option_direct_render = false; \
 int option_amiga_ydpi = 72; \
-bool option_widescreen = false; \
+int option_redraw_tile_size_x = 400; \
+int option_redraw_tile_size_y = 150; \
+int option_monitor_aspect_x = 0; \
+int option_monitor_aspect_y = 0; \
 bool option_accept_lang_locale = true; \
+int option_menu_refresh = 0; \
 
 #define EXTRA_OPTION_TABLE \
 { "url_file",		OPTION_STRING,	&option_url_file }, \
 { "hotlist_file",		OPTION_STRING,	&option_hotlist_file }, \
 { "use_pubscreen",	OPTION_STRING,	&option_use_pubscreen}, \
 { "screen_modeid",	OPTION_STRING,	&option_modeid}, \
+{ "screen_compositing",	OPTION_INTEGER,	&option_screen_compositing}, \
 { "cache_bitmaps",	OPTION_INTEGER,	&option_cache_bitmaps}, \
 { "theme",		OPTION_STRING,	&option_theme}, \
 { "clipboard_write_utf8",	OPTION_BOOL,	&option_utf8_clipboard}, \
@@ -132,7 +141,6 @@ bool option_accept_lang_locale = true; \
 { "new_tab_last",	OPTION_BOOL,	&option_new_tab_last}, \
 { "kiosk_mode",	OPTION_BOOL,	&option_kiosk_mode}, \
 { "search_engines_file",		OPTION_STRING,	&option_search_engines_file }, \
-{ "search_ico_file",		OPTION_STRING,	&option_search_ico_file }, \
 { "arexx_dir",		OPTION_STRING,	&option_arexx_dir }, \
 { "arexx_startup",		OPTION_STRING,	&option_arexx_startup }, \
 { "arexx_shutdown",		OPTION_STRING,	&option_arexx_shutdown }, \
@@ -163,6 +171,10 @@ bool option_accept_lang_locale = true; \
 { "cairo_renderer",	OPTION_INTEGER,	&option_cairo_renderer}, \
 { "direct_render",	OPTION_BOOL,	&option_direct_render}, \
 { "amiga_ydpi",	OPTION_INTEGER,	&option_amiga_ydpi}, \
-{ "widescreen",	OPTION_BOOL,	&option_widescreen}, \
-{ "accept_lang_locale",	OPTION_BOOL,	&option_accept_lang_locale},
+{ "redraw_tile_size_x",	OPTION_INTEGER,	&option_redraw_tile_size_x}, \
+{ "redraw_tile_size_y",	OPTION_INTEGER,	&option_redraw_tile_size_y}, \
+{ "monitor_aspect_x",	OPTION_INTEGER,	&option_monitor_aspect_x}, \
+{ "monitor_aspect_y",	OPTION_INTEGER,	&option_monitor_aspect_y}, \
+{ "accept_lang_locale",	OPTION_BOOL,	&option_accept_lang_locale}, \
+{ "menu_refresh",	OPTION_INTEGER,	&option_menu_refresh},
 #endif
