@@ -121,8 +121,10 @@ void ami_init_layers(struct gui_globals *gg, ULONG width, ULONG height)
 
 	struct BitMap *friend = NULL; /* Required to be NULL for Cairo and ARGB bitmaps */
 
-	if(!width) width = scrn->Width;
-	if(!height) height = scrn->Width;
+	if(option_redraw_tile_size_x <= 0) option_redraw_tile_size_x = scrn->Width;
+	if(option_redraw_tile_size_y <= 0) option_redraw_tile_size_y = scrn->Height;
+	if(!width) width = option_redraw_tile_size_x;
+	if(!height) height = option_redraw_tile_size_y;
 
 	gg->layerinfo = NewLayerInfo();
 	gg->areabuf = AllocVec(100,MEMF_PRIVATE | MEMF_CLEAR);
