@@ -38,6 +38,7 @@
 #include "amiga/drag.h"
 #include "amiga/options.h"
 #include "amiga/theme.h"
+#include "desktop/searchweb.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
 
@@ -96,6 +97,7 @@ char *ptrs32[AMI_LASTPOINTER+1] = {
 void ami_theme_init(void)
 {
 	char themefile[1024];
+	char searchico[1024];
 	BPTR lock = 0;
 
 	strcpy(themefile,option_theme);
@@ -121,6 +123,9 @@ void ami_theme_init(void)
 		UnLock(lock);
 		messages_load(themefile);
 	}
+
+	ami_get_theme_filename(searchico, "theme_search", false);
+	search_default_ico_location = (char *)strdup(searchico);
 }
 
 void ami_theme_throbber_setup(void)
