@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 - 2010 Chris Young <chris@unsatisfactorysoftware.co.uk>
+ * Copyright 2008 - 2012 Chris Young <chris@unsatisfactorysoftware.co.uk>
  *
  * This file is part of NetSurf, http://www.netsurf-browser.org/
  *
@@ -23,51 +23,62 @@
 #include <proto/space.h>
 #include <proto/label.h>
 #include <proto/scroller.h>
+#include <proto/exec.h>
+#include <proto/intuition.h>
+#include <proto/button.h>
+#include <proto/bitmap.h>
+#include <proto/graphics.h>
+#include <proto/asl.h>
+#include <proto/utility.h>
+#include <proto/dos.h>
+
+#include <libraries/gadtools.h>
+#include <gadgets/button.h>
 #include <classes/window.h>
 #include <gadgets/space.h>
 #include <images/label.h>
+#include <images/bitmap.h>
 #include <gadgets/layout.h>
 #include <gadgets/scroller.h>
 #include <reaction/reaction_macros.h>
-#include "amiga/gui.h"
-#include "content/urldb.h"
-#include <proto/exec.h>
-#include <assert.h>
-#include <proto/intuition.h>
-#include "amiga/tree.h"
-#include <proto/button.h>
-#include <gadgets/button.h>
-#include <string.h>
-#include "utils/messages.h"
-#include <proto/bitmap.h>
-#include <images/bitmap.h>
-#include <proto/graphics.h>
 #include <intuition/icclass.h>
-#include <proto/asl.h>
-#include <proto/utility.h>
-#include <libraries/gadtools.h>
-#include <proto/dos.h>
+#include <graphics/blitattr.h>
+
+#include "amiga/context_menu.h"
+#include "amiga/file.h"
+#include "amiga/gui.h"
+#include "amiga/tree.h"
 #include "amiga/utf8.h"
-#include "desktop/cookies.h"
-#include "desktop/history_global_core.h"
-#include "desktop/hotlist.h"
-#include "desktop/tree_url_node.h"
 #include "amiga/sslcert.h"
 #include "amiga/drag.h" /* drag icon stuff */
 #include "amiga/theme.h" /* pointers */
 #include "amiga/filetype.h"
 #include "amiga/options.h"
+#include "content/urldb.h"
+#include "desktop/cookies.h"
+#include "desktop/history_global_core.h"
+#include "desktop/hotlist.h"
+#include "desktop/tree_url_node.h"
 #include "utils/utils.h"
+#include "utils/messages.h"
+
+#include <assert.h>
+#include <string.h>
 
 #define AMI_TREE_MENU_ITEMS 21
 #define AMI_TREE_MENU_DELETE FULLMENUNUM(1,0,0)
 #define AMI_TREE_MENU_CLEAR FULLMENUNUM(1,3,0)
 
+enum {
+	GID_OPEN = GID_LAST,
+	GID_TREE_LAST
+};
+
+
 struct treeview_window {
-	struct Window *win;
-	Object *objects[GID_LAST];
-//	struct Gadget *gadgets[GID_LAST];
 	struct nsObject *node;
+	struct Window *win;
+	Object *objects[GID_TREE_LAST];
 	int type;
 	struct NewMenu *menu;
 	char *menu_name[AMI_TREE_MENU_ITEMS];
@@ -82,6 +93,11 @@ struct treeview_window {
 	int max_height;
 	struct gui_globals globals;
 	struct sslcert_session_data *ssl_data;
+	BOOL rmbtrapped;
+	char *wintitle;
+	char *sslerr;
+	char *sslaccept;
+	char *sslreject;
 };
 
 void ami_tree_draw(struct treeview_window *twin);
@@ -275,12 +291,13 @@ void ami_tree_drag_icon_show(struct treeview_window *twin)
 void ami_tree_drag_end(struct treeview_window *twin, int x, int y)
 {
 	struct gui_window_2 *gwin;
+	struct treeview_window *tw;
 	struct node *selected_node;
 	BOOL drag;
 
 	if(drag = ami_drag_in_progress()) ami_drag_icon_close(twin->win);
 
-	if(drag && (gwin = ami_window_at_pointer()))
+	if(drag && (twin != ami_window_at_pointer(AMINS_TVWINDOW)))
 	{
 		selected_node = tree_get_selected_node(tree_get_root(twin->tree));
 
@@ -290,8 +307,16 @@ void ami_tree_drag_end(struct treeview_window *twin, int x, int y)
 		}
 		else
 		{
-			browser_window_go(gwin->bw, tree_url_node_get_url(selected_node),
-				NULL, true);
+			if(gwin = ami_window_at_pointer(AMINS_WINDOW))
+			{
+				browser_window_go(gwin->bw, tree_url_node_get_url(selected_node),
+					NULL, true);
+			}
+			else if((tw = ami_window_at_pointer(AMINS_TVWINDOW)) &&
+				(tw != twin) && (tw->type == AMI_TREE_HOTLIST))
+			{
+				hotlist_add_page_xy(tree_url_node_get_url(selected_node), x, y);
+			}
 		}
 		tree_drag_end(twin->tree, twin->mouse_state,
 			twin->drag_x, twin->drag_y,
@@ -441,55 +466,22 @@ void ami_tree_menu(struct treeview_window *twin)
 
 void ami_tree_update_buttons(struct treeview_window *twin)
 {
-	BOOL launch_disable = FALSE;
-
 	if(twin->type == AMI_TREE_SSLCERT) return;
 
 	if(tree_node_has_selection(tree_get_root(twin->tree)))
 	{
-		struct node *selected_node =
-			tree_get_selected_node(tree_get_root(twin->tree));
-
 		OnMenu(twin->win, AMI_TREE_MENU_DELETE);
 		OnMenu(twin->win, AMI_TREE_MENU_CLEAR);
-
-		RefreshSetGadgetAttrs((struct Gadget *)twin->objects[GID_DEL],
-			twin->win, NULL,
-			GA_Disabled, FALSE,
-			TAG_DONE);
-
-		if((selected_node && (tree_node_is_folder(selected_node) == true)))
-			launch_disable = TRUE;
 	}
 	else
 	{
 		OffMenu(twin->win, AMI_TREE_MENU_DELETE);
 		OffMenu(twin->win, AMI_TREE_MENU_CLEAR);
-
-		RefreshSetGadgetAttrs((struct Gadget *)twin->objects[GID_DEL],
-			twin->win, NULL,
-			GA_Disabled, TRUE,
-			TAG_DONE);
-
-		launch_disable = TRUE;
-	}
-
-	if(twin->type != AMI_TREE_COOKIES)
-	{
-		RefreshSetGadgetAttrs((struct Gadget *)twin->objects[GID_OPEN],
-			twin->win, NULL,
-			GA_Disabled, launch_disable,
-			TAG_DONE);
 	}
 }
 
 void ami_tree_open(struct treeview_window *twin,int type)
 {
-	BOOL msel = TRUE,nothl = TRUE,launchdisable=FALSE;
-	static WORD gen=0;
-	char *wintitle;
-	char folderclosed[100],folderopen[100],item[100];
-
 	if(twin->win)
 	{
 		WindowToFront(twin->win);
@@ -502,35 +494,33 @@ void ami_tree_open(struct treeview_window *twin,int type)
 	switch(twin->type)
 	{
 		case AMI_TREE_HOTLIST:
-			nothl = FALSE;
-			wintitle = (char *)messages_get("Hotlist");
+			twin->wintitle = ami_utf8_easy((char *)messages_get("Hotlist"));
 		break;
 		case AMI_TREE_COOKIES:
-			nothl = TRUE;
-			launchdisable=TRUE;
-			wintitle = (char *)messages_get("Cookies");
+			twin->wintitle = ami_utf8_easy((char *)messages_get("Cookies"));
 		break;
 		case AMI_TREE_HISTORY:
-			nothl = TRUE;
-			wintitle = (char *)messages_get("GlobalHistory");
+			twin->wintitle = ami_utf8_easy((char *)messages_get("GlobalHistory"));
 		break;
 		case AMI_TREE_SSLCERT:
-			nothl = TRUE;
-			wintitle = (char *)messages_get("SSLCerts");
+			twin->wintitle = ami_utf8_easy((char *)messages_get("SSLCerts"));
+			twin->sslerr = ami_utf8_easy((char *)messages_get("SSLError"));
+			twin->sslaccept = ami_utf8_easy((char *)messages_get("Accept"));
+			twin->sslreject = ami_utf8_easy((char *)messages_get("Reject"));
 		break;
 	}
 
 	twin->scrollerhook.h_Entry = (void *)ami_tree_scroller_hook;
 	twin->scrollerhook.h_Data = twin;
 
-	ami_init_layers(&twin->globals, scrn->Width, scrn->Height);
+	ami_init_layers(&twin->globals, 0, 0);
 	ami_tree_menu(twin);
 
 	if(type == AMI_TREE_SSLCERT)
 	{
 		twin->objects[OID_MAIN] = WindowObject,
-      	    WA_ScreenTitle,nsscreentitle,
-           	WA_Title,wintitle,
+      	    WA_ScreenTitle, nsscreentitle,
+           	WA_Title, twin->wintitle,
            	WA_Activate, TRUE,
            	WA_DepthGadget, TRUE,
            	WA_DragBar, TRUE,
@@ -553,7 +543,7 @@ void ami_tree_open(struct treeview_window *twin,int type)
 			WINDOW_Position, WPOS_CENTERSCREEN,
 			WINDOW_ParentGroup, twin->objects[GID_MAIN] = VGroupObject,
 				LAYOUT_AddImage, LabelObject,
-					LABEL_Text, messages_get("SSLError"),
+					LABEL_Text, twin->sslerr,
 				LabelEnd,
 				LAYOUT_AddChild, twin->objects[GID_BROWSER] = SpaceObject,
 					GA_ID, GID_BROWSER,
@@ -563,12 +553,12 @@ void ami_tree_open(struct treeview_window *twin,int type)
 				LAYOUT_AddChild, HGroupObject,
 					LAYOUT_AddChild, twin->objects[GID_OPEN] = ButtonObject,
 						GA_ID,GID_OPEN,
-						GA_Text,messages_get("Accept"),
+						GA_Text, twin->sslaccept,
 						GA_RelVerify,TRUE,
 					ButtonEnd,
 					LAYOUT_AddChild, twin->objects[GID_CANCEL] = ButtonObject,
 						GA_ID,GID_CANCEL,
-						GA_Text,messages_get("Reject"),
+						GA_Text, twin->sslreject,
 						GA_RelVerify,TRUE,
 					ButtonEnd,
 				EndGroup,
@@ -607,7 +597,7 @@ void ami_tree_open(struct treeview_window *twin,int type)
 
 		twin->objects[OID_MAIN] = WindowObject,
       	    WA_ScreenTitle,nsscreentitle,
-           	WA_Title,wintitle,
+           	WA_Title, twin->wintitle,
            	WA_Activate, TRUE,
            	WA_DepthGadget, TRUE,
            	WA_DragBar, TRUE,
@@ -637,32 +627,6 @@ void ami_tree_open(struct treeview_window *twin,int type)
 					SPACE_Transparent,TRUE,
 					SPACE_BevelStyle, BVS_DISPLAY,
        			SpaceEnd,
-				LAYOUT_AddChild, HGroupObject,
-					LAYOUT_AddChild, twin->objects[GID_OPEN] = ButtonObject,
-						GA_ID,GID_OPEN,
-						GA_Text,messages_get("TreeLaunch"),
-						GA_RelVerify,TRUE,
-						GA_Disabled,launchdisable,
-					ButtonEnd,
-					LAYOUT_AddChild, twin->objects[GID_NEWF] = ButtonObject,
-						GA_ID,GID_NEWF,
-						BUTTON_AutoButton,BAG_POPDRAWER,
-						GA_RelVerify,TRUE,
-						GA_Disabled,nothl,
-					ButtonEnd,
-					LAYOUT_AddChild, twin->objects[GID_NEWB] = ButtonObject,
-						GA_ID,GID_NEWB,
-						BUTTON_AutoButton,BAG_POPFILE,
-						GA_RelVerify,TRUE,
-						GA_Disabled,nothl,
-					ButtonEnd,
-					LAYOUT_AddChild, twin->objects[GID_DEL] = ButtonObject,
-						GA_ID,GID_DEL,
-						GA_Text,messages_get("TreeDelete"),
-						GA_RelVerify,TRUE,
-					ButtonEnd,
-				EndGroup,
-				CHILD_WeightedHeight,0,
 			EndGroup,
 		EndWindow;
 	}
@@ -709,7 +673,14 @@ void ami_tree_close(struct treeview_window *twin)
 	}
 	FreeVec(twin->menu);
 	twin->menu = NULL;
-	if(twin->type == AMI_TREE_SSLCERT) ami_ssl_free(twin);
+	ami_utf8_free(twin->wintitle);
+	if(twin->type == AMI_TREE_SSLCERT) 
+	{
+		ami_utf8_free(twin->sslerr);
+		ami_utf8_free(twin->sslaccept);
+		ami_utf8_free(twin->sslreject);
+		ami_ssl_free(twin);
+	}
 }
 
 void ami_tree_update_quals(struct treeview_window *twin)
@@ -759,13 +730,9 @@ BOOL ami_tree_event(struct treeview_window *twin)
 				switch(result & WMHI_GADGETMASK)
 				{
 					case GID_OPEN:
-						if(twin->type == AMI_TREE_SSLCERT)
-						{
-							sslcert_accept(twin->ssl_data);
-							ami_tree_close(twin);
-							return TRUE;
-						}
-						else tree_launch_selected(twin->tree, true);
+						sslcert_accept(twin->ssl_data);
+						ami_tree_close(twin);
+						return TRUE;
 					break;
 
 					case GID_CANCEL:
@@ -773,29 +740,6 @@ BOOL ami_tree_event(struct treeview_window *twin)
 						ami_tree_close(twin);
 						return TRUE;
 					break;
-
-					case GID_NEWF:
-						hotlist_add_folder();
-					break;
-
-					case GID_NEWB:
-						hotlist_add_entry();
-					break;
-
-					case GID_DEL:
-						switch(twin->type)
-						{
-							case AMI_TREE_HISTORY:
-								history_global_delete_selected();
-							break;
-							case AMI_TREE_COOKIES:
-								cookies_delete_selected();
-							break;
-							case AMI_TREE_HOTLIST:
-								hotlist_delete_selected();
-							break;
-						}
-					break;
 				}
 			break;
 
@@ -805,6 +749,27 @@ BOOL ami_tree_event(struct treeview_window *twin)
 
 				GetAttr(SPACE_AreaBox, twin->objects[GID_BROWSER], (ULONG *)&bbox);
 
+				if((twin->win->MouseX - bbox->Left >=0) &&
+					(twin->win->MouseX - bbox->Width - bbox->Left <=0) &&
+					(twin->win->MouseY - bbox->Top >=0) &&
+					(twin->win->MouseY - bbox->Height - bbox->Top <=0))
+				{
+					if((twin->type != AMI_TREE_SSLCERT) &&
+						(twin->rmbtrapped == FALSE))
+					{
+						SetWindowAttr(twin->win, WA_RMBTrap, (APTR)(BOOL)TRUE, sizeof(BOOL));
+						twin->rmbtrapped = TRUE;
+					}
+				}
+				else
+				{
+					if(twin->rmbtrapped == TRUE)
+					{
+						SetWindowAttr(twin->win, WA_RMBTrap, (APTR)(BOOL)FALSE, sizeof(BOOL));
+						twin->rmbtrapped = FALSE;
+					}
+				}
+
 				GetAttr(SCROLLER_Top, twin->objects[OID_HSCROLL], (ULONG *)&xs);
 				x = twin->win->MouseX - bbox->Left + xs;
 
@@ -827,6 +792,9 @@ BOOL ami_tree_event(struct treeview_window *twin)
 					if((twin->win->MouseY > (bbox->Top + bbox->Height)) &&
 						((twin->win->MouseY - (bbox->Top + bbox->Height)) < AMI_DRAG_THRESHOLD))
 						drag_y_move = twin->win->MouseY - (bbox->Top + bbox->Height);
+
+					tree_mouse_action(twin->tree,
+						twin->mouse_state | twin->key_state, x, y);
 				}
 
 				if((x >= xs) && (y >= ys) && (x < bbox->Width + xs) &&
@@ -836,23 +804,25 @@ BOOL ami_tree_event(struct treeview_window *twin)
 
 					if(twin->mouse_state & BROWSER_MOUSE_PRESS_1)
 					{
-						tree_mouse_action(twin->tree,
-							BROWSER_MOUSE_DRAG_1 | twin->key_state, x, y);
-						twin->mouse_state = BROWSER_MOUSE_HOLDING_1 |
-											BROWSER_MOUSE_DRAG_ON;
-						if(twin->drag_x == 0) twin->drag_x = x;
-						if(twin->drag_y == 0) twin->drag_y = y;
-						ami_tree_drag_icon_show(twin);
+						if((abs(x - twin->drag_x) + abs(y - twin->drag_y)) > 2)
+						{
+							tree_mouse_action(twin->tree,
+								BROWSER_MOUSE_DRAG_1 | twin->key_state, x, y);
+							twin->mouse_state = BROWSER_MOUSE_HOLDING_1 |
+												BROWSER_MOUSE_DRAG_ON;
+							ami_tree_drag_icon_show(twin);
+						}
 					}
 					else if(twin->mouse_state & BROWSER_MOUSE_PRESS_2)
 					{
-						tree_mouse_action(twin->tree,
-							BROWSER_MOUSE_DRAG_2 | twin->key_state, x, y);
-						twin->mouse_state = BROWSER_MOUSE_HOLDING_2 |
-											BROWSER_MOUSE_DRAG_ON;
-						if(twin->drag_x == 0) twin->drag_x = x;
-						if(twin->drag_y == 0) twin->drag_y = y;
-						ami_tree_drag_icon_show(twin);
+						if((abs(x - twin->drag_x) + abs(y - twin->drag_y)) > 2)
+						{
+							tree_mouse_action(twin->tree,
+								BROWSER_MOUSE_DRAG_2 | twin->key_state, x, y);
+							twin->mouse_state = BROWSER_MOUSE_HOLDING_2 |
+												BROWSER_MOUSE_DRAG_ON;
+							ami_tree_drag_icon_show(twin);
+						}
 					}
 				}
 				twin->lastclick.tv_sec = 0;
@@ -875,9 +845,21 @@ BOOL ami_tree_event(struct treeview_window *twin)
 					{
 						case SELECTDOWN:
 							twin->mouse_state = BROWSER_MOUSE_PRESS_1;
+							if(twin->drag_x == 0) twin->drag_x = x;
+							if(twin->drag_y == 0) twin->drag_y = y;
 						break;
 						case MIDDLEDOWN:
 							twin->mouse_state = BROWSER_MOUSE_PRESS_2;
+							if(twin->drag_x == 0) twin->drag_x = x;
+							if(twin->drag_y == 0) twin->drag_y = y;
+						break;
+						case MENUDOWN:
+							if(tree_node_has_selection(tree_get_root(twin->tree)) == false)
+							{
+								tree_set_node_selected_at(twin->tree, x, y, true);
+							}
+							ami_context_menu_show_tree(twin->tree, twin->win, twin->type);
+
 						break;
 					}
 				}
@@ -1198,6 +1180,7 @@ BOOL ami_tree_event(struct treeview_window *twin)
 			break;
 
 			case WMHI_NEWSIZE:
+				ami_tree_resized(twin->tree, twin->max_width, twin->max_height, twin);
 				ami_tree_draw(twin);
 			break;
 
@@ -1235,8 +1218,10 @@ void ami_tree_redraw_request(int x, int y, int width, int height, void *data)
 	struct treeview_window *twin = data;
 	struct IBox *bbox;
 	int pos_x, pos_y;
+	int tile_x, tile_y, tile_w, tile_h;
 	struct redraw_context ctx = {
 		.interactive = true,
+		.background_images = true,
 		.plot = &amiplot
 	};
 
@@ -1253,10 +1238,42 @@ void ami_tree_redraw_request(int x, int y, int width, int height, void *data)
 	if(x - pos_x + width > bbox->Width) width = bbox->Width - (x - pos_x);
 	if(y - pos_y + height > bbox->Height) height = bbox->Height - (y - pos_y);
 
-	tree_draw(twin->tree, -pos_x, -pos_y, x, y, width, height, &ctx);
+	if(x < pos_x) {
+		width -= pos_x - x;
+		x = pos_x;
+	}
 
-	BltBitMapRastPort(twin->globals.bm, x - pos_x, y - pos_y, twin->win->RPort,
-		bbox->Left + x - pos_x, bbox->Top + y - pos_y, width, height, 0x0C0);
+	if(y < pos_y) {
+		height -= pos_y - y;
+		y = pos_y;
+	}
+
+	for(tile_y = y; tile_y < (y + height); tile_y += option_redraw_tile_size_y) {
+		tile_h = option_redraw_tile_size_y;
+		if(((y + height) - tile_y) < option_redraw_tile_size_y)
+			tile_h = (y + height) - tile_y;
+
+		for(tile_x = x; tile_x < (x + width); tile_x += option_redraw_tile_size_x) {
+			tile_w = option_redraw_tile_size_x;
+			if(((x + width) - tile_x) < option_redraw_tile_size_x)
+				tile_w = (x + width) - tile_x;
+
+			tree_draw(twin->tree, - tile_x, - tile_y,
+				tile_x, tile_y, tile_w, tile_h, &ctx);
+
+			BltBitMapTags(BLITA_SrcType, BLITT_BITMAP, 
+					BLITA_Source, twin->globals.bm,
+					BLITA_SrcX, 0,
+					BLITA_SrcY, 0,
+					BLITA_DestType, BLITT_RASTPORT, 
+					BLITA_Dest, twin->win->RPort,
+					BLITA_DestX, bbox->Left + tile_x - pos_x,
+					BLITA_DestY, bbox->Top + tile_y - pos_y,
+					BLITA_Width, tile_w,
+					BLITA_Height, tile_h,
+					TAG_DONE);
+		}
+	}
 
 	ami_update_pointer(twin->win, GUI_POINTER_DEFAULT);
 	glob = &browserglob;
