@@ -21,6 +21,7 @@
 #define NETSURF_VERSION_MAJOR "2"
 
 static const __attribute__((used)) char *verstag = "\0$VER: NetSurf " NETSURF_VERSION_MAJOR "." WT_REVID " (" WT_COMPILEDATE ")\0";
-const char * const versvn = "SVN " WT_REVID;
+const char * const versvn = "r" WT_REVID;
 const char * const verdate = WT_COMPILEDATE;
 const char * const verarexx = NETSURF_VERSION_MAJOR "." WT_REVID;
+const char * const wt_revid = WT_REVID;
