@@ -14,6 +14,12 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Module Description:
+ *
+ * This WinDom compo
+ *
+ *
  */
 
 #include <limits.h>
@@ -30,6 +36,12 @@
 #include "desktop/browser.h"
 #include "desktop/mouse.h"
 #include "desktop/textinput.h"
+#include "content/content.h"
+#include "content/hlcache.h"
+#include "content/urldb.h"
+#include "css/css.h"
+#include "render/box.h"
+#include "render/form.h"
 #include "utils/log.h"
 #include "utils/messages.h"
 
@@ -37,47 +49,43 @@
 #include "atari/browser_win.h"
 #include "atari/misc.h"
 #include "atari/global_evnt.h"
-#include "atari/browser_win.h"
 #include "atari/res/netsurf.rsh"
+#include "atari/redrawslots.h"
 #include "atari/browser.h"
 #include "atari/plot/plotter.h"
 #include "atari/plot.h"
-#include "atari/font.h"
+#include "atari/encoding.h"
+#include "atari/ctxmenu.h"
 #include "cflib.h"
 
-extern browser_mouse_state bmstate;
-extern int mouse_click_time[3];
-extern int mouse_hold_start[3];
 extern GEM_PLOTTER plotter;
 extern struct gui_window *input_window;
-extern short last_drag_x;
-extern short last_drag_y;
+
+static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect );
+static void browser_redraw_content( struct gui_window * gw, int xoff, int yoff,
+								struct rect * area );
+static void __CDECL browser_evnt_destroy( COMPONENT * c, long buff[8],
+										void * data);
+static void __CDECL browser_evnt_redraw( COMPONENT * c, long buff[8],
+										void * data);
+static void __CDECL browser_evnt_mbutton( COMPONENT * c, long buff[8],
+										void * data);
 
 
-
-static void __CDECL browser_evnt_wdestroy( WINDOW * c, short buff[8], void * data);
-COMPONENT *comp_widget_create( APPvar *app, WINDOW *win, int size, int flex );
-
-static bool frameinit = true;
-
-
-/* create an browser component */
+/*
+	Create an browser component.
+	Currently, this area is the area which is used to display HTML content.
+	However, it could also contains other areas, these need to be handled within
+	"browser_get_rect" function.
+*/
 struct s_browser * browser_create
 (
 	struct gui_window * gw,
 	struct browser_window *bw,
 	struct browser_window * clone,
-	int lt, int w, int flex 
+	int lt, int w, int flex
 )
 {
-	LGRECT cwork;
-	COMPONENT * scrollv, * scrollh, * drawable;
-
-	if( frameinit ) {
-		mt_FrameInit( &app );
-		frameinit = false;
-	}
-
 	CMP_BROWSER bnew = (CMP_BROWSER)malloc( sizeof(struct s_browser) );
 	if( bnew )
 	{
@@ -88,39 +96,34 @@ struct s_browser * browser_create
 			bw->scale = clone->scale;
 		else
 			bw->scale = 1;
-		bnew->redraw.areas_used = 0;
-		bnew->compwin = mt_WindCreate( &app, VSLIDE|HSLIDE, 1, 1, app.w, app.h);
-		bnew->compwin->w_u = 1;
-		bnew->compwin->h_u = 1;
-		/* needs to be adjusted when content width is known: */
-		bnew->compwin->ypos_max = w; 
-		bnew->compwin->xpos_max = w;
-		mt_WindSlider( &app, bnew->compwin, HSLIDER|VSLIDER);
-		bnew->comp = (COMPONENT*)comp_widget_create( (void*)&app, (WINDOW*)bnew->compwin, 1, 1 ); 
+		redraw_slots_init( &bnew->redraw, MAX_REDRW_SLOTS );
+		bnew->comp = (COMPONENT*)mt_CompCreate(&app, CLT_HORIZONTAL, 100, 1);
 		if( bnew->comp == NULL ) {
 			free(bnew);
 			return(NULL);
 		}
-		mt_EvntDataAdd( &app, bnew->compwin, WM_XBUTTON, 
-							browser_evnt_mbutton, (void*)gw, EV_BOT );
-		mt_CompEvntDataAttach( &app, bnew->comp, WM_REDRAW, 
-								browser_evnt_redraw, (void*)gw );
-		mt_EvntDataAttach( &app, bnew->compwin , WM_REDRAW, browser_evnt_redraw_x, NULL );
-		mt_EvntDataAttach( &app, bnew->compwin, WM_SLIDEXY, 
-								browser_evnt_slider, gw );
-		mt_EvntDataAttach( &app, bnew->compwin, WM_ARROWED, 
-								browser_evnt_arrowed, gw );	
-		mt_CompEvntDataAttach( &app, bnew->comp, WM_DESTROY, 
-								browser_evnt_destroy, (void*)bnew );
-		/* just stub, as an reminder: */
-		mt_EvntDataAttach( &app, bnew->compwin, WM_DESTROY, 
-								browser_evnt_wdestroy, (void*)bnew );
 
-		mt_CompDataAttach( &app, bnew->comp, CDT_OWNER, gw ); 
-		bnew->scroll.requested.y = 0;	
+		/* Attach events to the component: */
+		mt_CompEvntDataAdd( &app, bnew->comp, WM_XBUTTON,
+						browser_evnt_mbutton, (void*)gw, EV_BOT
+		);
+		mt_CompEvntDataAttach( &app, bnew->comp, WM_REDRAW,
+								browser_evnt_redraw, (void*)gw
+		);
+		mt_CompEvntDataAttach( &app, bnew->comp, WM_DESTROY,
+								browser_evnt_destroy, (void*)bnew
+		);
+
+		/* Set the gui_window owner. */
+		/* it is an link to the netsurf window system */
+		mt_CompDataAttach( &app, bnew->comp, CDT_OWNER, gw );
+
+		bnew->scroll.requested.y = 0;
 		bnew->scroll.requested.x = 0;
 		bnew->scroll.current.x = 0;
 		bnew->scroll.current.y = 0;
+		bnew->reformat_pending = false;
+
 	}
 	return( bnew );
 }
@@ -129,67 +132,42 @@ bool browser_destroy( struct s_browser * b )
 {
 
 	LOG(("%s\n", b->bw->name ));
+
 	assert( b != NULL );
 	assert( b->comp != NULL );
 	assert( b->bw != NULL );
 
-	if( b->compwin != NULL ) {
-		COMPONENT * old = b->comp;
-		WINDOW * oldwin = b->compwin;
-		b->comp = NULL;
-		b->compwin = NULL;
-		/* Im not sure if this is the right thing to do (after compdelete above,... */
-		/* listRemove should be used when we just remove an frame... */
-		/* (I encountered spurious events when not doing that...) */
-		/* listRemove( (LINKABLE*) oldwin ); */
-		/* listRemove( (LINKABLE*) old ); */
-		WindDelete( oldwin );
-		mt_CompDelete(&app,  old );	
+	if( b->comp != NULL ){
+		mt_CompDelete(&app,  b->comp );
 	}
 	return( true );
 }
 
+/*
+	Query the browser component for widget rectangles.
+*/
 void browser_get_rect( struct gui_window * gw, enum browser_rect type, LGRECT * out)
 {
-	GRECT work;
-	assert( out != NULL);
-	int slider_v_w = 20;
-	int slider_v_h = 22;
-	int slider_h_w = 20;
-	int slider_h_h = 22;
-
-	WindGetGrect( gw->browser->compwin, WF_WORKXYWH, &work);
-	if( type == BR_CONTENT ){
-		out->g_w = work.g_w;
-		out->g_h = work.g_h;
-		out->g_x = work.g_x;
-		out->g_y = work.g_y;
-		return;
-	}
-	
 	LGRECT cur;
+
+	/* Query component for it's current size: */
 	mt_CompGetLGrect(&app, gw->browser->comp, WF_WORKXYWH, &cur);
-	if( type == BR_HSLIDER ){
-		out->g_x = cur.g_x;
-		out->g_y = cur.g_y + work.g_h;
-		out->g_h = cur.g_h - work.g_h;
+
+	/* And extract the different widget dimensions: */
+	if( type == BR_CONTENT ){
 		out->g_w = cur.g_w;
-	}
-
-	if( type == BR_VSLIDER ){
-		out->g_x = cur.g_x + work.g_w;
+		out->g_h = cur.g_h;
+		out->g_x = cur.g_x;
 		out->g_y = cur.g_y;
-		out->g_w = cur.g_w - work.g_w;
-		out->g_h = work.g_h;
 	}
 
+	return;
 }
 
 /* Report an resize to the COMPONENT interface */
 void browser_update_rects(struct gui_window * gw )
 {
 	short buff[8];
-	LGRECT cmprect;
 	mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&buff[4]);
 	buff[0] = CM_REFLOW;
 	buff[1] = _AESapid;
@@ -202,8 +180,10 @@ void browser_set_content_size(struct gui_window * gw, int w, int h)
 	CMP_BROWSER b = gw->browser;
 	LGRECT work;
 	browser_get_rect( gw, BR_CONTENT, &work );
-	b->compwin->ypos_max = h;
-	b->compwin->xpos_max = w;
+
+	gw->root->handle->xpos_max = w;
+	gw->root->handle->ypos_max = h;
+
 	if( w < work.g_w + b->scroll.current.x || w < work.g_h + b->scroll.current.y ) {
 		/* let the scroll routine detect invalid scroll values... */
 		browser_scroll(gw, WA_LFLINE, b->scroll.current.x, true );
@@ -213,11 +193,6 @@ void browser_set_content_size(struct gui_window * gw, int w, int h)
 	}
 }
 
-static void __CDECL browser_evnt_wdestroy( WINDOW * c, short buff[8], void * data)
-{
-	struct s_browser * b = (struct s_browser*)data;
-	LOG((""));
-}
 
 static void __CDECL browser_evnt_destroy( COMPONENT * c, long buff[8], void * data)
 {
@@ -227,87 +202,34 @@ static void __CDECL browser_evnt_destroy( COMPONENT * c, long buff[8], void * da
 
 	assert( b != NULL );
 	assert( gw != NULL );
-
-	assert( b->comp == NULL );
-
 	free( b );
 	gw->browser = NULL;
 	LOG(("evnt_destroy done!"));
 }
 
+/*
+	Mouse Button handler for browser component.
+*/
 
-static void __CDECL browser_evnt_arrowed( WINDOW *win, short buff[8], void * data) 
+static void __CDECL browser_evnt_mbutton( COMPONENT * c, long buff[8], void * data)
 {
-	bool abs = false;
-	int value = BROWSER_SCROLL_SVAL;
-	struct gui_window * gw = data;
-	LGRECT cwork;
-
-	if( input_window == NULL || input_window != gw ) {
-		return;
-	}
-	browser_get_rect( gw, BR_CONTENT, &cwork );
-
-	switch( buff[4] ) {
-		case WA_UPPAGE:
-		case WA_DNPAGE:
-				value = cwork.g_h;
-			break;
-
-		case WA_LFPAGE:
-		case WA_RTPAGE:
-				value = cwork.g_w;
-			break;
-
-		default: 
-			break;
-	}
-	browser_scroll( gw, buff[4], value, abs );
-}
-
-void __CDECL browser_evnt_slider( WINDOW *win, short buff[8], void * data)
-{
-	int dx = buff[4];
-	int dy = buff[5];
-	GRECT work, screen;
-	struct gui_window * gw = data;
-
-	if (!dx && !dy) return;
-
-	if( input_window == NULL || input_window != gw ) {
-		return;
-	}
-
-	/* update the sliders _before_ we call redraw (which might depend on the slider possitions) */
-	mt_WindSlider( &app, win, (dx?HSLIDER:0) | (dy?VSLIDER:0) );
-
-	if( dy > 0 )
-		browser_scroll( gw, WA_DNPAGE, abs(dy), false );
-	else if ( dy < 0)
-		browser_scroll( gw, WA_UPPAGE, abs(dy), false );
-	if( dx > 0 )
-		browser_scroll( gw, WA_RTPAGE, abs(dx), false );
-	else if( dx < 0 )
-		browser_scroll( gw, WA_LFPAGE, abs(dx), false );
-}
-
-static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data)
-{
-	long lbuff[8];
-	short i;
 	short mx, my, dummy, mbut;
-	uint32_t tnow = clock()*1000 / CLOCKS_PER_SEC;
 	LGRECT cwork;
+	browser_mouse_state bmstate = 0;
 	struct gui_window * gw = data;
-	if( input_window != gw ) {
-		return;
+
+	if( input_window != gw ){
+		input_window = gw;
 	}
+
 	window_set_focus( gw, BROWSER, (void*)gw->browser );
 	browser_get_rect( gw, BR_CONTENT, &cwork );
-	mx = evnt.mx - cwork.g_x; 
-	my = evnt.my - cwork.g_y;
-	LOG(("mevent (%d) within %s at %d / %d\n", evnt.nb_click, gw->browser->bw->name, mx, my ));
 
+	/* convert screen coords to component coords: */
+	mx = evnt.mx - cwork.g_x;
+	my = evnt.my - cwork.g_y;
+
+	/* Translate GEM key state to netsurf mouse modifier */
 	if( evnt.mkstate & (K_RSHIFT | K_LSHIFT) ){
 		bmstate |= BROWSER_MOUSE_MOD_1;
 	} else {
@@ -323,67 +245,67 @@ static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data
 	} else {
 		bmstate &= ~(BROWSER_MOUSE_MOD_3);
 	}
-	int sx = (mx + gw->browser->scroll.current.x);
-	int sy = (my + gw->browser->scroll.current.y);
 
-	graf_mkstate(&dummy, &dummy, &mbut, &dummy);
-	/* todo: if we need right button click, increase loop count */
-	for( i = 1; i<2; i++) {
-		if( (mbut & i)  ) {
-			if( mouse_hold_start[i-1] == 0 ) {
-				mouse_hold_start[i-1] = tnow;
-				LOG(("Drag %d starts at %d,%d\n", i, sx, sy));
-				if( i == 1 ) {
-					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_1,sx,sy);
-					bmstate |= BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_ON;
-				}
-				if( i == 2 ) {
-					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_2,sx,sy);
-					bmstate |= BROWSER_MOUSE_HOLDING_2 | BROWSER_MOUSE_DRAG_ON;
-				}
+	/* convert component coords to scrolled content coords: */
+	int sx_origin = (mx + gw->browser->scroll.current.x);
+	int sy_origin = (my + gw->browser->scroll.current.y);
+
+	short rel_cur_x, rel_cur_y;
+	short prev_x=sx_origin, prev_y=sy_origin;
+	bool dragmode = 0;
+
+	/* Detect left mouse button state and compare with event state: */
+	graf_mkstate(&rel_cur_x, &rel_cur_y, &mbut, &dummy);
+	if( (mbut & 1) && (evnt.mbut & 1) ){
+		/* Mouse still pressed, report drag */
+		rel_cur_x = (rel_cur_x - cwork.g_x) + gw->browser->scroll.current.x;
+		rel_cur_y = (rel_cur_y - cwork.g_y) + gw->browser->scroll.current.y;
+		browser_window_mouse_click( gw->browser->bw,
+									BROWSER_MOUSE_DRAG_ON|BROWSER_MOUSE_DRAG_1,
+									sx_origin, sy_origin);
+		do{
+			if( abs(prev_x-rel_cur_x) > 5 || abs(prev_y-rel_cur_y) > 5 ){
+				browser_window_mouse_track( gw->browser->bw,
+									BROWSER_MOUSE_DRAG_ON|BROWSER_MOUSE_DRAG_1,
+									rel_cur_x, rel_cur_y);
+				prev_x = rel_cur_x;
+				prev_y = rel_cur_y;
+				dragmode = true;
 			} else {
-				if( i == 1 ) {
-					bmstate |= BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_DRAG_ON;
-				}
-				if( i == 2 ) {
-					bmstate |= BROWSER_MOUSE_DRAG_2 | BROWSER_MOUSE_DRAG_ON;
+				if( dragmode == false ){
+					browser_window_mouse_track( gw->browser->bw,BROWSER_MOUSE_PRESS_1,
+									rel_cur_x, rel_cur_y);
 				}
 			}
-
-			if( i != 0 ){
-				if( (abs(mx-last_drag_x)>5) || (abs(mx-last_drag_y)>5) ){	
-					browser_window_mouse_track( 
-						gw->browser->bw,
-						bmstate, 
-						sx, sy
-					);
-					last_drag_x = mx;
-					last_drag_y = my;
-				}
+			if( browser_redraw_required( gw ) ){
+				browser_redraw( gw );
 			}
-
+			graf_mkstate(&rel_cur_x, &rel_cur_y, &mbut, &dummy);
+			rel_cur_x = (rel_cur_x - cwork.g_x) + gw->browser->scroll.current.x;
+			rel_cur_y = (rel_cur_y - cwork.g_y) + gw->browser->scroll.current.y;
+		} while( mbut & 1 );
+		browser_window_mouse_track(gw->browser->bw, 0, rel_cur_x,rel_cur_y);
+	} else {
+		/* Right button pressed? */
+		if( (evnt.mbut & 2 ) ) {
+			context_popup( gw, evnt.mx, evnt.my );
 		} else {
-			mouse_click_time[i-1] = tnow; /* clock in ms */
-			/* check if this event was during an drag op: */
-			if( mouse_hold_start[i-1] == 0 ) {
-				if( i == 1) {
-					LOG(("Click within %s at %d / %d\n", gw->browser->bw->name, sx, sy ));
-					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_1,sx,sy);
-					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_CLICK_1,sx,sy);
-					bmstate &= ~( BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_CLICK_1);
-				}
-				if( i == 2 ) {
-					LOG(("Click within %s at %d / %d", gw->browser->bw->name, mx, my ));
-					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_PRESS_1,sx,sy);
-					browser_window_mouse_click(gw->browser->bw,BROWSER_MOUSE_CLICK_2,sx,sy);
-					bmstate &= ~( BROWSER_MOUSE_HOLDING_2 | BROWSER_MOUSE_DRAG_2 | BROWSER_MOUSE_CLICK_2);
-				}
-			}
-			mouse_hold_start[i-1] = 0;
+			browser_window_mouse_click(gw->browser->bw,
+										bmstate|BROWSER_MOUSE_PRESS_1,
+										sx_origin,sy_origin);
+			browser_window_mouse_click(gw->browser->bw,
+										bmstate|BROWSER_MOUSE_CLICK_1,
+										sx_origin,sy_origin);
 		}
 	}
+
+
 }
 
+
+/*
+	Report scroll event to the browser component.
+*/
 void browser_scroll( struct gui_window * gw, short mode, int value, bool abs )
 {
 	LGRECT work;
@@ -406,12 +328,12 @@ void browser_scroll( struct gui_window * gw, short mode, int value, bool abs )
 	switch( mode ) {
 
 		case WA_UPPAGE:
-		case WA_UPLINE: 
+		case WA_UPLINE:
 			if( max_y_scroll < 1 )
 				return;
 			if( abs == false )
 				b->scroll.requested.y -= value;
-			else 
+			else
 				b->scroll.requested.y = value - b->scroll.current.y;
 		break;
 
@@ -426,7 +348,7 @@ void browser_scroll( struct gui_window * gw, short mode, int value, bool abs )
 		break;
 
 		case WA_LFPAGE:
-		case WA_LFLINE: 
+		case WA_LFLINE:
 			if( max_x_scroll < 1 )
 				return;
 			if( abs == false )
@@ -445,7 +367,7 @@ void browser_scroll( struct gui_window * gw, short mode, int value, bool abs )
 				b->scroll.requested.x = value - b->scroll.current.x;
 		break;
 
-		default: break;				
+		default: break;
 	}
 
 	if( b->scroll.current.y + b->scroll.requested.y < 0 ) {
@@ -474,7 +396,7 @@ void browser_scroll( struct gui_window * gw, short mode, int value, bool abs )
 	perform the requested scrolling.
 	gw -> the browser window to act upon.
 	bwrect -> the dimensions of the browser, so that this function
-			  doesn't need to get it. 
+			  doesn't need to get it.
 */
 static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 {
@@ -487,7 +409,7 @@ static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 		return;
 
 	h = (short) abs( b->scroll.requested.y );
-	w = (short) abs( b->scroll.requested.x ); 
+	w = (short) abs( b->scroll.requested.x );
 
 	/* if the request exceeds the browser size, redraw the whole area */
 	if ( b->scroll.requested.y > bwrect.g_h || b->scroll.requested.y < -bwrect.g_h ||
@@ -511,11 +433,11 @@ static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 		dst.g_h = src.g_h;
 		plotter->copy_rect( plotter, src, dst );
 		b->scroll.current.y += b->scroll.requested.y;
-		browser_schedule_redraw( gw, 0, 0, bwrect.g_w, h ) ; 
+		browser_schedule_redraw( gw, 0, 0, bwrect.g_w, h );
 	}
 
 	if( b->scroll.requested.y > 0 ) {
-		/* scroll down */ 
+		/* scroll down */
 		src.g_x = 0;
 		src.g_y = h;
 		src.g_w = bwrect.g_w;
@@ -527,7 +449,7 @@ static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 		plotter->copy_rect( plotter, src, dst );
 		b->scroll.current.y += b->scroll.requested.y;
 		browser_schedule_redraw( gw, 0, bwrect.g_h - h, bwrect.g_w, bwrect.g_h );
-	} 
+	}
 
 	if( b->scroll.requested.x < 0 ) {
 		/* scroll to the left */
@@ -557,177 +479,84 @@ static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect )
 		plotter->copy_rect( plotter, src, dst );
 		b->scroll.current.x += b->scroll.requested.x;
 		browser_schedule_redraw( gw, bwrect.g_w - w, 0, bwrect.g_w, bwrect.g_h );
-	} 
+	}
 	b->scroll.requested.y = 0;
 	b->scroll.requested.x = 0;
-	gw->browser->compwin->xpos = b->scroll.current.x;
-	gw->browser->compwin->ypos = b->scroll.current.y;
+	if( b->caret.requested.g_w > 0 ){
+		b->caret.redraw = true;
+	}
 
-	mt_WindSlider( &app, gw->browser->compwin, HSLIDER|VSLIDER);
+	gw->root->handle->xpos = b->scroll.current.x;
+	gw->root->handle->ypos = b->scroll.current.y;
+
+	mt_WindSlider( &app, gw->root->handle, HSLIDER|VSLIDER );
 }
 
-
-bool browser_input( struct gui_window * gw, unsigned short nkc ) 
+/*
+	Report keypress to browser component.
+	The browser component doesn't listen for keyinput by itself.
+	parameter:
+		- gui_window ( compocnent owner ).
+		- unsigned short nkc ( CFLIB normalised key code )
+*/
+bool browser_input( struct gui_window * gw, unsigned short nkc )
 {
 	LGRECT work;
 	bool r = false;
 	unsigned char ascii = (nkc & 0xFF);
-	nkc = (nkc & (NKF_CTRL|NKF_SHIFT|0xFF));
-	browser_get_rect(gw, BR_CONTENT, &work);
+	long ucs4;
+	long ik = nkc_to_input_key( nkc, &ucs4 );
 
-	if( (nkc & NKF_CTRL) != 0  ) {
-		switch ( ascii ) {
-			case 'A':
-				r = browser_window_key_press(gw->browser->bw, KEY_SELECT_ALL);
-			break;
+	// pass event to specific control?
 
-			case 'C':
-				r = browser_window_key_press(gw->browser->bw, KEY_COPY_SELECTION);
-			break;
-
-			case 'X':
-				r = browser_window_key_press(gw->browser->bw, KEY_CUT_SELECTION);
-			break;
-
-			case 'V':
-				r = browser_window_key_press(gw->browser->bw, KEY_PASTE);
-			break;
-
-			default:
-			break;
-		}
-	}
-	if( (nkc & NKF_SHIFT) != 0 ) {
-		switch( ascii ) {
-
-			case NK_TAB:
-				r = browser_window_key_press(gw->browser->bw, KEY_SHIFT_TAB);			
-			break;
-
-			case NK_LEFT:
-				if( browser_window_key_press(gw->browser->bw, KEY_LINE_START) == false) {
-					browser_scroll( gw, WA_LFPAGE, work.g_w, false );
-					r = true;
-				}
-			break;
-
-			case NK_RIGHT:
-				if( browser_window_key_press(gw->browser->bw, KEY_LINE_END) == false) {
-					browser_scroll( gw, WA_RTPAGE, work.g_w, false );
-					r = true;
-				}
-			break;
-
-			case NK_UP:
-				if ( browser_window_key_press(gw->browser->bw, KEY_PAGE_UP) ==false ){
-					browser_scroll( gw, WA_UPPAGE, work.g_h, false );
-					r = true;
-				}
-			break;
-
-			case NK_DOWN:
-				if (browser_window_key_press(gw->browser->bw, KEY_PAGE_DOWN) == false) {
-					browser_scroll( gw, WA_DNPAGE, work.g_h, false );
-					r = true;
-				}
-			break;
-
-			default:
-			break;
-		}
-	}
-	if( (nkc & (NKF_SHIFT|NKF_CTRL) ) == 0 ) {
-		switch( ascii ) {
-			case NK_BS:
-				r = browser_window_key_press(gw->browser->bw, KEY_DELETE_LEFT);
-			break;
-
-			case NK_DEL:
-				r = browser_window_key_press(gw->browser->bw, KEY_DELETE_RIGHT);
-			break;
-
-			case NK_TAB:
-				r = browser_window_key_press(gw->browser->bw, KEY_TAB);
-			break;
-
-
-			case NK_ENTER:
-				r = browser_window_key_press(gw->browser->bw, KEY_NL);
-			break;
-
-			case NK_RET:
-				r = browser_window_key_press(gw->browser->bw, KEY_CR);
-			break;
-
-			case NK_ESC:
-				r = browser_window_key_press(gw->browser->bw, KEY_ESCAPE);		
-			break;
-
-			case NK_CLRHOME:
-				r = browser_window_key_press(gw->browser->bw, KEY_TEXT_START);
-			break;
-
-			case NK_RIGHT:
-				if (browser_window_key_press(gw->browser->bw, KEY_RIGHT) == false){
-					browser_scroll( gw, WA_RTLINE, 16, false );
-					r = true;
-				}
-			break;	
-
-			case NK_LEFT:
-				if (browser_window_key_press(gw->browser->bw, KEY_LEFT) == false) {
-					browser_scroll( gw, WA_LFLINE, 16, false );
-					r = true;
-				}
-			break;
-
-			case NK_UP:
-				if (browser_window_key_press(gw->browser->bw, KEY_UP) == false) {
-					browser_scroll( gw, WA_UPLINE, 16, false);
-					r = true;
-				}
-			break;
-
-			case NK_DOWN:
-				if (browser_window_key_press(gw->browser->bw, KEY_DOWN) == false) {
-					browser_scroll( gw, WA_DNLINE, 16, false);
-					r = true;
-				}
-			break;
-
-			case NK_M_PGUP:
-				if ( browser_window_key_press(gw->browser->bw, KEY_PAGE_UP) ==false ) {
-					browser_scroll( gw, WA_UPPAGE, work.g_h, false );
-					r = true;
-				}
-			break;
-
-			case NK_M_PGDOWN: 
-				if (browser_window_key_press(gw->browser->bw, KEY_PAGE_DOWN) == false) {
-					browser_scroll( gw, WA_DNPAGE, work.g_h, false );
-					r = true;
-				}
-			break;
-
-			default:
-			break;
-		}
-	}
-
-	if( r == false && ( (nkc & NKF_CTRL)==0)  ) {
+	if( ik == 0 ){
 		if (ascii >= 9 ) {
-			int ucs4 = atari_to_ucs4(ascii);
             r = browser_window_key_press(gw->browser->bw, ucs4 );
 		}
-	}
-	return( r );
-}
+	} else {
+		r = browser_window_key_press(gw->browser->bw, ik );
+		if( r == false ){
+			browser_get_rect(gw, BR_CONTENT, &work);
+			switch( ik ){
+				case KEY_LINE_START:
+					browser_scroll( gw, WA_LFPAGE, work.g_w, false );
+				break;
 
-static void __CDECL browser_evnt_redraw_x( WINDOW * c, short buf[8], void * data)
-{
-	/* just an stub to prevent wndclear */
-	/* Probably the browser redraw is better placed here? dunno! */
-	return;
+				case KEY_LINE_END:
+					browser_scroll( gw, WA_RTPAGE, work.g_w, false );
+				break;
+
+				case KEY_PAGE_UP:
+					browser_scroll( gw, WA_UPPAGE, work.g_h, false );
+				break;
+
+				case KEY_PAGE_DOWN:
+					browser_scroll( gw, WA_DNPAGE, work.g_h, false );
+				break;
+
+				case KEY_RIGHT:
+					browser_scroll( gw, WA_RTLINE, 16, false );
+				break;
+
+				case KEY_LEFT:
+					browser_scroll( gw, WA_LFLINE, 16, false );
+				break;
+
+				case KEY_UP:
+					browser_scroll( gw, WA_UPLINE, 16, false);
+				break;
+
+				case KEY_DOWN:
+					browser_scroll( gw, WA_DNLINE, 16, false);
+				break;
+
+				default:
+				break;
+			}
+		}
+	}
+
+	return( r );
 }
 
 /* determines if a browser window needs redraw */
@@ -739,9 +568,13 @@ bool browser_redraw_required( struct gui_window * gw)
 	if( b->bw->current_content == NULL )
 		return ( false );
 
+	/* disable redraws when the browser awaits WM_REDRAW caused by resize */
+	if( b->reformat_pending )
+		return( false );
+
 	ret = ( ((b->redraw.areas_used > 0) )
 			|| b->scroll.required
-			|| b->caret.redraw );
+			|| b->caret.redraw);
 	return( ret );
 }
 
@@ -762,22 +595,6 @@ void browser_schedule_redraw_rect(struct gui_window * gw, short x, short y, shor
 	browser_schedule_redraw( gw, x, y, x+w, y+h );
 }
 
-static inline bool rect_intersect( struct rect * box1, struct rect * box2 )
-{
-	if (box2->x1 < box1->x0)
- 	       return false;
-
-	if (box2->y1 < box1->y0)
-        	return false;
-
-	if (box2->x0 > box1->x1)
-        	return false;
-
-	if (box2->y0 > box1->y1)
-        	return false;
-
-	return true;
-}
 
 /*
 	schedule a redraw of content, coords are relative to the framebuffer
@@ -786,8 +603,6 @@ void browser_schedule_redraw(struct gui_window * gw, short x0, short y0, short x
 {
 	assert( gw != NULL );
 	CMP_BROWSER b = gw->browser;
-	int i;
-	struct rect area;
 	LGRECT work;
 
 	if( y1 < 0 || x1 < 0 )
@@ -799,107 +614,132 @@ void browser_schedule_redraw(struct gui_window * gw, short x0, short y0, short x
 	if( y0 > work.g_h )
 		return;
 
-	area.x0 = x0;
-	area.y0 = y0;
-	area.x1 = x1;
-	area.y1 = y1;
+	redraw_slot_schedule( &b->redraw, x0, y0, x1, y1 );
 
-	for( i=0; i<b->redraw.areas_used; i++) {
-		if( b->redraw.areas[i].x0 <= x0
-			&& b->redraw.areas[i].x1 >= x1
-			&& b->redraw.areas[i].y0 <= y0
-			&& b->redraw.areas[i].y1 >= y1 ){
-			/* the area is already queued for redraw */
-			return;
-		} else {
-			if( rect_intersect(&b->redraw.areas[i], &area ) ){
-				b->redraw.areas[i].x0 = MIN(b->redraw.areas[i].x0, x0);
-				b->redraw.areas[i].y0 = MIN(b->redraw.areas[i].y0, y0);
-				b->redraw.areas[i].x1 = MAX(b->redraw.areas[i].x1, x1);
-				b->redraw.areas[i].y1 = MAX(b->redraw.areas[i].y1, y1);
-				return;
-			}
-		}
-	} 
-
-	if( b->redraw.areas_used < MAX_REDRW_SLOTS ) {
-		b->redraw.areas[b->redraw.areas_used].x0 = x0;
-		b->redraw.areas[b->redraw.areas_used].x1 = x1;
-		b->redraw.areas[b->redraw.areas_used].y0 = y0;
-		b->redraw.areas[b->redraw.areas_used].y1 = y1;
-		b->redraw.areas_used++;
-	} else {
-		/*
-			we are out of available slots, merge box with last slot
-			this is dumb... but also a very rare case.
-		*/
-		b->redraw.areas[MAX_REDRW_SLOTS-1].x0 = MIN(b->redraw.areas[i].x0, x0);
-		b->redraw.areas[MAX_REDRW_SLOTS-1].y0 = MIN(b->redraw.areas[i].y0, y0);
-		b->redraw.areas[MAX_REDRW_SLOTS-1].x1 = MAX(b->redraw.areas[i].x1, x1);
-		b->redraw.areas[MAX_REDRW_SLOTS-1].y1 = MAX(b->redraw.areas[i].y1, y1);
-	}
-done:
 	return;
 }
 
-static void browser_redraw_content( struct gui_window * gw, int xoff, int yoff )
+static void browser_redraw_content( struct gui_window * gw, int xoff, int yoff,
+								struct rect * area )
 {
-	LGRECT work;
 	CMP_BROWSER b = gw->browser;
 
 	struct redraw_context ctx = {
 		.interactive = true,
+		.background_images = true,
 		.plot = &atari_plotters
 	};
 
-	LOG(("%s : %d,%d - %d,%d\n", b->bw->name, b->redraw.area.x0,
-		b->redraw.area.y0, b->redraw.area.x1, b->redraw.area.y1
+	LOG(("%s : %d,%d - %d,%d\n", b->bw->name, area->x0,
+		area->y0, area->x1, area->y1
 	));
 
 
 	browser_window_redraw( b->bw, -b->scroll.current.x,
-			-b->scroll.current.y, &b->redraw.area, &ctx );
+			-b->scroll.current.y, area, &ctx );
 
 }
 
-
-void browser_redraw_caret( struct gui_window * gw, GRECT * area )
+/*
+	area: the browser canvas
+*/
+void browser_restore_caret_background( struct gui_window * gw, LGRECT * area)
 {
-	GRECT caret;
-	struct s_browser * b = gw->browser;
-	if( b->caret.redraw == true ){
+	CMP_BROWSER b = gw->browser;
+	LGRECT rect;
+	if( area == NULL ){
+		browser_get_rect( gw, BR_CONTENT, &rect );
+		area = &rect;
+	}
+	/* This call restores the background and releases the memory: */
+	// TODO: only release memory/clear flag when the caret is not clipped.
+	// TODO: apply clipping.
+	w_put_bkgr( &app,
+			area->g_x-b->scroll.current.x+b->caret.current.g_x,
+			area->g_y-b->scroll.current.y+b->caret.current.g_y,
+			gw->browser->caret.current.g_w,
+			gw->browser->caret.current.g_h,
+			&gw->browser->caret.background
+	);
+	gw->browser->caret.background.fd_addr = NULL;
+}
+
+/*
+	area: the browser canvas
+*/
+void browser_redraw_caret( struct gui_window * gw, LGRECT * area )
+{
+	// TODO: only redraw caret when window is topped.
+	if( gw->browser->caret.redraw && gw->browser->caret.requested.g_w > 0 ){
+		LGRECT caret;
+		struct s_browser * b = gw->browser;
 		struct rect old_clip;
 		struct rect clip;
 
+		if( b->caret.current.g_w > 0 && b->caret.background.fd_addr != NULL ){
+			browser_restore_caret_background( gw, area );
+		}
+
 		caret = b->caret.requested;
-		caret.g_x -= gw->browser->scroll.current.x;
-		caret.g_y -= gw->browser->scroll.current.y;
-		clip.x0 = caret.g_x - 1;
-		clip.y0 = caret.g_y - 1;
-		clip.x1 = caret.g_x + caret.g_w + 1;
-		clip.y1 = caret.g_y + caret.g_h + 1;
+		caret.g_x -= b->scroll.current.x - area->g_x;
+		caret.g_y -= b->scroll.current.y - area->g_y;
+
+		if( !rc_lintersect( area, &caret ) ) {
+			return;
+		}
+
+		MFDB screen;
+		short pxy[8];
+
+		/* save background: */
+		//assert( b->caret.background.fd_addr == NULL );
+		init_mfdb( app.nplanes, caret.g_w, caret.g_h, 0,
+					&b->caret.background );
+		init_mfdb( 0, caret.g_w, caret.g_h, 0, &screen );
+		pxy[0] = caret.g_x;
+		pxy[1] = caret.g_y;
+		pxy[2] = caret.g_x + caret.g_w - 1;
+		pxy[3] = caret.g_y + caret.g_h - 1;
+		pxy[4] = 0;
+		pxy[5] = 0;
+		pxy[6] = caret.g_w - 1;
+		pxy[7] = caret.g_h - 1;
+		/* hide the mouse */
+		v_hide_c ( app.graf.handle);
+		/* copy screen image */
+		vro_cpyfm ( app.graf.handle, S_ONLY, pxy, &screen, &b->caret.background);
+		/* restore the mouse */
+		v_show_c ( app.graf.handle, 1);
+		/* draw caret: */
+		caret.g_x -= area->g_x;
+		caret.g_y -= area->g_y;
+		clip.x0 = caret.g_x;
+		clip.y0 = caret.g_y;
+		clip.x1 = caret.g_x + caret.g_w-1;
+		clip.y1 = caret.g_y + caret.g_h-1;
 		/* store old clip before adjusting it: */
 		plot_get_clip( &old_clip );
 		/* clip to cursor: */
 		plot_clip( &clip );
-		plot_rectangle( caret.g_x, caret.g_y, 
-			caret.g_x+caret.g_w, caret.g_y+caret.g_h,
-			plot_style_caret );
+		plot_line( caret.g_x, caret.g_y, caret.g_x, caret.g_y + caret.g_h,
+					plot_style_caret );
 		/* restore old clip area: */
 		plot_clip( &old_clip );
 		b->caret.current.g_x = caret.g_x + gw->browser->scroll.current.x;
 		b->caret.current.g_y = caret.g_y + gw->browser->scroll.current.y;
 		b->caret.current.g_w = caret.g_w;
-		b->caret.current.g_h = caret.g_h;		
+		b->caret.current.g_h = caret.g_h;
 	}
 }
 
-void browser_redraw( struct gui_window * gw ) 
+void browser_redraw( struct gui_window * gw )
 {
 	LGRECT bwrect;
 	struct s_browser * b = gw->browser;
 	short todo[4];
 	struct rect clip;
+	/* used for clipping of content redraw: */
+	struct rect redraw_area;
 
 	if( b->attached == false || b->bw->current_content == NULL ) {
 		return;
@@ -914,7 +754,8 @@ void browser_redraw( struct gui_window * gw )
 	clip.x1 = bwrect.g_w;
 	clip.y1 = bwrect.g_h;
 	plotter->clip( plotter, &clip );
-	plotter->lock(plotter);
+	if( plotter->lock(plotter) == 0 )
+		return;
 
 	if( b->scroll.required == true && b->bw->current_content != NULL) {
 		browser_process_scroll( gw, bwrect );
@@ -954,13 +795,13 @@ void browser_redraw( struct gui_window * gw )
 						area.g_x = b->redraw.areas[i].x0;
 						area.g_y = b->redraw.areas[i].y0;
 						area.g_w = b->redraw.areas[i].x1 - b->redraw.areas[i].x0;
-						area.g_h = b->redraw.areas[i].y1 - b->redraw.areas[i].y0; 
+						area.g_h = b->redraw.areas[i].y1 - b->redraw.areas[i].y0;
 						if (rc_intersect((GRECT *)&fbwork,(GRECT *)&area)) {
-							b->redraw.area.x0 = area.g_x;
-							b->redraw.area.y0 = area.g_y;
-							b->redraw.area.x1 = area.g_x + area.g_w;
-							b->redraw.area.y1 = area.g_y + area.g_h;
-							browser_redraw_content( gw, 0, 0 );
+							redraw_area.x0 = area.g_x;
+							redraw_area.y0 = area.g_y;
+							redraw_area.x1 = area.g_x + area.g_w;
+							redraw_area.y1 = area.g_y + area.g_h;
+							browser_redraw_content( gw, 0, 0, &redraw_area );
 						} else {
 							/*
 								the area should be kept scheduled for later redraw, but because this
@@ -982,7 +823,7 @@ void browser_redraw( struct gui_window * gw )
 		b->redraw.areas_used = 0;
 	}
 	if( b->caret.redraw == true && 	b->bw->current_content != NULL ) {
-		GRECT area;
+		LGRECT area;
 		todo[0] = bwrect.g_x;
 		todo[1] = bwrect.g_y;
 		todo[2] = todo[0] + bwrect.g_w;
@@ -1002,26 +843,27 @@ void browser_redraw( struct gui_window * gw )
 
 static void __CDECL browser_evnt_redraw( COMPONENT * c, long buff[8], void * data)
 {
-	short pxy[8];
-	WINDOW * w;
 	struct gui_window * gw = (struct gui_window *) data;
 	CMP_BROWSER b = gw->browser;
-	LGRECT work, lclip, rwork;
+	LGRECT work, lclip;
 
-	// TODO: maybe implement something like validate_gw()
-	// to fetch spurious redraw events? the function should
-	// traverse all gui_windows and see if gw  exists in the list
-
-	int xoff,yoff,width,heigth;
-	short cw, ch, cellw, cellh;
-	/* use that instead of browser_find_root() ? */
-	w = (WINDOW*)mt_CompGetPtr( &app, c, CF_WINDOW );
 	browser_get_rect( gw, BR_CONTENT, &work );
 	lclip = work;
 	if ( !rc_lintersect( (LGRECT*)&buff[4], &lclip ) ) return;
 
-	if( b->bw->current_content == NULL )
+	if( b->bw->current_content == NULL ){
+		short pxy[4];
+		pxy[0] = lclip.g_x;
+		pxy[1] = lclip.g_y;
+		pxy[2] = lclip.g_x + lclip.g_w - 1;
+		pxy[3] = lclip.g_y + lclip.g_h - 1;
+		vsf_color( gw->root->handle->graf->handle, WHITE );
+		vsf_perimeter( gw->root->handle->graf->handle, 0);
+		vsf_interior( gw->root->handle->graf->handle, FIS_SOLID );
+		vsf_style( gw->root->handle->graf->handle, 1);
+		v_bar( gw->root->handle->graf->handle, (short*)&pxy );
 		return;
+	}
 
 	/* convert redraw coords to framebuffer coords: */
 	lclip.g_x -= work.g_x;
@@ -1038,9 +880,21 @@ static void __CDECL browser_evnt_redraw( COMPONENT * c, long buff[8], void * dat
 	}
 
 	if( lclip.g_h > 0 && lclip.g_w > 0 ) {
-		browser_schedule_redraw( gw, lclip.g_x, lclip.g_y,
-			lclip.g_x + lclip.g_w, lclip.g_y + lclip.g_h
-		);
+
+		if( gw->browser->reformat_pending == true ){
+			LGRECT newsize;
+			gw->browser->reformat_pending = false;
+			browser_get_rect(gw, BR_CONTENT, &newsize);
+			/* this call will also schedule an redraw for the complete */
+			/* area. */
+			/* Resize must be handled here, because otherwise */
+			/* a redraw is scheduled twice (1. by the frontend, 2. by AES) */
+			browser_window_reformat(b->bw, false, newsize.g_w, newsize.g_h );
+		} else {
+			browser_schedule_redraw( gw, lclip.g_x, lclip.g_y,
+				lclip.g_x + lclip.g_w, lclip.g_y + lclip.g_h
+			);
+		}
 	}
 
 	return;
