@@ -19,9 +19,11 @@
 #ifndef NS_ATARI_BROWSER_H
 #define NS_ATARI_BROWSER_H
 
+#include "atari/redrawslots.h"
+
 /*
  Each browser_window in the Atari Port is represented by an  struct s_browser,
- which cosnist mainly of an WinDom COMPONENT.
+ which consist mainly of an WinDom COMPONENT.
 */
 
 /*
@@ -30,13 +32,6 @@
 */
 #define BROWSER_SCROLL_SVAL 64
 
-/*
-	MAX_REDRW_SLOTS
-	This is the number of redraw requests that an browser window can queue.
-	If a redraw is scheduled and all slots are used, the rectangle will
-	be merged to one of the existing slots.
- */
-#define MAX_REDRW_SLOTS	32
 
 enum browser_rect
 {
@@ -61,41 +56,37 @@ struct s_scroll_info
 
 /*
 	This struct holds information of the cursor within the browser
-	viewport. 
+	viewport.
 */
 struct s_caret
 {
-	GRECT requested;
-	GRECT current;
+	LGRECT requested;
+	LGRECT current;
 	bool redraw;
+	MFDB background;
 };
 
 /*
-	This struct holds scheduled redraw requests.
-*/
-struct rect;
-struct s_browser_redrw_info
-{
-	struct rect areas[MAX_REDRW_SLOTS];
-	short areas_used;
-	/* used for clipping of content redraw: */
-	struct rect area;
-};
+	This is the browser content area (viewport).
+	It is redrawable and scrollable. It is based on the WinDOM
+	Component window (undocumented feature).
 
-/*
-	This is the actual browser widget, containings GUI elements and
-	the current state of the browser widget.
+	It's an windom component containing it's own Window controls,
+	like scrollbars, resizer, etc.
+
+	Now that the NetSurf core handles frames, the advantages of this
+	choice have probably vanished.
 */
 struct s_browser
 {
 	int type;
 	COMPONENT * comp;
-	WINDOW * compwin;
 	struct browser_window * bw;
 	struct s_scroll_info scroll;
-	struct s_browser_redrw_info redraw; 
+	struct s_redrw_slots redraw;
 	struct s_caret caret;
 	bool attached;
+	bool reformat_pending;
 };
 
 struct s_browser * browser_create( struct gui_window * gw, struct browser_window * clone, struct browser_window *bw, int lt,  int w, int flex );
@@ -106,9 +97,11 @@ void browser_redraw( struct gui_window * gw );
 void browser_set_content_size(struct gui_window * gw, int w, int h);
 void browser_scroll( struct gui_window * gw, short MODE, int value, bool abs );
 struct gui_window * browser_find_root( struct gui_window * gw );
-static void browser_process_scroll( struct gui_window * gw, LGRECT bwrect );
 bool browser_redraw_required( struct gui_window * gw);
-
+void browser_redraw_caret( struct gui_window * gw, LGRECT * area);
+void browser_restore_caret_background(struct gui_window * gw, LGRECT * area);
+/* update loc / size of the browser widgets: */
+void browser_update_rects(struct gui_window * gw );
 /*
 	This queues an redraw to one of the slots.
 	The following strategy is used:
@@ -120,19 +113,7 @@ bool browser_redraw_required( struct gui_window * gw);
 	4. if no slot is available, it will simply merge the new rectangle with
    	the last available slot.
 */
-void browser_redraw_caret( struct gui_window * gw, GRECT * area );
-static void browser_redraw_content( struct gui_window * gw, int xoff, int yoff );
-
-/* update loc / size of the browser widgets: */
-void browser_update_rects(struct gui_window * gw );
 void browser_schedule_redraw_rect(struct gui_window * gw, short x, short y, short w, short h);
 void browser_schedule_redraw(struct gui_window * gw, short x, short y, short w, short h );
-static void __CDECL browser_evnt_resize( COMPONENT * c, long buff[8], void * data);
-static void __CDECL browser_evnt_destroy( COMPONENT * c, long buff[8], void * data);
-static void __CDECL browser_evnt_mbutton( WINDOW * c, short buff[8], void * data);
-static void __CDECL browser_evnt_arrowed( WINDOW *win, short buff[8], void * data);
-static void __CDECL browser_evnt_slider( WINDOW *win, short buff[8], void * data);
-static void __CDECL browser_evnt_redraw( COMPONENT * c, long buff[8], void * data);
-static void __CDECL browser_evnt_redraw_x( WINDOW * c, short buff[8], void * data);
 
 #endif
