@@ -56,219 +56,56 @@
 #include "atari/dragdrop.h"
 #include "atari/search.h"
 #include "atari/osspec.h"
+#include "atari/encoding.h"
+#include "atari/toolbar.h"
 
 extern void * h_gem_rsrc;
 extern struct gui_window *input_window;
 extern GEM_PLOTTER plotter;
-extern int mouse_click_time[3];
-extern int mouse_hold_start[3];
-extern browser_mouse_state bmstate;
-extern short last_drag_x;
-extern short last_drag_y;
 
 void __CDECL std_szd( WINDOW * win, short buff[8], void * );
 void __CDECL std_mvd( WINDOW * win, short buff[8], void * );
 
+
+/* -------------------------------------------------------------------------- */
+/* Static module methods follow here:                                         */
+/* -------------------------------------------------------------------------- */
+static void __CDECL evnt_window_icondraw( WINDOW *win, short buff[8], void *data );
+static void __CDECL evnt_window_newtop( WINDOW *win, short buff[8], void *data );
+void __CDECL evnt_window_resize( WINDOW *win, short buff[8], void * data );
+static void __CDECL evnt_window_move( WINDOW *win, short buff[8], void * data );
+static void __CDECL evnt_window_rt_resize( WINDOW *win, short buff[8], void * date );
+static void __CDECL evnt_window_close( WINDOW *win, short buff[8], void *data );
+static void __CDECL evnt_window_dd( WINDOW *win, short wbuff[8], void * data ) ;
+static void __CDECL evnt_window_destroy( WINDOW *win, short buff[8], void *data );
+static void __CDECL evnt_window_slider( WINDOW * win, short buff[8], void * data);
+static void __CDECL evnt_window_arrowed( WINDOW *win, short buff[8], void *data );
+static void __CDECL evnt_window_uniconify( WINDOW *win, short buff[8], void * data );
+static void __CDECL evnt_window_iconify( WINDOW *win, short buff[8], void * data );
+
 /* -------------------------------------------------------------------------- */
 /* Module public functions:                                                   */
 /* -------------------------------------------------------------------------- */
 
 
-static void __CDECL evnt_window_arrowed( WINDOW *win, short buff[8], void *data )
+int window_create( struct gui_window * gw,
+				struct browser_window * bw,
+				unsigned long inflags )
 {
-	bool abs = false;
-	LGRECT cwork;
-	int value = BROWSER_SCROLL_SVAL;
-
-	if( input_window == NULL ) {
-		return;
-	}
-
-	browser_get_rect( input_window, BR_CONTENT, &cwork );
-
-	switch( buff[4] ) {
-		case WA_UPPAGE:
-		case WA_DNPAGE:
-				value = cwork.g_h;
-			break;
-
-
-		case WA_LFPAGE:
-		case WA_RTPAGE:
-				value = cwork.g_w;
-			break;
-
-		default:
-			break;
-	}
-	browser_scroll( input_window, buff[4], value, abs );
-}
-
-/*
-	track the mouse state and
-	finally checks for released buttons.
- */
-static void window_track_mouse_state( LGRECT * bwrect, bool within, short mx, short my, short mbut, short mkstate ){
-	int i = 0;
-	int nx, ny;
-	struct gui_window * gw = input_window;
-
-	if( !gw ) {
-		bmstate = 0;
-		mouse_hold_start[0] = 0;
-		mouse_hold_start[1] = 0;
-		return;
-	}
-
-	/* todo: creat function find_browser_window( mx, my ) */
-	nx = (mx - bwrect->g_x + gw->browser->scroll.current.x);
-	ny = (my - bwrect->g_y + gw->browser->scroll.current.y);
-
-	if( mkstate & (K_RSHIFT | K_LSHIFT) ){
-		bmstate |= BROWSER_MOUSE_MOD_1;
-	} else {
-		bmstate &= ~(BROWSER_MOUSE_MOD_1);
-	}
-	if( (mkstate & K_CTRL) ){
-		bmstate |= BROWSER_MOUSE_MOD_2;
-	} else {
-		bmstate &= ~(BROWSER_MOUSE_MOD_2);
-	}
-	if( (mkstate & K_ALT) ){
-		bmstate |= BROWSER_MOUSE_MOD_3;
-	} else {
-		bmstate &= ~(BROWSER_MOUSE_MOD_3);
-	}
-
-	if( !(mbut&1) && !(mbut&2) ) {
-		if(bmstate & BROWSER_MOUSE_DRAG_ON )
-			bmstate &= ~( BROWSER_MOUSE_DRAG_ON );
-	}
-
-	/* todo: if we need right button click, increase loop count */
-	for( i = 1; i<2; i++ ) {
-		if( !(mbut & i) ) {
-			if( mouse_hold_start[i-1] > 0 ) {
-				mouse_hold_start[i-1] = 0;
-				if( i==1 ) {
-					LOG(("Drag for %d ended", i));
-					bmstate &= ~( BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_1 ) ;
-					if( within ) {
-						/* drag end */
-						browser_window_mouse_track(
-							gw->browser->bw, 0, nx, ny
-						);
-					}
-				}
-				if( i==2 ) {
-					bmstate &= ~( BROWSER_MOUSE_HOLDING_2 | BROWSER_MOUSE_DRAG_2 ) ;
-					LOG(("Drag for %d ended", i));
-					if( within ) {
-						/* drag end */
-						browser_window_mouse_track(
-							gw->browser->bw, 0, nx, ny
-						);
-					}
-				}
-			}
-		}
-	}
-}
-
-
-static void __CDECL evnt_window_m1( WINDOW * win, short buff[8], void * data)
-{
-	struct gui_window * gw = input_window;
-	static bool prev_url = false;
-	static bool prev_sb = false;
-	short mx, my, mbut, mkstate;
-	bool a = false;	/* flags if mouse is within controls or browser canvas */
-	bool within = false;
-	LGRECT urlbox, bwbox, sbbox;
-	int nx, ny; 	/* relative mouse position */
-
-
-	if( gw == NULL)
-		return;
-
-	if( gw != input_window ){
-		return;
-	}
-
-	graf_mkstate(&mx, &my, &mbut, &mkstate);
-
-	browser_get_rect( gw, BR_CONTENT, &bwbox );
-	if( gw->root->toolbar )
-		mt_CompGetLGrect(&app, gw->root->toolbar->url.comp, WF_WORKXYWH, &urlbox);
-	if( gw->root->statusbar )
-		mt_CompGetLGrect(&app, gw->root->statusbar->comp, WF_WORKXYWH, &sbbox);
-
-	if( mx > bwbox.g_x && mx < bwbox.g_x + bwbox.g_w &&
-		my > bwbox.g_y &&  my < bwbox.g_y + bwbox.g_h ){
-		within = true;
-	}
-
-	if( evnt.m1_flag == MO_LEAVE ) {
-		if( MOUSE_IS_DRAGGING() ){
-			window_track_mouse_state( &bwbox, within, mx, my, mbut, mkstate );
-		}
-		if( gw->root->toolbar && within == false ) {
-			if( (mx > urlbox.g_x && mx < urlbox.g_x + urlbox.g_w ) &&
-			 	(my > urlbox.g_y && my < + urlbox.g_y + urlbox.g_h )) {
-				gem_set_cursor( &gem_cursors.ibeam );
-				prev_url = a = true;
-			}
-		}
-		if( gw->root->statusbar && within == false /* && a == false */ ) {
-			if( mx >= sbbox.g_x + (sbbox.g_w-MOVER_WH) && mx <= sbbox.g_x + sbbox.g_w &&
-				my >= sbbox.g_y + (sbbox.g_h-MOVER_WH) && my <= sbbox.g_y + sbbox.g_h ) {
-				/* mouse within sizer box ( bottom right ) */
-				prev_sb = a =  true;
-				gem_set_cursor( &gem_cursors.sizenwse );
-			}
-		}
-		if( !a ) {
-			if( prev_sb )
-				gw->root->statusbar->resize_init = true;
-			if( prev_url || prev_sb ) {
-				gem_set_cursor( &gem_cursors.arrow );
-				prev_url = false;
-				prev_sb = false;
-			}
-			/* report mouse move in the browser window */
-			if( within ){
-				nx = mx - bwbox.g_x;
-				ny = my - bwbox.g_y;
-				if( ( abs(mx-last_drag_x)>5 || abs(mx-last_drag_y)>5 ) ||
-					!MOUSE_IS_DRAGGING() ){
-					browser_window_mouse_track(
-						input_window->browser->bw,
-						bmstate,
-						nx + gw->browser->scroll.current.x,
-						ny + gw->browser->scroll.current.y
-					);
-					if( MOUSE_IS_DRAGGING() ){
-						last_drag_x = mx;
-						last_drag_y = my;
-					}
-				}
-			}
-		}
-	} else {
-		/* set input window? */
-	}
-}
-
-int window_create( struct gui_window * gw, struct browser_window * bw, unsigned long inflags)
-{
-	short buff[8];
-	OBJECT * tbtree;
 	int err = 0;
 	bool tb, sb;
+	int flags;
+
 	tb = (inflags & WIDGET_TOOLBAR );
-	sb = (inflags & WIDGET_STATUSBAR); 
-	short w,h, wx, wy, wh, ww;
-	int flags = CLOSER | MOVER | NAME | FULLER | SMALLER ;
+	sb = (inflags & WIDGET_STATUSBAR );
+
+	flags = CLOSER | MOVER | NAME | FULLER | SMALLER;
+	if( inflags & WIDGET_SCROLL ){
+		flags |= (UPARROW | DNARROW | LFARROW | RTARROW | VSLIDE | HSLIDE);
+	}
+	if( inflags & WIDGET_RESIZE ){
+		flags |= ( SIZER );
+	}
 
 	gw->root = malloc( sizeof(struct s_gui_win_root) );
 	if( gw->root == NULL )
@@ -281,9 +118,16 @@ int window_create( struct gui_window * gw, struct browser_window * bw, unsigned
 		free( gw->root );
 		return( -1 );
 	}
+
+	/* set scroll / content granularity ( 1 unit ) */
+	gw->root->handle->w_u = 1;
+	gw->root->handle->h_u = 1;
+
+	/* Create Root component: */
 	gw->root->cmproot = mt_CompCreate(&app, CLT_VERTICAL, 1, 1);
 	WindSetPtr( gw->root->handle, WF_COMPONENT, gw->root->cmproot, NULL);
 
+	/* create toolbar component: */
 	if( tb ) {
 		gw->root->toolbar = tb_create( gw );
 		assert( gw->root->toolbar );
@@ -293,9 +137,11 @@ int window_create( struct gui_window * gw, struct browser_window * bw, unsigned
 		gw->root->toolbar = NULL;
 	}
 
+	/* create browser component: */
 	gw->browser = browser_create( gw, bw, NULL, CLT_HORIZONTAL, 1, 1 );
 	mt_CompAttach( &app, gw->root->cmproot,  gw->browser->comp );
 
+	/* create statusbar component: */
 	if( sb ) {
 		gw->root->statusbar = sb_create( gw );
 		mt_CompAttach( &app, gw->root->cmproot, gw->root->statusbar->comp );
@@ -308,34 +154,27 @@ int window_create( struct gui_window * gw, struct browser_window * bw, unsigned
 	/* Event Handlers: */
 	EvntDataAttach( gw->root->handle, WM_CLOSED, evnt_window_close, gw );
 	/* capture resize/move events so we can handle that manually */
-	EvntDataAttach( gw->root->handle, WM_SIZED, evnt_window_resize, gw );
-	if( !option_atari_realtime_move ) {
-		EvntDataAttach( gw->root->handle, WM_MOVED, evnt_window_move, gw );
-	} else {
-		EvntDataAdd( gw->root->handle, WM_MOVED, evnt_window_rt_resize, gw, EV_BOT );
-	}
-	EvntDataAttach( gw->root->handle, WM_FORCE_MOVE, evnt_window_rt_resize, gw );
-	EvntDataAttach( gw->root->handle, AP_DRAGDROP, evnt_window_dd, gw );
+	EvntDataAdd( gw->root->handle, WM_SIZED, evnt_window_rt_resize, gw, EV_BOT );
+	EvntDataAdd( gw->root->handle, WM_MOVED, evnt_window_rt_resize, gw, EV_BOT );
+	EvntDataAdd( gw->root->handle, WM_FULLED, evnt_window_rt_resize, gw, EV_BOT );
 	EvntDataAdd( gw->root->handle, WM_DESTROY,evnt_window_destroy, gw, EV_TOP );
 	EvntDataAdd( gw->root->handle, WM_ARROWED,evnt_window_arrowed, gw, EV_TOP );
 	EvntDataAdd( gw->root->handle, WM_NEWTOP, evnt_window_newtop, gw, EV_BOT);
 	EvntDataAdd( gw->root->handle, WM_TOPPED, evnt_window_newtop, gw, EV_BOT);
+	EvntDataAdd( gw->root->handle, WM_ICONIFY, evnt_window_iconify, gw, EV_BOT);
+	EvntDataAdd( gw->root->handle, WM_UNICONIFY, evnt_window_uniconify, gw, EV_BOT);
+	EvntDataAttach( gw->root->handle, AP_DRAGDROP, evnt_window_dd, gw );
 	EvntDataAttach( gw->root->handle, WM_ICONDRAW, evnt_window_icondraw, gw);
-	EvntDataAttach( gw->root->handle, WM_XM1, evnt_window_m1, gw );
+	EvntDataAttach( gw->root->handle, WM_SLIDEXY, evnt_window_slider, gw );
 
-	/*
-	OBJECT * tbut;
-	RsrcGaddr( h_gem_rsrc, R_TREE, FAVICO , &tbut );
-	window_set_icon(gw, &tbut[]);
-	*/
 	/* TODO: check if window is openend as "foreground" window... */
 	window_set_focus( gw, BROWSER, gw->browser);
+
 	return (err);
 }
 
 int window_destroy( struct gui_window * gw)
 {
-	short buff[8];
 	int err = 0;
 
 	search_destroy( gw );
@@ -352,20 +191,15 @@ int window_destroy( struct gui_window * gw)
 
 	search_destroy( gw );
 
-	LOG(("Freeing browser window"));
 	if( gw->browser )
 		browser_destroy( gw->browser );
 
-
-	/* destroy the icon: */
-	/*window_set_icon(gw, NULL, false );*/
-
 	/* needed? */ /*listRemove( (LINKABLE*)gw->root->cmproot ); */
-	LOG(("Freeing root window"));
 	if( gw->root ) {
 		/* TODO: check if no other browser is bound to this root window! */
+		/* only needed for tabs */
 		if( gw->root->title )
-			free( gw->root->title ); 
+			free( gw->root->title );
 		if( gw->root->cmproot )
 			mt_CompDelete( &app, gw->root->cmproot );
 		ApplWrite( _AESapid, WM_DESTROY, gw->root->handle->handle, 0, 0, 0, 0);
@@ -379,15 +213,18 @@ int window_destroy( struct gui_window * gw)
 
 
 
-void window_open( struct gui_window * gw)
+void window_open( struct gui_window * gw, GRECT pos )
 {
 	LGRECT br;
-	GRECT dim;
-	WindOpen(gw->root->handle, 20, 20, app.w/2, app.h/2 );
+
+	WindOpen(gw->root->handle, pos.g_x, pos.g_y, pos.g_w, pos.g_h );
+	WindClear( gw->root->handle );
 	WindSetStr( gw->root->handle, WF_NAME, (char *)"" );
+
 	/* apply focus to the root frame: */
 	long lfbuff[8] = { CM_GETFOCUS };
 	mt_CompEvntExec( gl_appvar, gw->browser->comp, lfbuff );
+
 	/* recompute the nested component sizes and positions: */
 	browser_update_rects( gw );
 	mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
@@ -398,38 +235,12 @@ void window_open( struct gui_window * gw)
 	if( gw->root->statusbar != NULL ){
 		gw->root->statusbar->attached = true;
 	}
-	snd_rdw( gw->root->handle );
+	tb_adjust_size( gw );
+	/*TBD: get already present content and set size? */
+	input_window = gw;
+	window_set_focus( gw, BROWSER, gw->browser );
 }
 
-/*
-TODO
-void window_set_icon(struct gui_window * gw, void * data, bool is_rsc )
-{
-	#define CDT_ICON_TYPE_OBJECT 1UL
-	#define CDT_ICON_TYPE_BITMAP 2UL
-	void * prev_type;
-	void * ico = DataSearch(&app, gw->root->handle, CDT_ICON );
-	if(ico != NULL) {
-		prev_type = DataSearch(&app, gw->root->handle, CDT_ICON_TYPE );
-		if( prev_type == (void*)CDT_ICON_TYPE_OBJECT ){
-			mt_ObjcFree( &app, (OBJECT*)ico );
-		}
-		if( prev_type == (void*)CDT_ICON_TYPE_BITMAP ){
-			bitmap_destroy(ico);
-		}
-	}
-	if( data != NULL ) {
-		DataAttach( &app, gw->root->handle, CDT_ICON, data);
-		if(is_rsc) {
-			DataAttach( &app, gw->root->handle, CDT_ICON_TYPE, CDT_ICON_TYPE_OBJECT);
-		} else {
-			DataAttach( &app, gw->root->handle, CDT_ICON_TYPE, CDT_ICON_TYPE_BITMAP);
-		}
-	}
-	#undef CDT_ICON_TYPE_OBJECT
-	#undef CDT_ICON_TYPE_BITMAP
-}
-*/
 
 
 /* update back forward buttons (see tb_update_buttons (bug) ) */
@@ -438,28 +249,6 @@ void window_update_back_forward( struct gui_window * gw)
 	tb_update_buttons( gw );
 }
 
-static void window_redraw_controls(struct gui_window *gw, uint32_t flags)
-{
-	LGRECT rect;
-	/* redraw sliders manually, dunno why this is needed (mt_WindSlider should do the job anytime)!*/
-
-	browser_get_rect( gw, BR_VSLIDER, &rect);
-	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
-		rect.g_x, rect.g_y, rect.g_w, rect.g_h );
-
-	browser_get_rect( gw, BR_HSLIDER, &rect);
-	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
-		rect.g_x, rect.g_y, rect.g_w, rect.g_h );
-
-	/* send redraw to toolbar & statusbar & scrollbars: */
-	mt_CompGetLGrect(&app, gw->root->toolbar->comp, WF_WORKXYWH, &rect);
-	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
-		rect.g_x, rect.g_y, rect.g_w, rect.g_h );
-	mt_CompGetLGrect(&app, gw->root->statusbar->comp, WF_WORKXYWH, &rect);	
-	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
-		rect.g_x, rect.g_y, rect.g_w, rect.g_h );
-}
-
 void window_set_stauts( struct gui_window * gw , char * text )
 {
 	if( gw->root == NULL )
@@ -480,6 +269,18 @@ void window_set_focus( struct gui_window * gw, enum focus_element_type type, voi
 		LOG(("Set focus: %p (%d)\n", element, type));
 		gw->root->focus.type = type;
 		gw->root->focus.element = element;
+		if( element != NULL ){
+			switch( type ){
+
+				case URL_WIDGET:
+					textarea_keypress(((struct s_url_widget*)(element))->textarea,
+										KEY_SELECT_ALL );
+					break;
+
+				default: break;
+
+			}
+		}
 	}
 }
 
@@ -509,7 +310,42 @@ bool window_widget_has_focus( struct gui_window * gw, enum focus_element_type t,
 	return( ( element == gw->root->focus.element && t == gw->root->focus.type) );
 }
 
-static void __CDECL evnt_window_dd( WINDOW *win, short wbuff[8], void * data ) 
+
+/* -------------------------------------------------------------------------- */
+/* Event Handlers:                                                            */
+/* -------------------------------------------------------------------------- */
+
+static void __CDECL evnt_window_arrowed( WINDOW *win, short buff[8], void *data )
+{
+	bool abs = false;
+	LGRECT cwork;
+	struct gui_window * gw = data;
+	int value = BROWSER_SCROLL_SVAL;
+
+	assert( gw != NULL );
+
+	browser_get_rect( gw, BR_CONTENT, &cwork );
+
+	switch( buff[4] ) {
+		case WA_UPPAGE:
+		case WA_DNPAGE:
+				value = cwork.g_h;
+			break;
+
+
+		case WA_LFPAGE:
+		case WA_RTPAGE:
+				value = cwork.g_w;
+			break;
+
+		default:
+			break;
+	}
+	browser_scroll( gw, buff[4], value, abs );
+}
+
+
+static void __CDECL evnt_window_dd( WINDOW *win, short wbuff[8], void * data )
 {
 	struct gui_window * gw = (struct gui_window *)data;
 	char file[DD_NAMEMAX];
@@ -533,7 +369,7 @@ static void __CDECL evnt_window_dd( WINDOW *win, short wbuff[8], void * data )
 	memset( ext, 0, 32);
 	strcpy( ext, "ARGS");
 	dd_msg = ddsexts( dd_hdl, ext);
-	if( dd_msg<0) 
+	if( dd_msg<0)
 		goto error;
 	dd_msg = ddrtry( dd_hdl, (char*)&name[0], (char*)&file[0], (char*)&ext[0], &size);
 	if( size+1 >= PATH_MAX )
@@ -544,83 +380,39 @@ static void __CDECL evnt_window_dd( WINDOW *win, short wbuff[8], void * data )
 		buff = (char*)alloca(sizeof(char)*(size+1));
 		if( buff != NULL )
 		{
-			if( Fread(dd_hdl, size, buff ) == size) 
+			if( Fread(dd_hdl, size, buff ) == size)
 			{
 				buff[size] = 0;
 			}
-			LOG(("file: %s, ext: %s, size: %d dropped at: %d,%d\n", 
-				(char*)buff, (char*)&ext, 
-				size, mx, my 
+			LOG(("file: %s, ext: %s, size: %d dropped at: %d,%d\n",
+				(char*)buff, (char*)&ext,
+				size, mx, my
 			));
 			{
-				int posx, posy;
-				struct box *box;
-				struct box *file_box = 0;
-				hlcache_handle *h;
-				int box_x, box_y;
 				LGRECT bwrect;
 				struct browser_window * bw = gw->browser->bw;
-				h = bw->current_content;
-				if (!bw->current_content || content_get_type(h) != CONTENT_HTML)
-					return;
 				browser_get_rect( gw, BR_CONTENT, &bwrect );
 				mx = mx - bwrect.g_x;
 				my = my - bwrect.g_y;
 				if( (mx < 0 || mx > bwrect.g_w) || (my < 0 || my > bwrect.g_h) )
 					return;
-				box = html_get_box_tree(h);
-				box_x = box->margin[LEFT];
-				box_y = box->margin[TOP];
 
-				while ((box = box_at_point(box, mx+gw->browser->scroll.current.x, my+gw->browser->scroll.current.y, &box_x, &box_y, &h))) 
-				{
-					if (box->style && css_computed_visibility(box->style) == CSS_VISIBILITY_HIDDEN)
-						continue;
-					if (box->gadget) 
-					{
-						switch (box->gadget->type) 
-						{
-							case GADGET_FILE:
-								file_box = box;
-							break;
-							/*
-							TODO: handle these
-							case GADGET_TEXTBOX:
-							case GADGET_TEXTAREA:
-							case GADGET_PASSWORD:
-							text_box = box;
-							break;
-							*/
-							default:
-								break;
-						}
-					}
-				} /* end While */
-				if ( !file_box )
-					return;
-				if (file_box) {
-					utf8_convert_ret ret;
-					char *utf8_fn;
+				utf8_convert_ret ret;
+				char *utf8_fn;
 
-					ret = local_encoding_to_utf8( buff, 0, &utf8_fn);
-					if (ret != UTF8_CONVERT_OK) {
+				ret = utf8_from_local_encoding( buff, 0, &utf8_fn);
+				if (ret != UTF8_CONVERT_OK) {
 						/* A bad encoding should never happen */
 						LOG(("utf8_from_local_encoding failed"));
 						assert(ret != UTF8_CONVERT_BADENC);
-						/* Load was for us - just no memory */
+						/* no memory */
 						return;
-					}
-					/* Found: update form input. */
-					free(file_box->gadget->value);
-					file_box->gadget->value = utf8_fn;
-					/* Redraw box. */
-					box_coords(file_box, &posx, &posy);
-					browser_schedule_redraw(bw->window, 
-						posx - gw->browser->scroll.current.x, 
-						posy - gw->browser->scroll.current.y,
-						posx - gw->browser->scroll.current.x + file_box->width,
-						posy - gw->browser->scroll.current.y + file_box->height);
 				}
+				browser_window_drop_file_at_point( gw->browser->bw,
+											mx+gw->browser->scroll.current.x,
+											my+gw->browser->scroll.current.y,
+											utf8_fn );
+				free( utf8_fn );
 			}
 		}
 	}
@@ -628,9 +420,6 @@ error:
 	ddclose( dd_hdl);
 }
 
-/* -------------------------------------------------------------------------- */
-/* Non Public Modul event handlers:                                           */
-/* -------------------------------------------------------------------------- */
 static void __CDECL evnt_window_destroy( WINDOW *win, short buff[8], void *data )
 {
 	LOG(("%s\n", __FUNCTION__ ));
@@ -648,19 +437,51 @@ static void __CDECL evnt_window_close( WINDOW *win, short buff[8], void *data )
 static void __CDECL evnt_window_newtop( WINDOW *win, short buff[8], void *data )
 {
 	input_window = (struct gui_window *) data;
-	LOG(("newtop: iw: %p, win: %p", input_window, win ));
+	window_set_focus( input_window, BROWSER, input_window->browser );
+	LOG(("newtop gui window: %p, WINDOW: %p", input_window, win ));
 	assert( input_window != NULL );
-
-	window_redraw_controls(input_window, 0);
 }
 
-static void __CDECL evnt_window_shaded( WINDOW *win, short buff[8], void *data )
+static void __CDECL evnt_window_slider( WINDOW * win, short buff[8], void * data)
 {
-	if(buff[0] == WM_SHADED){
-		LOG(("WM_SHADED, vis: %d, state: %d", GEMWIN_VISIBLE(win), win->status ));
-	}
-	if(buff[0] == WM_UNSHADED){
+	int dx = buff[4];
+	int dy = buff[5];
+	struct gui_window * gw = data;
 
+	if (!dx && !dy) return;
+
+	if( input_window == NULL || input_window != gw ) {
+		return;
+	}
+
+	/* 	update the sliders _before_ we call redraw
+		(which might depend on the slider possitions) */
+	WindSlider( win, (dx?HSLIDER:0) | (dy?VSLIDER:0) );
+
+	if( dy > 0 )
+		browser_scroll( gw, WA_DNPAGE, abs(dy), false );
+	else if ( dy < 0)
+		browser_scroll( gw, WA_UPPAGE, abs(dy), false );
+	if( dx > 0 )
+		browser_scroll( gw, WA_RTPAGE, abs(dx), false );
+	else if( dx < 0 )
+		browser_scroll( gw, WA_LFPAGE, abs(dx), false );
+}
+
+static void __CDECL evnt_window_uniconify( WINDOW *win, short buff[8], void * data )
+{
+	struct gui_window * gw = (struct gui_window *)data;
+
+	input_window = gw;
+	WindTop( gw->root->handle );
+	window_set_focus( gw, BROWSER, gw->browser );
+}
+
+static void __CDECL evnt_window_iconify( WINDOW *win, short buff[8], void * data )
+{
+	struct gui_window * gw = (struct gui_window *)data;
+	if( input_window == gw){
+		input_window = NULL;
 	}
 }
 
@@ -668,12 +489,10 @@ static void __CDECL evnt_window_icondraw( WINDOW *win, short buff[8], void * dat
 {
 	short x,y,w,h;
 	struct gui_window * gw = (struct gui_window*)data;
-	bool has_favicon = false;
 
 	WindClear( win);
 	WindGet( win, WF_WORKXYWH, &x, &y, &w, &h);
-
-	if( has_favicon == false ) {
+	if( gw->icon == NULL ) {
 		OBJECT * tree;
 		RsrcGaddr( h_gem_rsrc, R_TREE, ICONIFY , &tree );
 		tree->ob_x = x;
@@ -681,59 +500,25 @@ static void __CDECL evnt_window_icondraw( WINDOW *win, short buff[8], void * dat
 		tree->ob_width = w;
 		tree->ob_height = h;
 		mt_objc_draw( tree, 0, 8, buff[4], buff[5], buff[6], buff[7], app.aes_global );
+	} else {
+	    struct rect clip = { 0,0,w,h };
+        plotter->move( plotter, x, y );
+        plotter->resize( plotter, w, h );
+        plotter->clip(plotter, &clip );
+        plotter->bitmap_resize( plotter,  gw->icon, w, h  );
+        plotter->bitmap(
+			plotter,
+			( gw->icon->resized ) ? gw->icon->resized : gw->icon,
+			0, 0, 0xffffff, BITMAPF_NONE
+		);
 	}
 }
 
-static void __CDECL evnt_window_move( WINDOW *win, short buff[8], void * data )
-{
-	short mx,my, mb, ks;
-	short wx, wy, wh, ww, nx, ny;
-	short r;
-	short xoff, yoff;
-	if( option_atari_realtime_move  ) {
-		std_mvd( win, buff, &app );
-		evnt_window_rt_resize( win, buff, data );
-	} else { 
-		wind_get( win->handle, WF_CURRXYWH, &wx, &wy, &ww, &wh );
-		if( graf_dragbox( ww, wh, wx, wy, app.x-ww, app.y, app.w+ww, app.h+wh, &nx, &ny )){
-			buff[4] = nx;
-			buff[5] = ny;
-			buff[6] = ww;
-			buff[7] = wh;
-			std_mvd( win, buff, &app );
-			evnt_window_rt_resize( win, buff, data );
-		}
-	}
-}
-
-void __CDECL evnt_window_resize( WINDOW *win, short buff[8], void * data )
-{
-	short wx, wy, wh, ww, nw, nh;
-	short r;
-
-	wind_get( win->handle, WF_CURRXYWH, &wx, &wy, &ww, &wh );
-	r = graf_rubberbox(wx, wy, 20, 20, &nw, &nh);
-	if( nw < 40 && nw < 40 )
-		return;
-	buff[4] = wx;
-	buff[5] = wy;
-	buff[6] = nw;
-	buff[7] = nh;
-	std_szd( win, buff, &app );
-	evnt_window_rt_resize( win, buff, data );
-}
-
 /* perform the actual resize */
 static void __CDECL evnt_window_rt_resize( WINDOW *win, short buff[8], void * data )
 {
 	short x,y,w,h;
 	struct gui_window * gw;
-	LGRECT rect;
-
-	if(buff[0] == WM_FORCE_MOVE ) {
-		std_mvd(win, buff, &app);
-		std_szd(win, buff, &app);
-	}
 
 	wind_get( win->handle, WF_CURRXYWH, &x, &y, &w, &h );
 	gw = (struct gui_window *)data;
@@ -743,17 +528,27 @@ static void __CDECL evnt_window_rt_resize( WINDOW *win, short buff[8], void * da
 	if(gw->root->loc.g_w != w || gw->root->loc.g_h != h ){
 		/* report resize to component interface: */
 		browser_update_rects( gw );
-		mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
-		browser_get_rect( gw, BR_CONTENT, &rect );
-		if( gw->browser->bw->current_content != NULL )
-			browser_window_reformat(gw->browser->bw, false, rect.g_w, rect.g_h );
-		gw->root->toolbar->url.scrollx = 0;
-		window_redraw_controls(gw, 0);
-		/* TODO: recalculate scroll position, instead of zeroing? */
+		tb_adjust_size( gw );
+		if( gw->browser->bw->current_content != NULL ){
+			/* Reformat will happen when next redraw message arrives: */
+			gw->browser->reformat_pending = true;
+			if( sys_XAAES() ){
+				if( gw->root->loc.g_w > w || gw->root->loc.g_h > h ){
+					ApplWrite( _AESapid, WM_REDRAW, gw->root->handle->handle,
+								gw->root->loc.g_x, gw->root->loc.g_y,
+								gw->root->loc.g_w, gw->root->loc.g_h );
+				}
+			}
+			mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH,
+							(GRECT*)&gw->root->loc);
+		}
+		else {
+			WindClear( gw->root->handle );
+		}
 	} else {
 		if(gw->root->loc.g_x != x || gw->root->loc.g_y != y ){
-       			mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
+			mt_WindGetGrect( &app, gw->root->handle, WF_CURRXYWH, (GRECT*)&gw->root->loc);
 			browser_update_rects( gw );
-        	}
+		}
 	}
 }
