@@ -31,8 +31,10 @@
 #include "desktop/mouse.h"
 #include "desktop/textinput.h"
 #include "desktop/hotlist.h"
+#include "desktop/save_complete.h"
 #include "utils/log.h"
 #include "utils/messages.h"
+#include "utils/url.h"
 
 #include "atari/gui.h"
 #include "atari/browser_win.h"
@@ -45,6 +47,8 @@
 #include "atari/res/netsurf.rsh"
 #include "atari/search.h"
 #include "atari/options.h"
+#include "atari/findfile.h"
+#include "atari/settings.h"
 #include "cflib.h"
 
 extern const char * cfg_homepage_url;
@@ -68,6 +72,18 @@ extern short last_drag_y;
 #define NUM_MENU_TITLES 7
 static char * menu_titles[NUM_MENU_TITLES] = {NULL};
 
+/* Global event handlers: */
+static void __CDECL global_evnt_apterm( WINDOW * win, short buff[8] );
+static void __CDECL global_evnt_menu( WINDOW * win, short buff[8] );
+static void __CDECL global_evnt_m1( WINDOW * win, short buff[8] );
+static void __CDECL global_evnt_keybd( WINDOW * win, short buff[8],void * data);
+
+/* Menu event handlers: */
+static void __CDECL menu_about(WINDOW *win, int item, int title, void *data);
+
+static char * get_accel(int mode, char * message, struct s_accelerator * accel);
+static int parse_accel( char * message, struct s_accelerator * accel);
+
 
 /* Menu event handlers: */
 static void __CDECL menu_about(WINDOW *win, int item, int title, void *data)
@@ -89,19 +105,39 @@ static void __CDECL menu_open_url(WINDOW *win, int item, int title, void *data)
 {
 	struct gui_window * gw;
 	struct browser_window * bw ;
+	LOG(("%s", __FUNCTION__));
 
 	gw = input_window;
 	if( gw == NULL ) {
 		bw = browser_window_create("", 0, 0, true, false);
 		gw = bw->window;
+
 	}
-	/* TODO: reset url? */
-	LOG(("%s", __FUNCTION__));
+	/* Loose focus: */
+	window_set_focus( gw, WIDGET_NONE, NULL );
+
+	/* trigger on-focus event (select all text): */
+	window_set_focus( gw, URL_WIDGET, &gw->root->toolbar->url );
+
+	/* delete selection: */
+	tb_url_input( gw, NK_DEL );
 }
 
 static void __CDECL menu_open_file(WINDOW *win, int item, int title, void *data)
 {
+	struct gui_window * gw;
+	struct browser_window * bw ;
+
 	LOG(("%s", __FUNCTION__));
+
+	const char * filename = file_select( messages_get("OpenFile"), "" );
+	if( filename != NULL ){
+		char * url = local_file_to_url( filename );
+		if( url ){
+			bw = browser_window_create(url, NULL, NULL, true, false);
+			free( url );
+		}
+	}
 }
 
 static void __CDECL menu_close_win(WINDOW *win, int item, int title, void *data)
@@ -115,6 +151,41 @@ static void __CDECL menu_close_win(WINDOW *win, int item, int title, void *data)
 static void __CDECL menu_save_page(WINDOW *win, int item, int title, void *data)
 {
 	LOG(("%s", __FUNCTION__));
+	static bool init = true;
+	bool is_folder=false;
+	const char * path;
+
+	if( !input_window )
+		return;
+
+	if( init ){
+		init = false;
+		save_complete_init();
+	}
+
+	do {
+		// TODO: localize string
+		path = file_select( "Select folder", "" );
+		if( path ) {
+			printf("testing: %s\n", path );
+			// dumb check if the selection is an folder:
+			/*FILE * fp;
+			fp = fopen( path, "r" );
+			if( !fp ){
+				is_folder = true;
+			} else {
+				fclose( fp );
+				form_alert(1, "[1][Please select an folder or abort!][OK]");
+			}
+			*/
+			is_folder = true;
+		}
+	} while( !is_folder && path != NULL );
+
+	if( path != NULL ){
+		save_complete( input_window->browser->bw->current_content, path );
+	}
+
 }
 
 static void __CDECL menu_quit(WINDOW *win, int item, int title, void *data)
@@ -154,7 +225,9 @@ static void __CDECL menu_find(WINDOW *win, int item, int title, void *data)
 
 static void __CDECL menu_choices(WINDOW *win, int item, int title, void *data)
 {
+	static WINDOW * settings_dlg = NULL;
 	LOG(("%s", __FUNCTION__));
+	settings_dlg = open_settings();
 }
 
 static void __CDECL menu_stop(WINDOW *win, int item, int title, void *data)
@@ -176,7 +249,12 @@ static void __CDECL menu_reload(WINDOW *win, int item, int title, void *data)
 
 static void __CDECL menu_toolbars(WINDOW *win, int item, int title, void *data)
 {
+	static int state = 0;
 	LOG(("%s", __FUNCTION__));
+	if( input_window != null && input_window->root->toolbar != null ){
+		state = !state;
+		tb_hide( input_window, state );
+	}
 }
 
 static void __CDECL menu_savewin(WINDOW *win, int item, int title, void *data)
@@ -192,11 +270,26 @@ static void __CDECL menu_debug_render(WINDOW *win, int item, int title, void *da
 		if ( input_window->browser != NULL && input_window->browser->bw != NULL) {
 			LGRECT rect;
 			browser_get_rect( input_window, BR_CONTENT, &rect );
-			browser_window_reformat(input_window->browser->bw, false, rect.g_w, rect.g_h );
+			browser_window_reformat(input_window->browser->bw, false,
+									rect.g_w, rect.g_h );
+			MenuIcheck(NULL, MAINMENU_M_DEBUG_RENDER,
+						(html_redraw_debug) ? 1 : 0 );
 		}
 	}
 }
 
+static void __CDECL menu_fg_images(WINDOW *win, int item, int title, void *data)
+{
+	option_foreground_images = !option_foreground_images;
+	MenuIcheck( NULL, MAINMENU_M_FG_IMAGES, (option_foreground_images) ? 1 : 0);
+}
+
+static void __CDECL menu_bg_images(WINDOW *win, int item, int title, void *data)
+{
+	option_background_images = !option_background_images;
+	MenuIcheck( NULL, MAINMENU_M_BG_IMAGES, (option_background_images) ? 1 : 0);
+}
+
 static void __CDECL menu_back(WINDOW *win, int item, int title, void *data)
 {
 	LOG(("%s", __FUNCTION__));
@@ -224,6 +317,8 @@ static void __CDECL menu_home(WINDOW *win, int item, int title, void *data)
 static void __CDECL menu_lhistory(WINDOW *win, int item, int title, void *data)
 {
 	LOG(("%s", __FUNCTION__));
+	if( input_window == NULL )
+		return;
 }
 
 static void __CDECL menu_ghistory(WINDOW *win, int item, int title, void *data)
@@ -241,7 +336,7 @@ static void __CDECL menu_add_bookmark(WINDOW *win, int item, int title, void *da
 	if( input_window ) {
 		if( input_window->browser->bw->current_content != NULL ){
 			atari_hotlist_add_page(
-				content_get_url( input_window->browser->bw->current_content),
+				nsurl_access(hlcache_handle_get_url( input_window->browser->bw->current_content)),
 				NULL
 			);
 		}
@@ -283,13 +378,15 @@ static struct s_menu_item_evnt menu_evnt_tbl[] =
 	{T_VIEW, MAINMENU_M_TOOLBARS, "Toolbars", menu_toolbars, {0,0,0}, NULL},
 	{T_VIEW, MAINMENU_M_SAVEWIN, "", menu_savewin, {0,0,0}, NULL},
 	{T_VIEW, MAINMENU_M_DEBUG_RENDER, "", menu_debug_render, {0,0,0}, NULL},
+	{T_VIEW, MAINMENU_M_FG_IMAGES, "", menu_fg_images, {0,0,0}, NULL},
+	{T_VIEW, MAINMENU_M_BG_IMAGES, "", menu_bg_images, {0,0,0}, NULL},
 	{T_VIEW, MAINMENU_M_STOP, "Stop", menu_stop, {0,0,0}, NULL},
 	{T_NAV, MAINMENU_M_BACK, "Back", menu_back, {0,0,0}, NULL},
 	{T_NAV, MAINMENU_M_FORWARD, "Forward", menu_forward, {0,0,0}, NULL},
 	{T_NAV, MAINMENU_M_HOME, "Home", menu_home, {0,0,0}, NULL},
 	{T_UTIL, MAINMENU_M_LHISTORY, "HistLocal", menu_lhistory, {0,0,0}, NULL},
 	{T_UTIL, MAINMENU_M_GHISTORY, "HistGlobal", menu_ghistory, {0,0,0}, NULL},
-	{T_UTIL, MAINMENU_M_ADD_BOOKMARK, "HotlistAdd", menu_add_bookmark, {0,0,0}, NULL},
+	{T_UTIL, MAINMENU_M_ADD_BOOKMARK, "HotlistAdd", menu_add_bookmark, {'D',0,K_CTRL}, NULL},
 	{T_UTIL, MAINMENU_M_BOOKMARKS, "HotlistShow", menu_bookmarks, {0,0,0}, NULL},
 	{T_UTIL, MAINMENU_M_CHOICES, "Choices", menu_choices, {0,0,0}, NULL},
 	{T_UTIL, MAINMENU_M_VLOG, "Verbose Log", menu_vlog, {0,0,0}, NULL},
@@ -304,6 +401,65 @@ void __CDECL global_evnt_apterm( WINDOW * win, short buff[8] )
 	netsurf_quit = true;
 }
 
+
+static void __CDECL global_evnt_m1( WINDOW * win, short buff[8] )
+{
+	struct gui_window * gw = input_window;
+	static bool prev_url = false;
+	static short prev_x=0;
+	static short prev_y=0;
+	bool within = false;
+	LGRECT urlbox, bwbox, sbbox;
+	int nx, ny;
+
+	if( gw == NULL)
+		return;
+
+	if( prev_x == evnt.mx && prev_y == evnt.my ){
+		return;
+	}
+
+	short ghandle = wind_find( evnt.mx, evnt.my );
+	if( input_window->root->handle->handle == ghandle ){
+
+		// The window found at x,y is an gui_window
+		// and it's the input window.
+
+		browser_get_rect( gw, BR_CONTENT, &bwbox );
+
+		if( evnt.mx > bwbox.g_x && evnt.mx < bwbox.g_x + bwbox.g_w &&
+			evnt.my > bwbox.g_y &&  evnt.my < bwbox.g_y + bwbox.g_h ){
+			within = true;
+			browser_window_mouse_track(
+							input_window->browser->bw,
+							0,
+							evnt.mx - bwbox.g_x + gw->browser->scroll.current.x,
+							evnt.my - bwbox.g_y + gw->browser->scroll.current.y
+						);
+		}
+
+		if( gw->root->toolbar && within == false ) {
+			mt_CompGetLGrect(&app, gw->root->toolbar->url.comp, WF_WORKXYWH, &urlbox);
+			if( (evnt.mx > urlbox.g_x && evnt.mx < urlbox.g_x + urlbox.g_w ) &&
+				(evnt.my > urlbox.g_y && evnt.my < + urlbox.g_y + urlbox.g_h )) {
+				gem_set_cursor( &gem_cursors.ibeam );
+				prev_url = true;
+			} else {
+				if( prev_url ) {
+					gem_set_cursor( &gem_cursors.arrow );
+					prev_url = false;
+				}
+			}
+		}
+	} else {
+		gem_set_cursor( &gem_cursors.arrow );
+		prev_url = false;
+	}
+
+	prev_x = evnt.mx;
+	prev_y = evnt.my;
+}
+
 void __CDECL global_evnt_keybd( WINDOW * win, short buff[8], void * data)
 {
 	char sascii;
@@ -320,15 +476,14 @@ void __CDECL global_evnt_keybd( WINDOW * win, short buff[8], void * data)
 		return;
 	kstate = evnt.mkstate;
 	kcode = evnt.keybd;
-	nkc= gem_to_norm( (short)kstate, (short)kcode ); 
+	nkc= gem_to_norm( (short)kstate, (short)kcode );
 	nks = (nkc & 0xFF00);
 	if( kstate & (K_LSHIFT|K_RSHIFT))
 		kstate |= K_LSHIFT|K_RSHIFT;
 	if( window_url_widget_has_focus( gw ) ) {
 		/* make sure we report for the root window and report...: */
- 		done = tb_url_input( gw,  nkc );
-		if( done ) return;
-	} else  {
+		done = tb_url_input( gw, nkc );
+	}  else  {
 		gw_tmp = window_list;
 		/* search for active browser component: */
 		while( gw_tmp != NULL && done == false ) {
@@ -354,11 +509,11 @@ void __CDECL global_evnt_keybd( WINDOW * win, short buff[8], void * data)
 			} else {
 				/* the accel code hides in the keycode: */
 				if( menu_evnt_tbl[i].accel.keycode != 0) {
-					if( menu_evnt_tbl[i].accel.keycode == (nkc & 0xFF) && 
-						kstate == menu_evnt_tbl[i].accel.mod && 
+					if( menu_evnt_tbl[i].accel.keycode == (nkc & 0xFF) &&
+						kstate == menu_evnt_tbl[i].accel.mod &&
 						menu_evnt_tbl[i].menu_func != NULL) {
-							menu_evnt_tbl[i].menu_func( NULL, 
-								menu_evnt_tbl[i].rid, 
+							menu_evnt_tbl[i].menu_func( NULL,
+								menu_evnt_tbl[i].rid,
 								MAINMENU, buff
 							);
 							done = true;
@@ -372,7 +527,7 @@ void __CDECL global_evnt_keybd( WINDOW * win, short buff[8], void * data)
 }
 
 /*
-	mode = 0 -> return string ptr 
+	mode = 0 -> return string ptr
 				(build from accel definition in s_accelerator accel)
 	mode = 1 -> return ptr to (untranslated) NS accel string, if any
 */
@@ -464,7 +619,7 @@ static int parse_accel( char * message, struct s_accelerator * accel)
 		accel->mod |= K_CTRL;
 		s++;
 	}
-parsekey:
+
 	/* expect  F1/F10 or something like A, B, C ... : */
 	if(strlen(s) >= 2 && s[0] == 'F' ) {
 		if(s[1] >= 49 && s[1] <= 57) {
@@ -504,9 +659,11 @@ static void set_menu_title(int rid, const char * nsid)
 {
 	static int count=0;
 	char * msgstr;
-  msgstr = (char*)messages_get(nsid);
+	msgstr = (char*)messages_get(nsid);
 	if(msgstr != NULL) {
 		if(msgstr[0] != 0) {
+			// TODO: modify resource tree, adjust width of menu to chars
+			// actually used.
 			assert(count < NUM_MENU_TITLES);
 			menu_titles[count] = malloc( strlen(msgstr)+3 );
 			strcpy((char*)menu_titles[count], " ");
@@ -518,6 +675,13 @@ static void set_menu_title(int rid, const char * nsid)
 	}
 }
 
+void main_menu_update( void )
+{
+	MenuIcheck( NULL, MAINMENU_M_DEBUG_RENDER, (html_redraw_debug) ? 1 : 0);
+	MenuIcheck( NULL, MAINMENU_M_FG_IMAGES, (option_foreground_images) ? 1 : 0);
+	MenuIcheck( NULL, MAINMENU_M_BG_IMAGES, (option_background_images) ? 1 : 0);
+}
+
 
 /* Bind global and menu events to event handler functions, create accelerators */
 void bind_global_events( void )
@@ -530,9 +694,8 @@ void bind_global_events( void )
 	EvntDataAttach( NULL, WM_XKEYBD, global_evnt_keybd, (void*)&evnt_data );
 	EvntAttach( NULL, AP_TERM, global_evnt_apterm );
 	EvntAttach( NULL, MN_SELECTED,  global_evnt_menu );
-	/* EvntDataAttach( NULL, WM_XM1, global_evnt_m1, NULL ); */
+	EvntAttach( NULL, WM_XM1, global_evnt_m1 );
 
-	/* TODO: maybe instant redraw after this is better! */
 	set_menu_title( MAINMENU_T_FILE, "Page");
 	set_menu_title( MAINMENU_T_EDIT, "Edit" );
 	set_menu_title( MAINMENU_T_NAVIGATE, "Navigate");
@@ -572,10 +735,10 @@ void bind_global_events( void )
 			spare[1]=' ';
 			if( u != NULL && t != NULL ) {
 				LOG(("Menu Item %s: found NS accelerator, ascii: %c, scancode: %x, mod: %x",
-					m, 
-					menu_evnt_tbl[i].accel.ascii, 
-					menu_evnt_tbl[i].accel.keycode, 
-					menu_evnt_tbl[i].accel.mod 
+					m,
+					menu_evnt_tbl[i].accel.ascii,
+					menu_evnt_tbl[i].accel.keycode,
+					menu_evnt_tbl[i].accel.mod
 				));
 				/* Accelerator is defined in menu string: */
 				memcpy((char*)&spare[2], m, u-m-1);
@@ -583,10 +746,10 @@ void bind_global_events( void )
 			}
 			else if( t != NULL && u == NULL) {
 				LOG(("Menu Item %s: found RSC accelerator, ascii: %c, scancode: %x, mod: %x",
-					m, 
-					menu_evnt_tbl[i].accel.ascii, 
-					menu_evnt_tbl[i].accel.keycode, 
-					menu_evnt_tbl[i].accel.mod 
+					m,
+					menu_evnt_tbl[i].accel.ascii,
+					menu_evnt_tbl[i].accel.keycode,
+					menu_evnt_tbl[i].accel.mod
 				));
 				/* Accelerator is defined in struct: */
 				memcpy( (char*)&spare[2], m, strlen(m) );
@@ -607,10 +770,13 @@ void bind_global_events( void )
 	i=0;
 	while( menu_evnt_tbl[i].rid != -1 ) {
 		if( menu_evnt_tbl[i].menustr != NULL ) {
+			// TODO: modify resource tree, adjust width of menu to chars
+			// actually used.
 			MenuText( NULL, menu_evnt_tbl[i].rid, menu_evnt_tbl[i].menustr );
 		}
 		i++;
 	}
+	main_menu_update();
 	/* TODO: Fix pixel sizes for Titles and Items (for non-8px fonts) */
 }
 
@@ -629,15 +795,3 @@ void unbind_global_events( void )
 	}
 }
 
-/* send redraw to all browser windows */
-void snd_redraw( short x, short y, short w, short h)
-{
-	struct gui_window * gw;
-	gw = window_list;
-	while( gw != NULL && gw->browser->type == 0 ) {
-		ApplWrite( _AESapid, WM_REDRAW, gw->root->handle->handle, x, y, w, h);
-		gw = gw->next;
-	}
-	return;
-}
-
