@@ -68,7 +68,7 @@
 #include "atari/hotlist.h"
 #include "atari/login.h"
 #include "atari/global_evnt.h"
-#include "atari/font.h"
+#include "atari/encoding.h"
 #include "atari/res/netsurf.rsh"
 #include "atari/plot.h"
 #include "atari/clipboard.h"
@@ -78,40 +78,28 @@
 
 #define TODO() (0)/*printf("%s Unimplemented!\n", __FUNCTION__)*/
 
-char *default_stylesheet_url;
-char *adblock_stylesheet_url;
-char *quirks_stylesheet_url;
 char *tmp_clipboard;
 struct gui_window *input_window = NULL;
 struct gui_window *window_list = NULL;
 void * h_gem_rsrc;
-OBJECT * 	h_gem_menu;
+OBJECT * h_gem_menu;
 OBJECT **rsc_trindex;
 short vdih;
 short rsc_ntree;
-int mouse_click_time[3] = { INT_MAX, INT_MAX, INT_MAX };
-int mouse_hold_start[3];
-short last_drag_x;
-short last_drag_y;
 long next_poll;
-browser_mouse_state bmstate;
+bool rendering = false;
+
 
 /* Comandline / Options: */
 int cfg_width;
 int cfg_height;
 
+/* Defaults to option_homepage_url, commandline options overwrites that value */
 const char * cfg_homepage_url;
 
-extern GEM_PLOTTER plotter;
+/* path to choices file: */
+char options[PATH_MAX];
 
-void gui_multitask(void)
-{
-	short winloc[4];
-	short mx, my, dummy, aestop;
-	int flags = MU_MESAG | MU_KEYBD | MU_BUTTON | MU_TIMER;
-	evnt.timer = 0;
-	EvntWindom( flags );
-}
 
 void gui_poll(bool active)
 {
@@ -121,59 +109,29 @@ void gui_poll(bool active)
 	short mx, my, dummy;
 	short aestop;
 
-	evnt.timer = schedule_run(); 
+	evnt.timer = schedule_run();
 
-	wind_get( 0, WF_TOP, &aestop, &winloc[1], &winloc[2], &winloc[3]);
-	if( winloc[1] != _AESapid ){
-		aestop = 0;
-	}
-
-	if( (aestop > 0) && !active ) {
-		flags |= MU_M1;
-		wind_get( aestop, WF_WORKXYWH, &winloc[0],
-			&winloc[1], &winloc[2], &winloc[3] );
-		graf_mkstate( &mx, &my, &dummy, &dummy );
-		/* this can be improved a lot under XaAES - there is an event for mouse move */
-		if( mx >= winloc[0] && mx <= winloc[0] + winloc[2] &&
-			my >= winloc[1] && my <= winloc[1] + winloc[3] ){
-			/* Mouse is within the top window area */
-			evnt.m1_flag = MO_LEAVE;
-			evnt.m1_w = evnt.m1_h = 1;
-			evnt.m1_x = mx;
-			evnt.m1_y = my;
-		} else {
-			/* Mouse is outside of top window area. */
-			if( evnt.m1_flag == MO_LEAVE ) {
-				/* Previous move was inside the top window area */
-				struct gui_window * gw = input_window;
-				if(gw != NULL && gw->browser != NULL && gw->browser->bw != NULL ){
-					/* reset mouse state */
-					/* you could also track further, without reset, but */
-					/* when the mouse moves into native scroller area, the mouse is */
-					/* the native scroll bar code gets in between.. */
-                			mouse_hold_start[0] = 0;
-                			mouse_hold_start[1] = 0;
-					bmstate = 0;
-					browser_window_mouse_track( gw->browser->bw, bmstate, mx, my );
-				}
-			}
-			evnt.m1_flag = MO_ENTER;
-			evnt.m1_w = winloc[2];
-			evnt.m1_h = winloc[3];
-			evnt.m1_x = winloc[0];
-			evnt.m1_y = winloc[1];
-		}
-	}
-
-	/*printf("time: %d, active: %d\n", evnt.timer, active );*/
-	if( active ) {
+	if( active || rendering ) {
 		if( clock() >= next_poll ) {
 			evnt.timer = 0;
 			flags |= MU_TIMER;
 			EvntWindom( flags );
-			next_poll = clock() + CLOCKS_PER_SEC;
+			next_poll = clock() + (CLOCKS_PER_SEC>>2);
 		}
 	} else {
+		if( input_window != NULL ){
+			wind_get( 0, WF_TOP, &aestop, &winloc[1], &winloc[2], &winloc[3]);
+			if( winloc[1] == _AESapid ){
+				/* only check for mouse move when netsurf is on top: */
+				// move that into m1 event handler
+				graf_mkstate( &mx, &my, &dummy, &dummy );
+				flags |= MU_M1;
+				evnt.m1_flag = MO_LEAVE;
+				evnt.m1_w = evnt.m1_h = 1;
+				evnt.m1_x = mx;
+				evnt.m1_y = my;
+			}
+		}
 		flags |= MU_TIMER;
 		EvntWindom( flags );
 	}
@@ -183,10 +141,16 @@ void gui_poll(bool active)
 		if( browser_redraw_required( g ) ){
 			browser_redraw( g );
 		}
+		if( g->root->toolbar ){
+			if(g->root->toolbar->url.redraw ){
+				tb_url_redraw( g );
+			}
+		}
 	}
 	if( evnt.timer != 0 && !active ){
-		/* this suits for stuff with lower priority */ 
-		hotlist_redraw();
+		/* this suits for stuff with lower priority */
+		/* TBD: really be spare on redraws??? */
+		atari_treeview_redraw( hl.tv );
 	}
 }
 
@@ -196,9 +160,7 @@ gui_create_browser_window(struct browser_window *bw,
 			  bool new_tab)
 {
 	struct gui_window *gw=NULL;
-	struct gui_window * gwroot ;
-	short winloc[4];
-	LOG(( "gw: %p, BW: %p, clone %p, tab: %d\n" , gw,  bw, clone, 
+	LOG(( "gw: %p, BW: %p, clone %p, tab: %d\n" , gw,  bw, clone,
 		(int)new_tab
 	));
 
@@ -208,9 +170,13 @@ gui_create_browser_window(struct browser_window *bw,
 	memset( gw, 0, sizeof(struct gui_window) );
 
 	LOG(("new window: %p, bw: %p\n", gw, bw));
-	window_create(gw, bw, WIDGET_STATUSBAR|WIDGET_TOOLBAR );
+	window_create(gw, bw, WIDGET_STATUSBAR|WIDGET_TOOLBAR|WIDGET_RESIZE|WIDGET_SCROLL );
 	if( gw->root->handle ) {
-		window_open( gw );
+		GRECT pos = {
+			app.w/2-(cfg_width/2), (app.h/2)-(cfg_height/2)+16,
+			cfg_width, cfg_height
+		};
+		window_open( gw , pos );
 		/* Recalculate windows browser area now */
 		browser_update_rects( gw );
 		tb_update_buttons( gw );
@@ -257,11 +223,17 @@ void gui_window_destroy(struct gui_window *w)
 	if( w->next != NULL ) {
 		w->next->prev = w->prev;
 	}
-	if( input_window == w ) {
-		input_window = window_list;
-	}
 	free(w);
 	w = NULL;
+
+	w = window_list;
+	while( w != NULL ) {
+		if( w->root ) {
+			input_window = w;
+			break;
+		}
+		w = w->next;
+	}
 }
 
 void gui_window_get_dimensions(struct gui_window *w, int *width, int *height,
@@ -300,6 +272,21 @@ void gui_window_set_title(struct gui_window *gw, const char *title)
  */
 void gui_window_set_status(struct gui_window *w, const char *text)
 {
+	static char * msg_loading = NULL;
+	static char * msg_fetch = NULL;
+
+	if( msg_loading == NULL ){
+		msg_loading = messages_get("Loading");
+		msg_fetch = messages_get("Fetch");
+	}
+
+	if( (strncmp(msg_loading, text, 4) == 0)
+		||
+		(strncmp(msg_fetch, text, 4)) == 0 ) {
+			rendering = true;
+	} else {
+		rendering = false;
+	}
 
 	if (w == NULL || text == NULL )
 		return;
@@ -319,7 +306,6 @@ void gui_window_redraw_window(struct gui_window *gw)
 
 void gui_window_update_box(struct gui_window *gw, const struct rect *rect)
 {
-	LGRECT work;
 	CMP_BROWSER b;
 	if (gw == NULL)
 		return;
@@ -363,7 +349,7 @@ void gui_window_set_scroll(struct gui_window *w, int sx, int sy)
 			browser_scroll(w, WA_DNLINE, abs(sy), true );
 		}
 	}
-	return; 
+	return;
 
 }
 
@@ -385,7 +371,7 @@ void gui_window_update_extent(struct gui_window *gw)
 	oldx = gw->browser->scroll.current.x;
 	oldy = gw->browser->scroll.current.y;
 	if( gw->browser->bw->current_content != NULL ) {
-		browser_set_content_size( gw, 
+		browser_set_content_size( gw,
 			content_get_width(gw->browser->bw->current_content),
 			content_get_height(gw->browser->bw->current_content)
 		);
@@ -395,7 +381,7 @@ void gui_window_update_extent(struct gui_window *gw)
 
 void gui_clear_selection(struct gui_window *g)
 {
-	
+
 }
 
 
@@ -504,7 +490,7 @@ static void throbber_advance( void * data )
 		gw->root->toolbar->throbber.index = THROBBER_MIN_INDEX;
 	ApplWrite( _AESapid, WM_REDRAW,  gw->root->handle->handle,
 		work.g_x, work.g_y, work.g_w, work.g_h );
-	schedule(50, throbber_advance, gw );
+	schedule(100, throbber_advance, gw );
 }
 
 void gui_window_start_throbber(struct gui_window *w)
@@ -512,11 +498,13 @@ void gui_window_start_throbber(struct gui_window *w)
 	LGRECT work;
 	if (w == NULL)
 		return;
+	if( w->root->toolbar->throbber.running == true )
+		return;
 	mt_CompGetLGrect(&app, w->root->toolbar->throbber.comp,
 						WF_WORKXYWH, &work);
 	w->root->toolbar->throbber.running = true;
 	w->root->toolbar->throbber.index = THROBBER_MIN_INDEX;
-	schedule(50, throbber_advance, w );
+	schedule(100, throbber_advance, w );
 	ApplWrite( _AESapid, WM_REDRAW,  w->root->handle->handle,
 		work.g_x, work.g_y, work.g_w, work.g_h );
 }
@@ -526,6 +514,11 @@ void gui_window_stop_throbber(struct gui_window *w)
 	LGRECT work;
 	if (w == NULL)
 		return;
+	if( w->root->toolbar->throbber.running == false )
+		return;
+
+	schedule_remove(throbber_advance, w);
+
 	mt_CompGetLGrect(&app, w->root->toolbar->throbber.comp,
 						WF_WORKXYWH, &work);
 	w->root->toolbar->throbber.running = false;
@@ -536,25 +529,15 @@ void gui_window_stop_throbber(struct gui_window *w)
 /* Place caret in window */
 void gui_window_place_caret(struct gui_window *w, int x, int y, int height)
 {
-	LGRECT work;
 	if (w == NULL)
 		return;
-	CMP_BROWSER b = w->browser;
-	if( w->browser->caret.current.g_w > 0 ) 
+	if( w->browser->caret.current.g_w > 0 )
 		gui_window_remove_caret( w );
-	
 	w->browser->caret.requested.g_x = x;
 	w->browser->caret.requested.g_y = y;
-	w->browser->caret.requested.g_w = 2;
+	w->browser->caret.requested.g_w = 1;
 	w->browser->caret.requested.g_h = height;
 	w->browser->caret.redraw = true;
-	browser_schedule_redraw_rect( 
-		w, 
-		x - b->scroll.current.x, 
-		y - b->scroll.current.y, 
-		w->browser->caret.requested.g_w, 
-		w->browser->caret.requested.g_h 
-	);	
 	return;
 }
 
@@ -567,21 +550,19 @@ gui_window_remove_caret(struct gui_window *w)
 {
 	if (w == NULL)
 		return;
-	CMP_BROWSER b = w->browser;
-	w->browser->caret.requested.g_w = 0;
-	w->browser->caret.redraw = true;
-	browser_schedule_redraw_rect( w,
-		w->browser->caret.current.g_x - b->scroll.current.x,
-		w->browser->caret.current.g_y - b->scroll.current.y,
-		w->browser->caret.current.g_w,
-		w->browser->caret.current.g_h
-	);
+
+	if( w->browser->caret.background.fd_addr != NULL ){
+		browser_restore_caret_background( w, NULL );
+		w->browser->caret.requested.g_w = 0;
+		w->browser->caret.current.g_w = 0;
+	}
+	return;
 }
 
 void
 gui_window_set_icon(struct gui_window *g, hlcache_handle *icon)
 {
-	TODO();
+	g->icon = (icon != NULL) ? content_get_bitmap(icon) : NULL;
 }
 
 void
@@ -590,31 +571,6 @@ gui_window_set_search_ico(hlcache_handle *ico)
 	TODO();
 }
 
-bool
-save_complete_gui_save(const char *path,
-		       const char *filename,
-		       size_t len,
-		       const char *sourcedata,
-		       lwc_string *mime_type)
-{
-	TODO();
-	LOG(("%s", filename));
-	return false;
-}
-
-int
-save_complete_htmlSaveFileFormat(const char *path,
-				 const char *filename,
-				 xmlDocPtr cur,
-				 const char *encoding,
-				 int format)
-{
-	LOG(("%s", filename));
-	TODO();
-	return 0;
-}
-
-
 void gui_window_new_content(struct gui_window *w)
 {
 	w->browser->scroll.current.x = 0;
@@ -631,8 +587,8 @@ bool gui_window_scroll_start(struct gui_window *w)
 	return true;
 }
 
-bool gui_window_box_scroll_start(struct gui_window *w,
-				 int x0, int y0, int x1, int y1)
+bool gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+		const struct rect *rect)
 {
 	TODO();
 	return true;
@@ -674,7 +630,7 @@ void gui_paste_from_clipboard(struct gui_window *w, int x, int y)
 		utf8_convert_ret ret;
 		/* Clipboard is in local encoding so
 		 * convert to UTF8 */
-		ret = local_encoding_to_utf8(clip,
+		ret = utf8_from_local_encoding(clip,
 				clip_length, &utf8);
 		if (ret == UTF8_CONVERT_OK) {
 			browser_window_paste_text(w->browser->bw, utf8,
@@ -702,7 +658,7 @@ bool gui_add_to_clipboard(const char *text_utf8, size_t length_utf8, bool space)
 	size_t newlen = 0;
 	char * text = NULL;
 	char * text2 = NULL;
-	bool retval; 
+	bool retval;
 	int length = 0;
 	if( length_utf8 > 0 && text_utf8 != NULL ) {
 		utf8_to_local_encoding(text_utf8,length_utf8,&text);
@@ -719,7 +675,7 @@ bool gui_add_to_clipboard(const char *text_utf8, size_t length_utf8, bool space)
 		text = malloc(length + 2);
 		if( text == NULL ) {
 			goto error;
-		} 
+		}
 		text2 = text;
 		text[length+1] = 0;
 		memset(text, ' ', length+1);
@@ -727,7 +683,7 @@ bool gui_add_to_clipboard(const char *text_utf8, size_t length_utf8, bool space)
 	length = strlen(text);
 	if( tmp_clipboard != NULL ) {
 		oldlen = strlen( tmp_clipboard );
-	} 
+	}
 	newlen = oldlen + length + 1;
 	if( tmp_clipboard == NULL){
 		tmp_clipboard = malloc(newlen);
@@ -753,7 +709,7 @@ success:
 	retval = true;
 
 fin:
-	if( text2 != NULL ) 
+	if( text2 != NULL )
 		free( text2 );
 	return(retval);
 
@@ -762,7 +718,7 @@ fin:
 bool gui_commit_clipboard(void)
 {
 	int r = scrap_txt_write(&app, tmp_clipboard);
-	return( (r>0)?true:false );	
+	return( (r>0)?true:false );
 }
 
 
@@ -796,8 +752,8 @@ bool gui_copy_to_clipboard(struct selection *s)
 		if(selection_traverse(s, gui_selection_traverse_handler, NULL)){
 			ret = gui_commit_clipboard();
 		}
-	} 
-	gui_empty_clipboard();		
+	}
+	gui_empty_clipboard();
 	return ret;
 }
 
@@ -839,12 +795,13 @@ void gui_cert_verify(const char *url, const struct ssl_cert_info *certs,
 		nserror (*cb)(bool proceed, void *pw), void *cbpw)
 {
 	LOG((""));
-	
+
 	bool bres;
 	/*bres = verify_ssl_form_do(url, certs, num);
 	if( bres )
 		urldb_set_cert_permissions(url, true);
 	*/
+	// TODO: localize string
 	int b = form_alert(1, "[2][SSL Verify failed, continue?][Continue|Abort]");
 	bres = (b==1)? true : false;
 	LOG(("Trust: %d", bres ));
@@ -852,12 +809,6 @@ void gui_cert_verify(const char *url, const struct ssl_cert_info *certs,
 	cb(bres, cbpw);
 }
 
-
-static void *myrealloc(void *ptr, size_t len, void *pw)
-{
-	return realloc(ptr, len);
-}
-
 void gui_quit(void)
 {
 	LOG((""));
@@ -873,21 +824,17 @@ void gui_quit(void)
 
 	hotlist_destroy();
 
-	/* send WM_DESTROY to windows purely managed by windom: */
-
 	urldb_save_cookies(option_cookie_file);
-	urldb_save(option_url_file);	
+	urldb_save(option_url_file);
 
 	RsrcXtype( 0, rsc_trindex, rsc_ntree);
 	unbind_global_events();
 	MenuBar( h_gem_menu , 0 );
-	if( h_gem_rsrc == NULL ) {
+	if( h_gem_rsrc != NULL ) {
 		RsrcXfree(h_gem_rsrc );
 	}
 	LOG(("Shutting down plotter"));
 	atari_plotter_finalise();
-	LOG(("FrameExit"));
-	mt_FrameExit( &app );
 	if( tmp_clipboard != NULL ){
 		free( tmp_clipboard );
 		tmp_clipboard = NULL;
@@ -909,8 +856,14 @@ process_cmdline(int argc, char** argv)
 		cfg_width = option_window_width;
 		cfg_height = option_window_height;
 	} else {
-		cfg_width = 600;
-		cfg_height = 360;
+		if( sys_type() == SYS_TOS ){
+			/* on single tasking OS, start as fulled window: */
+			cfg_width = app.w;
+			cfg_height = app.h;
+		} else {
+			cfg_width = 600;
+			cfg_height = 360;
+		}
 	}
 
 	if (option_homepage_url != NULL && option_homepage_url[0] != '\0')
@@ -952,21 +905,25 @@ static inline void create_cursor(int flags, short mode, void * form, MFORM_EX *
 	}
 }
 
-char* gui_get_resource_url(const char *filename)
+nsurl *gui_get_resource_url(const char *path)
 {
 	char buf[PATH_MAX];
-	int len;
-	char * ret;
-	atari_find_resource((char*)&buf, filename, filename); 
-	/* TODO: handle failure? */
-	len = strlen( (char*)&buf ) + 1;
-	return( path_to_url((char*)&buf) );
+	char *raw;
+	nsurl *url = NULL;
+
+	atari_find_resource((char*)&buf, path, path);
+	raw = path_to_url((char*)&buf);
+	if (raw != NULL) {
+		nsurl_create(raw, &url);
+		free(raw);
+	}
+
+	return url;
 }
 
 static void gui_init(int argc, char** argv)
 {
-	char buf[PATH_MAX], sbuf[PATH_MAX];
-	int len;
+	char buf[PATH_MAX];
 	OBJECT * cursors;
 
 	atari_find_resource(buf, "netsurf.rsc", "./res/netsurf.rsc");
@@ -979,7 +936,7 @@ static void gui_init(int argc, char** argv)
 	rsc_ntree   = RsrcGhdr(h_gem_rsrc)->ntree;
 
 	RsrcGaddr( h_gem_rsrc, R_TREE, MAINMENU , &h_gem_menu );
-	RsrcXtype( RSRC_XALL, rsc_trindex, rsc_ntree);
+	RsrcXtype( RSRC_XTYPE, rsc_trindex, rsc_ntree);
 
 	create_cursor(0, POINT_HAND, NULL, &gem_cursors.hand );
 	create_cursor(0, TEXT_CRSR,  NULL, &gem_cursors.ibeam );
@@ -1009,19 +966,12 @@ static void gui_init(int argc, char** argv)
 	LOG(("Enabling core select menu"));
 	option_core_select_menu = true;
 
-	atari_find_resource(buf, "default.css", "./res/default.css");
-	default_stylesheet_url = path_to_url(buf);
-	LOG(("Using '%s' as Default CSS URL", default_stylesheet_url));
-
-	atari_find_resource(buf, "quirks.css", "./res/quirks.css");
-	quirks_stylesheet_url = path_to_url(buf);
-
-	if( strlen(option_url_file) ){	
+	if( strlen(option_url_file) ){
 		urldb_load(option_url_file);
 	}
 	if( strlen(option_cookie_file) ){
 		urldb_load_cookies(option_cookie_file);
-		LOG(("Loading cookies from: %s", option_cookie_file ));		
+		LOG(("Loading cookies from: %s", option_cookie_file ));
 	}
 
 	if (process_cmdline(argc,argv) != true)
@@ -1029,19 +979,11 @@ static void gui_init(int argc, char** argv)
 
 	nkc_init();
 	atari_plotter_init( option_atari_screen_driver, option_atari_font_driver );
-	LOG(("Knockout rendering: %s\n",  option_atari_knockout ? "yes" : "no"));
-	plot_set_knockout( option_atari_knockout );
-	/* Interface colours */
-	option_gui_colour_bg_1 = 0xFFFFFF; /** Background          (bbggrr) */
-	option_gui_colour_fg_1 = 0xFF0000; /** Foreground          (bbggrr) */
-	option_gui_colour_fg_2 = 0x000000; /** Foreground selected (bbggrr) */
 }
 
 static char *theapp = (char*)"NetSurf";
 static void gui_init2(int argc, char** argv)
 {
-	struct browser_window *bw;
-	const char *addr = NETSURF_HOMEPAGE;
 	MenuBar( h_gem_menu , 1 );
 	bind_global_events();
 	menu_register( -1, theapp);
@@ -1061,7 +1003,6 @@ static void gui_init2(int argc, char** argv)
  */
 int main(int argc, char** argv)
 {
-	char options[PATH_MAX];
 	char messages[PATH_MAX];
 
 	setbuf(stderr, NULL);
@@ -1082,10 +1023,9 @@ int main(int argc, char** argv)
 	graf_mouse( ARROW , NULL);
 	netsurf_main_loop();
 	netsurf_exit();
-
 	LOG(("ApplExit"));
 	ApplExit();
-#ifdef WITH_DBG_LOGFILE	
+#ifdef WITH_DBG_LOGFILE
 	fclose(stdout);
 	fclose(stderr);
 #endif
