@@ -21,21 +21,11 @@
 
 #include <windom.h>
 
-#define WM_FORCE_MOVE 1024 + 3
-
 struct point_s {
 	int x;
 	int y;
 };
 
-struct bbox_s {
-	int x0;
-	int y0;
-	int x1;
-	int y1;
-};
-
-typedef struct bbox_s BBOX;
 typedef struct point_s POINT;
 
 #define MFORM_EX_FLAG_USERFORM 0x01
@@ -82,10 +72,13 @@ struct s_focus_info
 };
 
 /* defines for data attached to components: */
-#define CDT_OBJECT	0x004f424aUL
-#define CDT_OWNER 	0x03UL
-#define CDT_ICON 	0x04UL
-#define CDT_ICON_TYPE	0x05UL
+#define CDT_OBJECT	            0x004f424aUL
+#define CDT_OWNER 	            0x03UL
+#define CDT_ICON 	            0x04UL
+#define CDT_ICON_TYPE	        0x05UL
+#define CDT_ICON_TYPE_NONE      0x00UL
+#define CDT_ICON_TYPE_OBJECT    0x01UL
+#define CDT_ICON_TYPE_BITMAP    0x02UL
 
 
 struct gui_window;
@@ -113,7 +106,8 @@ struct s_gui_win_root
 	float scale;
 	bool throbbing;
 	char * title;
-	GRECT loc;	/* current size of window on screen */
+	/* current size of window on screen: */
+	GRECT loc;
 };
 
 /*
@@ -125,11 +119,11 @@ struct s_gui_win_root
 struct gui_window {
 	struct s_gui_win_root * root;
 	CMP_BROWSER browser;
+    /* icon to be drawn when iconified, or NULL for default resource. */
+	struct bitmap * icon;
 	struct gui_window *next, *prev;
 };
 
 extern struct gui_window *window_list;
 
-#define MOUSE_IS_DRAGGING() (mouse_hold_start[0] || mouse_hold_start[1])
-
 #endif
