@@ -21,11 +21,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <mint/osbind.h>
-#include <mint/cookie.h>
 #include <windom.h>
 
 #include "desktop/cookies.h"
 #include "desktop/mouse.h"
+#include "desktop/cookies.h"
 #include "utils/messages.h"
 #include "utils/utils.h"
 #include "utils/url.h"
@@ -35,6 +35,8 @@
 #include "atari/toolbar.h"
 #include "atari/browser.h"
 #include "atari/misc.h"
+#include "atari/encoding.h"
+#include "cflib.h"
 
 extern void * h_gem_rsrc;
 
@@ -50,15 +52,9 @@ void warn_user(const char *warning, const char *detail)
 void die(const char *error)
 {
 	printf("%s\n", error);
-	sleep( 3 );
 	exit(1);
 }
 
-bool cookies_update(const char *domain, const struct cookie_data *data)
-{
-    return true;
-}
-
 /**
  * Return the filename part of a full path
  *
@@ -100,24 +96,43 @@ bool path_add_part(char *path, int length, const char *newpart)
 	return true;
 }
 
-struct gui_window * find_gui_window( WINDOW * win ){
+/*
+  TBD: make use of this function or remove it...
+*/
+struct gui_window * find_gui_window( unsigned long handle, short mode ){
 
-       struct gui_window * gw;
-        gw = window_list;
+	struct gui_window * gw;
+	gw = window_list;
 
-	if( win == NULL )
+	if( handle == 0 ){
 		return( NULL );
-
+	}
+	else if( mode == BY_WINDOM_HANDLE ){
+		WINDOW * win = (WINDOW*) handle;
         while( gw != NULL) {
                 if( gw->root->handle == win ) {
-                        return( gw );
+					return( gw );
                 }
                 else
-                        gw = gw->next;
+					gw = gw->next;
         }
+	}
+	else if( mode == BY_GEM_HANDLE ){
+		short ghandle = (short)handle;
+        while( gw != NULL) {
+                if( gw->root->handle != NULL
+					&& gw->root->handle->handle == ghandle ) {
+					return( gw );
+                }
+                else
+					gw = gw->next;
+        }
+	}
+
         return( NULL );
 }
 
+
 struct gui_window * find_cmp_window( COMPONENT * c )
 {
 	struct gui_window * gw;
@@ -154,7 +169,7 @@ OBJECT *get_tree( int idx) {
 void gem_set_cursor( MFORM_EX * cursor )
 {
 	static unsigned char flags = 255;
-	static int number = 255; 
+	static int number = 255;
 	if( flags == cursor->flags && number == cursor->number )
 		return;
 	if( cursor->flags & MFORM_EX_FLAG_USERFORM ) {
@@ -166,23 +181,189 @@ void gem_set_cursor( MFORM_EX * cursor )
 	flags = cursor->flags;
 }
 
+long nkc_to_input_key(short nkc, long * ucs4_out)
+{
+	unsigned char ascii = (nkc & 0xFF);
+	nkc = (nkc & (NKF_CTRL|NKF_SHIFT|0xFF));
+	long ik = 0;
+	*ucs4_out = 0;
+
+	/* shift + cntrl key: */
+	if( ((nkc & NKF_CTRL) == NKF_CTRL) && ((nkc & (NKF_SHIFT))!=0) ) {
+
+	}
+	/* cntrl key only: */
+	else if( (nkc & NKF_CTRL) == NKF_CTRL ) {
+		switch ( ascii ) {
+			case 'A':
+				ik = KEY_SELECT_ALL;
+			break;
+
+			case 'C':
+				ik = KEY_COPY_SELECTION;
+			break;
+
+			case 'X':
+				ik = KEY_CUT_SELECTION;
+			break;
+
+			case 'V':
+				ik = KEY_PASTE;
+			break;
+
+			default:
+			break;
+		}
+	}
+	/* shift key only: */
+	else if( (nkc & NKF_SHIFT) != 0 ) {
+		switch( ascii ) {
+			case NK_TAB:
+				ik = KEY_SHIFT_TAB;
+			break;
+
+			case NK_LEFT:
+				ik = KEY_LINE_START;
+			break;
+
+			case NK_RIGHT:
+				ik = KEY_LINE_END;
+			break;
+
+			case NK_UP:
+				ik = KEY_PAGE_UP;
+			break;
+
+			case NK_DOWN:
+				ik = KEY_PAGE_DOWN;
+			break;
+
+			default:
+			break;
+		}
+	}
+	/* No modifier keys: */
+	else {
+		switch( ascii ) {
+
+			case NK_INS:
+				ik = KEY_PASTE;
+				break;
+
+			case NK_BS:
+				ik = KEY_DELETE_LEFT;
+			break;
+
+			case NK_DEL:
+				ik = KEY_DELETE_RIGHT;
+			break;
+
+			case NK_TAB:
+				ik = KEY_TAB;
+			break;
+
+
+			case NK_ENTER:
+				ik = KEY_NL;
+			break;
+
+			case NK_RET:
+				ik = KEY_CR;
+			break;
+
+			case NK_ESC:
+				ik = KEY_ESCAPE;
+			break;
+
+			case NK_CLRHOME:
+				ik = KEY_TEXT_START;
+			break;
+
+			case NK_RIGHT:
+				ik = KEY_RIGHT;
+			break;
+
+			case NK_LEFT:
+				ik = KEY_LEFT;
+			break;
+
+			case NK_UP:
+				ik = KEY_UP;
+			break;
+
+			case NK_DOWN:
+				ik = KEY_DOWN;
+			break;
+
+			case NK_M_PGUP:
+				ik = KEY_PAGE_UP;
+			break;
+
+			case NK_M_PGDOWN:
+				ik = KEY_PAGE_DOWN;
+			break;
+
+			default:
+			break;
+		}
+	}
+
+	if( ik == 0 && ( (nkc & NKF_CTRL)==0)  ) {
+		if (ascii >= 9 ) {
+			*ucs4_out = atari_to_ucs4(ascii);
+		}
+	}
+	return ( ik );
+}
+
+/**
+ * Show default file selector
+ *
+ * \param title  The selector title.
+ * \param name	 Default file name
+ * \return a static char pointer or null if the user aborted the selection.
+ */
+const char * file_select( const char * title, const char * name ) {
+	static char path[PATH_MAX]=""; // First usage : current directory
+	static char fullname[PATH_MAX]="";
+	char tmpname[255];
+	char * use_title = (char*)title;
+
+	if( strlen(name)>254)
+		return( NULL );
+
+	strcpy( tmpname, name );
+
+	if( use_title == NULL ){
+		use_title = (char*)"";
+	}
+
+	if( FselInput( path, tmpname, (char*)"",  use_title, NULL, NULL)) {
+		strncpy( fullname, path, PATH_MAX-1 );
+		strncat( fullname, tmpname, PATH_MAX-strlen(fullname)-1 );
+		return( (const char*)&fullname  );
+	}
+	return( NULL );
+}
+
+
 void dbg_lgrect( char * str, LGRECT * r )
 {
-	printf("%s: x: %d, y: %d, w: %d, h: %d\n", str, 
+	printf("%s: x: %d, y: %d, w: %d, h: %d\n", str,
 		r->g_x, r->g_y, r->g_w, r->g_h );
 }
 
 void dbg_grect( char * str, GRECT * r )
 {
-	printf("%s: x: %d, y: %d, w: %d, h: %d\n", str, 
+	printf("%s: x: %d, y: %d, w: %d, h: %d\n", str,
 		r->g_x, r->g_y, r->g_w, r->g_h );
-} 
+}
 
 void dbg_pxy( char * str, short * pxy )
 {
-	printf("%s: x: %d, y: %d, w: %d, h: %d\n", str, 
+	printf("%s: x: %d, y: %d, w: %d, h: %d\n", str,
 		pxy[0], pxy[1], pxy[2], pxy[3] );
-} 
+}
 
 /* some LDG functions here to reduce dependencies */
 void * ldg_open( char * name, short * global )
@@ -195,7 +376,7 @@ void * ldg_find( char * name, short * ldg )
 	return( NULL );
 }
 
-int ldg_close( void * ldg, short * global ) 
+int ldg_close( void * ldg, short * global )
 {
 	return( 0 );
-} 
+}
