@@ -19,6 +19,10 @@
 #ifndef NS_ATARI_MISC_H
 #define NS_ATARI_MISC_H
 
+#include "cflib.h"
+#include "desktop/textinput.h"
+#include "atari/gui.h"
+
 #define SBUF8_TO_LBUF8(sbuf,lbuf)\
 	lbuf[0] = (long)sbuf[0];\
 	lbuf[1] = (long)sbuf[1];\
@@ -30,13 +34,21 @@
 	lbuf[7] = (long)sbuf[7];
 
 
-struct gui_window * find_gui_window( WINDOW * win );
+/* Modes for find_gui_window: */
+#define BY_WINDOM_HANDLE 0x0
+#define BY_GEM_HANDLE    0x1
+
+struct gui_window * find_gui_window( unsigned long, short mode );
 struct gui_window * find_cmp_window( COMPONENT * c );
 OBJECT *get_tree( int idx );
 char *get_rsc_string( int idx );
 void gem_set_cursor( MFORM_EX * cursor );
 void dbg_grect( char * str, GRECT * r );
+void dbg_lgrect( char * str, LGRECT * r );
+void dbg_pxy( char * str, short * pxy );
 void * ldg_open( char * name, short * global );
 void * ldg_find( char * name, short * ldg );
 int ldg_close( void * ldg, short * global );
+long nkc_to_input_key(short nkc, long * ucs4_out);
+const char * file_select( const char * title, const char * name );
 #endif
