@@ -41,7 +41,6 @@
 GEM_PLOTTER plotter = NULL;
 GEM_FONT_PLOTTER fplotter = NULL;
 
-extern APPvar * appv;
 extern short vdih;
 
 /*
@@ -56,11 +55,14 @@ int atari_plotter_init( char* drvrname, char * fdrvrname )
 	int err=0;
 	struct s_driver_table_entry * drvinfo;
 	int flags = 0;
+	unsigned long font_flags = 0;
 
-	if( option_atari_dither == 1) 
+	if( option_atari_dither == 1)
 		flags |= PLOT_FLAG_DITHER;
-	if( option_atari_transparency == 1 ) 
+	if( option_atari_transparency == 1 )
 		flags |= PLOT_FLAG_TRANS;
+	if( option_atari_font_monochrom == 1 )
+		font_flags |= FONTPLOT_FLAG_MONOGLYPH;
 
 	vdih = app.graf.handle;
 	if( verbose_log ) {
@@ -71,14 +73,18 @@ int atari_plotter_init( char* drvrname, char * fdrvrname )
 	drvinfo = get_screen_driver_entry( drvrname );
 
 	LOG(("using plotters: %s, %s", drvrname, fdrvrname));
-	fplotter = new_font_plotter(vdih, fdrvrname, 0, &err );
-	if(err)
-		die(("Unable to load font plotter %s -> %s", fdrvrname, plotter_err_str(err) ));
+	fplotter = new_font_plotter(vdih, fdrvrname, font_flags, &err );
+	if(err){
+		char * desc = plotter_err_str(err);
+		die(("Unable to load font plotter %s -> %s", fdrvrname, desc ));
+	}
 
 	plotter = new_plotter( vdih, drvrname, &loc_pos, drvinfo->max_bpp,
 							flags, fplotter, &err );
-	if(err)
-		die(("Unable to load graphics plotter %s -> %s", drvrname, plotter_err_str(err) ));
+	if(err){
+		char * desc = plotter_err_str(err);
+		die(("Unable to load graphics plotter %s -> %s", drvrname, desc ));
+	}
 
 	return( err );
 }
@@ -89,23 +95,14 @@ int atari_plotter_finalise( void )
 	delete_font_plotter( fplotter );
 }
 
-void plot_set_knockout( int set ){
-	bool * x = (bool*)&atari_plotters.option_knockout;
-	if( set == 0 ) {
-		*x = false;
-	} else {
-		*x = true;
-	}
-}
-
-bool plot_rectangle( int x0, int y0, int x1, int y1, 
+bool plot_rectangle( int x0, int y0, int x1, int y1,
 			const plot_style_t *style )
 {
 	plotter->rectangle( plotter, x0, y0, x1, y1, style );
 	return ( true );
 }
 
-static bool plot_line( int x0, int y0, int x1, int y1, 
+bool plot_line( int x0, int y0, int x1, int y1,
 			const plot_style_t *style )
 {
 	plotter->line( plotter, x0, y0, x1, y1, style );
@@ -160,33 +157,29 @@ static bool plot_bitmap(int x, int y, int width, int height,
 	bool repeat_y = (flags & BITMAPF_REPEAT_Y);
 	int bmpw,bmph;
 	struct rect clip;
-	
-	if( option_suppress_images != 0 ) {
-		return( true );
-	}
 
-	bmpw = bitmap_get_width(bitmap);	
+	bmpw = bitmap_get_width(bitmap);
 	bmph = bitmap_get_height(bitmap);
 
-	if ( repeat_x || repeat_y ) {	
+	if ( repeat_x || repeat_y ) {
 		plotter_get_clip( plotter, &clip );
-		if( repeat_x && width == 1 && repeat_y && height == 1 ){			
+		if( repeat_x && width == 1 && repeat_y && height == 1 ){
 			width = MAX( width, clip.x1 - x );
 			height = MAX( height,  clip.y1 - y );
 		}
 		else if( repeat_x && width == 1 ){
 			width = MAX( width, clip.x1 - x);
-		} 
+		}
 		else if( repeat_y && height == 1){
 			height = MAX( height, clip.y1 - y );
-		} 		
+		}
 	}
 
 	if(  width != bmpw || height != bmph ) {
 		plotter->bitmap_resize(plotter, bitmap, width, height );
 		if( bitmap->resized )
 			bm = bitmap->resized;
-		else 
+		else
 			bm = bitmap;
 	} else {
 		bm = bitmap;
@@ -204,7 +197,7 @@ static bool plot_bitmap(int x, int y, int width, int height,
 		int xf,yf;
 		int xoff = x;
 		int yoff = y;
-		
+
 		if (yoff > clip.y0 )
 			yoff = (clip.y0 - height) + ((yoff - clip.y0) % height);
 		if (xoff > clip.x0 )
@@ -252,5 +245,5 @@ const struct plotter_table atari_plotters = {
 	.flush = NULL,
 	.group_start = NULL,
 	.group_end = NULL,
-	.option_knockout = false
+	.option_knockout = true
 };
