@@ -32,14 +32,34 @@
 #include "atari/plot/eddi.h"
 #include "atari/plot/plotter.h"
 #include "atari/plot/plotter_vdi.h"
-#include "atari/plot/font_vdi.h"
-#include "atari/plot/font_internal.h"
-#include "atari/plot/font_freetype.h"
+#ifdef WITH_VDI_FONT_DRIVER
+ #include "atari/plot/font_vdi.h"
+#endif
+#ifdef WITH_INTERNAL_FONT_DRIVER
+ #include "atari/plot/font_internal.h"
+#endif
+#ifdef WITH_FREETYPE_FONT_DRIVER
+ #include "atari/plot/font_freetype.h"
+#endif
 #include "atari/gui.h"
 #include "utils/log.h"
 #include "atari/misc.h"
 #include "atari/osspec.h"
 
+
+/* get index to driver in driver list by name */
+static int drvrname_idx( char * name );
+
+/* Error code translations: */
+static const char * plot_error_codes[] =
+{
+	"None",
+	"ERR_BUFFERSIZE_EXCEEDS_SCREEN",
+	"ERR_NO_MEM",
+	"ERR_PLOTTER_NOT_AVAILABLE"
+};
+
+#ifdef WITH_8BPP_SUPPORT
 unsigned short vdi_web_pal[216][3] = {
 	{0x000,0x000,0x000}, {0x0c8,0x000,0x000}, {0x190,0x000,0x000}, {0x258,0x000,0x000}, {0x320,0x000,0x000}, {0x3e8,0x000,0x000},
 	{0x000,0x0c8,0x000}, {0x0c8,0x0c8,0x000}, {0x190,0x0c8,0x000}, {0x258,0x0c8,0x000}, {0x320,0x0c8,0x000}, {0x3e8,0x0c8,0x000},
@@ -78,6 +98,7 @@ unsigned short vdi_web_pal[216][3] = {
 	{0x000,0x320,0x3e8}, {0x0c8,0x320,0x3e8}, {0x190,0x320,0x3e8}, {0x258,0x320,0x3e8}, {0x320,0x320,0x3e8}, {0x3e8,0x320,0x3e8},
 	{0x000,0x3e8,0x3e8}, {0x0c8,0x3e8,0x3e8}, {0x190,0x3e8,0x3e8}, {0x258,0x3e8,0x3e8}, {0x320,0x3e8,0x3e8}, {0x3e8,0x3e8,0x3e8}
 };
+#endif
 
 static short prev_vdi_clip[4];
 struct s_vdi_sysinfo vdi_sysinfo;
@@ -85,16 +106,20 @@ struct s_vdi_sysinfo vdi_sysinfo;
 struct s_driver_table_entry screen_driver_table[] =
 {
 	{(char*)"vdi", ctor_plotter_vdi, 0, 32},
-	{(char*)"screen.ldg", ctor_plotter_vdi, PLOT_FLAG_OFFSCREEN, 32 },
-	{(char*)"vdi_offscreen", ctor_plotter_vdi, PLOT_FLAG_OFFSCREEN, 32},
 	{(char*)NULL, NULL, 0, 0 }
 };
 
 const struct s_font_driver_table_entry font_driver_table[] =
 {
+#ifdef WITH_VDI_FONT_DRIVER
 	{(char*)"vdi", ctor_font_plotter_vdi, 0},
+#endif
+#ifdef WITH_FREETYPE_FONT_DRIVER
 	{(char*)"freetype", ctor_font_plotter_freetype, 0},
+#endif
+#ifdef WITH_INTERNAL_FONT_DRIVER
 	{(char*)"internal", ctor_font_plotter_internal, 0},
+#endif
 	{(char*)NULL, NULL, 0}
 };
 
@@ -229,7 +254,7 @@ struct s_vdi_sysinfo * read_vdi_sysinfo( short vdih, struct s_vdi_sysinfo * info
 
 
 /*
-	lookup an plotter ID by name 
+	lookup an plotter ID by name
 */
 static int drvrname_idx( char * name )
 {
@@ -246,8 +271,8 @@ static int drvrname_idx( char * name )
 	}
 }
 
-/* 
-	lookup of font plotter ID by name 
+/*
+	lookup of font plotter ID by name
 */
 static int font_drvrname_idx( char * name )
 {
@@ -264,7 +289,7 @@ static int font_drvrname_idx( char * name )
 	}
 }
 
-/* 
+/*
 	Get an plotter info entry, the entry contains an pointer to ctor
 */
 struct s_driver_table_entry * get_screen_driver_entry( char * name )
@@ -276,8 +301,8 @@ struct s_driver_table_entry * get_screen_driver_entry( char * name )
 	return( &screen_driver_table[idx] );
 }
 
-/* 
-	Get an font plotter info entry, the entry contains an pointer to ctor. 
+/*
+	Get an font plotter info entry, the entry contains an pointer to ctor.
 */
 struct s_font_driver_table_entry * get_font_driver_entry( char * name )
 {
@@ -289,8 +314,8 @@ struct s_font_driver_table_entry * get_font_driver_entry( char * name )
 }
 
 
-/* 
-	Create an new text plotter object 
+/*
+	Create an new text plotter object
 */
 FONT_PLOTTER new_font_plotter( int vdihandle, char * name, unsigned long flags, int * error)
 {
@@ -301,8 +326,10 @@ FONT_PLOTTER new_font_plotter( int vdihandle, char * name, unsigned long flags,
 		*error = 0-ERR_NO_MEM;
 		return( NULL );
 	}
+	memset( fplotter, 0, sizeof(FONT_PLOTTER));
 	fplotter->vdi_handle = vdihandle;
 	fplotter->name = name;
+	fplotter->flags = 0;
 	fplotter->flags |= flags;
 	for( i = 0; ; i++) {
 		if( font_driver_table[i].name == NULL ) {
@@ -334,8 +361,8 @@ FONT_PLOTTER new_font_plotter( int vdihandle, char * name, unsigned long flags,
 static bool init=false;
 static int inst=0;
 
-/* 
-	Create an new plotter object 
+/*
+	Create an new plotter object
 */
 GEM_PLOTTER new_plotter(int vdihandle, char * name, GRECT * loc_size,
 						int virt_bpp, unsigned long flags, FONT_PLOTTER fplotter, int * error )
@@ -343,7 +370,7 @@ GEM_PLOTTER new_plotter(int vdihandle, char * name, GRECT * loc_size,
 	int res = 0-ERR_PLOTTER_NOT_AVAILABLE;
 	int i;
 	assert( fplotter != NULL );
-	
+
 	GEM_PLOTTER gemplotter = (GEM_PLOTTER)malloc( sizeof(struct s_gem_plotter) );
 	if( !gemplotter ) {
 		*error = 0-ERR_NO_MEM;
@@ -410,7 +437,7 @@ int delete_plotter( GEM_PLOTTER p )
 		p = NULL;
 		inst--;
 		if( inst == 0 ){
-			
+
 		}
 	}
 	else
@@ -444,7 +471,7 @@ int calc_chunked_buffer_size(int x, int y, int stride, int bpp)
    return( (x * (bpp >> 3)) * y );
 }
 
-/* 
+/*
 	x - x coord
 	y - y coord
 	stride - stride in bytes
@@ -485,7 +512,7 @@ void update_visible_rect( GEM_PLOTTER p )
 		CURFB(p).vis_h = common.g_h;
 		if( CURFB(p).x < screen.g_x )
 			CURFB(p).vis_x = frame.g_w - common.g_w;
-		else 
+		else
 			CURFB(p).vis_x = 0;
 		if( CURFB(p).y <screen.g_y )
 			CURFB(p).vis_y = frame.g_h - common.g_h;
@@ -494,7 +521,7 @@ void update_visible_rect( GEM_PLOTTER p )
 	} else {
 		CURFB(p).vis_w = CURFB(p).vis_h = 0;
 		CURFB(p).vis_x = CURFB(p).vis_y = 0;
-	}	
+	}
 }
 
 /*
@@ -603,12 +630,22 @@ void rgb_to_vdi1000( unsigned char * in, unsigned short * out )
 	return;
 }
 
+void vdi1000_to_rgb( unsigned short * in, unsigned char * out )
+{
+	double r = ((double)in[0]/1000); /* prozentsatz red */
+	double g = ((double)in[1]/1000);	/* prozentsatz green */
+	double b = ((double)in[2]/1000);	/* prozentsatz blue */
+	out[2] = 255 * r + 0.5;
+	out[1] = 255 * g + 0.5;
+	out[0] = 255 * b + 0.5;
+	return;
+}
 
 
 static short web_std_colors[6] = {0, 51, 102, 153, 204, 255};
 
-/* 
-	Convert an RGB color into an index into the 216 colors web pallette 
+/*
+	Convert an RGB color into an index into the 216 colors web pallette
 */
 short rgb_to_666_index(unsigned char r, unsigned char g, unsigned char b)
 {
@@ -616,6 +653,19 @@ short rgb_to_666_index(unsigned char r, unsigned char g, unsigned char b)
 	short i;
 	unsigned char rgb[3] = {r,g,b};
 	unsigned char tval[3];
+
+	int diff_a, diff_b, diff_c;
+	diff_a = abs(r-g);
+	diff_b = abs(r-b);
+	diff_c = abs(r-b);
+	if( diff_a < 2 && diff_b < 2 && diff_c < 2 ){
+		if( (r!=0XFF) && (g!=0XFF) && (g!=0XFF)  ){
+			if( ((r&0xF0)>>4) != 0 )
+				//printf("conv gray: %x -> %d\n", ((r&0xF0)>>4) , (OFFSET_CUST_PAL) + ((r&0xF0)>>4) );
+			return( (OFFSET_CUST_PAL - OFFSET_WEB_PAL) + ((r&0xF0)>>4) );
+		}
+	}
+
 	/* convert each 8bit color to 6bit web color: */
 	for( i=0; i<3; i++) {
 		if(0 == rgb[i] % web_std_colors[1] ) {
@@ -632,14 +682,17 @@ short rgb_to_666_index(unsigned char r, unsigned char g, unsigned char b)
 	return( tval[2]*36+tval[1]*6+tval[0] );
 }
 
+/*
+	bpp: bits per pixel,
 
+*/
 int init_mfdb(int bpp, int w, int h, uint32_t flags, MFDB * out )
 {
 	int dststride;
 	dststride = MFDB_STRIDE( w );
-	int size = MFDB_SIZE( bpp, dststride, h ); 
-	if( bpp > 0 ) { 
-		if( (flags & MFDB_FLAG_NOALLOC) == 0  ) { 
+	int size = MFDB_SIZE( bpp, dststride, h );
+	if( bpp > 0 ) {
+		if( (flags & MFDB_FLAG_NOALLOC) == 0  ) {
 			out->fd_addr = malloc( size );
 			if( out->fd_addr == NULL ){
 				return( 0 );
@@ -714,7 +767,7 @@ void plotter_vdi_clip( GEM_PLOTTER self, bool set)
 		newclip[3] = MIN(CURFB(self).y+CURFB(self).h, newclip[1] + (c->y1 - c->y0) )-1;
 		vs_clip( self->vdi_handle, 1, (short*)&newclip );
 	} else {
-		vs_clip( self->vdi_handle, 1, (short *)&prev_vdi_clip );	
+		vs_clip( self->vdi_handle, 1, (short *)&prev_vdi_clip );
 	}
 }
 
