@@ -57,24 +57,18 @@
 #define PLOT_FLAG_DITHER 	0x04		/* true if the plotter shall dither images */
 #define PLOT_FLAG_TRANS		0x08		/* true if the plotter supports transparent operations */
 
+/* Font Plotter flags: */
+#define FONTPLOT_FLAG_MONOGLYPH 0x01
+
 /* Flags for init_mfdb function: */
-#define MFDB_FLAG_STAND			0x01	
+#define MFDB_FLAG_STAND			0x01
 #define MFDB_FLAG_ZEROMEM		0x02
 #define MFDB_FLAG_NOALLOC		0x04
 
 /* Error codes: */
 #define ERR_BUFFERSIZE_EXCEEDS_SCREEN 1	/* The buffer allocated is larger than the screen */
-#define ERR_NO_MEM 2										/* Not enough memory for requested operation */
-#define ERR_PLOTTER_NOT_AVAILABLE 3			/* invalid plotter driver name passed */
-
-/* Error code translations: */
-static const char * plot_error_codes[] =
-{
-	"None",
-	"ERR_BUFFERSIZE_EXCEEDS_SCREEN",
-	"ERR_NO_MEM",
-	"ERR_PLOTTER_NOT_AVAILABLE"
-};
+#define ERR_NO_MEM 2					/* Not enough memory for requested operation */
+#define ERR_PLOTTER_NOT_AVAILABLE 3		/* invalid plotter driver name passed */
 
 /* Grapics & Font Plotter "Objects": */
 typedef struct s_font_plotter * FONT_PLOTTER;
@@ -91,10 +85,14 @@ typedef int (*_fpmf_str_split)( FONT_PLOTTER self, const plot_font_style_t *fsty
 typedef int (*_fpmf_pixel_pos)( FONT_PLOTTER self, const plot_font_style_t *fstyle,
 						const char *string, size_t length,
 						int x, size_t *char_offset, int *actual_x);
-typedef int (*_fpmf_text)( FONT_PLOTTER self, int x, int y, const char *text, 
+typedef int (*_fpmf_text)( FONT_PLOTTER self, int x, int y, const char *text,
 													size_t length, const plot_font_style_t *fstyle);
+
+typedef void (*_fpmf_draw_glyph)(FONT_PLOTTER self, GRECT * loc, uint8_t * pixdata,
+						int pitch, uint32_t colour);
 typedef int (*_fpmf_dtor)( FONT_PLOTTER self );
 
+
 /* prototype of the font plotter "object" */
 struct s_font_plotter
 {
@@ -108,6 +106,7 @@ struct s_font_plotter
 	_fpmf_str_split str_split;
 	_fpmf_pixel_pos pixel_pos;
 	_fpmf_text text;
+	_fpmf_draw_glyph draw_glyph;
 	_fpmf_dtor dtor;
 };
 
@@ -156,9 +155,9 @@ struct s_frame_buf
 /* declaration of plotter member functions ( _pmf_ prefix )*/
 typedef int (*_pmf_resize)(GEM_PLOTTER self, int w, int h);
 typedef	int (*_pmf_move)(GEM_PLOTTER self, short x, short y );
-typedef	void * (*_pmf_lock)(GEM_PLOTTER self);
 typedef	void * (*_pmf_create_framebuffer)(GEM_PLOTTER self);
 typedef	void * (*_pmf_switch_to_framebuffer)(GEM_PLOTTER self);
+typedef	int (*_pmf_lock)(GEM_PLOTTER self);
 typedef	int (*_pmf_unlock)(GEM_PLOTTER self);
 typedef	int (*_pmf_update_region)(GEM_PLOTTER self, GRECT region);
 typedef	int (*_pmf_update_screen_region)( GEM_PLOTTER self, GRECT region );
@@ -173,9 +172,11 @@ typedef	int (*_pmf_rectangle)(GEM_PLOTTER self, int x0, int y0, int x1, int y1,
 typedef	int (*_pmf_polygon)(GEM_PLOTTER self, const int *p, unsigned int n,  const plot_style_t * pstyle);
 typedef	int (*_pmf_path)(GEM_PLOTTER self, const float *p, unsigned int n, int fill, float width, int c, const float transform[6]);
 typedef	int (*_pmf_bitmap_resize) ( GEM_PLOTTER self, struct bitmap * bm, int nw, int nh );
+typedef int (*_pmf_bitmap_convert)( GEM_PLOTTER self, struct bitmap * img, int x, int y,
+				GRECT * clip, uint32_t bg, uint32_t flags, MFDB *out  );
 typedef	int (*_pmf_bitmap)(GEM_PLOTTER self, struct bitmap * bmp, int x, int y,
 				unsigned long bg, unsigned long flags );
-typedef int (*_pmf_plot_mfdb)(GEM_PLOTTER self, GRECT * loc, MFDB * mfdb, uint32_t flags);
+typedef int (*_pmf_plot_mfdb)(GEM_PLOTTER self, GRECT * loc, MFDB * mfdb, unsigned char fgcolor, uint32_t flags);
 typedef	int (*_pmf_text)(GEM_PLOTTER self, int x, int y, const char *text, size_t length, const plot_font_style_t *fstyle);
 typedef	int (*_pmf_dtor)(GEM_PLOTTER self);
 
@@ -189,11 +190,11 @@ struct s_gem_plotter
 	struct s_vdi_sysinfo * scr;
 	void * priv_data;
 	/* bit depth of framebuffers: */
-	int bpp_virt;     	
+	int bpp_virt;
 	struct rect clipping;
 	struct s_frame_buf fbuf[MAX_FRAMEBUFS];
 	/* current framebuffer index: */
-	int cfbi; 			
+	int cfbi;
 
 	FONT_PLOTTER font_plotter;
 	/* set new dimensions (realloc memory): */
@@ -218,6 +219,8 @@ struct s_gem_plotter
 	_pmf_path path;
 	/* scale an netsurf bitmap: */
 	_pmf_bitmap_resize bitmap_resize;
+	/* convert an ABGR (netsurf) bitmap to screen format, ready for vro_cpyfm */
+	_pmf_bitmap_convert bitmap_convert;
 	/* plot an netsurf bitmap into the buffer / screen: */
 	_pmf_bitmap bitmap;
 	/* plot an mfdb into the buffer / screen: */
@@ -247,9 +250,6 @@ struct s_font_driver_table_entry
 typedef struct s_driver_table_entry * PLOTTER_INFO;
 typedef struct s_font_driver_table_entry * FONT_PLOTTER_INFO;
 
-/* get index to driver in driver list by name */
-static int drvrname_idx( char * name );
-
 /* get s_driver_table_entry from driver table */
 struct s_driver_table_entry * get_screen_driver_entry(char * name);
 
@@ -303,26 +303,29 @@ void dump_font_drivers(void);
 void dump_plot_drivers(void);
 void dump_vdi_info(short);
 
-/* convert an rgb color to vdi1000 color */
+/* convert an vdi color to bgra */
+void vdi1000_to_rgb( unsigned short * in, unsigned char * out );
+
+/* convert an bgra color to vdi1000 color */
 void rgb_to_vdi1000( unsigned char * in, unsigned short * out );
 
 /* convert an rgb color to an index into the web palette */
 short rgb_to_666_index(unsigned char r, unsigned char g, unsigned char b);
 
 /*
-	setup an MFDB struct and allocate memory for it when it is needed. 
+	setup an MFDB struct and allocate memory for it when it is needed.
 	If bpp == 0, this function assumes that the MFDB shall point to the screen
-	and will not allocate any memory (mfdb.fd_addr == 0).  
-	The function will return 0 when the memory allocation fails 
+	and will not allocate any memory (mfdb.fd_addr == 0).
+	The function will return 0 when the memory allocation fails
 	( out of memory), otherwise it returns the size of the mfdb.fd_addr
-  as number of bytes.    
+  as number of bytes.
 */
 int init_mfdb(int bpp, int w, int h, uint32_t flags, MFDB * out );
 
 /* shared / static methods follows */
 
 /*
-	Get clipping for current framebuffer 
+	Get clipping for current framebuffer
 */
 int plotter_get_clip( GEM_PLOTTER self, struct rect * out );
 
@@ -332,7 +335,7 @@ int plotter_get_clip( GEM_PLOTTER self, struct rect * out );
 void plotter_get_clip_grect( GEM_PLOTTER self, GRECT * out );
 
 /*
-	Get current visible coords 
+	Get current visible coords
 */
 void plotter_get_visible_grect( GEM_PLOTTER self, GRECT * out );
 
@@ -343,7 +346,7 @@ int plotter_std_clip(GEM_PLOTTER self, const struct rect * clip);
 
 
 /*
-	convert framebuffer clipping to vdi clipping and activates it 
+	convert framebuffer clipping to vdi clipping and activates it
 */
 void plotter_vdi_clip( GEM_PLOTTER self, bool set);
 
@@ -359,20 +362,21 @@ void plotter_vdi_clip( GEM_PLOTTER self, bool set);
 /* some Well known indexes into the VDI palette */
 /* common indexes into the VDI palette */
 /* (only used when running with 256 colors or less ) */
-#define OFFSET_WEB_PAL 16		
+#define OFFSET_WEB_PAL 16
 #define OFFSET_CUST_PAL 232
 #define OFFSET_CUSTOM_COLOR 255	/* this one is used by the TC renderer */
 #define RGB_TO_VDI(c) rgb_to_666_index( (c&0xFF),(c&0xFF00)>>8,(c&0xFF0000)>>16)+OFFSET_WEB_PAL
-#define ABGR_TO_RGB(c)  ( ((c&0xFF)<<16) | (c&0xFF00) | ((c&0xFF0000)>>16) ) << 8 
+/* the name of this macro is crap - it should be named bgr_to_rgba ... or so */
+#define ABGR_TO_RGB(c)  ( ((c&0xFF)<<16) | (c&0xFF00) | ((c&0xFF0000)>>16) ) << 8
 
 /* calculate MFDB compatible rowstride (in number of bits) */
 #define MFDB_STRIDE( w ) (((w & 15) != 0) ? (w | 15)+1 : w)
 
-/* 
+/*
 Calculate size of an mfdb, params:
-	Bits per pixel, 
-	Word aligned rowstride (width) as returned by MFDB_STRIDE, 
-	height in pixels 
+	Bits per pixel,
+	Word aligned rowstride (width) as returned by MFDB_STRIDE,
+	height in pixels
 */
 #define MFDB_SIZE( bpp, stride, h ) ( ((stride >> 3) * h) * bpp )
 
