@@ -27,9 +27,8 @@
 #include "atari/plot/eddi.h"
 #include "atari/plot/plotter.h"
 #include "atari/plot/plotter_vdi.h"
-#include "atari/plot/font_vdi.h"
 
-/* assign vdi line style to dst ( netsurf type ) */ 
+/* assign vdi line style to dst ( netsurf type ) */
 #define NSLT2VDI(dst, src) \
 	dst = 0;\
 	switch( src->stroke_type ) {\
@@ -49,7 +48,7 @@
 static int dtor( GEM_PLOTTER self );
 static int resize( GEM_PLOTTER self, int w, int h );
 static int move( GEM_PLOTTER self, short x, short y );
-static void * lock( GEM_PLOTTER self );
+static int lock( GEM_PLOTTER self );
 static int unlock( GEM_PLOTTER self );
 static int update_region( GEM_PLOTTER self, GRECT region );
 static int update_screen_region( GEM_PLOTTER self, GRECT region );
@@ -63,33 +62,43 @@ static int rectangle(GEM_PLOTTER self,int x0, int y0, int x1, int y1, const plot
 static int polygon(GEM_PLOTTER self,const int *p, unsigned int n, const plot_style_t * pstyle);
 static int path(GEM_PLOTTER self,const float *p, unsigned int n, int fill, float width, int c, const float transform[6]);
 static int bitmap_resize( GEM_PLOTTER self, struct bitmap * img, int nw, int nh );
+static int bitmap_convert( GEM_PLOTTER self, struct bitmap * img, int x, int y,
+	GRECT * clip,uint32_t bg,uint32_t flags, MFDB *out  );
+static int bitmap_convert_8( GEM_PLOTTER self, struct bitmap * img,int x, int y,
+	GRECT * clip,uint32_t bg,uint32_t flags, MFDB *out  );
 static int bitmap( GEM_PLOTTER self, struct bitmap * bmp, int x, int y,
 					unsigned long bg, unsigned long flags );
-static int plot_mfdb( GEM_PLOTTER self, GRECT * where, MFDB * mfdb, uint32_t flags);
+static int plot_mfdb( GEM_PLOTTER self, GRECT * where, MFDB * mfdb, unsigned char fgcolor, uint32_t flags);
 static int text(GEM_PLOTTER self, int x, int y, const char *text,size_t length, const plot_font_style_t *fstyle);
 
+static inline void set_stdpx( MFDB * dst, int wdplanesz, int x, int y, unsigned char val );
+static inline unsigned char get_stdpx(MFDB * dst, int wdplanesz, int x, int y );
+
+
+#ifdef WITH_8BPP_SUPPORT
 static unsigned short sys_pal[256][3]; /*RGB*/
 static unsigned short pal[256][3];     /*RGB*/
-
-extern unsigned char rgb_web_pal[126][3];
+static char rgb_lookup[256][4];
 extern unsigned short vdi_web_pal[126][3];
+#endif
 extern struct s_vdi_sysinfo vdi_sysinfo;
 
 static HermesHandle hermes_pal_h; /* hermes palette handle */
 static HermesHandle hermes_cnv_h; /* hermes converter instance handle */
 static HermesHandle hermes_res_h;
-int32 * hermes_pal_p;
+
 
 static inline void vsl_rgbcolor( short vdih, uint32_t cin )
 {
 	if( vdi_sysinfo.scr_bpp > 8 ) {
 		unsigned short c[4];
-		rgb_to_vdi1000( (unsigned char*)&cin, (unsigned short*)&c );	
+		rgb_to_vdi1000( (unsigned char*)&cin, (unsigned short*)&c );
 		vs_color( vdih, OFFSET_CUSTOM_COLOR, (unsigned short*)&c[0] );
 		vsl_color( vdih, OFFSET_CUSTOM_COLOR );
 	} else {
-		if( vdi_sysinfo.scr_bpp >= 4 )
+		if( vdi_sysinfo.scr_bpp >= 4 ){
 			vsl_color( vdih, RGB_TO_VDI(cin) );
+		}
 		else
 			vsl_color( vdih, BLACK );
 	}
@@ -99,12 +108,13 @@ static inline void vsf_rgbcolor( short vdih, uint32_t cin )
 {
 	if( vdi_sysinfo.scr_bpp > 8 ) {
 		unsigned short c[4];
-		rgb_to_vdi1000( (unsigned char*)&cin, (unsigned short*)&c );	
+		rgb_to_vdi1000( (unsigned char*)&cin, (unsigned short*)&c );
 		vs_color( vdih, OFFSET_CUSTOM_COLOR, &c[0] );
 		vsf_color( vdih, OFFSET_CUSTOM_COLOR );
 	} else {
-		if( vdi_sysinfo.scr_bpp >= 4 )
+		if( vdi_sysinfo.scr_bpp >= 4 ){
 			vsf_color( vdih, RGB_TO_VDI(cin) );
+		}
 		else
 			vsf_color( vdih, WHITE );
 	}
@@ -135,6 +145,12 @@ int ctor_plotter_vdi(GEM_PLOTTER self )
 	self->path = path;
 	self->bitmap = bitmap;
 	self->bitmap_resize = bitmap_resize;
+#ifdef WITH_8BPP_SUPPORT
+	self->bitmap_convert =(app.nplanes > 8) ? bitmap_convert : bitmap_convert_8;
+#else
+	self->bitmap_convert = bitmap_convert;
+#endif
+	//self->bitmap_convert =bitmap_convert;
 	self->plot_mfdb = plot_mfdb;
 	self->text = text;
 	LOG(("Screen: x: %d, y: %d\n", vdi_sysinfo.scr_w, vdi_sysinfo.scr_h));
@@ -168,10 +184,19 @@ int ctor_plotter_vdi(GEM_PLOTTER self )
 	clip.x1 = FIRSTFB(self).w;
 	clip.y1 = FIRSTFB(self).h;
 	self->clip( self, &clip );
+
+	assert( Hermes_Init() );
 	/* store system palette & setup the new (web) palette: */
+#ifdef WITH_8BPP_SUPPORT
 	i = 0;
+
+	unsigned char * col;
+	unsigned char rgbcol[4];
+	unsigned char graytone=0;
 	if( app.nplanes <= 8 ){
 		for( i=0; i<=255; i++ ) {
+
+			// get the current color and save it for restore:
 			vq_color(self->vdi_handle, i, 1, (unsigned short*)&sys_pal[i][0] );
 			if( i<OFFSET_WEB_PAL ) {
 				pal[i][0] = sys_pal[i][0];
@@ -182,40 +207,27 @@ int ctor_plotter_vdi(GEM_PLOTTER self )
 					pal[i][0] = vdi_web_pal[i-OFFSET_WEB_PAL][0];
 					pal[i][1] = vdi_web_pal[i-OFFSET_WEB_PAL][1];
 					pal[i][2] = vdi_web_pal[i-OFFSET_WEB_PAL][2];
+					//set the new palette color to websafe value:
+					vs_color( self->vdi_handle, i, &pal[i][0] );
 				}
-				if( i >= OFFSET_CUST_PAL ) {
-					/* here we could define 22 additional colors... */ 
+				if( i >= OFFSET_CUST_PAL && i<OFFSET_CUST_PAL+16 ) {
+					/* here we define 20 additional gray colors... */
+					rgbcol[1] = rgbcol[2] = rgbcol[3] = ((graytone&0x0F) << 4);
+					rgb_to_vdi1000( &rgbcol[0], &pal[i][0] );
+					vs_color( self->vdi_handle, i, &pal[i][0] );
+					graytone++;
 				}
-				vs_color( self->vdi_handle, i, &pal[i][0] );
+
 			}
+			vdi1000_to_rgb( &pal[i][0],  &rgb_lookup[i][0] );
 		}
+
 	} else {
 		/* no need to change the palette - its application specific */
 	}
 
-	unsigned char * col;
-	assert( Hermes_Init() );
-/*
-	hermes_pal_h = Hermes_PaletteInstance();
-	hermes_pal_p = Hermes_PaletteGet(hermes_pal_h);
-	assert(hermes_pal_p);
 
-	for( i = 0; i<OFFSET_CUST_PAL; i++) {
-		col = (unsigned char *)(hermes_pal_p+i);
-		if( i < OFFSET_WEB_PAL ) {
-			col[0] = sys_pal[i][0];
-			col[1] = sys_pal[i][1];
-			col[2] = sys_pal[i][2];
-		}
-		if( i >= OFFSET_WEB_PAL ) {
-			col[0] = rgb_web_pal[i-OFFSET_WEB_PAL][0];
-			col[1] = rgb_web_pal[i-OFFSET_WEB_PAL][1];
-			col[2] = rgb_web_pal[i-OFFSET_WEB_PAL][2];
-		}
-		col[3] = 0;
-	}
-	Hermes_PaletteInvalidateCache(hermes_pal_h);
-*/
+#endif
 
 	unsigned long flags = ( self->flags & PLOT_FLAG_DITHER ) ? HERMES_CONVERT_DITHER : 0;
 	hermes_cnv_h = Hermes_ConverterInstance( flags );
@@ -238,8 +250,8 @@ int ctor_plotter_vdi(GEM_PLOTTER self )
 	DUMMY_PRIV(self)->vfmt.b = vdi_sysinfo.mask_b;
 	DUMMY_PRIV(self)->vfmt.a = vdi_sysinfo.mask_a;
 	DUMMY_PRIV(self)->vfmt.bits = self->bpp_virt;
-	DUMMY_PRIV(self)->vfmt.indexed = false;
-	DUMMY_PRIV(self)->vfmt.has_colorkey = false;
+	DUMMY_PRIV(self)->vfmt.indexed = ( app.nplanes <= 8 ) ? 1 : 0;
+	DUMMY_PRIV(self)->vfmt.has_colorkey = 0;
 
 	return( 1 );
 }
@@ -253,16 +265,15 @@ static int dtor( GEM_PLOTTER self )
 			free( self->fbuf[i].mem );
 	}
 
-	if( app.nplanes <= 8 ){
-		/* restore system palette */
-		for( i=0; i<=255; i++ ) {
-			vs_color( self->vdi_handle, i, &sys_pal[i][0] );
-		}
+#ifdef WITH_8BPP_SUPPORT
+	for( i=OFFSET_WEB_PAL; i<OFFSET_CUST_PAL+16; i++){
+		vs_color( self->vdi_handle, i, &sys_pal[i][0] );
 	}
+#endif
 
 	/* close Hermes stuff: */
 	Hermes_ConverterReturn( hermes_cnv_h );
-	/* Hermes_PaletteReturn( hermes_pal_h ); */
+
 	Hermes_Done();
 
 	if( self->priv_data != NULL ){
@@ -272,6 +283,7 @@ static int dtor( GEM_PLOTTER self )
 			free( DUMMY_PRIV(self)->buf_planar );
 		free( self->priv_data );
 	}
+	snapshot_destroy( self );
 	return( 1 );
 }
 
@@ -279,9 +291,9 @@ static int resize( GEM_PLOTTER self, int w, int h )
 {
 	if( w == CURFB(self).w && h == CURFB(self).h )
 		return( 1 );
+	/* todo: needed when using offscreen buffers...
 	int newsize = calc_chunked_buffer_size( w, h, w, self->bpp_virt );
 	LOG(("%s: %s, oldsize: %d\n", (char*)__FILE__, __FUNCTION__, CURFB(self).size ));
-	/* todo: needed when using offscreen buffers...
 	if( newsize > self->screen_buffer_size ) {
 		self->screen_buffer_size = newsize;
 		self->screen_buffer =realloc( self->screen_buffer , self->screen_buffer_size );
@@ -303,32 +315,31 @@ static int move( GEM_PLOTTER self,short x, short y )
 	CURFB(self).x = x;
 	CURFB(self).y = y;
 	update_visible_rect( self );
-
-   /*
-      for offscreen plotters:
-      copy current contents to new pos?
-      we could also copy content of our own screen buffer,
-      but only when it's unlocked
-		...nono, the user must do this manually. Because window move will already be handled by the OS
-   */
-   /*update_screen( self );*/
-   return( 1 );
+	return( 1 );
 }
 
 
-static void * lock( GEM_PLOTTER self )
+static int lock( GEM_PLOTTER self )
 {
 	LOG(("%s: %s\n", (char*)__FILE__, __FUNCTION__));
+	if( (self->flags & PLOT_FLAG_LOCKED) != 0 )
+		return(1);
 	self->flags |= PLOT_FLAG_LOCKED;
-	wind_update(BEG_UPDATE);
-	wind_update(BEG_MCTRL);
+	if( !wind_update(BEG_UPDATE|0x100) )
+		return(0);
+	if( !wind_update(BEG_MCTRL|0x100) ){
+		wind_update(END_UPDATE);
+		return(0);
+	}
 	graf_mouse(M_OFF, NULL);
-	return( NULL );
+	return( 1 );
 }
 
 static int unlock( GEM_PLOTTER self )
 {
 	LOG(("%s: %s\n", (char*)__FILE__, __FUNCTION__));
+	if( (self->flags & PLOT_FLAG_LOCKED) == 0 )
+		return(1);
 	self->flags &=  ~PLOT_FLAG_LOCKED;
 	wind_update(END_MCTRL);
 	wind_update(END_UPDATE);
@@ -424,7 +435,7 @@ static int update_screen( GEM_PLOTTER self )
 	LOG(("area: x:%d ,y:%d ,w:%d ,h:%d, from: %p (offset: %d) \n",
 		target.g_x, target.g_y,
          target.g_w, target.g_h,
-		((char*)CURFB(self).mem)+src_offset, src_offset 
+		((char*)CURFB(self).mem)+src_offset, src_offset
 	));
 
    return( 1 );
@@ -435,6 +446,8 @@ static int put_pixel(GEM_PLOTTER self, int x, int y, int color )
    return( 1 );
 }
 
+/* copy an rectangle from the plot buffer to screen */
+/* because this is an on-screen plotter, this is an screen to screen copy. */
 static int copy_rect( GEM_PLOTTER self, GRECT src, GRECT dst )
 {
 	MFDB devmf;
@@ -542,6 +555,7 @@ static int line(GEM_PLOTTER self,int x0, int y0, int x1, int y1, const plot_styl
 		sw = 1;
 	NSLT2VDI(lt, pstyle)
 	vsl_type( self->vdi_handle, (lt&0x0F) );
+	/* if the line style is not available within VDI system,define own style: */
 	if( (lt&0x0F) == 7 ){
 		vsl_udsty(self->vdi_handle, ((lt&0xFFFF00) >> 8) );
 	}
@@ -554,38 +568,107 @@ static int line(GEM_PLOTTER self,int x0, int y0, int x1, int y1, const plot_styl
 
 static int rectangle(GEM_PLOTTER self,int x0, int y0, int x1, int y1,  const plot_style_t * pstyle)
 {
-   short pxy[10];
-
+	short pxy[4];
 	GRECT r, rclip, sclip;
+	int sw = pstyle->stroke_width;
+	uint32_t lt;
 
+	/* current canvas clip: */
 	rclip.g_x = self->clipping.x0;
 	rclip.g_y = self->clipping.y0;
 	rclip.g_w = self->clipping.x1 - self->clipping.x0;
 	rclip.g_h = self->clipping.y1 - self->clipping.y0;
 
+	/* physical clipping: */
 	sclip.g_x = rclip.g_x;
 	sclip.g_y = rclip.g_y;
 	sclip.g_w = CURFB(self).vis_w;
 	sclip.g_h = CURFB(self).vis_h;
+
 	rc_intersect(&sclip, &rclip);
 	r.g_x = x0;
 	r.g_y = y0;
 	r.g_w = x1 - x0;
 	r.g_h = y1 - y0;
+
 	if( !rc_intersect( &rclip, &r ) ) {
 		return( 1 );
 	}
-	vsf_rgbcolor( self->vdi_handle, pstyle->fill_colour );
-	vsf_perimeter( self->vdi_handle, 0);
-	vsf_interior( self->vdi_handle, FIS_SOLID );
+	if( pstyle->stroke_type != PLOT_OP_TYPE_NONE ){
+		/*
+			manually draw the line, because we do not need vdi clipping
+			for vertical / horizontal line draws.
+		*/
+		if( sw == 0)
+			sw = 1;
 
-	pxy[0] = CURFB(self).x + r.g_x;
-	pxy[1] = CURFB(self).y + r.g_y;
-	pxy[2] = CURFB(self).x + r.g_x + r.g_w -1;
-	pxy[3] = CURFB(self).y + r.g_y + r.g_h -1;
+		NSLT2VDI(lt, pstyle);
+		vsl_type( self->vdi_handle, (lt&0x0F) );
+		/*
+			if the line style is not available within VDI system,
+			define own style:
+		*/
+		if( (lt&0x0F) == 7 ){
+			vsl_udsty(self->vdi_handle, ((lt&0xFFFF00) >> 8) );
+		}
+		vsl_width( self->vdi_handle, (short)sw );
+		vsl_rgbcolor( self->vdi_handle, pstyle->stroke_colour );
+		/* top border: */
+		if( r.g_y == y0){
+			pxy[0] = CURFB(self).x + r.g_x;
+			pxy[1] = CURFB(self).y + r.g_y ;
+			pxy[2] = CURFB(self).x + r.g_x + r.g_w;
+			pxy[3] = CURFB(self).y + r.g_y;
+			v_pline(self->vdi_handle, 2, (short *)&pxy );
+		}
+
+		/* right border: */
+		if( r.g_x + r.g_w == x1 ){
+			pxy[0] = CURFB(self).x + r.g_x + r.g_w;
+			pxy[1] = CURFB(self).y + r.g_y;
+			pxy[2] = CURFB(self).x + r.g_x + r.g_w;
+			pxy[3] = CURFB(self).y + r.g_y + r.g_h;
+			v_pline(self->vdi_handle, 2, (short *)&pxy );
+		}
+
+		/* bottom border: */
+		if( r.g_y+r.g_h == y1 ){
+			pxy[0] = CURFB(self).x + r.g_x;
+			pxy[1] = CURFB(self).y + r.g_y+r.g_h;
+			pxy[2] = CURFB(self).x + r.g_x+r.g_w;
+			pxy[3] = CURFB(self).y + r.g_y+r.g_h;
+			v_pline(self->vdi_handle, 2, (short *)&pxy );
+		}
+
+		/* left border: */
+		if( r.g_x == x0 ){
+			pxy[0] = CURFB(self).x + r.g_x;
+			pxy[1] = CURFB(self).y + r.g_y;
+			pxy[2] = CURFB(self).x + r.g_x;
+			pxy[3] = CURFB(self).y + r.g_y + r.g_h;
+			v_pline(self->vdi_handle, 2, (short *)&pxy );
+		}
+	}
+
+	if( pstyle->fill_type != PLOT_OP_TYPE_NONE ){
+
+		short stroke_width = (short)(pstyle->stroke_type != PLOT_OP_TYPE_NONE) ?
+								pstyle->stroke_width : 0;
+
+		vsf_rgbcolor( self->vdi_handle, pstyle->fill_colour );
+		vsf_perimeter( self->vdi_handle, 0);
+		vsf_interior( self->vdi_handle, FIS_SOLID );
+
+
+		pxy[0] = CURFB(self).x + r.g_x + stroke_width;
+		pxy[1] = CURFB(self).y + r.g_y + stroke_width;
+		pxy[2] = CURFB(self).x + r.g_x + r.g_w -1 - stroke_width ;
+		pxy[3] = CURFB(self).y + r.g_y + r.g_h -1 - stroke_width;
+
+		vsf_style( self->vdi_handle, 1);
+		v_bar( self->vdi_handle, (short*)&pxy );
+	}
 
-	vsf_style( self->vdi_handle, 1);
-	v_bar( self->vdi_handle, (short*)&pxy );
 	return ( 1 );
 }
 
@@ -596,11 +679,6 @@ static int polygon(GEM_PLOTTER self,const int *p, unsigned int n,  const plot_st
 	short d[4];
 	if( vdi_sysinfo.maxpolycoords > 0 )
 		assert( (signed int)n < vdi_sysinfo.maxpolycoords );
-/*
-	Does this double check make sense? 
-	else 
-		assert( n < vdi_sysinfo.maxintin ); 
-*/
 	plotter_vdi_clip( self, 1);
 	vsf_interior( self->vdi_handle, FIS_SOLID );
 	vsf_style( self->vdi_handle, 1);
@@ -618,15 +696,15 @@ static int polygon(GEM_PLOTTER self,const int *p, unsigned int n,  const plot_st
 		vsl_rgbcolor( self->vdi_handle, pstyle->stroke_colour);
 		v_pline(self->vdi_handle, n+1,  (short *)&pxy );
 	}
-	plotter_vdi_clip( self, 0); 
+	plotter_vdi_clip( self, 0);
 	return ( 1 );
 }
 
 static int path(GEM_PLOTTER self,const float *p, unsigned int n, int fill, float width,
 			int c, const float transform[6])
 {
-   LOG(("%s: %s\n", (char*)__FILE__, __FUNCTION__));
-   return ( 1 );
+	LOG(("%s: %s\n", (char*)__FILE__, __FUNCTION__));
+	return ( 1 );
 }
 
 
@@ -642,7 +720,7 @@ static inline uint32_t ablend(uint32_t pixel, uint32_t scrpixel)
     g  = ((pixel & 0x00FF00) * opacity +
     	(scrpixel & 0x00FF00) * transp) >> 8;
 
-    return ((rb & 0xFF00FF) | (g & 0xFF00)) << 8; 
+    return ((rb & 0xFF00FF) | (g & 0xFF00)) << 8;
 }
 
 static int bitmap_resize( GEM_PLOTTER self, struct bitmap * img, int nw, int nh )
@@ -671,9 +749,9 @@ static int bitmap_resize( GEM_PLOTTER self, struct bitmap * img, int nw, int nh
 	}
 
 	/* allocate an converter, only for resizing */
-	err = Hermes_ConverterRequest( hermes_res_h, 
-			&DUMMY_PRIV(self)->nsfmt, 
-			&DUMMY_PRIV(self)->nsfmt 
+	err = Hermes_ConverterRequest( hermes_res_h,
+			&DUMMY_PRIV(self)->nsfmt,
+			&DUMMY_PRIV(self)->nsfmt
 	);
 	if( err == 0 ) {
 		return( -ERR_PLOTTER_NOT_AVAILABLE );
@@ -700,16 +778,13 @@ static int bitmap_resize( GEM_PLOTTER self, struct bitmap * img, int nw, int nh
 	return( 0 );
 }
 
-static struct bitmap * snapshot_create(GEM_PLOTTER self, int x, int y, int w, int h)
+// create snapshot, native screen format
+static MFDB * snapshot_create_native_mfdb( GEM_PLOTTER self, int x, int y, int w, int h)
 {
 	MFDB scr;
 	short pxy[8];
-	int err;
-
-	/* make sure the screen format is pixel packed... */
-	/* no method to convert planar screen to pixel packed ... right now */
-	assert( vdi_sysinfo.vdiformat == VDI_FORMAT_PACK  );	
 
+	/* allocate memory for the snapshot */
 	{
 		int scr_stride = MFDB_STRIDE( w );
 		int scr_size = ( ((scr_stride >> 3) * h) * vdi_sysinfo.scr_bpp );
@@ -719,7 +794,7 @@ static struct bitmap * snapshot_create(GEM_PLOTTER self, int x, int y, int w, in
 			DUMMY_PRIV(self)->size_buf_scr = scr_size;
 		} else {
 			if( scr_size > DUMMY_PRIV(self)->size_buf_scr ) {
-				DUMMY_PRIV(self)->buf_scr.fd_addr = realloc( 
+				DUMMY_PRIV(self)->buf_scr.fd_addr = realloc(
 					DUMMY_PRIV(self)->buf_scr.fd_addr, scr_size
 				);
 				DUMMY_PRIV(self)->size_buf_scr = scr_size;
@@ -732,10 +807,9 @@ static struct bitmap * snapshot_create(GEM_PLOTTER self, int x, int y, int w, in
 		DUMMY_PRIV(self)->buf_scr.fd_nplanes = vdi_sysinfo.scr_bpp;
 		DUMMY_PRIV(self)->buf_scr.fd_w = scr_stride;
 		DUMMY_PRIV(self)->buf_scr.fd_h = h;
-		DUMMY_PRIV(self)->buf_scr.fd_wdwidth = scr_stride >> 4;	
+		DUMMY_PRIV(self)->buf_scr.fd_wdwidth = scr_stride >> 4;
 		assert( DUMMY_PRIV(self)->buf_scr.fd_addr != NULL );
 	}
-
 	init_mfdb( 0, w, h, 0, &scr );
 	pxy[0] = x;
 	pxy[1] = y;
@@ -743,33 +817,86 @@ static struct bitmap * snapshot_create(GEM_PLOTTER self, int x, int y, int w, in
 	pxy[3] = pxy[1] + h-1;
 	pxy[4] = 0;
 	pxy[5] = 0;
-	pxy[6] = pxy[2];
-	pxy[7] = pxy[3];
-	vro_cpyfm( self->vdi_handle, S_ONLY, (short*)&pxy, 
-				&scr,  &DUMMY_PRIV(self)->buf_scr 
+	pxy[6] = w-1;
+	pxy[7] = h-1;
+	vro_cpyfm(
+			self->vdi_handle, S_ONLY, (short*)&pxy,
+			&scr,  &DUMMY_PRIV(self)->buf_scr
 	);
 
-	/* convert screen buffer to ns format: */
+	return( &DUMMY_PRIV(self)->buf_scr );
+}
+
+// create snapshot, vdi std. format
+static MFDB * snapshot_create_std_mfdb(GEM_PLOTTER self, int x, int y, int w, int h)
+{
+	/* allocate memory for the snapshot */
+	{
+		int scr_stride = MFDB_STRIDE( w );
+		int scr_size = ( ((scr_stride >> 3) * h) * app.nplanes );
+		if( DUMMY_PRIV(self)->size_buf_std == 0 ){
+			/* init screen mfdb */
+			DUMMY_PRIV(self)->buf_std.fd_addr = malloc( scr_size );
+			DUMMY_PRIV(self)->size_buf_std = scr_size;
+		} else {
+			if( scr_size > DUMMY_PRIV(self)->size_buf_std ) {
+				DUMMY_PRIV(self)->buf_std.fd_addr = realloc(
+					DUMMY_PRIV(self)->buf_std.fd_addr, scr_size
+				);
+				DUMMY_PRIV(self)->size_buf_std = scr_size;
+			}
+		}
+		if( DUMMY_PRIV(self)->buf_std.fd_addr == NULL ) {
+			DUMMY_PRIV(self)->size_buf_std = 0;
+			return( NULL );
+		}
+		DUMMY_PRIV(self)->buf_std.fd_nplanes = app.nplanes;
+		DUMMY_PRIV(self)->buf_std.fd_w = scr_stride;
+		DUMMY_PRIV(self)->buf_std.fd_h = h;
+		DUMMY_PRIV(self)->buf_std.fd_stand = 1;
+		DUMMY_PRIV(self)->buf_std.fd_wdwidth = scr_stride >> 4;
+		assert( DUMMY_PRIV(self)->buf_std.fd_addr != NULL );
+	}
+	MFDB * native = snapshot_create_native_mfdb( self, x,y,w,h );
+	assert( native );
+
+	vr_trnfm( self->vdi_handle, native, &DUMMY_PRIV(self)->buf_std );
+	return( &DUMMY_PRIV(self)->buf_std );
+}
+
+/*
+	This will create an snapshot of the screen in netsurf ABGR format
+*/
+static struct bitmap * snapshot_create(GEM_PLOTTER self, int x, int y, int w, int h)
+{
+	int err;
+	MFDB * native;
+
+	native = snapshot_create_native_mfdb( self, x, y, w, h );
+
+	/* allocate buffer for result bitmap: */
 	if( DUMMY_PRIV(self)->buf_scr_compat == NULL ) {
 		DUMMY_PRIV(self)->buf_scr_compat = bitmap_create(w, h, 0);
 	} else {
-		DUMMY_PRIV(self)->buf_scr_compat = bitmap_realloc( w, h, 
-			DUMMY_PRIV(self)->buf_scr_compat->bpp, 
-			w * DUMMY_PRIV(self)->buf_scr_compat->bpp, 
-			BITMAP_GROW, 
-			DUMMY_PRIV(self)->buf_scr_compat );	
+		DUMMY_PRIV(self)->buf_scr_compat = bitmap_realloc( w, h,
+			DUMMY_PRIV(self)->buf_scr_compat->bpp,
+			w * DUMMY_PRIV(self)->buf_scr_compat->bpp,
+			BITMAP_GROW,
+			DUMMY_PRIV(self)->buf_scr_compat );
 	}
-	err = Hermes_ConverterRequest( hermes_cnv_h, 
-			&DUMMY_PRIV(self)->vfmt, 
+
+	/* convert screen buffer to ns format: */
+	err = Hermes_ConverterRequest( hermes_cnv_h,
+			&DUMMY_PRIV(self)->vfmt,
 			&DUMMY_PRIV(self)->nsfmt
 	);
 	assert( err != 0 );
 	err = Hermes_ConverterCopy( hermes_cnv_h,
-		DUMMY_PRIV(self)->buf_scr.fd_addr,
+		native->fd_addr,
 		0,			/* x src coord of top left in pixel coords */
 		0,			/* y src coord of top left in pixel coords */
 		w, h,
-		DUMMY_PRIV(self)->buf_scr.fd_w * vdi_sysinfo.pixelsize, /* stride as bytes */
+		native->fd_w * vdi_sysinfo.pixelsize, /* stride as bytes */
 		DUMMY_PRIV(self)->buf_scr_compat->pixdata,
 		0,			/* x dst coord of top left in pixel coords */
 		0,			/* y dst coord of top left in pixel coords */
@@ -780,47 +907,320 @@ static struct bitmap * snapshot_create(GEM_PLOTTER self, int x, int y, int w, in
 	return( (struct bitmap * )DUMMY_PRIV(self)->buf_scr_compat );
 }
 
-static void snapshot_suspend(GEM_PLOTTER self ) 
+static void snapshot_suspend(GEM_PLOTTER self )
 {
 	if( DUMMY_PRIV(self)->size_buf_scr > CONV_KEEP_LIMIT  ) {
-		DUMMY_PRIV(self)->buf_scr.fd_addr = realloc( 
+		DUMMY_PRIV(self)->buf_scr.fd_addr = realloc(
 			DUMMY_PRIV(self)->buf_scr.fd_addr, CONV_KEEP_LIMIT
 		);
 		if( DUMMY_PRIV(self)->buf_scr.fd_addr != NULL ) {
 			DUMMY_PRIV(self)->size_buf_scr = CONV_KEEP_LIMIT;
 		} else {
 			DUMMY_PRIV(self)->size_buf_scr = 0;
-		} 
-	} 
+		}
+	}
+
+	if( DUMMY_PRIV(self)->size_buf_std > CONV_KEEP_LIMIT  ) {
+		DUMMY_PRIV(self)->buf_std.fd_addr = realloc(
+			DUMMY_PRIV(self)->buf_std.fd_addr, CONV_KEEP_LIMIT
+		);
+		if( DUMMY_PRIV(self)->buf_std.fd_addr != NULL ) {
+			DUMMY_PRIV(self)->size_buf_std = CONV_KEEP_LIMIT;
+		} else {
+			DUMMY_PRIV(self)->size_buf_std = 0;
+		}
+	}
 
 	if( bitmap_buffer_size( DUMMY_PRIV(self)->buf_scr_compat ) > CONV_KEEP_LIMIT ) {
-		int w = 0; 
+		int w = 0;
 		int h = 1;
 		w = (CONV_KEEP_LIMIT / DUMMY_PRIV(self)->buf_scr_compat->bpp);
 		assert( CONV_KEEP_LIMIT == w*DUMMY_PRIV(self)->buf_scr_compat->bpp );
-		DUMMY_PRIV(self)->buf_scr_compat = bitmap_realloc( w, h, 
-			DUMMY_PRIV(self)->buf_scr_compat->bpp, 
+		DUMMY_PRIV(self)->buf_scr_compat = bitmap_realloc( w, h,
+			DUMMY_PRIV(self)->buf_scr_compat->bpp,
 			CONV_KEEP_LIMIT, BITMAP_SHRINK, DUMMY_PRIV(self)->buf_scr_compat
 		);
-		
-	} 
+	}
 }
 
-static void snapshot_destroy( GEM_PLOTTER self ) 
+static void snapshot_destroy( GEM_PLOTTER self )
 {
 	if( DUMMY_PRIV(self)->buf_scr.fd_addr ) {
 		free( DUMMY_PRIV(self)->buf_scr.fd_addr  );
 		DUMMY_PRIV(self)->buf_scr.fd_addr = NULL;
 	}
 
+	if( DUMMY_PRIV(self)->buf_std.fd_addr ) {
+		free( DUMMY_PRIV(self)->buf_std.fd_addr  );
+		DUMMY_PRIV(self)->buf_std.fd_addr = NULL;
+	}
+
 	if( DUMMY_PRIV(self)->buf_scr_compat ) {
 		bitmap_destroy( DUMMY_PRIV(self)->buf_scr_compat );
 		DUMMY_PRIV(self)->buf_scr_compat = NULL;
 	}
-} 
+}
+
+inline void set_stdpx( MFDB * dst, int wdplanesz, int x, int y, unsigned char val )
+{
+	short * buf;
+	short whichbit = (1<<(15-(x%16)));
+
+	buf = dst->fd_addr;
+	buf += ((dst->fd_wdwidth*(y))+(x>>4));
+
+	*buf = (val&1) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<1)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<2)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<3)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<4)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<5)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<6)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+
+	buf += wdplanesz;
+	*buf = (val&(1<<7)) ? ((*buf)|(whichbit)) : ((*buf)&~(whichbit));
+}
+
+inline unsigned char get_stdpx(MFDB * dst, int wdplanesz, int x, int y )
+{
+	unsigned char ret=0;
+	short * buf;
+	short whichbit = (1<<(15-(x%16)));
+
+	buf = dst->fd_addr;
+	buf += ((dst->fd_wdwidth*(y))+(x>>4));
+
+	if( *buf & whichbit )
+		ret |= 1;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 2;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 4;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 8;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 16;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 32;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 64;
+
+	buf += wdplanesz;
+	if( *buf & whichbit )
+		ret |= 128;
+
+	return( ret );
+}
+
+#ifdef WITH_8BPP_SUPPORT
+static int bitmap_convert_8( GEM_PLOTTER self,
+	struct bitmap * img,
+	int x,
+	int y,
+	GRECT * clip,
+	uint32_t bg,
+	uint32_t flags,
+	MFDB *out  )
+{
+
+	int dststride;						/* stride of dest. image */
+	int dstsize;						/* size of dest. in byte */
+	int err;
+	int bw;
+	struct bitmap * scrbuf = NULL;
+	struct bitmap * bm;
+	bool transp = ( ( (img->opaque == false) || ( (flags & BITMAP_MONOGLYPH) != 0) )
+					&& ((self->flags & PLOT_FLAG_TRANS) != 0) );
+
+	assert( clip->g_h > 0 );
+	assert( clip->g_w > 0 );
+
+	bm = img;
+	bw = bitmap_get_width( img );
+
+	dststride = MFDB_STRIDE( clip->g_w );
+	dstsize = ( ((dststride >> 3) * clip->g_h) * self->bpp_virt );
+
+	/* (re)allocate buffer for out image: */
+	/* altough the buffer is named "buf_packed" on 8bit systems */
+	/* it's not... */
+	if( dstsize > DUMMY_PRIV(self)->size_buf_packed) {
+		int blocks = (dstsize / (CONV_BLOCK_SIZE-1))+1;
+		if( DUMMY_PRIV(self)->buf_packed == NULL )
+			DUMMY_PRIV(self)->buf_packed =(void*)malloc( blocks * CONV_BLOCK_SIZE );
+		 else
+			DUMMY_PRIV(self)->buf_packed =(void*)realloc(
+													DUMMY_PRIV(self)->buf_packed,
+													blocks * CONV_BLOCK_SIZE
+												);
+		assert( DUMMY_PRIV(self)->buf_packed );
+		if( DUMMY_PRIV(self)->buf_packed == NULL ) {
+			return( 0-ERR_NO_MEM );
+		}
+		DUMMY_PRIV(self)->size_buf_packed = blocks * CONV_BLOCK_SIZE;
+	}
+
+
+	/*
+		on 8 bit systems we must convert the TC (ABGR) image
+		to vdi standard format. ( only tested for 256 colors )
+		and then convert it to native format
+	*/
+
+	// realloc mem for stdform
+	MFDB stdform;
+	if( transp ){
+		if( ((self->flags & PLOT_FLAG_TRANS) != 0) || ( (flags & BITMAP_MONOGLYPH) != 0) ) {
+			// point image to snapshot buffer, otherwise allocate mem
+			MFDB * bg = snapshot_create_std_mfdb( self, x+clip->g_x,y+clip->g_y, clip->g_w, clip->g_h );
+			stdform.fd_addr = bg->fd_addr;
+		} else {
+			if( dstsize > DUMMY_PRIV(self)->size_buf_planar) {
+				int blocks = (dstsize / (CONV_BLOCK_SIZE-1))+1;
+				if( DUMMY_PRIV(self)->buf_planar == NULL )
+					DUMMY_PRIV(self)->buf_planar =(void*)malloc( blocks * CONV_BLOCK_SIZE );
+				 else
+					DUMMY_PRIV(self)->buf_planar =(void*)realloc(
+															DUMMY_PRIV(self)->buf_planar,
+															blocks * CONV_BLOCK_SIZE
+														);
+				assert( DUMMY_PRIV(self)->buf_planar );
+				if( DUMMY_PRIV(self)->buf_planar == NULL ) {
+					return( 0-ERR_NO_MEM );
+				}
+				DUMMY_PRIV(self)->size_buf_planar = blocks * CONV_BLOCK_SIZE;
+			}
+			stdform.fd_addr = DUMMY_PRIV(self)->buf_planar;
+		}
+	}
+	stdform.fd_w = dststride;
+	stdform.fd_h = clip->g_h;
+	stdform.fd_wdwidth = dststride >> 4;
+	stdform.fd_stand = 1;
+	stdform.fd_nplanes = (short)self->bpp_virt;
+	stdform.fd_r1 = stdform.fd_r2 = stdform.fd_r3 = 0;
+
+	int img_stride = bitmap_get_rowstride(bm);
+	uint32_t prev_pixel = 0x12345678;
+	unsigned long col = 0;
+	unsigned char val = 0;
+	uint32_t * row;
+	uint32_t pixel;
+	int wdplanesize = stdform.fd_wdwidth*stdform.fd_h;
+
+
+	// apply transparency.
+	if( transp ){
+		unsigned long bgcol = 0;
+		unsigned char prev_col = 0;
+
+
+		for( y=0; y<clip->g_h; y++ ){
+
+			row = (uint32_t *)(bm->pixdata + (img_stride * (y+clip->g_y)));
+
+			for( x=0; x<clip->g_w; x++ ){
+
+				pixel = row[x+clip->g_x];
+
+				if( (pixel&0xFF) == 0 ){
+					continue;
+				}
+
+				if( (pixel&0xFF) < 0xF0 ){
+					col = get_stdpx( &stdform, wdplanesize,x,y );
+					if( (col != prev_col) || (y == 0) )
+						bgcol = (((rgb_lookup[col][2] << 16) | (rgb_lookup[col][1] << 8) | (rgb_lookup[col][0]))<<8);
+					if( prev_col != col || prev_pixel != pixel ){
+						prev_col = col;
+						pixel = ablend( pixel, bgcol );
+						prev_pixel = pixel;
+						pixel = pixel >> 8;
+						/* convert pixel value to vdi color index: */
+						col = ( ((pixel&0xFF)<<16)
+									| (pixel&0xFF00)
+									| ((pixel&0xFF0000)>>16) );
+						val = RGB_TO_VDI( col );
+					}
+					set_stdpx( &stdform, wdplanesize, x,y, val );
+				} else {
+					if( pixel != prev_pixel ){
+						/* convert pixel value to vdi color index: */
+						pixel = pixel >> 8;
+						col = ( ((pixel&0xFF)<<16)
+									| (pixel&0xFF00)
+									| ((pixel&0xFF0000)>>16) );
+						val = RGB_TO_VDI( col );
+						prev_pixel = pixel;
+					}
+					set_stdpx( &stdform, wdplanesize, x,y, val );
+				}
+			}
+		}
+	} else {
+		for( y=0; y<clip->g_h; y++ ){
+
+			row = (uint32_t *)(bm->pixdata + (img_stride * (y+clip->g_y)));
+
+			for( x=0; x<clip->g_w; x++ ){
+
+				pixel = row[x+clip->g_x];
+				if( pixel != prev_pixel ){
+					/* convert pixel value to vdi color index: */
+					pixel = pixel >> 8;
+					col = ( ((pixel&0xFF)<<16)
+								| (pixel&0xFF00)
+								| ((pixel&0xFF0000)>>16) );
+					val = RGB_TO_VDI( col );
+					prev_pixel = pixel;
+				}
+				set_stdpx( &stdform, wdplanesize, x,y, val );
+			}
+		}
+	}
+
+	// convert into native format:
+	MFDB native;
+	native.fd_addr = DUMMY_PRIV(self)->buf_packed;
+	native.fd_w = dststride;
+	native.fd_h = clip->g_h;
+	native.fd_wdwidth = dststride >> 4;
+	native.fd_stand = 0;
+	native.fd_nplanes = (short)self->bpp_virt;
+	native.fd_r1 = native.fd_r2 = native.fd_r3 = 0;
+	vr_trnfm( self->vdi_handle, &stdform, &native );
+	*out = native;
+
+	return(0);
+}
+#endif
 
 /* convert bitmap to the virutal (chunked) framebuffer format */
-static int convert_bitmap( GEM_PLOTTER self,
+static int bitmap_convert( GEM_PLOTTER self,
 	struct bitmap * img,
 	int x,
 	int y,
@@ -838,18 +1238,18 @@ static int convert_bitmap( GEM_PLOTTER self,
 
 	assert( clip->g_h > 0 );
 	assert( clip->g_w > 0 );
-	
+
 	bm = img;
 	bw = bitmap_get_width( img );
 
 	/* rem. if eddi xy is installed, we could directly access the screen! */
 	/* apply transparency to the image: */
-	if( (img->opaque == false) 
-		&& ( (self->flags & PLOT_FLAG_TRANS) != 0) 
-		&& ( 
-			(vdi_sysinfo.vdiformat == VDI_FORMAT_PACK ) 
-			|| 
-			( (flags & BITMAP_MONOGLYPH) != 0) 
+	if( (img->opaque == false)
+		&& ( (self->flags & PLOT_FLAG_TRANS) != 0)
+		&& (
+			(vdi_sysinfo.vdiformat == VDI_FORMAT_PACK )
+			||
+			( (flags & BITMAP_MONOGLYPH) != 0)
 		) ) {
 		uint32_t * imgpixel;
 		uint32_t * screenpixel;
@@ -860,13 +1260,13 @@ static int convert_bitmap( GEM_PLOTTER self,
 		if( scrbuf != NULL ) {
 			/* copy blended pixels the new buffer (which contains screen content): */
 			int img_stride = bitmap_get_rowstride(bm);
-			int screen_stride = bitmap_get_rowstride(scrbuf); 
+			int screen_stride = bitmap_get_rowstride(scrbuf);
 			for( img_y = clip->g_y, screen_y = 0; screen_y < clip->g_h; screen_y++, img_y++) {
 				imgpixel = (uint32_t *)(bm->pixdata + (img_stride * img_y));
 				screenpixel = (uint32_t *)(scrbuf->pixdata + (screen_stride * screen_y));
-				for( img_x = clip->g_x, screen_x = 0; screen_x < clip->g_w; screen_x++, img_x++ ) {	
+				for( img_x = clip->g_x, screen_x = 0; screen_x < clip->g_w; screen_x++, img_x++ ) {
 					if( (imgpixel[img_x] & 0xFF) == 0xFF ) {
-						screenpixel[screen_x] = imgpixel[img_x]; 
+						screenpixel[screen_x] = imgpixel[img_x];
 					} else {
 						if( (imgpixel[img_x] & 0x0FF) != 0 ) {
 							screenpixel[screen_x] = ablend( imgpixel[img_x], screenpixel[screen_x]);
@@ -888,9 +1288,9 @@ static int convert_bitmap( GEM_PLOTTER self,
 		if( DUMMY_PRIV(self)->buf_packed == NULL )
 			DUMMY_PRIV(self)->buf_packed =(void*)malloc( blocks * CONV_BLOCK_SIZE );
 		 else
-			DUMMY_PRIV(self)->buf_packed =(void*)realloc( 
-													DUMMY_PRIV(self)->buf_packed, 
-													blocks * CONV_BLOCK_SIZE 
+			DUMMY_PRIV(self)->buf_packed =(void*)realloc(
+													DUMMY_PRIV(self)->buf_packed,
+													blocks * CONV_BLOCK_SIZE
 												);
 		assert( DUMMY_PRIV(self)->buf_packed );
 		if( DUMMY_PRIV(self)->buf_packed == NULL ) {
@@ -909,9 +1309,9 @@ static int convert_bitmap( GEM_PLOTTER self,
 	out->fd_r1 = out->fd_r2 = out->fd_r3 = 0;
 
 	err = Hermes_ConverterRequest(
-			hermes_cnv_h, 
-			&DUMMY_PRIV(self)->nsfmt, 
-			&DUMMY_PRIV(self)->vfmt 
+			hermes_cnv_h,
+			&DUMMY_PRIV(self)->nsfmt,
+			&DUMMY_PRIV(self)->vfmt
 	);
 	assert( err != 0 );
 	/* convert image to virtual format: */
@@ -988,47 +1388,48 @@ static int bitmap( GEM_PLOTTER self, struct bitmap * bmp, int x, int y,
 	pxy[5] = CURFB(self).y + loc.g_y;
 	pxy[6] = CURFB(self).x + loc.g_x + off.g_w-1;
 	pxy[7] = CURFB(self).y + loc.g_y + off.g_h-1;
-	if( convert_bitmap( self, bmp, pxy[4], pxy[5], &off, bg, flags, &src_mf) != 0 ) {
-		return( true ); 
+	/* Convert the Bitmap to native screen format - ready for output*/
+	/* This includes blending transparent pixels */
+	if( self->bitmap_convert( self, bmp, pxy[4], pxy[5], &off, bg, flags, &src_mf) != 0 ) {
+		return( true );
 	}
 	vro_cpyfm( self->vdi_handle, S_ONLY, (short*)&pxy, &src_mf,  &scrmf);
 	convert_bitmap_done( self );
 	return( true );
 }
 
-static int plot_mfdb (GEM_PLOTTER self, GRECT * loc, MFDB * insrc, uint32_t flags)
+static int plot_mfdb (GEM_PLOTTER self, GRECT * loc, MFDB * insrc, unsigned char fgcolor, uint32_t flags)
 {
 
 	MFDB screen, tran;
 	MFDB * src;
 	short pxy[8];
-	short pxyclip[4];
-	short c[2] = {OFFSET_CUSTOM_COLOR, WHITE};
+	short c[2] = {fgcolor, WHITE};
 	GRECT off;
 
 	plotter_get_clip_grect( self, &off );
 	if( rc_intersect(loc, &off) == 0 ){
-		return( 1 ); 
+		return( 1 );
 	}
-	
+
 	init_mfdb( 0, loc->g_w, loc->g_h, 0, &screen );
 
 	if( insrc->fd_stand ){
-		int size = init_mfdb( insrc->fd_nplanes, loc->g_w, loc->g_h, 
-			MFDB_FLAG_NOALLOC, 
-			&tran 
+		int size = init_mfdb( insrc->fd_nplanes, loc->g_w, loc->g_h,
+			MFDB_FLAG_NOALLOC,
+			&tran
 		);
 		if( DUMMY_PRIV(self)->size_buf_scr == 0 ){
 			DUMMY_PRIV(self)->buf_scr.fd_addr = malloc( size );
 			DUMMY_PRIV(self)->size_buf_scr = size;
 		} else {
 			if( size > DUMMY_PRIV(self)->size_buf_scr ) {
-				DUMMY_PRIV(self)->buf_scr.fd_addr = realloc( 
+				DUMMY_PRIV(self)->buf_scr.fd_addr = realloc(
 					DUMMY_PRIV(self)->buf_scr.fd_addr, size
 				);
 				DUMMY_PRIV(self)->size_buf_scr = size;
 			}
-		}		
+		}
 		tran.fd_addr = DUMMY_PRIV(self)->buf_scr.fd_addr;
 		vr_trnfm( self->vdi_handle, insrc, &tran );
 		src = &tran;
@@ -1049,17 +1450,8 @@ static int plot_mfdb (GEM_PLOTTER self, GRECT * loc, MFDB * insrc, uint32_t flag
 	if( flags & PLOT_FLAG_TRANS && src->fd_nplanes == 1){
 		vrt_cpyfm( self->vdi_handle, MD_TRANS, (short*)pxy, src, &screen, (short*)&c );
 	} else {
-		/* this method only plots transparent bitmaps, right now... */ 
+		/* this method only plots transparent bitmaps, right now... */
 	}
-
-	/* TODO: shrink conversion buffer? 
-		no, it requires time. 
-	if( insrc->fd_stand ){ 
-
-	}
-
-	*/
-	
 	return( 1 );
 }
 
