@@ -61,53 +61,26 @@ static void __CDECL evnt_tv_keybd( WINDOW *win, short buff[8], void * data )
 	bool r=false;
 	long kstate = 0;
 	long kcode = 0;
+	long ucs4;
+	long ik;
 	unsigned short nkc = 0;
 	unsigned short nks = 0;
 	unsigned char ascii;
+
 	NSTREEVIEW tv = (NSTREEVIEW) data;
 	kstate = evnt.mkstate;
 	kcode = evnt.keybd;
 	nkc= gem_to_norm( (short)kstate, (short)kcode );
 	ascii = (nkc & 0xFF);
-	nkc = (nkc & (NKF_CTRL|NKF_SHIFT|0xFF));
-	if( (nkc & (NKF_SHIFT|NKF_CTRL) ) == 0 ) {
-		switch( ascii ) {
-			case NK_BS:
-				r = tree_keypress( tv->tree, KEY_DELETE_LEFT);
-			break;
+	ik = nkc_to_input_key( nkc, &ucs4 );
 
-			case NK_DEL:
-				r = tree_keypress( tv->tree, KEY_DELETE_RIGHT);
-			break;
-
-			case NK_ENTER:
-				r = tree_keypress( tv->tree, KEY_NL);
-			break;
-
-			case NK_RET:
-				r = tree_keypress( tv->tree, KEY_CR);
-			break;
-
-			case NK_ESC:
-				r = tree_keypress( tv->tree, KEY_ESCAPE);		
-			break;
-
-			case NK_RIGHT:
-				r = tree_keypress( tv->tree, KEY_RIGHT );
-			break;	
-
-			case NK_LEFT:
-				r = tree_keypress( tv->tree, KEY_LEFT );
-			break;
-
-			default:
-			break;
+	if( ik == 0 ){
+		if (ascii >= 9 ) {
+            r = tree_keypress( tv->tree, ucs4 );
 		}
+	} else {
+		r = tree_keypress( tv->tree, ik );
 	}
-	if ( r == false && ( (nkc & NKF_CTRL)==0)  ) {
-		int ucs4 = atari_to_ucs4(ascii);
-		tree_keypress( tv->tree, ucs4 );
-	}	
 }
 
 
@@ -130,18 +103,18 @@ static void __CDECL evnt_tv_redraw( WINDOW *win, short buff[8], void * data )
 		clip.g_h = work.g_h + clip.g_y;
 		clip.g_y = 0;
 	}
-	if( clip.g_h > 0 && clip.g_w > 0 ) { 
-		atari_treeview_request_redraw( win->xpos*win->w_u + clip.g_x, win->ypos*win->h_u + clip.g_y, 
-			clip.g_w, clip.g_h, tv
+	if( clip.g_h > 0 && clip.g_w > 0 ) {
+		atari_treeview_request_redraw(
+										win->xpos*win->w_u + clip.g_x,
+										win->ypos*win->h_u + clip.g_y,
+										clip.g_w, clip.g_h, tv
 		);
 	}
 }
 
-
 static void __CDECL evnt_tv_mbutton( WINDOW *win, short buff[8], void * data )
 {
 	GRECT work;
-	bool ignore=false;
 	NSTREEVIEW tv = (NSTREEVIEW) data;
 	if( tv == NULL )
 		return;
@@ -151,91 +124,79 @@ static void __CDECL evnt_tv_mbutton( WINDOW *win, short buff[8], void * data )
 	}
 
 	WindGetGrect( tv->window, WF_WORKXYWH, &work );
-	int rx = (evnt.mx-work.g_x)+(win->xpos*win->w_u);
-	int ry = (evnt.my-work.g_y)+(win->ypos*win->h_u);
 
-	if( rx >= 0 && ry >= 0 && evnt.mx < work.g_x + work.g_w &&
-		evnt.my < work.g_y + work.g_h ){
+	/* mouse click relative origin: */
+	short origin_rel_x = (evnt.mx-work.g_x)+(win->xpos*win->w_u);
+	short origin_rel_y = (evnt.my-work.g_y)+(win->ypos*win->h_u);
 
-		short mx, my, dummy, mbut;
-		uint32_t tnow = clock()*1000 / CLOCKS_PER_SEC;
+	if( origin_rel_x >= 0 && origin_rel_y >= 0
+		&& evnt.mx < work.g_x + work.g_w
+		&& evnt.my < work.g_y + work.g_h )
+	{
+		int bms;
+		bool ignore=false;
+		short cur_rel_x, cur_rel_y, dummy, mbut;
 
-		if( evnt.mkstate & (K_RSHIFT | K_LSHIFT) ){
-			bmstate |= BROWSER_MOUSE_MOD_1;
-		} else {
-			bmstate &= ~(BROWSER_MOUSE_MOD_1);
-		}
-		if( (evnt.mkstate & K_CTRL) ){
-			bmstate |= BROWSER_MOUSE_MOD_2;
-		} else {
-			bmstate &= ~(BROWSER_MOUSE_MOD_2);
-		}
-		if( (evnt.mkstate & K_ALT) ){
-			bmstate |= BROWSER_MOUSE_MOD_3;
-		} else {
-			bmstate &= ~(BROWSER_MOUSE_MOD_3);
+		if( evnt.nb_click == 2 ){
+			tree_mouse_action(tv->tree,
+							BROWSER_MOUSE_CLICK_1 | BROWSER_MOUSE_DOUBLE_CLICK,
+							origin_rel_x, origin_rel_y );
+			return;
 		}
 
-		graf_mkstate(&dummy, &dummy, &mbut, &dummy);
-		if( evnt.nb_click == 2 ) {
-			bmstate =  BROWSER_MOUSE_CLICK_1 | BROWSER_MOUSE_DOUBLE_CLICK;
-			mouse_hold_start[0] = 0;
-		} else {
-			if( mbut & 1 ) {
-				bmstate |= BROWSER_MOUSE_DRAG_ON;
-				if( mouse_hold_start[0] == 0) {
-					mouse_hold_start[0] = tnow;
-					tv->startdrag.x = rx;
-					tv->startdrag.y = ry;
-					bmstate |=  BROWSER_MOUSE_DRAG_1;
-					gem_set_cursor(&gem_cursors.cross);
-				} else {
-					/* todo: add more isual indication (grafbox?) */
-					ignore = true;		
-					gem_set_cursor(&gem_cursors.cross);	
-				}
-			} else {
-				if( bmstate & BROWSER_MOUSE_DRAG_ON ){
-					bmstate = 0;
-					tree_drag_end(tv->tree, bmstate, tv->startdrag.x, tv->startdrag.y, rx, ry);
-					gem_set_cursor(&gem_cursors.arrow);
-					ignore = true;
-				} else {
-					bmstate =  BROWSER_MOUSE_CLICK_1;	
-					mouse_hold_start[0] = 0;
-				}
+		graf_mkstate(&cur_rel_x, &cur_rel_x, &mbut, &dummy);
+		if( (mbut&1) == 0 ){
+			bms = BROWSER_MOUSE_CLICK_1 | BROWSER_MOUSE_PRESS_1;
+			if( evnt.nb_click == 2 ) {
+				bms = BROWSER_MOUSE_DOUBLE_CLICK;
 			}
-		}
-		if( !ignore ) {
-			tree_mouse_action(tv->tree, bmstate, rx, ry );
-		}
-		bmstate &= ~(BROWSER_MOUSE_DOUBLE_CLICK | BROWSER_MOUSE_CLICK_1 );
-	} 
-}
+			tree_mouse_action(tv->tree, bms, origin_rel_x, origin_rel_y );
+		} else {
+			/* button still pressed */
 
-static void __CDECL evnt_tv_m1( WINDOW *win, short buff[8], void * data)
-{
-	GRECT work;
-	NSTREEVIEW tv = (NSTREEVIEW) data;
-	if( tv == NULL )
-		return;
+			short prev_x = origin_rel_x;
+			short prev_y = origin_rel_y;
 
-	if( bmstate & BROWSER_MOUSE_DRAG_ON ) {
+			cur_rel_x = origin_rel_x;
+			cur_rel_y = origin_rel_y;
 
-		short mbut, dummy;
-		graf_mkstate(&dummy, &dummy, &mbut, &dummy);
-		if( !(mbut & 1) ){
-			WindGetGrect( tv->window, WF_WORKXYWH, &work );
-			int rx = (evnt.mx-work.g_x)+(win->xpos*win->w_u);
-			int ry = (evnt.my-work.g_y)+(win->ypos*win->h_u);
-			bmstate = 0;
-			tree_drag_end(tv->tree, bmstate, tv->startdrag.x, tv->startdrag.y, rx, ry);
+			if( tree_is_edited(tv->tree) ){
+				gem_set_cursor(&gem_cursors.ibeam);
+			} else {
+				gem_set_cursor(&gem_cursors.hand);
+			}
+
+			tv->startdrag.x = origin_rel_x;
+			tv->startdrag.y = origin_rel_y;
+
+			tree_mouse_action( tv->tree,
+								BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_DRAG_ON ,
+								cur_rel_x, cur_rel_y );
+			do{
+				if( abs(prev_x-cur_rel_x) > 5 || abs(prev_y-cur_rel_y) > 5 ){
+					tree_mouse_action( tv->tree,
+								BROWSER_MOUSE_HOLDING_1 | BROWSER_MOUSE_DRAG_ON,
+								cur_rel_x, cur_rel_y);
+					prev_x = cur_rel_x;
+					prev_y = cur_rel_y;
+				}
+
+				if( tv->redraw )
+					atari_treeview_redraw( tv );
+				/* sample mouse button state: */
+				graf_mkstate(&cur_rel_x, &cur_rel_y, &mbut, &dummy);
+				cur_rel_x = (cur_rel_x-work.g_x)+(win->xpos*win->w_u);
+				cur_rel_y = (cur_rel_y-work.g_y)+(win->ypos*win->h_u);
+			} while( mbut & 1 );
+
+			tree_drag_end(tv->tree, 0, tv->startdrag.x, tv->startdrag.y,
+							cur_rel_x, cur_rel_y );
 			gem_set_cursor(&gem_cursors.arrow);
 		}
-	}	
+	}
 }
 
-NSTREEVIEW atari_treeview_create( uint32_t flags, WINDOW *win ) 
+NSTREEVIEW atari_treeview_create( uint32_t flags, WINDOW *win )
 {
 	if( win == NULL )
 		return( NULL );
@@ -252,11 +213,10 @@ NSTREEVIEW atari_treeview_create( uint32_t flags, WINDOW *win )
 
 	win->w_u = 16;
 	win->h_u = 16;
-	
+
 	EvntDataAdd( new->window, WM_XBUTTON, evnt_tv_mbutton, new, EV_BOT );
 	EvntDataAttach( new->window, WM_REDRAW, evnt_tv_redraw, new );
 	EvntDataAttach( new->window, WM_XKEYBD, evnt_tv_keybd, new );
-	EvntDataAttach( new->window, WM_XM1, evnt_tv_m1, new );
 
 	return(new);
 }
@@ -275,7 +235,7 @@ void atari_treeview_close( NSTREEVIEW tv )
 	}
 }
 
-void atari_treeview_destroy( NSTREEVIEW tv ) 
+void atari_treeview_destroy( NSTREEVIEW tv )
 {
 	if( tv != NULL ){
 		tv->disposing = true;
@@ -314,22 +274,25 @@ void atari_treeview_redraw( NSTREEVIEW tv)
 
 			struct redraw_context ctx = {
 				.interactive = true,
+				.background_images = true,
 				.plot = &atari_plotters
 			};
 
 			plotter->resize(plotter, work.g_w, work.g_h);
 			plotter->move(plotter, work.g_x, work.g_y );
+			if( plotter->lock( plotter ) == 0 )
+				return;
 
 			todo[0] = work.g_x;
 			todo[1] = work.g_y;
 			todo[2] = todo[0] + work.g_w-1;
 			todo[3] = todo[1] + work.g_h-1;
 			vs_clip(plotter->vdi_handle, 1, (short*)&todo );
-			
-			if( wind_get(tv->window->handle, WF_FIRSTXYWH, 
+
+			if( wind_get(tv->window->handle, WF_FIRSTXYWH,
 							&todo[0], &todo[1], &todo[2], &todo[3] )!=0 ) {
 				while (todo[2] && todo[3]) {
-					
+
 					/* convert screen to treeview coords: */
 					todo[0] = todo[0] - work.g_x + tv->window->xpos*tv->window->w_u;
 					todo[1] = todo[1] - work.g_y + tv->window->ypos*tv->window->h_u;
@@ -343,18 +306,20 @@ void atari_treeview_redraw( NSTREEVIEW tv)
 					}
 
 					if (rc_intersect((GRECT *)&tv->rdw_area,(GRECT *)&todo)) {
-						tree_draw(tv->tree, -tv->window->xpos*16, -tv->window->ypos*16, 
+						tree_draw(tv->tree, -tv->window->xpos*16, -tv->window->ypos*16,
 							todo[0], todo[1], todo[2], todo[3], &ctx
 						);
 					}
-					if (wind_get(tv->window->handle, WF_NEXTXYWH, 
+					if (wind_get(tv->window->handle, WF_NEXTXYWH,
 							&todo[0], &todo[1], &todo[2], &todo[3])==0) {
 						break;
 					}
 				}
 			} else {
+				plotter->unlock( plotter );
 				return;
 			}
+			plotter->unlock( plotter );
 			vs_clip(plotter->vdi_handle, 0, (short*)&todo);
 			tv->redraw = false;
 			tv->rdw_area.g_x = 65000;
@@ -379,7 +344,7 @@ void atari_treeview_redraw( NSTREEVIEW tv)
  */
 void atari_treeview_request_redraw(int x, int y, int w, int h, void *pw)
 {
-	if (pw != NULL) {
+	if ( pw != NULL ) {
 		NSTREEVIEW tv = (NSTREEVIEW) pw;
 		if( tv->redraw == false ){
 			tv->redraw = true;
@@ -395,7 +360,7 @@ void atari_treeview_request_redraw(int x, int y, int w, int h, void *pw)
 			int oldy1 = tv->rdw_area.g_y + tv->rdw_area.g_h;
 			tv->rdw_area.g_x = MIN(tv->rdw_area.g_x, x);
 			tv->rdw_area.g_y = MIN(tv->rdw_area.g_y, y);
-			tv->rdw_area.g_w = ( oldx1 > newx1 ) ? oldx1 - tv->rdw_area.g_x : newx1 - tv->rdw_area.g_x;  
+			tv->rdw_area.g_w = ( oldx1 > newx1 ) ? oldx1 - tv->rdw_area.g_x : newx1 - tv->rdw_area.g_x;
 			tv->rdw_area.g_h = ( oldy1 > newy1 ) ? oldy1 - tv->rdw_area.g_y : newy1 - tv->rdw_area.g_y;
 		}
 	}
@@ -436,8 +401,8 @@ void atari_treeview_resized(struct tree *tree, int width, int height, void *pw)
 
 void atari_treeview_scroll_visible(int y, int height, void *pw)
 {
-	/* we don't support dragging within the treeview */
-	/* so we don't need to implement this */
+	/* we don't support dragging outside the treeview */
+	/* so we don't need to implement this? */
 }
 
 /**
