@@ -783,7 +783,7 @@ void nsbeos_window_expose_event(BView *view, gui_window *g, BMessage *message)
 	float scale = g->bw->scale;
 	struct rect clip;
 
-	struct redraw_context ctx = { true, &nsbeos_plotters };
+	struct redraw_context ctx = { true, true, &nsbeos_plotters };
 
 	assert(g);
 	assert(g->bw);
@@ -1603,8 +1603,8 @@ bool gui_window_scroll_start(struct gui_window *g)
 	return true;
 }
 
-bool gui_window_box_scroll_start(struct gui_window *g,
-		    int x0, int y0, int x1, int y1)
+bool gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+		const struct rect *rect)
 {
 	return true;
 }
