@@ -23,7 +23,7 @@
 #define UNIMPL() NSLog( @"Function '%s' unimplemented", __func__ )
 
 bool save_complete_gui_save(const char *path, const char *filename,
-							size_t len, const char *sourcedata, content_type type)
+							size_t len, const char *sourcedata, lwc_string *mime_type)
 {
 	UNIMPL();
 	return false;
