@@ -82,7 +82,7 @@ nserror content__init(struct content *c, const content_handler *handler,
 	struct content_user *user_sentinel;
 	nserror error;
 	
-	LOG(("url %s -> %p", llcache_handle_get_url(llcache), c));
+	LOG(("url %s -> %p", nsurl_access(llcache_handle_get_url(llcache)), c));
 
 	user_sentinel = talloc(c, struct content_user);
 	if (user_sentinel == NULL) {
@@ -274,7 +274,8 @@ void content_convert(struct content *c)
 	if (c->locked == true)
 		return;
 	
-	LOG(("content %s (%p)", llcache_handle_get_url(c->llcache), c));
+	LOG(("content %s (%p)",
+			nsurl_access(llcache_handle_get_url(c->llcache)), c));
 
 	if (c->handler->data_complete != NULL) {
 		c->locked = true;
@@ -353,7 +354,7 @@ void content__reformat(struct content *c, bool background,
 	assert(c->status == CONTENT_STATUS_READY ||
 			c->status == CONTENT_STATUS_DONE);
 	assert(c->locked == false);
-	LOG(("%p %s", c, llcache_handle_get_url(c->llcache)));
+	LOG(("%p %s", c, nsurl_access(llcache_handle_get_url(c->llcache))));
 	c->available_width = width;
 	if (c->handler->reformat != NULL) {
 
@@ -375,8 +376,11 @@ void content__reformat(struct content *c, bool background,
 
 void content_destroy(struct content *c)
 {
+	struct content_rfc5988_link *link;
+
 	assert(c);
-	LOG(("content %p %s", c, llcache_handle_get_url(c->llcache)));
+	LOG(("content %p %s", c,
+			nsurl_access(llcache_handle_get_url(c->llcache))));
 	assert(c->locked == false);
 
 	if (c->handler->destroy != NULL)
@@ -387,6 +391,12 @@ void content_destroy(struct content *c)
 
 	lwc_string_unref(c->mime_type);
 
+	/* release metadata links */
+	link = c->links;
+	while (link != NULL) {
+		link = content__free_rfc5988_link(link);
+	}
+
 	talloc_free(c);
 }
 
@@ -541,7 +551,8 @@ bool content_add_user(struct content *c,
 	struct content_user *user;
 
 	LOG(("content %s (%p), user %p %p",
-			llcache_handle_get_url(c->llcache), c, callback, pw));
+			nsurl_access(llcache_handle_get_url(c->llcache)),
+			c, callback, pw));
 	user = talloc(c, struct content_user);
 	if (!user)
 		return false;
@@ -568,7 +579,8 @@ void content_remove_user(struct content *c,
 {
 	struct content_user *user, *next;
 	LOG(("content %s (%p), user %p %p",
-			llcache_handle_get_url(c->llcache), c, callback, pw));
+			nsurl_access(llcache_handle_get_url(c->llcache)), c,
+			callback, pw));
 
 	/* user_list starts with a sentinel */
 	for (user = c->user_list; user->next != 0 &&
@@ -665,7 +677,8 @@ void content_open(hlcache_handle *h, struct browser_window *bw,
 {
 	struct content *c = hlcache_handle_get_content(h);
 	assert(c != 0);
-	LOG(("content %p %s", c, llcache_handle_get_url(c->llcache)));
+	LOG(("content %p %s", c,
+			nsurl_access(llcache_handle_get_url(c->llcache))));
 	if (c->handler->open != NULL)
 		c->handler->open(c, bw, page, box, params);
 }
@@ -681,7 +694,8 @@ void content_close(hlcache_handle *h)
 {
 	struct content *c = hlcache_handle_get_content(h);
 	assert(c != 0);
-	LOG(("content %p %s", c, llcache_handle_get_url(c->llcache)));
+	LOG(("content %p %s", c,
+			nsurl_access(llcache_handle_get_url(c->llcache))));
 	if (c->handler->close != NULL)
 		c->handler->close(c);
 }
@@ -719,6 +733,32 @@ void content_get_contextual_content(struct hlcache_handle *h,
 }
 
 
+bool content_scroll_at_point(struct hlcache_handle *h,
+		int x, int y, int scrx, int scry)
+{
+	struct content *c = hlcache_handle_get_content(h);
+	assert(c != 0);
+
+	if (c->handler->scroll_at_point != NULL)
+		return c->handler->scroll_at_point(c, x, y, scrx, scry);
+
+	return false;
+}
+
+
+bool content_drop_file_at_point(struct hlcache_handle *h,
+		int x, int y, char *file)
+{
+	struct content *c = hlcache_handle_get_content(h);
+	assert(c != 0);
+
+	if (c->handler->drop_file_at_point != NULL)
+		return c->handler->drop_file_at_point(c, x, y, file);
+
+	return false;
+}
+
+
 void content_add_error(struct content *c, const char *token,
 		unsigned int line)
 {
@@ -738,6 +778,97 @@ bool content__set_title(struct content *c, const char *title)
 	return true;
 }
 
+struct content_rfc5988_link *
+content_find_rfc5988_link(hlcache_handle *h, lwc_string *rel)
+{
+	struct content *c = hlcache_handle_get_content(h);
+	struct content_rfc5988_link *link = c->links;
+	bool rel_match = false;
+
+	while (link != NULL) {
+		lwc_string_caseless_isequal(link->rel, rel, &rel_match);
+		if (rel_match) {
+			break;
+		}
+		link = link->next;
+	}
+	return link;
+}
+
+struct content_rfc5988_link *
+content__free_rfc5988_link(struct content_rfc5988_link *link) 
+{
+	struct content_rfc5988_link *next;
+
+	next = link->next;
+
+	lwc_string_unref(link->rel);
+	nsurl_unref(link->href);
+	if (link->hreflang != NULL) {
+		lwc_string_unref(link->hreflang);
+	}
+	if (link->type != NULL) {
+		lwc_string_unref(link->type);
+	}
+	if (link->media != NULL) {
+		lwc_string_unref(link->media);
+	}
+	if (link->sizes != NULL) {
+		lwc_string_unref(link->sizes);
+	}
+	free(link);
+
+	return next;
+}
+
+bool content__add_rfc5988_link(struct content *c, 
+		const struct content_rfc5988_link *link)
+{
+	struct content_rfc5988_link *newlink;	
+	union content_msg_data msg_data;
+
+	/* a link relation must be present for it to be a link */
+	if (link->rel == NULL) {
+		return false;
+	}
+
+	/* a link href must be present for it to be a link */
+	if (link->href == NULL) {
+		return false;
+	}
+
+	newlink = calloc(1, sizeof(struct content_rfc5988_link));
+	if (newlink == NULL) {
+		return false; 
+	}
+
+	/* copy values */
+	newlink->rel = lwc_string_ref(link->rel);
+	newlink->href = nsurl_ref(link->href);
+	if (link->hreflang != NULL) {
+		newlink->hreflang = lwc_string_ref(link->hreflang);
+	}
+	if (link->type != NULL) {
+		newlink->type = lwc_string_ref(link->type);
+	}
+	if (link->media != NULL) {
+		newlink->media = lwc_string_ref(link->media);
+	}
+	if (link->sizes != NULL) {
+		newlink->sizes = lwc_string_ref(link->sizes);
+	}
+
+	/* add to metadata link to list */
+	newlink->next = c->links;
+	c->links = newlink;
+
+	/* broadcast the data */
+	msg_data.rfc5988_link = newlink;
+	content_broadcast(c, CONTENT_MSG_LINK, msg_data);
+
+	return true;
+}
+
 /**
  * Retrieve computed type of content
  *
@@ -779,12 +910,7 @@ lwc_string *content__get_mime_type(struct content *c)
  * \param c  Content to retrieve URL from
  * \return Pointer to URL, or NULL if not found.
  */
-const char *content_get_url(hlcache_handle *h)
-{
-	return content__get_url(hlcache_handle_get_content(h));
-}
-
-const char *content__get_url(struct content *c)
+nsurl *content_get_url(struct content *c)
 {
 	if (c == NULL)
 		return NULL;
@@ -808,7 +934,8 @@ const char *content__get_title(struct content *c)
 	if (c == NULL)
 		return NULL;
 
-	return c->title != NULL ? c->title : llcache_handle_get_url(c->llcache);
+	return c->title != NULL ? c->title :
+			nsurl_access(llcache_handle_get_url(c->llcache));
 }
 
 /**
@@ -963,12 +1090,12 @@ void content__invalidate_reuse_data(struct content *c)
  * \param c  Content to retrieve refresh URL from
  * \return Pointer to URL, or NULL if none
  */
-const char *content_get_refresh_url(hlcache_handle *h)
+nsurl *content_get_refresh_url(hlcache_handle *h)
 {
 	return content__get_refresh_url(hlcache_handle_get_content(h));
 }
 
-const char *content__get_refresh_url(struct content *c)
+nsurl *content__get_refresh_url(struct content *c)
 {
 	if (c == NULL)
 		return NULL;
@@ -1152,7 +1279,7 @@ nserror content__clone(const struct content *c, struct content *nc)
 	}
 	
 	if (c->refresh != NULL) {
-		nc->refresh = talloc_strdup(nc, c->refresh);
+		nc->refresh = nsurl_ref(c->refresh);
 		if (nc->refresh == NULL) {
 			return NSERROR_NOMEM;
 		}
