@@ -33,6 +33,7 @@
 #include "utils/config.h"
 #include "utils/errors.h"
 #include "utils/http.h"
+#include "utils/nsurl.h"
 #include "utils/types.h"
 #include "content/content_factory.h"
 #include "content/content_type.h"
@@ -71,9 +72,21 @@ typedef enum {
 	CONTENT_MSG_REDRAW,    /**< needs redraw (eg. new animation frame) */
 	CONTENT_MSG_REFRESH,   /**< wants refresh */
 	CONTENT_MSG_DOWNLOAD,  /**< download, not for display */
-	CONTENT_MSG_FAVICON_REFRESH, /**< favicon has been refreshed (eg. new animation frame) */
+	CONTENT_MSG_LINK,      /**< RFC5988 link */
 } content_msg;
 
+/** RFC5988 metadata link */
+struct content_rfc5988_link {
+	struct content_rfc5988_link *next; /**< next rfc5988_link in list */
+
+	lwc_string *rel; /**< the link relationship - must be present */
+	nsurl *href; /* the link href - must be present */
+	lwc_string *hreflang;
+	lwc_string *type;
+	lwc_string *media;
+	lwc_string *sizes;
+};
+
 /** Extra data for some content_msg messages. */
 union content_msg_data {
 	const char *error;	/**< Error message, for CONTENT_MSG_ERROR. */
@@ -95,27 +108,29 @@ union content_msg_data {
 	bool background;
 	/** Low-level cache handle, for CONTENT_MSG_DOWNLOAD */
 	struct llcache_handle *download;
+	/** rfc5988 link data  CONTENT_MSG_RFC5988_LINK */
+	struct content_rfc5988_link *rfc5988_link;
 };
 
-
+/** parameters to content redraw */
 struct content_redraw_data {
-	int x;			/** coordinate for top-left of redraw */
-	int y;			/** coordinate for top-left of redraw */
+	int x; /**< coordinate for top-left of redraw */
+	int y; /**< coordinate for top-left of redraw */
 
 	/** dimensions to render content at
 	 *  (for scaling contents with intrinsic dimensions) */
-	int width;		/* horizontal */
-	int height;		/* vertical */
+	int width; /**< horizontal dimension */
+	int height; /**< vertical dimension */
 
-	/** the background colour */
+	/** The background colour */
 	colour background_colour;
 
 	/** Scale for redraw
 	 *  (for scaling contents without intrinsic dimensions) */
-	float scale;		/* scale factor */
+	float scale; /**< Scale factor for redraw */
 
-	bool repeat_x;		/* whether content is tiled in x direction */
-	bool repeat_y;		/* whether content is tiled in y direction */
+	bool repeat_x; /**< whether content is tiled in x direction */
+	bool repeat_y; /**< whether content is tiled in y direction */
 };
 
 /* The following are for hlcache */
@@ -136,6 +151,7 @@ bool content_is_shareable(struct content *c);
 content_status content__get_status(struct content *c);
 
 const struct llcache_handle *content_get_llcache_handle(struct content *c);
+nsurl *content_get_url(struct content *c);
 
 struct content *content_clone(struct content *c);
 
@@ -160,11 +176,16 @@ void content_close(struct hlcache_handle *h);
 struct selection *content_get_selection(struct hlcache_handle *h);
 void content_get_contextual_content(struct hlcache_handle *h,
 		int x, int y, struct contextual_content *data);
+bool content_scroll_at_point(struct hlcache_handle *h,
+		int x, int y, int scrx, int scry);
+bool content_drop_file_at_point(struct hlcache_handle *h,
+		int x, int y, char *file);
+struct content_rfc5988_link *content_find_rfc5988_link(struct hlcache_handle *c,
+		lwc_string *rel);
 
 /* Member accessors */
 content_type content_get_type(struct hlcache_handle *c);
 lwc_string *content_get_mime_type(struct hlcache_handle *c);
-const char *content_get_url(struct hlcache_handle *c);
 const char *content_get_title(struct hlcache_handle *c);
 content_status content_get_status(struct hlcache_handle *c);
 const char *content_get_status_message(struct hlcache_handle *c);
@@ -174,7 +195,7 @@ int content_get_available_width(struct hlcache_handle *c);
 const char *content_get_source_data(struct hlcache_handle *c, 
 		unsigned long *size);
 void content_invalidate_reuse_data(struct hlcache_handle *c);
-const char *content_get_refresh_url(struct hlcache_handle *c);
+nsurl *content_get_refresh_url(struct hlcache_handle *c);
 struct bitmap *content_get_bitmap(struct hlcache_handle *c);
 bool content_get_opaque(struct hlcache_handle *h);
 bool content_get_quirks(struct hlcache_handle *c);
