@@ -68,7 +68,11 @@ struct content_handler {
 	void (*close)(struct content *c);
 	struct selection * (*get_selection)(struct content *c);
 	void (*get_contextual_content)(struct content *c, int x, int y,
-				struct contextual_content *data);
+			struct contextual_content *data);
+	bool (*scroll_at_point)(struct content *c, int x, int y,
+			int scrx, int scry);
+	bool (*drop_file_at_point)(struct content *c, int x, int y,
+			char *file);
 	nserror (*clone)(const struct content *old, struct content **newc);
 	bool (*matches_quirks)(const struct content *c, bool quirks);
 	content_type (*type)(void);
@@ -106,8 +110,9 @@ struct content {
 	bool quirks;		/**< Content is in quirks mode */
 	char *fallback_charset;	/**< Fallback charset, or NULL */
 
-	/** URL for refresh request, in standard form as from url_join. */
-	char *refresh;
+	nsurl *refresh;		/**< URL for refresh request */
+
+	struct content_rfc5988_link *links; /**< list of metadata links */
 
 	unsigned int time;		/**< Creation time,
 					  if LOADING or READY,
@@ -161,6 +166,11 @@ void content_broadcast(struct content *c, content_msg msg,
 void content_add_error(struct content *c, const char *token,
 		unsigned int line);
 
+bool content__add_rfc5988_link(struct content *c, 
+		const struct content_rfc5988_link *link);
+struct content_rfc5988_link *content__free_rfc5988_link(
+		struct content_rfc5988_link *link);
+
 void content__reformat(struct content *c, bool background,
 		int width, int height);
 void content__request_redraw(struct content *c,
@@ -169,7 +179,6 @@ void content__request_redraw(struct content *c,
 bool content__set_title(struct content *c, const char *title);
 
 lwc_string *content__get_mime_type(struct content *c);
-const char *content__get_url(struct content *c);
 const char *content__get_title(struct content *c);
 const char *content__get_status_message(struct content *c);
 int content__get_width(struct content *c);
@@ -177,7 +186,7 @@ int content__get_height(struct content *c);
 int content__get_available_width(struct content *c);
 const char *content__get_source_data(struct content *c, unsigned long *size);
 void content__invalidate_reuse_data(struct content *c);
-const char *content__get_refresh_url(struct content *c);
+nsurl *content__get_refresh_url(struct content *c);
 struct bitmap *content__get_bitmap(struct content *c);
 bool content__get_opaque(struct content *c);
 
