@@ -48,7 +48,7 @@
 #include "desktop/options.h"
 #include "utils/log.h"
 #include "utils/messages.h"
-#include "utils/url.h"
+#include "utils/nsurl.h"
 #include "utils/utils.h"
 #include "utils/ring.h"
 
@@ -59,7 +59,8 @@ bool fetch_active;	/**< Fetches in progress, please call fetch_poll(). */
 
 /** Information about a fetcher for a given scheme. */
 typedef struct scheme_fetcher_s {
-	char *scheme_name;		/**< The scheme. */
+	lwc_string *scheme_name;		/**< The scheme. */
+	fetcher_can_fetch can_fetch;		/**< Ensure an URL can be fetched. */
 	fetcher_setup_fetch setup_fetch;	/**< Set up a fetch. */
 	fetcher_start_fetch start_fetch;	/**< Start a fetch. */
 	fetcher_abort_fetch abort_fetch;	/**< Abort a fetch. */
@@ -76,13 +77,12 @@ static scheme_fetcher *fetchers = NULL;
 /** Information for a single fetch. */
 struct fetch {
 	fetch_callback callback;/**< Callback function. */
-	char *url;		/**< URL. */
-	char *referer;		/**< Referer URL. */
+	nsurl *url;		/**< URL. */
+	nsurl *referer;		/**< Referer URL. */
 	bool send_referer;	/**< Valid to send the referer */
 	bool verifiable;	/**< Transaction is verifiable */
 	void *p;		/**< Private data for callback. */
-	char *host;		/**< Host part of URL. */
-        lwc_string *lwc_host;	/**< Host part of URL, interned */
+	lwc_string *host;	/**< Host part of URL, interned */
 	long http_code;		/**< HTTP response code, or 0. */
 	scheme_fetcher *ops;	/**< Fetcher operations for this fetch,
 				     NULL if not set. */
@@ -101,11 +101,13 @@ static void fetch_dispatch_jobs(void);
 static bool fetch_choose_and_dispatch(void);
 static bool fetch_dispatch_job(struct fetch *fetch);
 
+/* Static lwc_strings */
+static lwc_string *fetch_http_lwc;
+static lwc_string *fetch_https_lwc;
+
 
 /**
  * Initialise the fetcher.
- *
- * Must be called once before any other function.
  */
 
 void fetch_init(void)
@@ -116,6 +118,18 @@ void fetch_init(void)
 	fetch_resource_register();
 	fetch_about_register();
 	fetch_active = false;
+
+	if (lwc_intern_string("http", SLEN("http"), &fetch_http_lwc) !=
+			lwc_error_ok) {
+		die("Failed to initialise the fetch module "
+				"(couldn't intern \"http\").");
+	}
+
+	if (lwc_intern_string("https", SLEN("https"), &fetch_https_lwc) !=
+			lwc_error_ok) {
+		die("Failed to initialise the fetch module "
+				"(couldn't intern \"https\").");
+	}
 }
 
 
@@ -130,17 +144,21 @@ void fetch_quit(void)
 	while (fetchers != NULL) {
 		if (fetchers->refcount != 1) {
 			LOG(("Fetcher for scheme %s still active?!",
-					fetchers->scheme_name));
+					lwc_string_data(fetchers->scheme_name)));
 			/* We shouldn't do this, but... */
 			fetchers->refcount = 1;
 		}
 		fetch_unref_fetcher(fetchers);
 	}
+
+	lwc_string_unref(fetch_http_lwc);
+	lwc_string_unref(fetch_https_lwc);
 }
 
 
-bool fetch_add_fetcher(const char *scheme,
+bool fetch_add_fetcher(lwc_string *scheme,
 		  fetcher_initialise initialiser,
+		  fetcher_can_fetch can_fetch,
 		  fetcher_setup_fetch setup_fetch,
 		  fetcher_start_fetch start_fetch,
 		  fetcher_abort_fetch abort_fetch,
@@ -156,13 +174,9 @@ bool fetch_add_fetcher(const char *scheme,
 		finaliser(scheme);
 		return false;
 	}
-	new_fetcher->scheme_name = strdup(scheme);
-	if (new_fetcher->scheme_name == NULL) {
-		free(new_fetcher);
-		finaliser(scheme);
-		return false;
-	}
+	new_fetcher->scheme_name = scheme;
 	new_fetcher->refcount = 0;
+	new_fetcher->can_fetch = can_fetch;
 	new_fetcher->setup_fetch = setup_fetch;
 	new_fetcher->start_fetch = start_fetch;
 	new_fetcher->abort_fetch = abort_fetch;
@@ -180,7 +194,7 @@ void fetch_unref_fetcher(scheme_fetcher *fetcher)
 {
 	if (--fetcher->refcount == 0) {
 		fetcher->finaliser(fetcher->scheme_name);
-		free(fetcher->scheme_name);
+		lwc_string_unref(fetcher->scheme_name);
 		if (fetcher == fetchers) {
 			fetchers = fetcher->next_fetcher;
 			if (fetchers)
@@ -217,59 +231,35 @@ void fetch_unref_fetcher(scheme_fetcher *fetcher)
  *
  */
 
-struct fetch * fetch_start(const char *url, const char *referer,
+struct fetch * fetch_start(nsurl *url, nsurl *referer,
 			   fetch_callback callback,
 			   void *p, bool only_2xx, const char *post_urlenc,
 			   const struct fetch_multipart_data *post_multipart,
 			   bool verifiable, const char *headers[])
 {
-	char *host;
 	struct fetch *fetch;
-	url_func_result res;
-	char *scheme = NULL, *ref_scheme = NULL;
 	scheme_fetcher *fetcher = fetchers;
+	lwc_string *scheme;
+	bool match;
 
 	fetch = malloc(sizeof (*fetch));
 	if (fetch == NULL)
 		return NULL;
 
-	res = url_host(url, &host);
-	if (res != URL_FUNC_OK) {
-		/* we only fail memory exhaustion */
-		if (res == URL_FUNC_NOMEM)
-			goto failed;
-
-		host = strdup("");
-		if (host == NULL)
-			goto failed;
-	}
-
 	/* The URL we're fetching must have a scheme */
-	res = url_scheme(url, &scheme);
-	if (res != URL_FUNC_OK)
+	scheme = nsurl_get_component(url, NSURL_SCHEME);
+	if (scheme == NULL)
 		goto failed;
 
-	if (referer) {
-		res = url_scheme(referer, &ref_scheme);
-		if (res != URL_FUNC_OK) {
-			/* we only fail memory exhaustion */
-			if (res == URL_FUNC_NOMEM)
-				goto failed;
-
-			ref_scheme = NULL;
-		}
-	}
-
 #ifdef DEBUG_FETCH_VERBOSE
-	LOG(("fetch %p, url '%s'", fetch, url));
+	LOG(("fetch %p, url '%s'", fetch, nsurl_access(url)));
 #endif
 
 	/* construct a new fetch structure */
 	fetch->callback = callback;
-	fetch->url = strdup(url);
+	fetch->url = nsurl_ref(url);
 	fetch->verifiable = verifiable;
 	fetch->p = p;
-	fetch->host = host;
 	fetch->http_code = 0;
 	fetch->r_prev = NULL;
 	fetch->r_next = NULL;
@@ -278,15 +268,14 @@ struct fetch * fetch_start(const char *url, const char *referer,
 	fetch->fetcher_handle = NULL;
 	fetch->ops = NULL;
 	fetch->fetch_is_active = false;
-        fetch->lwc_host = NULL;
-        
-        if (lwc_intern_string(host, strlen(host), &fetch->lwc_host) != lwc_error_ok)
-                goto failed;
+	fetch->host = nsurl_get_component(url, NSURL_HOST);
         
 	if (referer != NULL) {
-		fetch->referer = strdup(referer);
-		if (fetch->referer == NULL)
-			goto failed;
+		lwc_string *ref_scheme;
+		fetch->referer = nsurl_ref(referer);
+
+		ref_scheme = nsurl_get_component(referer, NSURL_SCHEME);
+		/* Not a problem if referer has no scheme */
 
 		/* Determine whether to send the Referer header */
 		if (option_send_referer && ref_scheme != NULL) {
@@ -300,11 +289,16 @@ struct fetch * fetch_start(const char *url, const char *referer,
 			 * request from a page served over http. The inverse
 			 * (https -> http) should not send the referer (15.1.3)
 			 */
-			if (strcasecmp(scheme, ref_scheme) == 0 ||
-					(strcasecmp(scheme, "https") == 0 &&
-					strcasecmp(ref_scheme, "http") == 0))
+			bool match1;
+			bool match2;
+			lwc_string_isequal(scheme, ref_scheme, &match);
+			lwc_string_isequal(scheme, fetch_https_lwc, &match1);
+			lwc_string_isequal(ref_scheme, fetch_http_lwc, &match2);
+			if (match == true || (match1 == true && match2 == true))
 				fetch->send_referer = true;
 		}
+		if (ref_scheme != NULL)
+			lwc_string_unref(ref_scheme);
 	}
 
 	if (fetch->url == NULL)
@@ -312,7 +306,8 @@ struct fetch * fetch_start(const char *url, const char *referer,
 
 	/* Pick the scheme ops */
 	while (fetcher) {
-		if (strcmp(fetcher->scheme_name, scheme) == 0) {
+		lwc_string_isequal(fetcher->scheme_name, scheme, &match);
+		if (match == true) {
 			fetch->ops = fetcher;
 			break;
 		}
@@ -323,8 +318,8 @@ struct fetch * fetch_start(const char *url, const char *referer,
 		goto failed;
 
 	/* Got a scheme fetcher, try and set up the fetch */
-	fetch->fetcher_handle =
-		fetch->ops->setup_fetch(fetch, url, only_2xx, post_urlenc,
+	fetch->fetcher_handle = fetch->ops->setup_fetch(fetch, url,
+					only_2xx, post_urlenc,
 					post_multipart, headers);
 
 	if (fetch->fetcher_handle == NULL)
@@ -334,8 +329,7 @@ struct fetch * fetch_start(const char *url, const char *referer,
 	fetch_ref_fetcher(fetch->ops);
 
 	/* these aren't needed past here */
-	free(scheme);
-	free(ref_scheme);
+	lwc_string_unref(scheme);
 
 	/* Dump us in the queue and ask the queue to run. */
 	RING_INSERT(queue_ring, fetch);
@@ -344,13 +338,16 @@ struct fetch * fetch_start(const char *url, const char *referer,
 	return fetch;
 
 failed:
-	free(host);
-	free(ref_scheme);
-	free(scheme);
-	free(fetch->url);
-	free(fetch->referer);
-        if (fetch->lwc_host != NULL)
-                lwc_string_unref(fetch->lwc_host);
+	if (scheme != NULL)
+		lwc_string_unref(scheme);
+
+	if (fetch->host != NULL)
+		lwc_string_unref(fetch->host);
+	if (fetch->url != NULL)
+		nsurl_unref(fetch->url);
+	if (fetch->referer != NULL)
+		nsurl_unref(fetch->referer);
+
 	free(fetch);
 
 	return NULL;
@@ -363,8 +360,10 @@ failed:
 void fetch_dispatch_jobs(void)
 {
 	int all_active, all_queued;
+#ifdef DEBUG_FETCH_VERBOSE
 	struct fetch *q;
 	struct fetch *f;
+#endif
 
 	if (!queue_ring)
 		return; /* Nothing to do, the queue is empty */
@@ -373,26 +372,22 @@ void fetch_dispatch_jobs(void)
 
 #ifdef DEBUG_FETCH_VERBOSE
 	LOG(("queue_ring %i, fetch_ring %i", all_queued, all_active));
-#endif
 
 	q = queue_ring;
 	if (q) {
 		do {
-#ifdef DEBUG_FETCH_VERBOSE
 			LOG(("queue_ring: %s", q->url));
-#endif
 			q = q->r_next;
 		} while (q != queue_ring);
 	}
 	f = fetch_ring;
 	if (f) {
 		do {
-#ifdef DEBUG_FETCH_VERBOSE
 			LOG(("fetch_ring: %s", f->url));
-#endif
 			f = f->r_next;
 		} while (f != fetch_ring);
 	}
+#endif
 
 	while ( all_queued && all_active < option_max_fetchers ) {
 		/*LOG(("%d queued, %d fetching", all_queued, all_active));*/
@@ -421,6 +416,7 @@ void fetch_dispatch_jobs(void)
  */
 bool fetch_choose_and_dispatch(void)
 {
+	bool same_host;
 	struct fetch *queueitem;
 	queueitem = queue_ring;
 	do {
@@ -429,11 +425,20 @@ bool fetch_choose_and_dispatch(void)
 		 */
 		int countbyhost;
 		RING_COUNTBYLWCHOST(struct fetch, fetch_ring, countbyhost,
-				queueitem->lwc_host);
+				queueitem->host);
 		if (countbyhost < option_max_fetchers_per_host) {
 			/* We can dispatch this item in theory */
 			return fetch_dispatch_job(queueitem);
 		}
+		/* skip over other items with the same host */
+		same_host = true;
+		while (same_host == true && queueitem->r_next != queue_ring) {
+			if (lwc_string_isequal(queueitem->host,
+					queueitem->r_next->host, &same_host) ==
+					lwc_error_ok && same_host == true) {
+				queueitem = queueitem->r_next;
+			}
+		}
 		queueitem = queueitem->r_next;
 	} while (queueitem != queue_ring);
 	return false;
@@ -448,7 +453,7 @@ bool fetch_dispatch_job(struct fetch *fetch)
 	RING_REMOVE(queue_ring, fetch);
 #ifdef DEBUG_FETCH_VERBOSE
 	LOG(("Attempting to start fetch %p, fetcher %p, url %s", fetch,
-	     fetch->fetcher_handle, fetch->url));
+			fetch->fetcher_handle, nsurl_access(fetch->url)));
 #endif
 	if (!fetch->ops->start_fetch(fetch->fetcher_handle)) {
 		RING_INSERT(queue_ring, fetch); /* Put it back on the end of the queue */
@@ -469,7 +474,8 @@ void fetch_abort(struct fetch *f)
 {
 	assert(f);
 #ifdef DEBUG_FETCH_VERBOSE
-	LOG(("fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle, f->url));
+	LOG(("fetch %p, fetcher %p, url '%s'", f, f->fetcher_handle,
+			nsurl_access(f->url)));
 #endif
 	f->ops->abort_fetch(f->fetcher_handle);
 }
@@ -486,12 +492,11 @@ void fetch_free(struct fetch *f)
 #endif
 	f->ops->free_fetch(f->fetcher_handle);
 	fetch_unref_fetcher(f->ops);
-	free(f->url);
-	free(f->host);
-	if (f->referer)
-		free(f->referer);
-	if (f->lwc_host != NULL)
-                lwc_string_unref(f->lwc_host);
+	nsurl_unref(f->url);
+	if (f->referer != NULL)
+		nsurl_unref(f->referer);
+	if (f->host != NULL)
+		lwc_string_unref(f->host);
         free(f);
 }
 
@@ -512,9 +517,12 @@ void fetch_poll(void)
 	if (!fetch_active)
 		return; /* No point polling, there's no fetch active. */
 	while (fetcher != NULL) {
-		/* LOG(("Polling fetcher for %s", fetcher->scheme_name)); */
 		next_fetcher = fetcher->next_fetcher;
-		fetcher->poll_fetcher(fetcher->scheme_name);
+		if (fetcher->poll_fetcher != NULL) {
+			/* LOG(("Polling fetcher for %s",
+			   lwc_string_data(fetcher->scheme_name))); */
+			fetcher->poll_fetcher(fetcher->scheme_name);
+		}
 		fetcher = next_fetcher;
 	}
 }
@@ -527,24 +535,23 @@ void fetch_poll(void)
  * \return  true if the scheme is supported
  */
 
-bool fetch_can_fetch(const char *url)
+bool fetch_can_fetch(const nsurl *url)
 {
-	const char *semi;
-	size_t len;
 	scheme_fetcher *fetcher = fetchers;
-
-	if ((semi = strchr(url, ':')) == NULL)
-		return false;
-	len = semi - url;
+	bool match;
+	lwc_string *scheme = nsurl_get_component(url, NSURL_SCHEME);
 
 	while (fetcher != NULL) {
-		if (strlen(fetcher->scheme_name) == len &&
-		    strncmp(fetcher->scheme_name, url, len) == 0)
-			return true;
+		lwc_string_isequal(fetcher->scheme_name, scheme, &match);
+		if (match == true)
+			break;
+
 		fetcher = fetcher->next_fetcher;
 	}
 
-	return false;
+	lwc_string_unref(scheme);
+
+	return fetcher == NULL ? false : fetcher->can_fetch(url);
 }
 
 
@@ -571,20 +578,6 @@ long fetch_http_code(struct fetch *fetch)
 	return fetch->http_code;
 }
 
-
-/**
- * Get the referer
- *
- * \param fetch  fetch to retrieve referer from
- * \return Pointer to referer string, or NULL if none.
- */
-const char *fetch_get_referer(struct fetch *fetch)
-{
-	assert(fetch);
-
-	return fetch->referer;
-}
-
 /**
  * Determine if a fetch was verifiable
  *
@@ -671,12 +664,9 @@ void fetch_multipart_data_destroy(struct fetch_multipart_data *list)
 }
 
 void
-fetch_send_callback(fetch_msg msg, struct fetch *fetch, const void *data,
-		unsigned long size, fetch_error_code errorcode)
+fetch_send_callback(const fetch_msg *msg, struct fetch *fetch)
 {
-	/*LOG(("Fetcher sending callback. Fetch %p, fetcher %p data %p size %lu",
-	     fetch, fetch->fetcher_handle, data, size)); */
-	fetch->callback(msg, fetch->p, data, size, errorcode);
+	fetch->callback(msg, fetch->p);
 }
 
 
@@ -716,11 +706,10 @@ fetch_set_http_code(struct fetch *fetch, long http_code)
 	fetch->http_code = http_code;
 }
 
-const char *
-fetch_get_referer_to_send(struct fetch *fetch)
+const char *fetch_get_referer_to_send(struct fetch *fetch)
 {
 	if (fetch->send_referer)
-		return fetch->referer;
+		return nsurl_access(fetch->referer);
 	return NULL;
 }
 
@@ -736,7 +725,7 @@ fetch_set_cookie(struct fetch *fetch, const char *data)
 		/* If the transaction's verifiable, we don't require
 		 * that the request uri and the parent domain match,
 		 * so don't pass in any referer/parent in this case. */
-		urldb_set_cookie(data, fetch->url, NULL);
+		urldb_set_cookie(data, nsurl_access(fetch->url), NULL);
 	} else if (fetch->referer != NULL) {
 		/* Permit the cookie to be set if the fetch is unverifiable
 		 * and the fetch URI domain matches the referer. */
@@ -745,7 +734,8 @@ fetch_set_cookie(struct fetch *fetch, const char *data)
 		 * where a nested object requests a fetch, the origin URI
 		 * is the nested object's parent URI, whereas the referer
 		 * for the fetch will be the nested object's URI. */
-		urldb_set_cookie(data, fetch->url, fetch->referer);
+		urldb_set_cookie(data, nsurl_access(fetch->url),
+				nsurl_access(fetch->referer));
 	}
 }
 
