@@ -39,6 +39,8 @@
 #include <limits.h>
 #include <stdarg.h>
 
+#include <libwapcaplet/libwapcaplet.h>
+
 #include "utils/config.h"
 #include "content/dirlist.h"
 #include "content/fetch.h"
@@ -52,6 +54,7 @@
 #include "utils/utils.h"
 #include "utils/ring.h"
 #include "utils/testament.h"
+#include "image/image_cache.h"
 
 struct fetch_about_context;
 
@@ -66,7 +69,7 @@ struct fetch_about_context {
 	bool aborted; /**< Flag indicating fetch has been aborted */
 	bool locked; /**< Flag indicating entry is already entered */
 
-	char *url; /**< The full url the fetch refers to */
+	nsurl *url; /**< The full url the fetch refers to */
 
 	fetch_about_handler handler;
 };
@@ -74,12 +77,11 @@ struct fetch_about_context {
 static struct fetch_about_context *ring = NULL;
 
 /** issue fetch callbacks with locking */
-static inline bool fetch_about_send_callback(fetch_msg msg,
-		struct fetch_about_context *ctx, const void *data,
-		unsigned long size, fetch_error_code errorcode)
+static inline bool fetch_about_send_callback(const fetch_msg *msg,
+		struct fetch_about_context *ctx)
 {
 	ctx->locked = true;
-	fetch_send_callback(msg, ctx->fetchh, data, size, errorcode);
+	fetch_send_callback(msg, ctx->fetchh);
 	ctx->locked = false;
 
 	return ctx->aborted;
@@ -89,6 +91,7 @@ static bool fetch_about_send_header(struct fetch_about_context *ctx,
 		const char *fmt, ...)
 {
 	char header[64];
+	fetch_msg msg;
 	va_list ap;
 
 	va_start(ap, fmt);
@@ -97,8 +100,11 @@ static bool fetch_about_send_header(struct fetch_about_context *ctx,
 
 	va_end(ap);
 
-	fetch_about_send_callback(FETCH_HEADER, ctx, header, strlen(header),
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_HEADER;
+	msg.data.header_or_data.buf = (const uint8_t *) header;
+	msg.data.header_or_data.len = strlen(header);
+
+	fetch_about_send_callback(&msg, ctx);
 
 	return ctx->aborted;
 }
@@ -108,24 +114,26 @@ static bool fetch_about_send_header(struct fetch_about_context *ctx,
 
 static bool fetch_about_blank_handler(struct fetch_about_context *ctx)
 {
-	char buffer[2];
-	int code = 200;
+	fetch_msg msg;
+	const char buffer[2] = { ' ', '\0' };
 
 	/* content is going to return ok */
-	fetch_set_http_code(ctx->fetchh, code);
+	fetch_set_http_code(ctx->fetchh, 200);
 
 	/* content type */
 	if (fetch_about_send_header(ctx, "Content-Type: text/html"))
 		goto fetch_about_blank_handler_aborted;
 
-	buffer[0] = ' ';
-	buffer[1] = 0;
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, strlen(buffer),
-			FETCH_ERROR_NO_ERROR))
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+	msg.data.header_or_data.len = strlen(buffer);
+
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_blank_handler_aborted;
 
-	fetch_about_send_callback(FETCH_FINISHED, ctx, 0, 0,
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_FINISHED;
+
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 
@@ -136,11 +144,15 @@ fetch_about_blank_handler_aborted:
 
 static bool fetch_about_credits_handler(struct fetch_about_context *ctx)
 {
+	fetch_msg msg;
+
 	/* content is going to return redirect */
 	fetch_set_http_code(ctx->fetchh, 302);
 
-	fetch_about_send_callback(FETCH_REDIRECT, ctx, "resource:credits.html",
-			0, FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_REDIRECT;
+	msg.data.redirect = "resource:credits.html";
+
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 }
@@ -148,18 +160,134 @@ static bool fetch_about_credits_handler(struct fetch_about_context *ctx)
 
 static bool fetch_about_licence_handler(struct fetch_about_context *ctx)
 {
+	fetch_msg msg;
+
 	/* content is going to return redirect */
 	fetch_set_http_code(ctx->fetchh, 302);
 
-	fetch_about_send_callback(FETCH_REDIRECT, ctx, "resource:licence.html",
-			0, FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_REDIRECT;
+	msg.data.redirect = "resource:licence.html";
+
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 }
 
+/** Handler to generate about:cache page.
+ *
+ * Shows details of current iamge cache
+ *
+ */
+static bool fetch_about_imagecache_handler(struct fetch_about_context *ctx)
+{
+	fetch_msg msg;
+	char buffer[1024]; /* output buffer */
+	int code = 200;
+	int slen;
+	unsigned int cent_loop = 0;
+	int res = 0;
 
+	/* content is going to return ok */
+	fetch_set_http_code(ctx->fetchh, code);
+
+	/* content type */
+	if (fetch_about_send_header(ctx, "Content-Type: text/html"))
+		goto fetch_about_imagecache_handler_aborted;
+
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+
+	/* page head */
+	slen = snprintf(buffer, sizeof buffer, 
+			"<html>\n<head>\n"
+			"<title>NetSurf Browser Image Cache Status</title>\n"
+			"<link rel=\"stylesheet\" type=\"text/css\" "
+			"href=\"resource:internal.css\">\n"
+			"</head>\n"
+			"<body id =\"cachelist\">\n"
+			"<p class=\"banner\">"
+			"<a href=\"http://www.netsurf-browser.org/\">"
+			"<img src=\"resource:netsurf.png\" alt=\"NetSurf\"></a>"
+			"</p>\n"
+			"<h1>NetSurf Browser Image Cache Status</h1>\n"	);
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
+		goto fetch_about_imagecache_handler_aborted;
+
+	/* image cache summary */
+	slen = image_cache_snsummaryf(buffer, sizeof(buffer), 
+		"<p>Configured limit of %a hysteresis of %b"
+		"<p>Total bitmap size in use %c (in %d)"
+		"<p>Age %es"
+		"<p>Peak size %f (in %g)"
+		"<p>Peak image count %h (size %i)"
+		"<p>Cache total/hit/miss/fail (counts) %j/%k/%l/%m "
+				"(%pj%%/%pk%%/%pl%%/%pm%%)"
+		"<p>Cache total/hit/miss/fail (size) %n/%o/%q/%r "
+				"(%pn%%/%po%%/%pq%%/%pr%%)" 
+		"<p>Total images never rendered: %s "
+				"(includes %t that were converted)"
+		"<p>Total number of excessive conversions: %u "
+				"(from %v images converted more than once)"
+		"<p>Bitmap of size %w had most (%x) conversions");
+	if (slen >= (int) (sizeof(buffer))) 
+		goto fetch_about_imagecache_handler_aborted; /* overflow */
+
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
+		goto fetch_about_imagecache_handler_aborted;
+
+
+	/* image cache entry table */
+	slen = snprintf(buffer, sizeof buffer, 
+			"<table class=\"config\">\n"
+			"<tr><th>Entry</th>"
+			"<th>Content Key</th>"
+			"<th>Redraw Count</th>"
+			"<th>Conversion Count</th>"
+			"<th>Last Redraw</th>"
+			"<th>Bitmap Age</th>"
+			"<th>Bitmap Size</th></tr>\n");
+	do {
+		res = image_cache_snentryf(buffer + slen, sizeof buffer - slen,
+				cent_loop,
+				"<tr><td>%e</td><td>%k</td><td>%r</td><td>%c</td><td>%a</td><td>%g</td><td>%s</td></tr>\n");
+		if (res <= 0) 
+			break; /* last option */
+
+		if (res >= (int) (sizeof buffer - slen)) {
+			/* last entry would not fit in buffer, submit buffer */
+			msg.data.header_or_data.len = slen;
+			if (fetch_about_send_callback(&msg, ctx))
+				goto fetch_about_imagecache_handler_aborted;
+			slen = 0;
+		} else {
+			/* normal addition */
+			slen += res;
+			cent_loop++;
+		}
+	} while (res > 0);
+
+	slen += snprintf(buffer + slen, sizeof buffer - slen, 
+			 "</table>\n</body>\n</html>\n");
+
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
+		goto fetch_about_imagecache_handler_aborted;
+
+	msg.type = FETCH_FINISHED;
+	fetch_about_send_callback(&msg, ctx);
+
+	return true;
+
+fetch_about_imagecache_handler_aborted:
+	return false;
+}
+
+/** Handler to generate about:config page */
 static bool fetch_about_config_handler(struct fetch_about_context *ctx)
 {
+	fetch_msg msg;
 	char buffer[1024];
 	int code = 200;
 	int slen;
@@ -173,6 +301,9 @@ static bool fetch_about_config_handler(struct fetch_about_context *ctx)
 	if (fetch_about_send_header(ctx, "Content-Type: text/html"))
 		goto fetch_about_config_handler_aborted;
 
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+
 	slen = snprintf(buffer, sizeof buffer, 
 			"<html>\n<head>\n"
 			"<title>NetSurf Browser Config</title>\n"
@@ -197,8 +328,8 @@ static bool fetch_about_config_handler(struct fetch_about_context *ctx)
 
 		if (res >= (int) (sizeof buffer - slen)) {
 			/* last entry would not fit in buffer, submit buffer */
-			if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, 
-					slen, FETCH_ERROR_NO_ERROR))
+			msg.data.header_or_data.len = slen;
+			if (fetch_about_send_callback(&msg, ctx))
 				goto fetch_about_config_handler_aborted;
 			slen = 0;
 		} else {
@@ -211,12 +342,12 @@ static bool fetch_about_config_handler(struct fetch_about_context *ctx)
 	slen += snprintf(buffer + slen, sizeof buffer - slen, 
 			 "</table>\n</body>\n</html>\n");
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_config_handler_aborted;
 
-	fetch_about_send_callback(FETCH_FINISHED, ctx, 0, 0,
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_FINISHED;
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 
@@ -224,11 +355,13 @@ fetch_about_config_handler_aborted:
 	return false;
 }
 
+
 /** Generate the text of a Choices file which represents the current
  * in use options. 
  */
 static bool fetch_about_choices_handler(struct fetch_about_context *ctx)
 {
+	fetch_msg msg;
 	char buffer[1024];
 	int code = 200;
 	int slen;
@@ -242,6 +375,9 @@ static bool fetch_about_choices_handler(struct fetch_about_context *ctx)
 	if (fetch_about_send_header(ctx, "Content-Type: text/plain"))
 		goto fetch_about_choices_handler_aborted;
 
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+
 	slen = snprintf(buffer, sizeof buffer, 
 		 "# Automatically generated current NetSurf browser Choices\n");
 
@@ -255,8 +391,8 @@ static bool fetch_about_choices_handler(struct fetch_about_context *ctx)
 
 		if (res >= (int) (sizeof buffer - slen)) {
 			/* last entry would not fit in buffer, submit buffer */
-			if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, 
-					slen, FETCH_ERROR_NO_ERROR))
+			msg.data.header_or_data.len = slen;
+			if (fetch_about_send_callback(&msg, ctx))
 				goto fetch_about_choices_handler_aborted;
 			slen = 0;
 		} else {
@@ -266,12 +402,12 @@ static bool fetch_about_choices_handler(struct fetch_about_context *ctx)
 		}
 	} while (res > 0);
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_choices_handler_aborted;
 
-	fetch_about_send_callback(FETCH_FINISHED, ctx, 0, 0,
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_FINISHED;
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 
@@ -286,6 +422,7 @@ typedef struct { const char *leaf; const char modtype; } modification_t;
 static bool fetch_about_testament_handler(struct fetch_about_context *ctx)
 {
 	static modification_t modifications[] = WT_MODIFICATIONS;
+	fetch_msg msg;
 	char buffer[1024];
 	int code = 200;
 	int slen;
@@ -299,11 +436,14 @@ static bool fetch_about_testament_handler(struct fetch_about_context *ctx)
 	if (fetch_about_send_header(ctx, "Content-Type: text/plain"))
 		goto fetch_about_testament_handler_aborted;
 
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+
 	slen = snprintf(buffer, sizeof buffer, 
 		 "# Automatically generated by NetSurf build system\n\n");
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_testament_handler_aborted;
 	
 	slen = snprintf(buffer, sizeof buffer, 
@@ -319,9 +459,9 @@ static bool fetch_about_testament_handler(struct fetch_about_context *ctx)
 			"# This NetSurf was built from a branch.\n\n"
 #endif
 			);
-	
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+
+	msg.data.header_or_data.len = slen;	
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_testament_handler_aborted;
 
 	
@@ -329,16 +469,16 @@ static bool fetch_about_testament_handler(struct fetch_about_context *ctx)
 			"Built by %s (%s) from %s at revision %s\n\n",
 			GECOS, USERNAME, WT_BRANCHPATH, WT_REVID);
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_testament_handler_aborted;
 	
 	slen = snprintf(buffer, sizeof buffer, 
 			"Built on %s in %s\n\n",
 			WT_HOSTNAME, WT_ROOT);
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_testament_handler_aborted;
 	
 	if (WT_MODIFIED > 0) {
@@ -350,8 +490,8 @@ static bool fetch_about_testament_handler(struct fetch_about_context *ctx)
 				"Working tree is not modified.\n");
 	}
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_testament_handler_aborted;
 	
 	for (i = 0; i < WT_MODIFIED; ++i) {
@@ -359,14 +499,14 @@ static bool fetch_about_testament_handler(struct fetch_about_context *ctx)
 				"  %c  %s\n",
 				modifications[i].modtype,
 				modifications[i].leaf);
-		if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-					      FETCH_ERROR_NO_ERROR))
+		msg.data.header_or_data.len = slen;
+		if (fetch_about_send_callback(&msg, ctx))
 			goto fetch_about_testament_handler_aborted;
 		
 	}
-	
-	fetch_about_send_callback(FETCH_FINISHED, ctx, 0, 0,
-			FETCH_ERROR_NO_ERROR);
+
+	msg.type = FETCH_FINISHED;	
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 
@@ -376,11 +516,15 @@ fetch_about_testament_handler_aborted:
 
 static bool fetch_about_logo_handler(struct fetch_about_context *ctx)
 {
+	fetch_msg msg;
+
 	/* content is going to return redirect */
 	fetch_set_http_code(ctx->fetchh, 302);
 
-	fetch_about_send_callback(FETCH_REDIRECT, ctx, "resource:netsurf.png",
-			0, FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_REDIRECT;
+	msg.data.redirect = "resource:netsurf.png";
+
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 }
@@ -390,20 +534,26 @@ static bool fetch_about_about_handler(struct fetch_about_context *ctx);
 
 struct about_handlers {
 	const char *name; /**< name to match in url */
+	int name_len;
+	lwc_string *lname; /**< Interned name */
 	fetch_about_handler handler; /* handler for the url */
 	bool hidden; /* Flag indicating if entry should be show in listing */
 };
 
+/** List of about paths and their handlers */
 struct about_handlers about_handler_list[] = { 
-	{ "credits", fetch_about_credits_handler, false },
-	{ "licence", fetch_about_licence_handler, false },
-	{ "license", fetch_about_licence_handler, true },
-	{ "config", fetch_about_config_handler, false },
-	{ "Choices", fetch_about_choices_handler, false },
-	{ "testament", fetch_about_testament_handler, false },
-	{ "about", fetch_about_about_handler, true },
-	{ "logo", fetch_about_logo_handler, true },
-	{ "blank", fetch_about_blank_handler, true } /* The default */
+	{ "credits", SLEN("credits"), NULL, fetch_about_credits_handler, false },
+	{ "licence", SLEN("licence"), NULL, fetch_about_licence_handler, false },
+	{ "license", SLEN("license"), NULL, fetch_about_licence_handler, true },
+	{ "config", SLEN("config"), NULL, fetch_about_config_handler, false },
+	{ "Choices", SLEN("Choices"), NULL, fetch_about_choices_handler, false },
+	{ "testament", SLEN("testament"), NULL, fetch_about_testament_handler, false },
+	{ "about", SLEN("about"), NULL, fetch_about_about_handler, true },
+	{ "logo", SLEN("logo"), NULL, fetch_about_logo_handler, true },
+	/* details about the cache */
+	{ "imagecache", SLEN("iamgecache"), NULL, fetch_about_imagecache_handler, true },
+	/* The default blank page */
+	{ "blank", SLEN("blank"), NULL, fetch_about_blank_handler, true } 
 };
 
 #define about_handler_list_len (sizeof(about_handler_list) / sizeof(struct about_handlers))
@@ -416,6 +566,7 @@ struct about_handlers about_handler_list[] = {
  */
 static bool fetch_about_about_handler(struct fetch_about_context *ctx)
 {
+	fetch_msg msg;
 	char buffer[1024];
 	int code = 200;
 	int slen;
@@ -429,6 +580,9 @@ static bool fetch_about_about_handler(struct fetch_about_context *ctx)
 	if (fetch_about_send_header(ctx, "Content-Type: text/html"))
 		goto fetch_about_config_handler_aborted;
 
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+
 	slen = snprintf(buffer, sizeof buffer, 
 			"<html>\n<head>\n"
 			"<title>NetSurf List of About pages</title>\n"
@@ -458,8 +612,8 @@ static bool fetch_about_about_handler(struct fetch_about_context *ctx)
 
 		if (res >= (int)(sizeof buffer - slen)) {
 			/* last entry would not fit in buffer, submit buffer */
-			if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, 
-					slen, FETCH_ERROR_NO_ERROR))
+			msg.data.header_or_data.len = slen;
+			if (fetch_about_send_callback(&msg, ctx))
 				goto fetch_about_config_handler_aborted;
 			slen = 0;
 		} else {
@@ -471,12 +625,12 @@ static bool fetch_about_about_handler(struct fetch_about_context *ctx)
 	slen += snprintf(buffer + slen, sizeof buffer - slen, 
 			 "</ul>\n</body>\n</html>\n");
 
-	if (fetch_about_send_callback(FETCH_DATA, ctx, buffer, slen,
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = slen;
+	if (fetch_about_send_callback(&msg, ctx))
 		goto fetch_about_config_handler_aborted;
 
-	fetch_about_send_callback(FETCH_FINISHED, ctx, 0, 0,
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_FINISHED;
+	fetch_about_send_callback(&msg, ctx);
 
 	return true;
 
@@ -486,20 +640,44 @@ fetch_about_config_handler_aborted:
 
 
 /** callback to initialise the about fetcher. */
-static bool fetch_about_initialise(const char *scheme)
+static bool fetch_about_initialise(lwc_string *scheme)
 {
+	unsigned int abt_loop = 0;
+	lwc_error error;
+
+	for (abt_loop = 0; abt_loop < about_handler_list_len; abt_loop++) {
+		error = lwc_intern_string(about_handler_list[abt_loop].name, 
+					about_handler_list[abt_loop].name_len, 
+					&about_handler_list[abt_loop].lname);
+		if (error != lwc_error_ok) {
+			while (abt_loop-- != 0) {
+				lwc_string_unref(about_handler_list[abt_loop].lname);
+			}
+			return false;
+		}
+	}
+
 	return true;
 }
 
-/** callback to initialise the about fetcher. */
-static void fetch_about_finalise(const char *scheme)
+/** callback to finalise the about fetcher. */
+static void fetch_about_finalise(lwc_string *scheme)
 {
+	unsigned int abt_loop = 0;
+	for (abt_loop = 0; abt_loop < about_handler_list_len; abt_loop++) {
+		lwc_string_unref(about_handler_list[abt_loop].lname);
+	}
+}
+
+static bool fetch_about_can_fetch(const nsurl *url)
+{
+	return true;
 }
 
 /** callback to set up a about fetch context. */
 static void *
 fetch_about_setup(struct fetch *fetchh,
-		 const char *url,
+		 nsurl *url,
 		 bool only_2xx,
 		 const char *post_urlenc,
 		 const struct fetch_multipart_data *post_multipart,
@@ -507,25 +685,31 @@ fetch_about_setup(struct fetch *fetchh,
 {
 	struct fetch_about_context *ctx;
 	unsigned int handler_loop;
-	struct url_components urlcomp;
+	lwc_string *path;
+	bool match;
 
 	ctx = calloc(1, sizeof(*ctx));
 	if (ctx == NULL)
 		return NULL;
 
-	url_get_components(url, &urlcomp);
+	path = nsurl_get_component(url, NSURL_PATH);
 
 	for (handler_loop = 0; 
 	     handler_loop < about_handler_list_len; 
 	     handler_loop++) {
 		ctx->handler = about_handler_list[handler_loop].handler;
-		if (strcmp(about_handler_list[handler_loop].name, urlcomp.path) == 0)
+		if (lwc_string_isequal(path, 
+				       about_handler_list[handler_loop].lname, 
+				       &match) == lwc_error_ok && match) {
 			break;
+		}		
 	}
 
-	url_destroy_components(&urlcomp);
+	if (path != NULL)
+		lwc_string_unref(path);
 
 	ctx->fetchh = fetchh;
+	ctx->url = nsurl_ref(url);
 
 	RING_INSERT(ring, ctx);
 
@@ -536,7 +720,7 @@ fetch_about_setup(struct fetch *fetchh,
 static void fetch_about_free(void *ctx)
 {
 	struct fetch_about_context *c = ctx;
-	free(c->url);
+	nsurl_unref(c->url);
 	RING_REMOVE(ring, c);
 	free(ctx);
 }
@@ -561,7 +745,7 @@ static void fetch_about_abort(void *ctx)
 
 
 /** callback to poll for additional about fetch contents */
-static void fetch_about_poll(const char *scheme)
+static void fetch_about_poll(lwc_string *scheme)
 {
 	struct fetch_about_context *c, *next;
 
@@ -603,8 +787,17 @@ static void fetch_about_poll(const char *scheme)
 
 void fetch_about_register(void)
 {
-	fetch_add_fetcher("about",
+	lwc_string *scheme;
+
+	if (lwc_intern_string("about", SLEN("about"),
+			&scheme) != lwc_error_ok) {
+		die("Failed to initialise the fetch module "
+				"(couldn't intern \"about\").");
+	}
+
+	fetch_add_fetcher(scheme,
 		fetch_about_initialise,
+		fetch_about_can_fetch,
 		fetch_about_setup,
 		fetch_about_start,
 		fetch_about_abort,
