@@ -37,6 +37,8 @@
 #include <time.h>
 #include <sys/stat.h>
 
+#include <libwapcaplet/libwapcaplet.h>
+
 #include "utils/config.h"
 #include <openssl/ssl.h>
 #include "content/fetch.h"
@@ -58,6 +60,10 @@
  */
 #include <desktop/browser.h>
 
+/* uncomment this to use scheduler based calling
+#define FETCHER_CURLL_SCHEDULED 1
+*/
+
 /** SSL certificate info */
 struct cert_info {
 	X509 *cert;		/**< Pointer to certificate */
@@ -72,8 +78,8 @@ struct curl_fetch_info {
 	bool abort;		/**< Abort requested. */
 	bool stopped;		/**< Download stopped on purpose. */
 	bool only_2xx;		/**< Only HTTP 2xx responses acceptable. */
-	char *url;		/**< URL of this fetch. */
-	char *host;		/**< The hostname of this fetch. */
+	nsurl *url;		/**< URL of this fetch. */
+	lwc_string *host;	/**< The hostname of this fetch. */
 	struct curl_slist *headers;	/**< List of request headers. */
 	char *location;		/**< Response Location header, or 0. */
 	unsigned long content_length;	/**< Response Content-Length, or 0. */
@@ -89,7 +95,7 @@ struct curl_fetch_info {
 
 struct cache_handle {
 	CURL *handle; /**< The cached cURL handle */
-	char *host;	   /**< The host for which this handle is cached */
+	lwc_string *host;   /**< The host for which this handle is cached */
 
 	struct cache_handle *r_prev; /**< Previous cached handle in ring. */
 	struct cache_handle *r_next; /**< Next cached handle in ring. */
@@ -105,24 +111,25 @@ static bool curl_with_openssl;
 static char fetch_error_buffer[CURL_ERROR_SIZE]; /**< Error buffer for cURL. */
 static char fetch_proxy_userpwd[100];	/**< Proxy authentication details. */
 
-static bool fetch_curl_initialise(const char *scheme);
-static void fetch_curl_finalise(const char *scheme);
-static void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
+static bool fetch_curl_initialise(lwc_string *scheme);
+static void fetch_curl_finalise(lwc_string *scheme);
+static bool fetch_curl_can_fetch(const nsurl *url);
+static void * fetch_curl_setup(struct fetch *parent_fetch, nsurl *url,
 		 bool only_2xx, const char *post_urlenc,
 		 const struct fetch_multipart_data *post_multipart,
 		 const char **headers);
 static bool fetch_curl_start(void *vfetch);
 static bool fetch_curl_initiate_fetch(struct curl_fetch_info *fetch,
 		CURL *handle);
-static CURL *fetch_curl_get_handle(char *host);
-static void fetch_curl_cache_handle(CURL *handle, char *hostname);
+static CURL *fetch_curl_get_handle(lwc_string *host);
+static void fetch_curl_cache_handle(CURL *handle, lwc_string *host);
 static CURLcode fetch_curl_set_options(struct curl_fetch_info *f);
 static CURLcode fetch_curl_sslctxfun(CURL *curl_handle, void *_sslctx,
 				     void *p);
 static void fetch_curl_abort(void *vf);
 static void fetch_curl_stop(struct curl_fetch_info *f);
 static void fetch_curl_free(void *f);
-static void fetch_curl_poll(const char *scheme_ignored);
+static void fetch_curl_poll(lwc_string *scheme_ignored);
 static void fetch_curl_done(CURL *curl_handle, CURLcode result);
 static int fetch_curl_progress(void *clientp, double dltotal, double dlnow,
 		double ultotal, double ulnow);
@@ -155,6 +162,7 @@ void fetch_curl_register(void)
 	CURLcode code;
 	curl_version_info_data *data;
 	int i;
+	lwc_string *scheme;
 
 	LOG(("curl_version %s", curl_version()));
 
@@ -224,19 +232,38 @@ void fetch_curl_register(void)
 	data = curl_version_info(CURLVERSION_NOW);
 
 	for (i = 0; data->protocols[i]; i++) {
-		/* Ignore non-http(s) protocols */
-		if (strcmp(data->protocols[i], "http") != 0 &&
-				strcmp(data->protocols[i], "https") != 0)
-			continue;
+		if (strcmp(data->protocols[i], "http") == 0) {
+			if (lwc_intern_string("http", SLEN("http"),
+					&scheme) != lwc_error_ok) {
+				die("Failed to initialise the fetch module "
+						"(couldn't intern \"http\").");
+			}
 
-		if (!fetch_add_fetcher(data->protocols[i],
-				       fetch_curl_initialise,
-				       fetch_curl_setup,
-				       fetch_curl_start,
-				       fetch_curl_abort,
-				       fetch_curl_free,
-				       fetch_curl_poll,
-				       fetch_curl_finalise)) {
+		} else if (strcmp(data->protocols[i], "https") == 0) {
+			if (lwc_intern_string("https", SLEN("https"),
+					&scheme) != lwc_error_ok) {
+				die("Failed to initialise the fetch module "
+						"(couldn't intern \"https\").");
+			}
+
+		} else {
+			/* Ignore non-http(s) protocols */
+			continue;
+		}
+
+		if (!fetch_add_fetcher(scheme,
+				fetch_curl_initialise,
+				fetch_curl_can_fetch,
+				fetch_curl_setup,
+				fetch_curl_start,
+				fetch_curl_abort,
+				fetch_curl_free,
+#ifdef FETCHER_CURLL_SCHEDULED
+				       NULL,
+#else
+				fetch_curl_poll,
+#endif
+				fetch_curl_finalise)) {
 			LOG(("Unable to register cURL fetcher for %s",
 					data->protocols[i]));
 		}
@@ -253,9 +280,9 @@ curl_easy_setopt_failed:
  * Initialise a cURL fetcher.
  */
 
-bool fetch_curl_initialise(const char *scheme)
+bool fetch_curl_initialise(lwc_string *scheme)
 {
-	LOG(("Initialise cURL fetcher for %s", scheme));
+	LOG(("Initialise cURL fetcher for %s", lwc_string_data(scheme)));
 	curl_fetchers_registered++;
 	return true; /* Always succeeds */
 }
@@ -265,10 +292,12 @@ bool fetch_curl_initialise(const char *scheme)
  * Finalise a cURL fetcher
  */
 
-void fetch_curl_finalise(const char *scheme)
+void fetch_curl_finalise(lwc_string *scheme)
 {
+	struct cache_handle *h;
+
 	curl_fetchers_registered--;
-	LOG(("Finalise cURL fetcher %s", scheme));
+	LOG(("Finalise cURL fetcher %s", lwc_string_data(scheme)));
 	if (curl_fetchers_registered == 0) {
 		CURLMcode codem;
 		/* All the fetchers have been finalised. */
@@ -282,8 +311,19 @@ void fetch_curl_finalise(const char *scheme)
 
 		curl_global_cleanup();
 	}
+
+	/* Free anything remaining in the cached curl handle ring */
+	while (curl_handle_ring != NULL) {
+		h = curl_handle_ring;
+		RING_REMOVE(curl_handle_ring, h);
+		lwc_string_unref(h->host);
+	}
 }
 
+bool fetch_curl_can_fetch(const nsurl *url)
+{
+	return nsurl_has_component(url, NSURL_HOST);
+}
 
 /**
  * Start fetching data for the given URL.
@@ -307,68 +347,54 @@ void fetch_curl_finalise(const char *scheme)
  * callbacks will contain this.
  */
 
-void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
+void * fetch_curl_setup(struct fetch *parent_fetch, nsurl *url,
 		 bool only_2xx, const char *post_urlenc,
 		 const struct fetch_multipart_data *post_multipart,
 		 const char **headers)
 {
-	char *host;
 	struct curl_fetch_info *fetch;
 	struct curl_slist *slist;
-	url_func_result res;
 	int i;
 
 	fetch = malloc(sizeof (*fetch));
-	if (!fetch)
+	if (fetch == NULL)
 		return 0;
 
 	fetch->fetch_handle = parent_fetch;
 
-	res = url_host(url, &host);
-	if (res != URL_FUNC_OK) {
-		/* we only fail memory exhaustion */
-		if (res == URL_FUNC_NOMEM)
-			goto failed;
-
-		host = strdup("");
-		if (!host)
-			goto failed;
-	}
-
-	LOG(("fetch %p, url '%s'", fetch, url));
+	LOG(("fetch %p, url '%s'", fetch, nsurl_access(url)));
 
 	/* construct a new fetch structure */
-	fetch->curl_handle = 0;
+	fetch->curl_handle = NULL;
 	fetch->had_headers = false;
 	fetch->abort = false;
 	fetch->stopped = false;
 	fetch->only_2xx = only_2xx;
-	fetch->url = strdup(url);
-	fetch->headers = 0;
-	fetch->host = host;
-	fetch->location = 0;
+	fetch->headers = NULL;
+	fetch->url = nsurl_ref(url);
+	fetch->host = nsurl_get_component(url, NSURL_HOST);
+	fetch->location = NULL;
 	fetch->content_length = 0;
 	fetch->http_code = 0;
-	fetch->cookie_string = 0;
-	fetch->realm = 0;
-	fetch->post_urlenc = 0;
-	fetch->post_multipart = 0;
+	fetch->cookie_string = NULL;
+	fetch->realm = NULL;
+	fetch->post_urlenc = NULL;
+	fetch->post_multipart = NULL;
 	if (post_urlenc)
 		fetch->post_urlenc = strdup(post_urlenc);
 	else if (post_multipart)
 		fetch->post_multipart = fetch_curl_post_convert(post_multipart);
-	fetch->http_code = 0;
 	memset(fetch->cert_data, 0, sizeof(fetch->cert_data));
 	fetch->last_progress_update = 0;
 
-	if (!fetch->url ||
-	    (post_urlenc && !fetch->post_urlenc) ||
-	    (post_multipart && !fetch->post_multipart))
+	if (fetch->host == NULL ||
+		(post_multipart != NULL && fetch->post_multipart == NULL) ||
+			(post_urlenc != NULL && fetch->post_urlenc == NULL))
 		goto failed;
 
 #define APPEND(list, value) \
 	slist = curl_slist_append(list, value);		\
-	if (!slist)					\
+	if (slist == NULL)				\
 		goto failed;				\
 	list = slist;
 
@@ -379,7 +405,8 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 	 * which fails with lighttpd, so disable it (see bug 1429054) */
 	APPEND(fetch->headers, "Expect:");
 
-	if (option_accept_language && option_accept_language[0] != '\0') {
+	if (option_accept_language != NULL && 
+			option_accept_language[0] != '\0') {
 		char s[80];
 		snprintf(s, sizeof s, "Accept-Language: %s, *;q=0.1",
 				option_accept_language);
@@ -387,7 +414,8 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 		APPEND(fetch->headers, s);
 	}
 
-	if (option_accept_charset && option_accept_charset[0] != '\0') {
+	if (option_accept_charset != NULL && 
+			option_accept_charset[0] != '\0') {
 		char s[80];
 		snprintf(s, sizeof s, "Accept-Charset: %s, *;q=0.1",
 				option_accept_charset);
@@ -396,21 +424,23 @@ void * fetch_curl_setup(struct fetch *parent_fetch, const char *url,
 	}
 
 	/* And add any headers specified by the caller */
-	for (i = 0; headers[i]; i++) {
+	for (i = 0; headers[i] != NULL; i++) {
 		APPEND(fetch->headers, headers[i]);
 	}
 
 	return fetch;
 
 failed:
-	free(host);
-	free(fetch->url);
+	if (fetch->host != NULL)
+		lwc_string_unref(fetch->host);
+
+	nsurl_unref(fetch->url);
 	free(fetch->post_urlenc);
 	if (fetch->post_multipart)
 		curl_formfree(fetch->post_multipart);
 	curl_slist_free_all(fetch->headers);
 	free(fetch);
-	return 0;
+	return NULL;
 }
 
 
@@ -462,14 +492,14 @@ bool fetch_curl_initiate_fetch(struct curl_fetch_info *fetch, CURL *handle)
  * Find a CURL handle to use to dispatch a job
  */
 
-CURL *fetch_curl_get_handle(char *host)
+CURL *fetch_curl_get_handle(lwc_string *host)
 {
 	struct cache_handle *h;
 	CURL *ret;
-	RING_FINDBYHOST(curl_handle_ring, h, host);
+	RING_FINDBYLWCHOST(curl_handle_ring, h, host);
 	if (h) {
 		ret = h->handle;
-		free(h->host);
+		lwc_string_unref(h->host);
 		RING_REMOVE(curl_handle_ring, h);
 		free(h);
 	} else {
@@ -483,11 +513,11 @@ CURL *fetch_curl_get_handle(char *host)
  * Cache a CURL handle for the provided host (if wanted)
  */
 
-void fetch_curl_cache_handle(CURL *handle, char *host)
+void fetch_curl_cache_handle(CURL *handle, lwc_string *host)
 {
 	struct cache_handle *h = 0;
 	int c;
-	RING_FINDBYHOST(curl_handle_ring, h, host);
+	RING_FINDBYLWCHOST(curl_handle_ring, h, host);
 	if (h) {
 		/* Already have a handle cached for this hostname */
 		curl_easy_cleanup(handle);
@@ -506,8 +536,8 @@ void fetch_curl_cache_handle(CURL *handle, char *host)
 			curl_handle_ring = h->r_next;
 			curl_easy_cleanup(h->handle);
 			h->handle = handle;
-			free(h->host);
-			h->host = strdup(host);
+			lwc_string_unref(h->host);
+			h->host = lwc_string_ref(host);
 		} else {
 			/* Actually, we don't want to cache any handles */
 			curl_easy_cleanup(handle);
@@ -518,7 +548,7 @@ void fetch_curl_cache_handle(CURL *handle, char *host)
 	/* The table isn't full yet, so make a shiny new handle to add to the ring */
 	h = (struct cache_handle*)malloc(sizeof(struct cache_handle));
 	h->handle = handle;
-	h->host = strdup(host);
+	h->host = lwc_string_ref(host);
 	RING_INSERT(curl_handle_ring, h);
 }
 
@@ -540,7 +570,7 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 		return code;					\
 	}
 
-	SETOPT(CURLOPT_URL, f->url);
+	SETOPT(CURLOPT_URL, nsurl_access(f->url));
 	SETOPT(CURLOPT_PRIVATE, f);
 	SETOPT(CURLOPT_WRITEDATA, f);
 	SETOPT(CURLOPT_WRITEHEADER, f);
@@ -561,14 +591,14 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 		SETOPT(CURLOPT_HTTPGET, 1L);
 	}
 
-	f->cookie_string = urldb_get_cookie(f->url);
+	f->cookie_string = urldb_get_cookie(nsurl_access(f->url));
 	if (f->cookie_string) {
 		SETOPT(CURLOPT_COOKIE, f->cookie_string);
 	} else {
 		SETOPT(CURLOPT_COOKIE, NULL);
 	}
 
-	if ((auth = urldb_get_auth_details(f->url, NULL)) != NULL) {
+	if ((auth = urldb_get_auth_details(nsurl_access(f->url), NULL)) != NULL) {
 		SETOPT(CURLOPT_HTTPAUTH, CURLAUTH_ANY);
 		SETOPT(CURLOPT_USERPWD, auth);
 	} else {
@@ -576,7 +606,7 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 	}
 
 	if (option_http_proxy && option_http_proxy_host &&
-			strncmp(f->url, "file:", 5) != 0) {
+			strncmp(nsurl_access(f->url), "file:", 5) != 0) {
 		SETOPT(CURLOPT_PROXY, option_http_proxy_host);
 		SETOPT(CURLOPT_PROXYPORT, (long) option_http_proxy_port);
 		if (option_http_proxy_auth != OPTION_HTTP_PROXY_AUTH_NONE) {
@@ -596,7 +626,7 @@ fetch_curl_set_options(struct curl_fetch_info *f)
 		SETOPT(CURLOPT_PROXY, NULL);
 	}
 
-	if (urldb_get_cert_permissions(f->url)) {
+	if (urldb_get_cert_permissions(nsurl_access(f->url))) {
 		/* Disable certificate verification */
 		SETOPT(CURLOPT_SSL_VERIFYPEER, 0L);
 		SETOPT(CURLOPT_SSL_VERIFYHOST, 0L);
@@ -641,7 +671,7 @@ void fetch_curl_abort(void *vf)
 {
 	struct curl_fetch_info *f = (struct curl_fetch_info *)vf;
 	assert(f);
-	LOG(("fetch %p, url '%s'", f, f->url));
+	LOG(("fetch %p, url '%s'", f, nsurl_access(f->url)));
 	if (f->curl_handle) {
 		f->abort = true;
 	} else {
@@ -662,7 +692,7 @@ void fetch_curl_stop(struct curl_fetch_info *f)
 	CURLMcode codem;
 
 	assert(f);
-	LOG(("fetch %p, url '%s'", f, f->url));
+	LOG(("fetch %p, url '%s'", f, nsurl_access(f->url)));
 
 	if (f->curl_handle) {
 		/* remove from curl multi handle */
@@ -689,8 +719,8 @@ void fetch_curl_free(void *vf)
 
 	if (f->curl_handle)
 		curl_easy_cleanup(f->curl_handle);
-	free(f->url);
-	free(f->host);
+	nsurl_unref(f->url);
+	lwc_string_unref(f->host);
 	free(f->location);
 	free(f->cookie_string);
 	free(f->realm);
@@ -716,7 +746,7 @@ void fetch_curl_free(void *vf)
  * Must be called regularly to make progress on fetches.
  */
 
-void fetch_curl_poll(const char *scheme_ignored)
+void fetch_curl_poll(lwc_string *scheme_ignored)
 {
 	int running, queue;
 	CURLMcode codem;
@@ -746,10 +776,12 @@ void fetch_curl_poll(const char *scheme_ignored)
 		}
 		curl_msg = curl_multi_info_read(fetch_curl_multi, &queue);
 	}
-	
+
+#ifdef FETCHER_CURLL_SCHEDULED
 	if (running != 0) {
-		schedule(10, (schedule_callback_fn)fetch_curl_poll, fetch_curl_poll);
+		schedule(1, (schedule_callback_fn)fetch_curl_poll, fetch_curl_poll);
 	}
+#endif
 }
 
 
@@ -761,9 +793,9 @@ void fetch_curl_poll(const char *scheme_ignored)
 
 void fetch_curl_done(CURL *curl_handle, CURLcode result)
 {
+	fetch_msg msg;
 	bool finished = false;
 	bool error = false;
-	fetch_error_code errorcode = FETCH_ERROR_NO_ERROR;
 	bool cert = false;
 	bool abort_fetch;
 	struct curl_fetch_info *f;
@@ -778,10 +810,18 @@ void fetch_curl_done(CURL *curl_handle, CURLcode result)
 	assert(code == CURLE_OK);
 
 	abort_fetch = f->abort;
-	LOG(("done %s", f->url));
+	LOG(("done %s", nsurl_access(f->url)));
 
-	if (abort_fetch == false && result == CURLE_OK) {
-		/* fetch completed normally */
+	if (abort_fetch == false && (result == CURLE_OK ||
+			(result == CURLE_WRITE_ERROR && f->stopped == false))) {
+		/* fetch completed normally or the server fed us a junk gzip 
+		 * stream (usually in the form of garbage at the end of the 
+		 * stream). Curl will have fed us all but the last chunk of 
+		 * decoded data, which is sad as, if we'd received the last 
+		 * chunk, too, we'd be able to render the whole object.
+		 * As is, we'll just have to accept that the end of the
+		 * object will be truncated in this case and leave it to
+		 * the content handlers to cope. */
 		if (f->stopped ||
 				(!f->had_headers &&
 					fetch_curl_process_headers(f)))
@@ -796,7 +836,6 @@ void fetch_curl_done(CURL *curl_handle, CURLcode result)
 			; /* redirect with partial body, or similar */
 		else {
 			error = true;
-			errorcode = FETCH_ERROR_PARTIAL_FILE;
 		}
 	} else if (result == CURLE_WRITE_ERROR && f->stopped)
 		/* CURLE_WRITE_ERROR occurs when fetch_curl_data
@@ -808,23 +847,19 @@ void fetch_curl_done(CURL *curl_handle, CURLcode result)
 		memset(f->cert_data, 0, sizeof(f->cert_data));
 		cert = true;
 	}
-	else if (result == CURLE_COULDNT_RESOLVE_HOST) {
-		error = true;
-		errorcode = FETCH_ERROR_COULDNT_RESOLVE_HOST;
-	}
 	else {
 		LOG(("Unknown cURL response code %d", result));
 		error = true;
-		errorcode = FETCH_ERROR_MISC;
 	}
 
 	fetch_curl_stop(f);
 
 	if (abort_fetch)
 		; /* fetch was aborted: no callback */
-	else if (finished)
-		fetch_send_callback(FETCH_FINISHED, f->fetch_handle, 0, 0, errorcode);
-	else if (cert) {
+	else if (finished) {
+		msg.type = FETCH_FINISHED;
+		fetch_send_callback(&msg, f->fetch_handle);
+	} else if (cert) {
 		int i;
 		BIO *mem;
 		BUF_MEM *buf;
@@ -900,14 +935,17 @@ void fetch_curl_done(CURL *curl_handle, CURLcode result)
 			if (certs[i].cert->references == 0)
 				X509_free(certs[i].cert);
 		}
-		errorcode = FETCH_ERROR_CERT;
-		fetch_send_callback(FETCH_CERT_ERR, f->fetch_handle,
-				&ssl_certs, i, errorcode);
 
+		msg.type = FETCH_CERT_ERR;
+		msg.data.cert_err.certs = ssl_certs;
+		msg.data.cert_err.num_certs = i;
+		fetch_send_callback(&msg, f->fetch_handle);
+	} else if (error) {
+		msg.type = FETCH_ERROR;
+		msg.data.error = fetch_error_buffer;
+
+		fetch_send_callback(&msg, f->fetch_handle);
 	}
-	else if (error)
-		fetch_send_callback(FETCH_ERROR, f->fetch_handle,
-				fetch_error_buffer, 0, errorcode);
 
 	fetch_free(f->fetch_handle);
 }
@@ -923,11 +961,14 @@ int fetch_curl_progress(void *clientp, double dltotal, double dlnow,
 	static char fetch_progress_buffer[256]; /**< Progress buffer for cURL */
 	struct curl_fetch_info *f = (struct curl_fetch_info *) clientp;
 	unsigned int time_now_cs;
-	double percent;
+	fetch_msg msg;
 
 	if (f->abort)
 		return 0;
 
+	msg.type = FETCH_PROGRESS;
+	msg.data.progress = fetch_progress_buffer;
+
 	/* Rate limit each fetch's progress notifications to 2 a second */
 #define UPDATES_PER_SECOND 2
 #define UPDATE_DELAY_CS (100 / UPDATES_PER_SECOND)
@@ -939,22 +980,16 @@ int fetch_curl_progress(void *clientp, double dltotal, double dlnow,
 #undef UPDATES_PERS_SECOND
 
 	if (dltotal > 0) {
-		percent = dlnow * 100.0f / dltotal;
 		snprintf(fetch_progress_buffer, 255,
 				messages_get("Progress"),
 				human_friendly_bytesize(dlnow),
 				human_friendly_bytesize(dltotal));
-		fetch_send_callback(FETCH_PROGRESS, f->fetch_handle,
-				fetch_progress_buffer,
-				(unsigned long) percent,
-				FETCH_ERROR_NO_ERROR);
+		fetch_send_callback(&msg, f->fetch_handle);
 	} else {
 		snprintf(fetch_progress_buffer, 255,
 				messages_get("ProgressU"),
 				human_friendly_bytesize(dlnow));
-		fetch_send_callback(FETCH_PROGRESS, f->fetch_handle,
-				fetch_progress_buffer, 0,
-				FETCH_ERROR_NO_ERROR);
+		fetch_send_callback(&msg, f->fetch_handle);
 	}
 
 	return 0;
@@ -987,6 +1022,7 @@ size_t fetch_curl_data(char *data, size_t size, size_t nmemb,
 {
 	struct curl_fetch_info *f = _f;
 	CURLcode code;
+	fetch_msg msg;
 
 	/* ensure we only have to get this information once */
 	if (!f->http_code)
@@ -1005,17 +1041,16 @@ size_t fetch_curl_data(char *data, size_t size, size_t nmemb,
 		return size * nmemb;
 	}
 
-	/*LOG(("fetch %p, size %lu", f, size * nmemb));*/
-
 	if (f->abort || (!f->had_headers && fetch_curl_process_headers(f))) {
 		f->stopped = true;
 		return 0;
 	}
 
 	/* send data to the caller */
-	/*LOG(("FETCH_DATA"));*/
-	fetch_send_callback(FETCH_DATA, f->fetch_handle, data, size * nmemb, 
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) data;
+	msg.data.header_or_data.len = size * nmemb;
+	fetch_send_callback(&msg, f->fetch_handle);
 
 	if (f->abort) {
 		f->stopped = true;
@@ -1037,6 +1072,7 @@ size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
 {
 	struct curl_fetch_info *f = _f;
 	int i;
+	fetch_msg msg;
 	size *= nmemb;
 
 	if (f->abort) {
@@ -1044,8 +1080,10 @@ size_t fetch_curl_header(char *data, size_t size, size_t nmemb,
 		return 0;
 	}
 
-	fetch_send_callback(FETCH_HEADER, f->fetch_handle, data, size,
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_HEADER;
+	msg.data.header_or_data.buf = (const uint8_t *) data;
+	msg.data.header_or_data.len = size;
+	fetch_send_callback(&msg, f->fetch_handle);
 
 #define SKIP_ST(o) for (i = (o); i < (int) size && (data[i] == ' ' || data[i] == '\t'); i++)
 
@@ -1118,6 +1156,7 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 {
 	long http_code;
 	CURLcode code;
+	fetch_msg msg;
 
 	f->had_headers = true;
 
@@ -1133,32 +1172,34 @@ bool fetch_curl_process_headers(struct curl_fetch_info *f)
 
 	if (http_code == 304 && !f->post_urlenc && !f->post_multipart) {
 		/* Not Modified && GET request */
-		fetch_send_callback(FETCH_NOTMODIFIED, f->fetch_handle, 0, 0, 
-				FETCH_ERROR_NO_ERROR);
+		msg.type = FETCH_NOTMODIFIED;
+		fetch_send_callback(&msg, f->fetch_handle);
 		return true;
 	}
 
 	/* handle HTTP redirects (3xx response codes) */
 	if (300 <= http_code && http_code < 400 && f->location != 0) {
 		LOG(("FETCH_REDIRECT, '%s'", f->location));
-		fetch_send_callback(FETCH_REDIRECT, f->fetch_handle,
-				f->location, 0,	FETCH_ERROR_NO_ERROR);
+		msg.type = FETCH_REDIRECT;
+		msg.data.redirect = f->location;
+		fetch_send_callback(&msg, f->fetch_handle);
 		return true;
 	}
 
 	/* handle HTTP 401 (Authentication errors) */
 	if (http_code == 401) {
-		fetch_send_callback(FETCH_AUTH, f->fetch_handle, f->realm, 0,
-				FETCH_ERROR_AUTHENTICATION);
+		msg.type = FETCH_AUTH;
+		msg.data.auth.realm = f->realm;
+		fetch_send_callback(&msg, f->fetch_handle);
 		return true;
 	}
 
 	/* handle HTTP errors (non 2xx response codes) */
-	if (f->only_2xx && strncmp(f->url, "http", 4) == 0 &&
+	if (f->only_2xx && strncmp(nsurl_access(f->url), "http", 4) == 0 &&
 			(http_code < 200 || 299 < http_code)) {
-		fetch_send_callback(FETCH_ERROR, f->fetch_handle,
-				messages_get("Not2xx"), 0, 
-				FETCH_ERROR_HTTP_NOT2);
+		msg.type = FETCH_ERROR;
+		msg.data.error = messages_get("Not2xx");
+		fetch_send_callback(&msg, f->fetch_handle);
 		return true;
 	}
 
