@@ -34,6 +34,8 @@
 #include <limits.h>
 #include <stdarg.h>
 
+#include <libwapcaplet/libwapcaplet.h>
+
 #include "utils/config.h"
 #include "content/dirlist.h"
 #include "content/fetch.h"
@@ -59,7 +61,7 @@ struct fetch_file_context {
 	bool aborted; /**< Flag indicating fetch has been aborted */
 	bool locked; /**< Flag indicating entry is already entered */
 
-	char *url; /**< The full url the fetch refers to */
+	nsurl *url; /**< The full url the fetch refers to */
 	char *path; /**< The actual path to be used with open() */
 
 	time_t file_etag; /**< Request etag for file (previous st.m_time) */
@@ -68,12 +70,11 @@ struct fetch_file_context {
 static struct fetch_file_context *ring = NULL;
 
 /** issue fetch callbacks with locking */
-static inline bool fetch_file_send_callback(fetch_msg msg,
-		struct fetch_file_context *ctx, const void *data,
-		unsigned long size, fetch_error_code errorcode)
+static inline bool fetch_file_send_callback(const fetch_msg *msg,
+		struct fetch_file_context *ctx)
 {
 	ctx->locked = true;
-	fetch_send_callback(msg, ctx->fetchh, data, size, errorcode);
+	fetch_send_callback(msg, ctx->fetchh);
 	ctx->locked = false;
 
 	return ctx->aborted;
@@ -82,6 +83,7 @@ static inline bool fetch_file_send_callback(fetch_msg msg,
 static bool fetch_file_send_header(struct fetch_file_context *ctx,
 		const char *fmt, ...)
 {
+	fetch_msg msg;
 	char header[64];
 	va_list ap;
 
@@ -91,27 +93,34 @@ static bool fetch_file_send_header(struct fetch_file_context *ctx,
 
 	va_end(ap);
 
-	fetch_file_send_callback(FETCH_HEADER, ctx, header, strlen(header), 
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_HEADER;
+	msg.data.header_or_data.buf = (const uint8_t *) header;
+	msg.data.header_or_data.len = strlen(header);
+	fetch_file_send_callback(&msg, ctx);
 
 	return ctx->aborted;
 }
 
 /** callback to initialise the file fetcher. */
-static bool fetch_file_initialise(const char *scheme)
+static bool fetch_file_initialise(lwc_string *scheme)
 {
 	return true;
 }
 
 /** callback to initialise the file fetcher. */
-static void fetch_file_finalise(const char *scheme)
+static void fetch_file_finalise(lwc_string *scheme)
 {
 }
 
+static bool fetch_file_can_fetch(const nsurl *url)
+{
+	return true;
+}
+
 /** callback to set up a file fetch context. */
 static void *
 fetch_file_setup(struct fetch *fetchh,
-		 const char *url,
+		 nsurl *url,
 		 bool only_2xx,
 		 const char *post_urlenc,
 		 const struct fetch_multipart_data *post_multipart,
@@ -124,18 +133,13 @@ fetch_file_setup(struct fetch *fetchh,
 	if (ctx == NULL)
 		return NULL;
 
-	ctx->path = url_to_path(url);
+	ctx->path = url_to_path(nsurl_access(url));
 	if (ctx->path == NULL) {
 		free(ctx);
 		return NULL;
 	}
 
-	ctx->url = strdup(url);
-	if (ctx->url == NULL) {
-		free(ctx->path);
-		free(ctx);
-		return NULL;
-	}
+	ctx->url = nsurl_ref(url);
 
 	/* Scan request headers looking for If-None-Match */
 	for (i = 0; headers[i] != NULL; i++) {
@@ -165,7 +169,7 @@ fetch_file_setup(struct fetch *fetchh,
 static void fetch_file_free(void *ctx)
 {
 	struct fetch_file_context *c = ctx;
-	free(c->url);
+	nsurl_unref(c->url);
 	free(c->path);
 	RING_REMOVE(ring, c);
 	free(ctx);
@@ -207,6 +211,7 @@ static int fetch_file_errno_to_http_code(int error_no)
 
 static void fetch_file_process_error(struct fetch_file_context *ctx, int code)
 {
+	fetch_msg msg;
 	char buffer[1024];
 	const char *title;
 	char key[8];
@@ -224,14 +229,16 @@ static void fetch_file_process_error(struct fetch_file_context *ctx, int code)
 	snprintf(buffer, sizeof buffer, "<html><head><title>%s</title></head>"
 			"<body><h1>%s</h1>"
 			"<p>Error %d while fetching file %s</p></body></html>",
-			title, title, code, ctx->url);
+			title, title, code, nsurl_access(ctx->url));
 
-	if (fetch_file_send_callback(FETCH_DATA, ctx, buffer, strlen(buffer), 
-			FETCH_ERROR_NO_ERROR))
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+	msg.data.header_or_data.len = strlen(buffer);
+	if (fetch_file_send_callback(&msg, ctx))
 		goto fetch_file_process_error_aborted;
 
-	fetch_file_send_callback(FETCH_FINISHED, ctx, 0, 0, 
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_FINISHED;
+	fetch_file_send_callback(&msg, ctx);
 
 fetch_file_process_error_aborted:
 	return;
@@ -242,6 +249,7 @@ fetch_file_process_error_aborted:
 static void fetch_file_process_plain(struct fetch_file_context *ctx,
 				     struct stat *fdstat)
 {
+	fetch_msg msg;
 	char *buf;
 	size_t buf_size;
 
@@ -253,8 +261,8 @@ static void fetch_file_process_plain(struct fetch_file_context *ctx,
 	/* Check if we can just return not modified */
 	if (ctx->file_etag != 0 && ctx->file_etag == fdstat->st_mtime) {
 		fetch_set_http_code(ctx->fetchh, 304);
-		fetch_file_send_callback(FETCH_NOTMODIFIED, ctx, 0, 0,
-				FETCH_ERROR_NO_ERROR);
+		msg.type = FETCH_NOTMODIFIED;
+		fetch_file_send_callback(&msg, ctx);
 		return;
 	}
 
@@ -274,9 +282,10 @@ static void fetch_file_process_plain(struct fetch_file_context *ctx,
 	/* allocate the buffer storage */
 	buf = malloc(buf_size);
 	if (buf == NULL) {
-		fetch_file_send_callback(FETCH_ERROR, ctx,
-			"Unable to allocate memory for file data buffer",
-			0, FETCH_ERROR_MEMORY);
+		msg.type = FETCH_ERROR;
+		msg.data.error =
+			"Unable to allocate memory for file data buffer";
+		fetch_file_send_callback(&msg, ctx);
 		close(fd);
 		return;
 	}
@@ -307,29 +316,32 @@ static void fetch_file_process_plain(struct fetch_file_context *ctx,
 	while (tot_read < fdstat->st_size) {
 		res = read(fd, buf, buf_size);
 		if (res == -1) {
-			fetch_file_send_callback(FETCH_ERROR, ctx, 
-					"Error reading file", 0, 
-					FETCH_ERROR_PARTIAL_FILE);
+			msg.type = FETCH_ERROR;
+			msg.data.error = "Error reading file";
+			fetch_file_send_callback(&msg, ctx);
 			goto fetch_file_process_aborted;
 		}
 
 		if (res == 0) {
-			fetch_file_send_callback(FETCH_ERROR, ctx, 
-					"Unexpected EOF reading file", 0, 
-					FETCH_ERROR_PARTIAL_FILE);
+			msg.type = FETCH_ERROR;
+			msg.data.error = "Unexpected EOF reading file";
+			fetch_file_send_callback(&msg, ctx);
 			goto fetch_file_process_aborted;
 		}
 
 		tot_read += res;
 
-		if (fetch_file_send_callback(FETCH_DATA, ctx, buf, res, 
-				FETCH_ERROR_NO_ERROR))
+		msg.type = FETCH_DATA;
+		msg.data.header_or_data.buf = (const uint8_t *) buf;
+		msg.data.header_or_data.len = res;
+		if (fetch_file_send_callback(&msg, ctx))
 			break;
 	}
 
-	if (ctx->aborted == false)
-		fetch_file_send_callback(FETCH_FINISHED, ctx, 0, 0, 
-				FETCH_ERROR_NO_ERROR);
+	if (ctx->aborted == false) {
+		msg.type = FETCH_FINISHED;
+		fetch_file_send_callback(&msg, ctx);
+	}
 
 fetch_file_process_aborted:
 
@@ -395,6 +407,7 @@ static char *gen_nice_title(char *path)
 static void fetch_file_process_dir(struct fetch_file_context *ctx,
 				   struct stat *fdstat)
 {
+	fetch_msg msg;
 	char buffer[1024]; /* Output buffer */
 	bool even = false; /* formatting flag */
 	char *title; /* pretty printed title */
@@ -428,32 +441,32 @@ static void fetch_file_process_dir(struct fetch_file_context *ctx,
 	if (fetch_file_send_header(ctx, "Content-Type: text/html"))
 		goto fetch_file_process_dir_aborted;
 
+	msg.type = FETCH_DATA;
+	msg.data.header_or_data.buf = (const uint8_t *) buffer;
+
 	/* directory listing top */
 	dirlist_generate_top(buffer, sizeof buffer);
-	if (fetch_file_send_callback(FETCH_DATA, ctx, buffer, strlen(buffer), 
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = strlen(buffer);
+	if (fetch_file_send_callback(&msg, ctx))
 		goto fetch_file_process_dir_aborted;
 
 	/* directory listing title */
 	title = gen_nice_title(ctx->path);
 	dirlist_generate_title(title, buffer, sizeof buffer);
 	free(title);
-	if (fetch_file_send_callback(FETCH_DATA, ctx, buffer, strlen(buffer), 
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = strlen(buffer);
+	if (fetch_file_send_callback(&msg, ctx))
 		goto fetch_file_process_dir_aborted;
 
 	/* Print parent directory link */
-	res = url_parent(ctx->url, &up);
+	res = url_parent(nsurl_access(ctx->url), &up);
 	if (res == URL_FUNC_OK) {
-		res = url_compare(ctx->url, up, false, &compare);
+		res = url_compare(nsurl_access(ctx->url), up, false, &compare);
 		if ((res == URL_FUNC_OK) && compare == false) {
 			dirlist_generate_parent_link(up, buffer, sizeof buffer);
 
-			fetch_file_send_callback(FETCH_DATA, ctx,
-						 buffer,
-						 strlen(buffer),
-						 FETCH_ERROR_NO_ERROR);
-
+			msg.data.header_or_data.len = strlen(buffer);
+			fetch_file_send_callback(&msg, ctx);
 		}
 		free(up);
 
@@ -464,8 +477,8 @@ static void fetch_file_process_dir(struct fetch_file_context *ctx,
 
 	/* directory list headings */
 	dirlist_generate_headings(buffer, sizeof buffer);
-	if (fetch_file_send_callback(FETCH_DATA, ctx, buffer, strlen(buffer), 
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = strlen(buffer);
+	if (fetch_file_send_callback(&msg, ctx))
 		goto fetch_file_process_dir_aborted;
 
 	while ((ent = readdir(scandir)) != NULL) {
@@ -535,10 +548,8 @@ static void fetch_file_process_dir(struct fetch_file_context *ctx,
 
 		free(path);
 
-		if (fetch_file_send_callback(FETCH_DATA, ctx,
-					     buffer,
-					     strlen(buffer),
-					     FETCH_ERROR_NO_ERROR))
+		msg.data.header_or_data.len = strlen(buffer);
+		if (fetch_file_send_callback(&msg, ctx))
 			goto fetch_file_process_dir_aborted;
 
 		even = !even;
@@ -546,13 +557,12 @@ static void fetch_file_process_dir(struct fetch_file_context *ctx,
 
 	/* directory listing bottom */
 	dirlist_generate_bottom(buffer, sizeof buffer);
-	if (fetch_file_send_callback(FETCH_DATA, ctx, buffer, strlen(buffer), 
-			FETCH_ERROR_NO_ERROR))
+	msg.data.header_or_data.len = strlen(buffer);
+	if (fetch_file_send_callback(&msg, ctx))
 		goto fetch_file_process_dir_aborted;
 
-
-	fetch_file_send_callback(FETCH_FINISHED, ctx, 0, 0, 
-			FETCH_ERROR_NO_ERROR);
+	msg.type = FETCH_FINISHED;
+	fetch_file_send_callback(&msg, ctx);
 
 fetch_file_process_dir_aborted:
 
@@ -589,7 +599,7 @@ static void fetch_file_process(struct fetch_file_context *ctx)
 }
 
 /** callback to poll for additional file fetch contents */
-static void fetch_file_poll(const char *scheme)
+static void fetch_file_poll(lwc_string *scheme)
 {
 	struct fetch_file_context *c, *next;
 
@@ -631,8 +641,16 @@ static void fetch_file_poll(const char *scheme)
 
 void fetch_file_register(void)
 {
-	fetch_add_fetcher("file",
+	lwc_string *scheme;
+
+	if (lwc_intern_string("file", SLEN("file"), &scheme) != lwc_error_ok) {
+		die("Failed to initialise the fetch module "
+				"(couldn't intern \"file\").");
+	}
+
+	fetch_add_fetcher(scheme,
 		fetch_file_initialise,
+		fetch_file_can_fetch,
 		fetch_file_setup,
 		fetch_file_start,
 		fetch_file_abort,
