@@ -52,6 +52,8 @@ struct hlcache_retrieval_ctx {
 	content_type accepted_types;	/**< Accepted types */
 
 	hlcache_child_context child;	/**< Child context */
+
+	bool migrate_target;		/**< Whether this context is the migration target */
 };
 
 /** High-level cache handle */
@@ -176,7 +178,7 @@ void hlcache_finalise(void)
 
 		if (entry->content != NULL) {
 			LOG(("	%p : %s (%d users)", entry,
-			     content_get_url(&entry_handle), content_count_users(entry->content)));
+			     nsurl_access(hlcache_handle_get_url(&entry_handle)), content_count_users(entry->content)));
 		} else {
 			LOG(("	%p", entry));
 		}
@@ -225,8 +227,8 @@ nserror hlcache_poll(void)
 }
 
 /* See hlcache.h for documentation */
-nserror hlcache_handle_retrieve(const char *url, uint32_t flags,
-		const char *referer, llcache_post_data *post,
+nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
+		nsurl *referer, llcache_post_data *post,
 		hlcache_handle_callback cb, void *pw,
 		hlcache_child_context *child,
 		content_type accepted_types, hlcache_handle **result)
@@ -291,7 +293,8 @@ nserror hlcache_handle_release(hlcache_handle *handle)
 		RING_ITERATE_START(struct hlcache_retrieval_ctx,
 				   hlcache->retrieval_ctx_ring,
 				   ictx) {
-			if (ictx->handle == handle) {
+			if (ictx->handle == handle && 
+					ictx->migrate_target == false) {
 				/* This is the nascent context for us,
 				 * so abort the fetch */
 				llcache_handle_abort(ictx->llcache);
@@ -342,7 +345,8 @@ nserror hlcache_handle_abort(hlcache_handle *handle)
 		RING_ITERATE_START(struct hlcache_retrieval_ctx,
 				   hlcache->retrieval_ctx_ring,
 				   ictx) {
-			if (ictx->handle == handle) {
+			if (ictx->handle == handle && 
+					ictx->migrate_target == false) {
 				/* This is the nascent context for us,
 				 * so abort the fetch */
 				llcache_handle_abort(ictx->llcache);
@@ -416,6 +420,33 @@ nserror hlcache_handle_clone(hlcache_handle *handle, hlcache_handle **result)
 	return NSERROR_CLONE_FAILED;
 }
 
+/* See hlcache.h for documentation */
+nsurl *hlcache_handle_get_url(const hlcache_handle *handle)
+{
+	nsurl *result = NULL;
+
+	assert(handle != NULL);
+
+	if (handle->entry != NULL) {
+		result = content_get_url(handle->entry->content);
+	} else {
+		RING_ITERATE_START(struct hlcache_retrieval_ctx,
+				   hlcache->retrieval_ctx_ring,
+				   ictx) {
+			if (ictx->handle == handle) {
+				/* This is the nascent context for us */
+				result = llcache_handle_get_url(ictx->llcache);
+
+				/* And stop */
+				RING_ITERATE_STOP(hlcache->retrieval_ctx_ring,
+						ictx);
+			}
+		} RING_ITERATE_END(hlcache->retrieval_ctx_ring, ictx);
+	}
+
+	return result;
+}
+
 /******************************************************************************
  * High-level cache internals						      *
  ******************************************************************************/
@@ -491,6 +522,7 @@ nserror hlcache_llcache_callback(llcache_handle *handle,
 	case LLCACHE_EVENT_HAD_HEADERS:
 		error = mimesniff_compute_effective_type(handle, NULL, 0,
 				ctx->flags & HLCACHE_RETRIEVE_SNIFF_TYPE,
+				ctx->accepted_types == CONTENT_IMAGE,
 				&effective_type);
 		if (error == NSERROR_OK || error == NSERROR_NOT_FOUND) {
 			/* If the sniffer was successful or failed to find
@@ -514,6 +546,7 @@ nserror hlcache_llcache_callback(llcache_handle *handle,
 		error = mimesniff_compute_effective_type(handle,
 				event->data.data.buf, event->data.data.len,
 				ctx->flags & HLCACHE_RETRIEVE_SNIFF_TYPE,
+				ctx->accepted_types == CONTENT_IMAGE,
 				&effective_type);
 		if (error != NSERROR_OK) {
 			assert(0 && "MIME sniff failed with data");
@@ -530,7 +563,7 @@ nserror hlcache_llcache_callback(llcache_handle *handle,
 		/* DONE event before we could determine the effective MIME type.
 		 */
 		error = mimesniff_compute_effective_type(handle,
-				NULL, 0, false, &effective_type);
+				NULL, 0, false, false, &effective_type);
 		if (error == NSERROR_OK) {
 			error = hlcache_migrate_ctx(ctx, effective_type);
 
@@ -580,21 +613,22 @@ nserror hlcache_migrate_ctx(hlcache_retrieval_ctx *ctx,
 	content_type type = CONTENT_NONE;
 	nserror error = NSERROR_OK;
 
-	/* Unlink the context to prevent recursion */
-	RING_REMOVE(hlcache->retrieval_ctx_ring, ctx);
+	ctx->migrate_target = true;
 
 	if (effective_type != NULL &&
 			hlcache_type_is_acceptable(effective_type,
 			ctx->accepted_types, &type)) {
 		error = hlcache_find_content(ctx, effective_type);
 		if (error != NSERROR_OK && error != NSERROR_NEED_DATA) {
-			hlcache_event hlevent;
+			if (ctx->handle->cb != NULL) {
+				hlcache_event hlevent;
 
-			hlevent.type = CONTENT_MSG_ERROR;
-			hlevent.data.error = messages_get("MiscError");
+				hlevent.type = CONTENT_MSG_ERROR;
+				hlevent.data.error = messages_get("MiscError");
 
-			ctx->handle->cb(ctx->handle, &hlevent,
-					ctx->handle->pw);
+				ctx->handle->cb(ctx->handle, &hlevent,
+						ctx->handle->pw);
+			}
 
 			llcache_handle_abort(ctx->llcache);
 			llcache_handle_release(ctx->llcache);
@@ -617,10 +651,7 @@ nserror hlcache_migrate_ctx(hlcache_retrieval_ctx *ctx,
 		/* Ensure caller knows we need data */
 		error = NSERROR_NEED_DATA;
 	} else {
-		/* Unacceptable type: abort fetch and report error */
-		llcache_handle_abort(ctx->llcache);
-		llcache_handle_release(ctx->llcache);
-
+		/* Unacceptable type: report error */
 		if (ctx->handle->cb != NULL) {
 			hlcache_event hlevent;
 
@@ -630,9 +661,15 @@ nserror hlcache_migrate_ctx(hlcache_retrieval_ctx *ctx,
 			ctx->handle->cb(ctx->handle, &hlevent,
 					ctx->handle->pw);
 		}
+
+		llcache_handle_abort(ctx->llcache);
+		llcache_handle_release(ctx->llcache);
 	}
 
+	ctx->migrate_target = false;
+
 	/* No longer require retrieval context */
+	RING_REMOVE(hlcache->retrieval_ctx_ring, ctx);
 	free((char *) ctx->child.charset);
 	free(ctx);
 
