@@ -26,6 +26,7 @@
 #include "content/content.h"
 #include "content/llcache.h"
 #include "utils/errors.h"
+#include "utils/nsurl.h"
 
 /** High-level cache handle */
 typedef struct hlcache_handle hlcache_handle;
@@ -129,8 +130,8 @@ nserror hlcache_poll(void);
  *
  * \todo Is there any way to sensibly reduce the number of parameters here?
  */
-nserror hlcache_handle_retrieve(const char *url, uint32_t flags,
-		const char *referer, llcache_post_data *post,
+nserror hlcache_handle_retrieve(nsurl *url, uint32_t flags,
+		nsurl *referer, llcache_post_data *post,
 		hlcache_handle_callback cb, void *pw,
 		hlcache_child_context *child, 
 		content_type accepted_types, hlcache_handle **result);
@@ -189,4 +190,12 @@ struct content *hlcache_handle_get_content(const hlcache_handle *handle);
  */
 nserror hlcache_handle_clone(hlcache_handle *handle, hlcache_handle **result);
 
+/**
+ * Retrieve the URL associated with a high level cache handle
+ *
+ * \param handle  The handle to inspect
+ * \return  Pointer to URL.
+ */
+nsurl *hlcache_handle_get_url(const hlcache_handle *handle);
+
 #endif
