@@ -31,7 +31,7 @@
 #include "content/urldb.h"
 #include "utils/log.h"
 #include "utils/messages.h"
-#include "utils/url.h"
+#include "utils/nsurl.h"
 #include "utils/utils.h"
 
 /** Define to enable tracing of llcache operations. */
@@ -73,7 +73,7 @@ typedef struct llcache_object_user {
 /** Low-level cache object fetch context */
 typedef struct {
 	uint32_t flags;			/**< Fetch flags */
-	char *referer;			/**< Referring URL, or NULL if none */
+	nsurl *referer;			/**< Referring URL, or NULL if none */
 	llcache_post_data *post;	/**< POST data, or NULL for GET */	
 
 	struct fetch *fetch;		/**< Fetch handle for this object */
@@ -119,7 +119,7 @@ struct llcache_object {
 	llcache_object *prev;		/**< Previous in list */
 	llcache_object *next;		/**< Next in list */
 
-	char *url;			/**< Post-redirect URL for object */
+	nsurl *url;			/**< Post-redirect URL for object */
 	bool has_query;			/**< URL has a query segment */
   
 	/** \todo We need a generic dynamic buffer object */
@@ -159,15 +159,21 @@ struct llcache_s {
 /** low level cache state */
 static struct llcache_s *llcache = NULL;
 
+/* Static lwc_strings */
+static lwc_string *llcache_file_lwc;
+static lwc_string *llcache_about_lwc;
+static lwc_string *llcache_resource_lwc;
+
+
 static nserror llcache_object_user_new(llcache_handle_callback cb, void *pw,
 		llcache_object_user **user);
 static nserror llcache_object_user_destroy(llcache_object_user *user);
 
-static nserror llcache_object_retrieve(const char *url, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+static nserror llcache_object_retrieve(nsurl *url, uint32_t flags,
+		nsurl *referer, const llcache_post_data *post,
 		uint32_t redirect_count, llcache_object **result);
-static nserror llcache_object_retrieve_from_cache(const char *url, 
-		uint32_t flags, const char *referer, 
+static nserror llcache_object_retrieve_from_cache(nsurl *url, 
+		uint32_t flags, nsurl *referer, 
 		const llcache_post_data *post, uint32_t redirect_count,
 		llcache_object **result);
 static bool llcache_object_is_fresh(const llcache_object *object);
@@ -175,11 +181,11 @@ static nserror llcache_object_cache_update(llcache_object *object);
 static nserror llcache_object_clone_cache_data(llcache_object *source,
 		llcache_object *destination, bool deep);
 static nserror llcache_object_fetch(llcache_object *object, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+		nsurl *referer, const llcache_post_data *post,
 		uint32_t redirect_count);
 static nserror llcache_object_refetch(llcache_object *object);
 
-static nserror llcache_object_new(const char *url, llcache_object **result);
+static nserror llcache_object_new(nsurl *url, llcache_object **result);
 static nserror llcache_object_destroy(llcache_object *object);
 static nserror llcache_object_add_user(llcache_object *object,
 		llcache_object_user *user);
@@ -205,18 +211,17 @@ static nserror llcache_post_data_clone(const llcache_post_data *orig,
 
 static nserror llcache_query_handle_response(bool proceed, void *cbpw);
 
-static void llcache_fetch_callback(fetch_msg msg, void *p, const void *data, 
-		unsigned long size, fetch_error_code errorcode);
+static void llcache_fetch_callback(const fetch_msg *msg, void *p);
 static nserror llcache_fetch_redirect(llcache_object *object, 
 		const char *target, llcache_object **replacement);
 static nserror llcache_fetch_notmodified(llcache_object *object,
 		llcache_object **replacement);
-static nserror llcache_fetch_split_header(const char *data, size_t len, 
+static nserror llcache_fetch_split_header(const uint8_t *data, size_t len, 
 		char **name, char **value);
 static nserror llcache_fetch_parse_header(llcache_object *object,
-		const char *data, size_t len, char **name, char **value);
+		const uint8_t *data, size_t len, char **name, char **value);
 static nserror llcache_fetch_process_header(llcache_object *object, 
-		const char *data, size_t len);
+		const uint8_t *data, size_t len);
 static nserror llcache_fetch_process_data(llcache_object *object, 
 		const uint8_t *data, size_t len);
 static nserror llcache_fetch_auth(llcache_object *object,
@@ -265,6 +270,19 @@ llcache_initialise(llcache_query_callback cb, void *pw, uint32_t llcache_limit)
 	llcache->query_cb_pw = pw;
 	llcache->limit = llcache_limit;
 
+	/* Create static scheme strings */
+	if (lwc_intern_string("file", SLEN("file"),
+			&llcache_file_lwc) != lwc_error_ok)
+		return NSERROR_NOMEM;
+
+	if (lwc_intern_string("about", SLEN("about"),
+			&llcache_about_lwc) != lwc_error_ok)
+		return NSERROR_NOMEM;
+
+	if (lwc_intern_string("resource", SLEN("resource"),
+			&llcache_resource_lwc) != lwc_error_ok)
+		return NSERROR_NOMEM;
+
 	LOG(("llcache initialised with a limit of %d bytes", llcache_limit));
 
 	return NSERROR_OK;
@@ -317,6 +335,11 @@ void llcache_finalise(void)
 		llcache_object_destroy(object);
 	}
 
+	/* Unref static scheme lwc strings */
+	lwc_string_unref(llcache_file_lwc);
+	lwc_string_unref(llcache_about_lwc);
+	lwc_string_unref(llcache_resource_lwc);
+
 	free(llcache);
 	llcache = NULL;
 }
@@ -343,8 +366,8 @@ nserror llcache_poll(void)
 }
 
 /* See llcache.h for documentation */
-nserror llcache_handle_retrieve(const char *url, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+nserror llcache_handle_retrieve(nsurl *url, uint32_t flags,
+		nsurl *referer, const llcache_post_data *post,
 		llcache_handle_callback cb, void *pw,
 		llcache_handle **result)
 {
@@ -524,7 +547,7 @@ nserror llcache_handle_invalidate_cache_data(llcache_handle *handle)
 }
 
 /* See llcache.h for documentation */
-const char *llcache_handle_get_url(const llcache_handle *handle)
+nsurl *llcache_handle_get_url(const llcache_handle *handle)
 {
 	return handle->object != NULL ? handle->object->url : NULL;
 }
@@ -680,16 +703,14 @@ static nserror llcache_send_event_to_users(llcache_object *object,
  * \param result	  Pointer to location to recieve retrieved object
  * \return NSERROR_OK on success, appropriate error otherwise
  */
-nserror llcache_object_retrieve(const char *url, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+nserror llcache_object_retrieve(nsurl *url, uint32_t flags,
+		nsurl *referer, const llcache_post_data *post,
 		uint32_t redirect_count, llcache_object **result)
 {
 	nserror error;
 	llcache_object *obj;
 	bool has_query;
-	url_func_result res;
-	struct url_components components;
-	char *defragmented_url;
+	nsurl *defragmented_url;
 
 #ifdef LLCACHE_TRACE
 	LOG(("Retrieve %s (%x, %s, %p)", url, flags, referer, post));
@@ -703,26 +724,22 @@ nserror llcache_object_retrieve(const char *url, uint32_t flags,
 	 */
 
 	/* Look for a query segment */
-	res = url_get_components(url, &components);
-	if (res == URL_FUNC_NOMEM)
-		return NSERROR_NOMEM;
+	has_query = nsurl_has_component(url, NSURL_QUERY);
 
-	has_query = (components.query != NULL);
-	
-	components.fragment = NULL;
-
-	defragmented_url = url_reform_components(&components);
-
-	url_destroy_components(&components);
-
-	if (defragmented_url == NULL)
-		return NSERROR_NOMEM;
+	/* Get rid of any url fragment */
+	if (nsurl_has_component(url, NSURL_FRAGMENT)) {
+		error = nsurl_defragment(url, &defragmented_url);
+		if (error != NSERROR_OK)
+			return error;
+	} else {
+		defragmented_url = nsurl_ref(url);
+	}
 
 	if (flags & LLCACHE_RETRIEVE_FORCE_FETCH || post != NULL) {
 		/* Create new object */
 		error = llcache_object_new(defragmented_url, &obj);
 		if (error != NSERROR_OK) {
-			free(defragmented_url);
+			nsurl_unref(defragmented_url);
 			return error;
 		}
 
@@ -731,17 +748,17 @@ nserror llcache_object_retrieve(const char *url, uint32_t flags,
 				redirect_count);
 		if (error != NSERROR_OK) {
 			llcache_object_destroy(obj);
-			free(defragmented_url);
+			nsurl_unref(defragmented_url);
 			return error;
 		}
 
 		/* Add new object to uncached list */
 		llcache_object_add_to_list(obj, &llcache->uncached_objects);
 	} else {
-		error = llcache_object_retrieve_from_cache(defragmented_url, flags, referer,
-				post, redirect_count, &obj);
+		error = llcache_object_retrieve_from_cache(defragmented_url,
+				flags, referer, post, redirect_count, &obj);
 		if (error != NSERROR_OK) {
-			free(defragmented_url);
+			nsurl_unref(defragmented_url);
 			return error;
 		}
 
@@ -756,7 +773,7 @@ nserror llcache_object_retrieve(const char *url, uint32_t flags,
 	
 	*result = obj;
 	
-	free(defragmented_url);
+	nsurl_unref(defragmented_url);
 	
 	return NSERROR_OK;
 }
@@ -772,8 +789,8 @@ nserror llcache_object_retrieve(const char *url, uint32_t flags,
  * \param result	  Pointer to location to recieve retrieved object
  * \return NSERROR_OK on success, appropriate error otherwise
  */
-nserror llcache_object_retrieve_from_cache(const char *url, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+nserror llcache_object_retrieve_from_cache(nsurl *url, uint32_t flags,
+		nsurl *referer, const llcache_post_data *post,
 		uint32_t redirect_count, llcache_object **result)
 {
 	nserror error;
@@ -785,13 +802,11 @@ nserror llcache_object_retrieve_from_cache(const char *url, uint32_t flags,
 
 	/* Search for the most recently fetched matching object */
 	for (obj = llcache->cached_objects; obj != NULL; obj = obj->next) {
-		bool match;
 
 		if ((newest == NULL || 
 				obj->cache.req_time > newest->cache.req_time) &&
-				url_compare(obj->url, url, true, 
-					&match) == URL_FUNC_OK &&
-				match == true) {
+				nsurl_compare(obj->url, url,
+						NSURL_COMPLETE) == true) {
 			newest = obj;
 		}
 	}
@@ -1004,31 +1019,26 @@ nserror llcache_object_clone_cache_data(llcache_object *source,
  * \pre There must not be a fetch in progress for \a object
  */
 nserror llcache_object_fetch(llcache_object *object, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+		nsurl *referer, const llcache_post_data *post,
 		uint32_t redirect_count)
 {
 	nserror error;
-	char *referer_clone = NULL;
+	nsurl *referer_clone = NULL;
 	llcache_post_data *post_clone = NULL;
 
 #ifdef LLCACHE_TRACE
 	LOG(("Starting fetch for %p", object));
 #endif
 
-	if (referer != NULL) {
-		referer_clone = strdup(referer);
-		if (referer_clone == NULL)
-			return NSERROR_NOMEM;
-	}
-
 	if (post != NULL) {
 		error = llcache_post_data_clone(post, &post_clone);
-		if (error != NSERROR_OK) {
-			free(referer_clone);
+		if (error != NSERROR_OK)
 			return error;
-		}
 	}
 
+	if (referer != NULL)
+		referer_clone = nsurl_ref(referer);
+
 	object->fetch.flags = flags;
 	object->fetch.referer = referer_clone;
 	object->fetch.post = post_clone;
@@ -1136,21 +1146,17 @@ nserror llcache_object_refetch(llcache_object *object)
  * \param result  Pointer to location to receive result
  * \return NSERROR_OK on success, appropriate error otherwise
  */
-nserror llcache_object_new(const char *url, llcache_object **result)
+nserror llcache_object_new(nsurl *url, llcache_object **result)
 {
 	llcache_object *obj = calloc(1, sizeof(llcache_object));
 	if (obj == NULL)
 		return NSERROR_NOMEM;
 
 #ifdef LLCACHE_TRACE
-	LOG(("Created object %p (%s)", obj, url));
+	LOG(("Created object %p (%s)", obj, nsurl_access(url)));
 #endif
 
-	obj->url = strdup(url);
-	if (obj->url == NULL) {
-		free(obj);
-		return NSERROR_NOMEM;
-	}
+	obj->url = nsurl_ref(url);
 
 	*result = obj;
 
@@ -1175,7 +1181,7 @@ nserror llcache_object_destroy(llcache_object *object)
 	LOG(("Destroying object %p", object));
 #endif
 
-	free(object->url);
+	nsurl_unref(object->url);
 	free(object->source_data);
 
 	if (object->fetch.fetch != NULL) {
@@ -1183,7 +1189,8 @@ nserror llcache_object_destroy(llcache_object *object)
 		object->fetch.fetch = NULL;
 	}
 
-	free(object->fetch.referer);
+	if (object->fetch.referer != NULL)
+		nsurl_unref(object->fetch.referer);
 
 	if (object->fetch.post != NULL) {
 		if (object->fetch.post->type == LLCACHE_POST_URL_ENCODED) {
@@ -1806,29 +1813,27 @@ nserror llcache_query_handle_response(bool proceed, void *cbpw)
 /**
  * Handler for fetch events
  *
- * \param msg	     Type of fetch event
+ * \param msg	     Fetch event
  * \param p	     Our private data
- * \param data	     Event data
- * \param size	     Length of data in bytes
- * \param errorcode  Reason for fetch error
  */
-void llcache_fetch_callback(fetch_msg msg, void *p, const void *data, 
-		unsigned long size, fetch_error_code errorcode)
+void llcache_fetch_callback(const fetch_msg *msg, void *p)
 {
 	nserror error = NSERROR_OK;
 	llcache_object *object = p;
 	llcache_event event;
 
 #ifdef LLCACHE_TRACE
-	LOG(("Fetch event %d for %p", msg, object));
+	LOG(("Fetch event %d for %p", msg->type, object));
 #endif
 
-	switch (msg) {
+	switch (msg->type) {
 	case FETCH_HEADER:
 		/* Received a fetch header */
 		object->fetch.state = LLCACHE_FETCH_HEADERS;
 
-		error = llcache_fetch_process_header(object, data, size);
+		error = llcache_fetch_process_header(object, 
+				msg->data.header_or_data.buf, 
+				msg->data.header_or_data.len);
 		break;
 
 	/* 3xx responses */
@@ -1841,7 +1846,8 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 			object->candidate = NULL;
 		}
 
-		error = llcache_fetch_redirect(object, data, &object);
+		error = llcache_fetch_redirect(object, 
+				msg->data.redirect, &object);
 		break;
 	case FETCH_NOTMODIFIED:
 		/* Conditional request determined that cached object is fresh */
@@ -1866,7 +1872,7 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 			 */
 			long http_code = fetch_http_code(object->fetch.fetch);
 
-			if ((http_code != 200 && http_code != 203) &&
+			if ((http_code != 200 && http_code != 203) ||
 				(object->has_query && 
 				(object->cache.max_age == INVALID_AGE &&
 					object->cache.expires == 0))) {
@@ -1883,7 +1889,9 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 
 		object->fetch.state = LLCACHE_FETCH_DATA;
 
-		error = llcache_fetch_process_data(object, data, size);
+		error = llcache_fetch_process_data(object, 
+				msg->data.header_or_data.buf,
+				msg->data.header_or_data.len);
 		break;
 	case FETCH_FINISHED:
 		/* Finished fetching */
@@ -1925,7 +1933,7 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 		/** \todo Consider using errorcode for something */
 
 		event.type = LLCACHE_EVENT_ERROR;
-		event.data.msg = data;
+		event.data.msg = msg->data.error;
 		
 		error = llcache_send_event_to_users(object, &event);
 		
@@ -1933,8 +1941,8 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 	case FETCH_PROGRESS:
 		/* Progress update */
 		event.type = LLCACHE_EVENT_PROGRESS;
-		event.data.msg = data;
-		
+		event.data.msg = msg->data.progress;
+
 		error = llcache_send_event_to_users(object, &event);
 		
 		break;
@@ -1949,7 +1957,7 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 			object->candidate = NULL;
 		}
 
-		error = llcache_fetch_auth(object, data);
+		error = llcache_fetch_auth(object, msg->data.auth.realm);
 		break;
 	case FETCH_CERT_ERR:
 		/* Something went wrong when validating TLS certificates */
@@ -1960,7 +1968,9 @@ void llcache_fetch_callback(fetch_msg msg, void *p, const void *data,
 			object->candidate = NULL;
 		}
 
-		error = llcache_fetch_cert_error(object, data, size);
+		error = llcache_fetch_cert_error(object, 
+				msg->data.cert_err.certs, 
+				msg->data.cert_err.num_certs);
 		break;
 	}
 
@@ -1994,10 +2004,10 @@ nserror llcache_fetch_redirect(llcache_object *object, const char *target,
 	llcache_object *dest;
 	llcache_object_user *user, *next;
 	const llcache_post_data *post = object->fetch.post;
-	char *url, *absurl;
-	char *scheme;
-	char *object_scheme;
-	url_func_result result;
+	nsurl *url;
+	lwc_string *scheme;
+	lwc_string *object_scheme;
+	bool match;
 	/* Extract HTTP response code from the fetch object */
 	long http_code = fetch_http_code(object->fetch.fetch);
 
@@ -2026,57 +2036,41 @@ nserror llcache_fetch_redirect(llcache_object *object, const char *target,
 #undef REDIRECT_LIMIT
 
 	/* Make target absolute */
-	result = url_join(target, object->url, &absurl);
-	if (result != URL_FUNC_OK) {
-		return NSERROR_NOMEM;
-	}
-
-	/* Ensure target is normalised */
-	result = url_normalize(absurl, &url);
-
-	/* No longer require absolute url */
-	free(absurl);
-
-	if (result != URL_FUNC_OK) {
-		return NSERROR_NOMEM;
-	}
+	error = nsurl_join(object->url, target, &url);
+	if (error != NSERROR_OK)
+		return error;
 
 	/* Reject attempts to redirect from unvalidated to validated schemes
 	 * A "validated" scheme is one over which we have some guarantee that
 	 * the source is trustworthy. */
-	result = url_scheme(object->url, &object_scheme);
-	if (result != URL_FUNC_OK) {
-		free(url);
-		return NSERROR_NOMEM;
-	}
-
-	result = url_scheme(url, &scheme);
-	if (result != URL_FUNC_OK) {
-		free(object_scheme);
-		free(url);
-		return NSERROR_NOMEM;
-	}
+	object_scheme = nsurl_get_component(object->url, NSURL_SCHEME);
+	scheme = nsurl_get_component(url, NSURL_SCHEME);
 
 	/* resource: and about: are allowed to redirect anywhere */
-	if ((strcasecmp(object_scheme, "resource") != 0) &&
-	    (strcasecmp(object_scheme, "about") != 0)) {
+	if ((lwc_string_isequal(object_scheme, llcache_resource_lwc,
+			&match) == lwc_error_ok && match == false) &&
+	    (lwc_string_isequal(object_scheme, llcache_about_lwc,
+			&match) == lwc_error_ok && match == false)) {
 		/* file, about and resource are not valid redirect targets */
-		if ((strcasecmp(scheme, "file") == 0) ||
-		    (strcasecmp(scheme, "about") == 0) ||
-		    (strcasecmp(scheme, "resource") == 0)) {
-			free(object_scheme);
-			free(scheme);
-			free(url);
+		if ((lwc_string_isequal(object_scheme, llcache_file_lwc,
+				&match) == lwc_error_ok && match == true) ||
+		    (lwc_string_isequal(object_scheme, llcache_about_lwc,
+				&match) == lwc_error_ok && match == true) ||
+		    (lwc_string_isequal(object_scheme, llcache_resource_lwc,
+				&match) == lwc_error_ok && match == true)) {
+			lwc_string_unref(object_scheme);
+			lwc_string_unref(scheme);
+			nsurl_unref(url);
 			return NSERROR_OK;
 		}
 	}
 
-	free(scheme);
-	free(object_scheme);
+	lwc_string_unref(scheme);
+	lwc_string_unref(object_scheme);
 
 	/* Bail out if we've no way of handling this URL */
 	if (fetch_can_fetch(url) == false) {
-		free(url);
+		nsurl_unref(url);
 		return NSERROR_OK;
 	}
 
@@ -2085,7 +2079,7 @@ nserror llcache_fetch_redirect(llcache_object *object, const char *target,
 		post = NULL;
 	} else if (http_code != 307 || post != NULL) {
 		/** \todo 300, 305, 307 with POST */
-		free(url);
+		nsurl_unref(url);
 		return NSERROR_OK;
 	}
 
@@ -2095,7 +2089,7 @@ nserror llcache_fetch_redirect(llcache_object *object, const char *target,
 			object->fetch.redirect_count + 1, &dest);
 
 	/* No longer require url */
-	free(url);
+	nsurl_unref(url);
 
 	if (error != NSERROR_OK)
 		return error;
@@ -2187,17 +2181,17 @@ nserror llcache_fetch_notmodified(llcache_object *object,
  * \param value	 Pointer to location to receive header value
  * \return NSERROR_OK on success, appropriate error otherwise
  */
-nserror llcache_fetch_split_header(const char *data, size_t len, char **name,
-		char **value)
+nserror llcache_fetch_split_header(const uint8_t *data, size_t len, 
+		char **name, char **value)
 {
 	char *n, *v;
-	const char *colon;
+	const uint8_t *colon;
 
 	/* Find colon */
-	colon = strchr(data, ':');
+	colon = (const uint8_t *) strchr((const char *) data, ':');
 	if (colon == NULL) {
 		/* Failed, assume a key with no value */
-		n = strdup(data);
+		n = strdup((const char *) data);
 		if (n == NULL)
 			return NSERROR_NOMEM;
 
@@ -2221,7 +2215,7 @@ nserror llcache_fetch_split_header(const char *data, size_t len, char **name,
 				colon[-1] == '\n'))
 			colon--;
 
-		n = strndup(data, colon - data);
+		n = strndup((const char *) data, colon - data);
 		if (n == NULL)
 			return NSERROR_NOMEM;
 
@@ -2244,7 +2238,7 @@ nserror llcache_fetch_split_header(const char *data, size_t len, char **name,
 			len--;
 		}
 
-		v = strndup(colon, len - (colon - data));
+		v = strndup((const char *) colon, len - (colon - data));
 		if (v == NULL) {
 			free(n);
 			return NSERROR_NOMEM;
@@ -2271,8 +2265,8 @@ nserror llcache_fetch_split_header(const char *data, size_t len, char **name,
  *	 the cache control data for the object if an interesting
  *	 header is encountered
  */
-nserror llcache_fetch_parse_header(llcache_object *object, const char *data, 
-		size_t len, char **name, char **value)
+nserror llcache_fetch_parse_header(llcache_object *object, 
+		const uint8_t *data, size_t len, char **name, char **value)
 {
 	nserror error;
 
@@ -2363,8 +2357,8 @@ nserror llcache_fetch_parse_header(llcache_object *object, const char *data,
  * \param len	  Byte length of header
  * \return NSERROR_OK on success, appropriate error otherwise
  */
-nserror llcache_fetch_process_header(llcache_object *object, const char *data,
-		size_t len)
+nserror llcache_fetch_process_header(llcache_object *object, 
+		const uint8_t *data, size_t len)
 {
 	nserror error;
 	char *name, *value;
@@ -2382,7 +2376,7 @@ nserror llcache_fetch_process_header(llcache_object *object, const char *data,
 	 * that we might have computed, and start again.
 	 */
 	/** \todo Properly parse the response line */
-	if (strncmp(data, "HTTP/", SLEN("HTTP/")) == 0) {
+	if (strncmp((const char *) data, "HTTP/", SLEN("HTTP/")) == 0) {
 		time_t req_time = object->cache.req_time;
 
 		llcache_invalidate_cache_control_data(object);
@@ -2470,9 +2464,9 @@ nserror llcache_fetch_auth(llcache_object *object, const char *realm)
 	/* If there was no realm, then default to the URL */
 	/** \todo If there was no WWW-Authenticate header, use response body */
 	if (realm == NULL)
-		realm = object->url;
+		realm = nsurl_access(object->url);
 
-	auth = urldb_get_auth_details(object->url, realm);
+	auth = urldb_get_auth_details(nsurl_access(object->url), realm);
 
 	if (auth == NULL || object->fetch.tried_with_auth == true) {
 		/* No authentication details, or tried what we had, so ask */
