@@ -28,6 +28,7 @@
 #include <stdint.h>
 
 #include "utils/errors.h"
+#include "utils/nsurl.h"
 
 struct ssl_cert_info;
 struct fetch_multipart_data;
@@ -106,7 +107,7 @@ typedef enum {
 typedef struct {
 	llcache_query_type type;	/**< Type of query */
 
-	const char *url;		/**< URL being fetched */
+	nsurl *url;			/**< URL being fetched */
 
 	union {
 		struct {
@@ -189,8 +190,8 @@ void llcache_clean(void);
  * \param result   Pointer to location to recieve cache handle
  * \return NSERROR_OK on success, appropriate error otherwise
  */
-nserror llcache_handle_retrieve(const char *url, uint32_t flags,
-		const char *referer, const llcache_post_data *post,
+nserror llcache_handle_retrieve(nsurl *url, uint32_t flags,
+		nsurl *referer, const llcache_post_data *post,
 		llcache_handle_callback cb, void *pw,
 		llcache_handle **result);
 
@@ -253,7 +254,7 @@ nserror llcache_handle_invalidate_cache_data(llcache_handle *handle);
  * \param handle  Handle to retrieve URL from
  * \return Post-redirect URL of cache object
  */
-const char *llcache_handle_get_url(const llcache_handle *handle);
+nsurl *llcache_handle_get_url(const llcache_handle *handle);
 
 /**
  * Retrieve source data of a low-level cache object
