@@ -54,12 +54,14 @@
 #include "desktop/selection.h"
 #include "desktop/textinput.h"
 #include "desktop/plotters.h"
+#include "desktop/js.h"
 
 #include "render/form.h"
 #include "render/html.h"
 #include "render/textplain.h"
 #include "utils/log.h"
 #include "utils/messages.h"
+#include "utils/nsurl.h"
 #include "utils/schedule.h"
 #include "utils/url.h"
 #include "utils/utils.h"
@@ -79,7 +81,6 @@ static void browser_window_convert_to_download(struct browser_window *bw,
 		llcache_handle *stream);
 static void browser_window_start_throbber(struct browser_window *bw);
 static void browser_window_stop_throbber(struct browser_window *bw);
-static void browser_window_set_icon(struct browser_window *bw);
 static void browser_window_destroy_children(struct browser_window *bw);
 static void browser_window_destroy_internal(struct browser_window *bw);
 static void browser_window_set_scale_internal(struct browser_window *bw,
@@ -332,7 +333,6 @@ void browser_window_get_position(struct browser_window *bw, bool root,
 	while (bw) {
 		switch (bw->browser_window_type) {
 
-		case BROWSER_WINDOW_FRAME:
 		case BROWSER_WINDOW_FRAMESET:
 			*pos_x += bw->x * bw->scale;
 			*pos_y += bw->y * bw->scale;
@@ -342,6 +342,9 @@ void browser_window_get_position(struct browser_window *bw, bool root,
 			/* There is no offset to the root browser window */
 			break;
 
+		case BROWSER_WINDOW_FRAME:
+			/* Iframe and Frame handling is identical;
+			 * fall though */
 		case BROWSER_WINDOW_IFRAME:
 			*pos_x += (bw->x - scrollbar_get_offset(bw->scroll_x)) *
 					bw->scale;
@@ -377,16 +380,31 @@ void browser_window_set_position(struct browser_window *bw, int x, int y)
 
 /* exported interface, documented in browser.h */
 void browser_window_set_drag_type(struct browser_window *bw,
-		browser_drag_type type)
+		browser_drag_type type, const struct rect *rect)
 {
 	struct browser_window *top_bw = browser_window_get_root(bw);
-
-	if (type == DRAGGING_NONE)
-		top_bw->drag_window = NULL;
-	else
-		top_bw->drag_window = bw;
+	gui_drag_type gtype;
 
 	bw->drag_type = type;
+
+	if (type == DRAGGING_NONE) {
+		top_bw->drag_window = NULL;
+	} else {
+		top_bw->drag_window = bw;
+
+		switch (type) {
+		case DRAGGING_SCR_X:
+		case DRAGGING_SCR_Y:
+		case DRAGGING_CONTENT_SCROLLBAR:
+			gtype = GDRAGGING_SCROLLBAR;
+			break;
+		default:
+			gtype = GDRAGGING_OTHER;
+			break;
+		}
+
+		gui_window_drag_start(top_bw->window, gtype, rect);
+	}
 }
 
 /* exported interface, documented in browser.h */
@@ -526,6 +544,101 @@ void browser_window_get_contextual_content(struct browser_window *bw,
 	browser_window__get_contextual_content(bw, x, y, data);
 }
 
+/* exported interface, documented in browser.h */
+bool browser_window_scroll_at_point(struct browser_window *bw,
+		int x, int y, int scrx, int scry)
+{
+	bool handled_scroll = false;
+	assert(bw != NULL);
+
+	/* Handle (i)frame scroll offset (core-managed browser windows only) */
+	x += scrollbar_get_offset(bw->scroll_x);
+	y += scrollbar_get_offset(bw->scroll_y);
+
+	if (bw->children) {
+		/* Browser window has children, so pass request on to
+		 * appropriate child */
+		struct browser_window *bwc;
+		int cur_child;
+		int children = bw->rows * bw->cols;
+
+		/* Loop through all children of bw */
+		for (cur_child = 0; cur_child < children; cur_child++) {
+			/* Set current child */
+			bwc = &bw->children[cur_child];
+
+			/* Skip this frame if (x, y) coord lies outside */
+			if (x < bwc->x || bwc->x + bwc->width < x ||
+					y < bwc->y || bwc->y + bwc->height < y)
+				continue;
+
+			/* Pass request into this child */
+			return browser_window_scroll_at_point(bwc,
+					(x - bwc->x), (y - bwc->y),
+					scrx, scry);
+		}
+	}
+
+	/* Try to scroll any current content */
+	if (bw->current_content != NULL && content_scroll_at_point(
+			bw->current_content, x, y, scrx, scry) == true)
+		/* Scroll handled by current content */
+		return true;
+
+	/* Try to scroll this window, if scroll not already handled */
+	if (handled_scroll == false) {
+		if (bw->scroll_y && scrollbar_scroll(bw->scroll_y, scry))
+			handled_scroll = true;
+
+		if (bw->scroll_x && scrollbar_scroll(bw->scroll_x, scrx))
+			handled_scroll = true;
+	}
+
+	return handled_scroll;
+}
+
+/* exported interface, documented in browser.h */
+bool browser_window_drop_file_at_point(struct browser_window *bw,
+		int x, int y, char *file)
+{
+	assert(bw != NULL);
+
+	/* Handle (i)frame scroll offset (core-managed browser windows only) */
+	x += scrollbar_get_offset(bw->scroll_x);
+	y += scrollbar_get_offset(bw->scroll_y);
+
+	if (bw->children) {
+		/* Browser window has children, so pass request on to
+		 * appropriate child */
+		struct browser_window *bwc;
+		int cur_child;
+		int children = bw->rows * bw->cols;
+
+		/* Loop through all children of bw */
+		for (cur_child = 0; cur_child < children; cur_child++) {
+			/* Set current child */
+			bwc = &bw->children[cur_child];
+
+			/* Skip this frame if (x, y) coord lies outside */
+			if (x < bwc->x || bwc->x + bwc->width < x ||
+					y < bwc->y || bwc->y + bwc->height < y)
+				continue;
+
+			/* Pass request into this child */
+			return browser_window_drop_file_at_point(bwc,
+					(x - bwc->x), (y - bwc->y),
+					file);
+		}
+	}
+
+	/* Pass file drop on to any content */
+	if (bw->current_content != NULL)
+		return content_drop_file_at_point(bw->current_content,
+				x, y, file);
+
+	return false;
+}
+
 
 /**
  * Create and open a new root browser window with the given page.
@@ -549,6 +662,8 @@ struct browser_window *browser_window_create(const char *url,
 		return NULL;
 	}
 
+	bw->jsctx = js_newcontext();
+
 	/* Initialise common parts */
 	browser_window_initialise_common(bw, clone);
 
@@ -572,8 +687,9 @@ struct browser_window *browser_window_create(const char *url,
 		return NULL;
 	}
 
-	if (url)
+	if (url) {
 		browser_window_go(bw, url, referer, history_add);
+	}
 
 	
 	return bw;
@@ -702,9 +818,6 @@ void browser_window_go_post(struct browser_window *bw, const char *url,
 		bool verifiable, hlcache_handle *parent)
 {
 	hlcache_handle *c;
-	char *url2;
-	char *fragment;
-	url_func_result res;
 	int depth = 0;
 	struct browser_window *cur;
 	uint32_t fetch_flags = 0;
@@ -713,6 +826,9 @@ void browser_window_go_post(struct browser_window *bw, const char *url,
 	hlcache_child_context child;
 	nserror error;
 
+	nsurl *nsref = NULL;
+	nsurl *nsurl;
+
 	LOG(("bw %p, url %s", bw, url));
 	assert(bw);
 	assert(url);
@@ -742,29 +858,39 @@ void browser_window_go_post(struct browser_window *bw, const char *url,
 		child.quirks = content_get_quirks(parent);
 	}
 
-	/* Normalize the request URL */
-	res = url_normalize(url, &url2);
-	if (res != URL_FUNC_OK) {
-		LOG(("failed to normalize url %s", url));
+	error = nsurl_create(url, &nsurl);
+	if (error != NSERROR_OK) {
 		return;
 	}
 
+	if (referer != NULL) {
+		error = nsurl_create(referer, &nsref);
+		if (error != NSERROR_OK) {
+			nsurl_unref(nsurl);
+			return;
+		}
+	}
+
 	/* Get download out of the way */
 	if (download) {
 		llcache_handle *l;
+		struct browser_window *root;
+
+		root = browser_window_get_root(bw);
+		assert(root != NULL);
 
 		fetch_flags |= LLCACHE_RETRIEVE_FORCE_FETCH;
 		fetch_flags |= LLCACHE_RETRIEVE_STREAM_DATA;
 
-		error = llcache_handle_retrieve(url2, fetch_flags, referer, 
+		error = llcache_handle_retrieve(nsurl, fetch_flags, nsref, 
 				fetch_is_post ? &post : NULL,
 				NULL, NULL, &l);
 		if (error == NSERROR_NO_FETCH_HANDLER) {
-			gui_launch_url(url2);
+			gui_launch_url(nsurl_access(nsurl));
 		} else if (error != NSERROR_OK) {
 			LOG(("Failed to fetch download: %d", error));
 		} else {
-			error = download_context_create(l, bw->window);
+			error = download_context_create(l, root->window);
 			if (error != NSERROR_OK) {
 				LOG(("Failed creating download context: %d", 
 						error));
@@ -773,52 +899,47 @@ void browser_window_go_post(struct browser_window *bw, const char *url,
 			}
 		}
 
-		free(url2);
+		nsurl_unref(nsurl);
+		if (nsref != NULL)
+			nsurl_unref(nsref);
 
 		return;
 	}
 
-	free(bw->frag_id);
+	if (bw->frag_id != NULL)
+		lwc_string_unref(bw->frag_id);
 	bw->frag_id = NULL;
 
-	/* find any fragment identifier on end of URL */
-	res = url_fragment(url2, &fragment);
-	if (res == URL_FUNC_NOMEM) {
-		free(url2);
-		warn_user("NoMemory", 0);
-		return;
-	} else if (res == URL_FUNC_OK) {
+	if (nsurl_has_component(nsurl, NSURL_FRAGMENT)) {
 		bool same_url = false;
 
-		bw->frag_id = fragment;
+		bw->frag_id = nsurl_get_component(nsurl, NSURL_FRAGMENT);
 
 		/* Compare new URL with existing one (ignoring fragments) */
 		if (bw->current_content != NULL && 
-				content_get_url(bw->current_content) != NULL) {
-			res = url_compare(content_get_url(bw->current_content),
-					url2, true, &same_url);
-			if (res == URL_FUNC_NOMEM) {
-				free(url2);
-				warn_user("NoMemory", 0);
-				return;
-			} else if (res == URL_FUNC_FAILED) {
-				same_url = false;
-			}
+				hlcache_handle_get_url(bw->current_content) != NULL) {
+			same_url = nsurl_compare(nsurl,
+					hlcache_handle_get_url(bw->current_content),
+					NSURL_COMPLETE);
 		}
 
 		/* if we're simply moving to another ID on the same page,
 		 * don't bother to fetch, just update the window.
 		 */
 		if (same_url && fetch_is_post == false && 
-				strchr(url2, '?') == 0) {
-			free(url2);
+				nsurl_has_component(nsurl, NSURL_QUERY) ==
+						false) {
+			nsurl_unref(nsurl);
+			if (nsref != NULL)
+				nsurl_unref(nsref);
 			if (add_to_history)
 				history_add(bw->history, bw->current_content,
-						bw->frag_id);
+						bw->frag_id == NULL ? NULL :
+						lwc_string_data(bw->frag_id));
 			browser_window_update(bw, false);
 			if (bw->current_content != NULL) {
 				browser_window_refresh_url_bar(bw,
-					content_get_url(bw->current_content),
+					hlcache_handle_get_url(bw->current_content),
 					bw->frag_id);
 			}
 			return;
@@ -829,38 +950,214 @@ void browser_window_go_post(struct browser_window *bw, const char *url,
 	browser_window_remove_caret(bw);
 	browser_window_destroy_children(bw);
 
-	LOG(("Loading '%s'", url2));
+	LOG(("Loading '%s'", nsurl_access(nsurl)));
 
 	browser_window_set_status(bw, messages_get("Loading"));
 	bw->history_add = add_to_history;
 
-	error = hlcache_handle_retrieve(url2,
-			fetch_flags | HLCACHE_RETRIEVE_MAY_DOWNLOAD | 
-					HLCACHE_RETRIEVE_SNIFF_TYPE, 
-			referer,
+	/* fresh javascript compartment */
+	bw->jsglobal = js_newcompartment(bw->jsctx);
+
+	/* Verifiable fetches may trigger a download */
+	if (verifiable)
+		fetch_flags |= HLCACHE_RETRIEVE_MAY_DOWNLOAD;
+
+	error = hlcache_handle_retrieve(nsurl,
+			fetch_flags | HLCACHE_RETRIEVE_SNIFF_TYPE,
+			nsref,
 			fetch_is_post ? &post : NULL,
 			browser_window_callback, bw,
 			parent != NULL ? &child : NULL,
 			CONTENT_ANY, &c);
 	if (error == NSERROR_NO_FETCH_HANDLER) {
-		gui_launch_url(url2);
-		free(url2);
+		gui_launch_url(nsurl_access(nsurl));
+		nsurl_unref(nsurl);
+		if (nsref != NULL)
+			nsurl_unref(nsref);
 		return;
 	} else if (error != NSERROR_OK) {
-		free(url2);
+		nsurl_unref(nsurl);
+		if (nsref != NULL)
+			nsurl_unref(nsref);
 		browser_window_set_status(bw, messages_get("NoMemory"));
 		warn_user("NoMemory", 0);
 		return;
 	}
 
-	free(url2);
-
 	bw->loading_content = c;
 	browser_window_start_throbber(bw);
-	browser_window_refresh_url_bar(bw, url, NULL);
+	browser_window_refresh_url_bar(bw, nsurl, NULL);
+
+	nsurl_unref(nsurl);
+	if (nsref != NULL)
+		nsurl_unref(nsref);
 }
 
 
+
+/**
+ * Callback for fetchcache() for browser window favicon fetches.
+ */
+
+static nserror browser_window_favicon_callback(hlcache_handle *c,
+		const hlcache_event *event, void *pw)
+{
+	struct browser_window *bw = pw;
+
+	switch (event->type) {
+	case CONTENT_MSG_DONE:
+		if (bw->current_favicon != NULL) {
+			content_status status = 
+					content_get_status(bw->current_favicon);
+
+			if ((status == CONTENT_STATUS_READY) ||
+					(status == CONTENT_STATUS_DONE))
+				content_close(bw->current_favicon);
+
+			hlcache_handle_release(bw->current_favicon);
+		}
+
+		bw->current_favicon = c;
+		bw->loading_favicon = NULL;
+
+		/* content_get_bitmap on the hlcache_handle should give 
+		 *   us the favicon bitmap at this point
+		 */
+		gui_window_set_icon(bw->window, c);
+		break;
+
+	case CONTENT_MSG_ERROR:
+
+		/* clean up after ourselves */
+		if (c == bw->loading_favicon)
+			bw->loading_favicon = NULL;
+		else if (c == bw->current_favicon) {
+			bw->current_favicon = NULL;
+		}
+
+		hlcache_handle_release(c);
+
+		if (bw->failed_favicon == false) {
+			nsurl *nsref = NULL;
+			nsurl *nsurl;
+			nserror error;
+
+			bw->failed_favicon = true;
+
+			error = nsurl_create("resource:favicon.ico", &nsurl);
+			if (error != NSERROR_OK) {
+				LOG(("Unable to create default location url"));
+			} else {
+
+				hlcache_handle_retrieve(nsurl,
+						HLCACHE_RETRIEVE_SNIFF_TYPE, 
+						nsref, NULL,
+						browser_window_favicon_callback,
+						bw, NULL, CONTENT_IMAGE, 
+						&bw->loading_favicon);
+
+				nsurl_unref(nsurl);
+			}
+
+		}
+		break;
+
+	default:
+		break;
+	}
+	return NSERROR_OK;
+}
+
+static void browser_window_update_favicon(hlcache_handle *c,
+		struct browser_window *bw, struct content_rfc5988_link *link)
+{
+	lwc_string *icon_str;
+	nsurl *nsref = NULL;
+	nsurl *nsurl;
+	nserror error;
+
+	assert(c != NULL);
+	assert(bw !=NULL);
+
+	if (bw->window == NULL)
+		/* Not top-level browser window; not interested */
+		return;
+	
+	/* already fetching the favicon - use that */
+	if (bw->loading_favicon != NULL) 
+		return;
+
+	bw->failed_favicon = false;
+
+	if (link == NULL) {
+		/* look for favicon metadata link */
+		if (lwc_intern_string("icon", SLEN("icon"), 
+				&icon_str) == lwc_error_ok) {
+			link = content_find_rfc5988_link(c, icon_str);
+			lwc_string_unref(icon_str);
+		}
+	}
+
+	if (link == NULL) {
+		if (lwc_intern_string("shortcut icon", SLEN("shortcut icon"), 
+				&icon_str) == lwc_error_ok) {
+			link = content_find_rfc5988_link(c, icon_str);
+			lwc_string_unref(icon_str);
+		}
+	}
+
+	if (link == NULL) {
+		lwc_string *scheme;
+		bool speculative_default = false;
+
+		nsurl = hlcache_handle_get_url(c);
+
+		scheme = nsurl_get_component(nsurl, NSURL_SCHEME);
+
+		/* If the document was fetched over http(s), then speculate 
+		 * that there's a favicon living at /favicon.ico */
+		if ((lwc_string_length(scheme) == SLEN("HTTP") &&
+				strcasecmp(lwc_string_data(scheme), 
+						"http") == 0) ||
+		    (lwc_string_length(scheme) == SLEN("HTTPS") &&
+				strcasecmp(lwc_string_data(scheme), 
+						"https") == 0)) {
+			speculative_default = true;
+		}
+
+		lwc_string_unref(scheme);
+
+		if (speculative_default) {
+			/* no favicon via link, try for the default location */
+			error = nsurl_join(nsurl, "/favicon.ico", &nsurl);
+		} else {
+			bw->failed_favicon = true;
+			error = nsurl_create("resource:favicon.ico", &nsurl);
+		}
+		if (error != NSERROR_OK) {
+			LOG(("Unable to create default location url"));
+			return;
+		}
+	} else {
+		nsurl = nsurl_ref(link->href);
+	}
+
+	if (link == NULL) {
+		LOG(("fetching general favicon from '%s'", 
+		     nsurl_access(nsurl)));
+	} else {
+		LOG(("fetching favicon rel:%s '%s'",
+				lwc_string_data(link->rel),
+				nsurl_access(nsurl)));
+	}
+
+	hlcache_handle_retrieve(nsurl, HLCACHE_RETRIEVE_SNIFF_TYPE, 
+			nsref, NULL, browser_window_favicon_callback, 
+			bw, NULL, CONTENT_IMAGE, &bw->loading_favicon);
+
+	nsurl_unref(nsurl);
+}
+
 /**
  * Callback for fetchcache() for browser window fetches.
  */
@@ -878,8 +1175,8 @@ nserror browser_window_callback(hlcache_handle *c,
 
 		if (bw->current_content != NULL) {
 			browser_window_refresh_url_bar(bw,
-				content_get_url(bw->current_content),
-				bw->frag_id);
+					hlcache_handle_get_url(bw->current_content),
+					bw->frag_id);
 		}
 		break;
 
@@ -930,7 +1227,7 @@ nserror browser_window_callback(hlcache_handle *c,
 			gui_window_new_content(bw->window);
 
 		browser_window_refresh_url_bar(bw,
-				content_get_url(bw->current_content),
+				hlcache_handle_get_url(bw->current_content),
 				bw->frag_id);
 
 		/* new content; set scroll_to_top */
@@ -940,9 +1237,10 @@ nserror browser_window_callback(hlcache_handle *c,
 
 		/* history */
 		if (bw->history_add && bw->history) {
-			const char *url = content_get_url(c);
+			const char *url = nsurl_access(hlcache_handle_get_url(c));
 
-			history_add(bw->history, c, bw->frag_id);
+			history_add(bw->history, c, bw->frag_id == NULL ? NULL :
+					lwc_string_data(bw->frag_id));
 			if (urldb_add_url(url)) {
 				urldb_set_url_title(url, content_get_title(c));
 				urldb_update_url_visit_data(url);
@@ -983,7 +1281,7 @@ nserror browser_window_callback(hlcache_handle *c,
 		browser_window_update(bw, false);
 		browser_window_set_status(bw, content_get_status_message(c));
 		browser_window_stop_throbber(bw);
-		browser_window_set_icon(bw);
+		browser_window_update_favicon(c, bw, NULL);
 
 		history_update(bw->history, c);
 		hotlist_visited(c);
@@ -1045,13 +1343,6 @@ nserror browser_window_callback(hlcache_handle *c,
 			.y1 = event->data.redraw.y + event->data.redraw.height
 		};
 
-		if (bw->browser_window_type == BROWSER_WINDOW_FRAME) {
-			rect.x0 -= scrollbar_get_offset(bw->scroll_x);
-			rect.y0 -= scrollbar_get_offset(bw->scroll_y);
-			rect.x1 -= scrollbar_get_offset(bw->scroll_x);
-			rect.y1 -= scrollbar_get_offset(bw->scroll_y);
-		}
-
 		browser_window_update_box(bw, &rect);
 	}
 		break;
@@ -1060,12 +1351,36 @@ nserror browser_window_callback(hlcache_handle *c,
 		bw->refresh_interval = event->data.delay * 100;
 		break;
 		
-	case CONTENT_MSG_FAVICON_REFRESH:
-		/* Cause the GUI to update */
-		if (bw->browser_window_type == BROWSER_WINDOW_NORMAL) {
-			gui_window_set_icon(bw->window,
-					html_get_favicon(bw->current_content));
+	case CONTENT_MSG_LINK: /* content has an rfc5988 link element */
+	{
+		lwc_string *icon_str;
+		lwc_string *shortcut_icon_str;
+		bool icon_match = false;
+		bool shortcut_icon_match = false;
+
+		if (lwc_intern_string("icon", SLEN("icon"), 
+				&icon_str) == lwc_error_ok) {
+			lwc_string_caseless_isequal(
+					event->data.rfc5988_link->rel, 
+					icon_str, &icon_match);
+			lwc_string_unref(icon_str);
 		}
+
+		if (lwc_intern_string("shortcut icon", SLEN("shortcut icon"), 
+				&shortcut_icon_str) == lwc_error_ok) {
+			lwc_string_caseless_isequal(
+					event->data.rfc5988_link->rel, 
+					shortcut_icon_str, 
+					&shortcut_icon_match);
+			lwc_string_unref(shortcut_icon_str);
+		}
+
+		if (icon_match || shortcut_icon_match) {
+			/* it's a favicon perhaps start a fetch for it */
+			browser_window_update_favicon(c, bw,
+					event->data.rfc5988_link);
+		}
+	}
 		break;
 
 	default:
@@ -1132,14 +1447,15 @@ void browser_window_set_dimensions(struct browser_window *bw,
 void browser_window_convert_to_download(struct browser_window *bw,
 		llcache_handle *stream)
 {
+	struct browser_window *root = browser_window_get_root(bw);
 	nserror error;
 
-	error = download_context_create(stream, bw->window);
+	assert(root != NULL);
+
+	error = download_context_create(stream, root->window);
 	if (error != NSERROR_OK) {
 		llcache_handle_abort(stream);
 		llcache_handle_release(stream);
-
-		return;
 	}
 
 	/* remove content from browser window */
@@ -1171,14 +1487,14 @@ void browser_window_refresh(void *p)
 
 	/* Ignore if the refresh URL has gone
 	 * (may happen if a fetch error occurred) */
-	refresh = content_get_refresh_url(bw->current_content);
+	refresh = nsurl_access(content_get_refresh_url(bw->current_content));
 	if (refresh == NULL)
 		return;
 
 	/* mark this content as invalid so it gets flushed from the cache */
 	content_invalidate_reuse_data(bw->current_content);
 
-	url = content_get_url(bw->current_content);
+	url = nsurl_access(hlcache_handle_get_url(bw->current_content));
 	if (url != NULL && strcmp(url, refresh) == 0)
 		history_add = false;
 
@@ -1255,24 +1571,6 @@ bool browser_window_check_throbber(struct browser_window *bw)
 	return false;
 }
 
-/**
- * when ready, set icon at top level
- * \param bw browser_window
- * current implementation ignores lower-levels' link rels completely
- */
-void browser_window_set_icon(struct browser_window *bw)
-{
-	while (bw->parent)
-		bw = bw->parent;
-
-	if (bw->current_content != NULL && 
-			content_get_type(bw->current_content) == CONTENT_HTML)
-		gui_window_set_icon(bw->window,
-				html_get_favicon(bw->current_content));
-	else
-		gui_window_set_icon(bw->window, NULL);
-}
-
 /**
  * Redraw browser window, set extent to content, and update title.
  *
@@ -1343,10 +1641,10 @@ void browser_window_update(struct browser_window *bw, bool scroll_to_top)
 			browser_window_set_scroll(bw, x, y);
 		}
 
-		rect.x0 = 0;
-		rect.y0 = 0;
-		rect.x1 = bw->width;
-		rect.y1 = bw->height;
+		rect.x0 = scrollbar_get_offset(bw->scroll_x);
+		rect.y0 = scrollbar_get_offset(bw->scroll_y);
+		rect.x1 = rect.x0 + bw->width;
+		rect.y1 = rect.y0 + bw->height;
 
 		browser_window_update_box(bw, &rect);
 	}
@@ -1427,7 +1725,8 @@ void browser_window_stop(struct browser_window *bw)
 
 	if (bw->current_content != NULL) {
 		browser_window_refresh_url_bar(bw, 
-				content_get_url(bw->current_content), bw->frag_id);
+				hlcache_handle_get_url(bw->current_content),
+				bw->frag_id);
 	}
 
 	browser_window_stop_throbber(bw);
@@ -1478,7 +1777,8 @@ void browser_window_reload(struct browser_window *bw, bool all)
 
 	content_invalidate_reuse_data(bw->current_content);
 
-	browser_window_go(bw, content_get_url(bw->current_content), 0, false);
+	browser_window_go(bw, nsurl_access(
+			hlcache_handle_get_url(bw->current_content)), 0, false);
 }
 
 
@@ -1617,6 +1917,10 @@ void browser_window_destroy_internal(struct browser_window *bw)
 		}
 	}
 
+	/* Destroying a search context causes it to redraw any deselected,
+	 * content areas, so do this first */
+	browser_window_search_destroy_context(bw);
+
 	/* Destruction order is important: we must ensure that the frontend 
 	 * destroys any window(s) associated with this browser window before 
 	 * we attempt any destructive cleanup. 
@@ -1643,20 +1947,44 @@ void browser_window_destroy_internal(struct browser_window *bw)
 		bw->current_content = NULL;
 	}
 
+	if (bw->loading_favicon != NULL) {
+		hlcache_handle_abort(bw->loading_favicon);
+		hlcache_handle_release(bw->loading_favicon);
+		bw->loading_favicon = NULL;
+	}
+
+	if (bw->current_favicon != NULL) {
+		content_status status = content_get_status(bw->current_favicon);
+
+		if (status == CONTENT_STATUS_READY ||
+		    status == CONTENT_STATUS_DONE)
+			content_close(bw->current_favicon);
+
+		hlcache_handle_release(bw->current_favicon);
+		bw->current_favicon = NULL;
+	}
+
 	if (bw->box != NULL) {
 		bw->box->iframe = NULL;
 		bw->box = NULL;
 	}
 
+	if (bw->jsctx != NULL) {
+		js_destroycontext(bw->jsctx);
+	}
+
 	/* These simply free memory, so are safe here */
+
+	if (bw->frag_id != NULL)
+		lwc_string_unref(bw->frag_id);
+
 	history_destroy(bw->history);
 
 	free(bw->name);
-	free(bw->frag_id);
 	free(bw->status_text);
 	bw->status_text = NULL;
-	LOG(("Status text cache match:miss %d:%d", 
-	     bw->status_match, bw->status_miss));
+	LOG(("Status text cache match:miss %d:%d",
+			bw->status_match, bw->status_miss));
 }
 
 
@@ -1728,12 +2056,12 @@ void browser_window_set_scale_internal(struct browser_window *bw, float scale)
 	c = bw->current_content;
 
 	if (c != NULL) {
-	  	if (content_can_reformat(c) == false) {
+		if (content_can_reformat(c) == false) {
 			browser_window_update(bw, false);
-	  	} else {
+		} else {
 			bw->reformat_pending = true;
 			browser_reformat_pending = true;
-	 	}
+		}
 	}
 
 	for (i = 0; i < (bw->cols * bw->rows); i++)
@@ -1751,11 +2079,9 @@ void browser_window_set_scale_internal(struct browser_window *bw, float scale)
  * \param frag	Additional fragment. May be NULL if none.
  */
 
-void browser_window_refresh_url_bar(struct browser_window *bw, const char *url,
-		const char *frag)
+void browser_window_refresh_url_bar(struct browser_window *bw, nsurl *url,
+		lwc_string *frag)
 {
-	char *url_buf;
-
 	assert(bw);
 	assert(url);
 
@@ -1768,20 +2094,19 @@ void browser_window_refresh_url_bar(struct browser_window *bw, const char *url,
 		/* With no fragment, we may as well pass url straight through
 		 * saving a malloc, copy, free cycle.
 		 */
-		gui_window_set_url(bw->window, url);
+		gui_window_set_url(bw->window, nsurl_access(url));
 	} else {
-		url_buf = malloc(strlen(url) + 1 /* # */ +
-				strlen(frag) + 1 /* \0 */);
-		if (url_buf != NULL) {
-			/* This sprintf is safe because of the above size
-			 * calculation, thus we don't need snprintf
-			 */
-			sprintf(url_buf, "%s#%s", url, frag);
-			gui_window_set_url(bw->window, url_buf);
-			free(url_buf);
-		} else {
+		nsurl *display_url;
+		nserror error;
+
+		error = nsurl_refragment(url, frag, &display_url);
+		if (error != NSERROR_OK) {
 			warn_user("NoMemory", 0);
+			return;
 		}
+
+		gui_window_set_url(bw->window, nsurl_access(display_url));
+		nsurl_unref(display_url);
 	}
 }
 
@@ -1980,9 +2305,6 @@ void browser_window_mouse_track(struct browser_window *bw,
 		browser_window_get_position(drag_bw, true, &off_x, &off_y);
 
 		if (drag_bw->browser_window_type == BROWSER_WINDOW_FRAME) {
-			off_x -= scrollbar_get_offset(drag_bw->scroll_x);
-			off_y -= scrollbar_get_offset(drag_bw->scroll_y);
-
 			browser_window_mouse_track(drag_bw, mouse,
 					x - off_x, y - off_y);
 
@@ -2244,10 +2566,8 @@ void browser_window_mouse_drag_end(struct browser_window *bw,
 
 	switch (bw->drag_type) {
 	case DRAGGING_SELECTION:
-		/* Drag handled by content handler */
-		break;
-
 	case DRAGGING_OTHER:
+	case DRAGGING_CONTENT_SCROLLBAR:
 		/* Drag handled by content handler */
 		break;
 
@@ -2276,7 +2596,7 @@ void browser_window_mouse_drag_end(struct browser_window *bw,
 		break;
 
 	default:
-		browser_window_set_drag_type(bw, DRAGGING_NONE);
+		browser_window_set_drag_type(bw, DRAGGING_NONE, NULL);
 		break;
 	}
 }
@@ -2309,7 +2629,7 @@ void browser_window_redraw_rect(struct browser_window *bw, int x, int y,
 
 void browser_window_page_drag_start(struct browser_window *bw, int x, int y)
 {
-	bw->drag_type = DRAGGING_PAGE_SCROLL;
+	browser_window_set_drag_type(bw, DRAGGING_PAGE_SCROLL, NULL);
 
 	bw->drag_start_x = x;
 	bw->drag_start_y = y;
