@@ -63,6 +63,7 @@ typedef enum {
 	DRAGGING_FRAME,
 	DRAGGING_SCR_X,
 	DRAGGING_SCR_Y,
+	DRAGGING_CONTENT_SCROLLBAR,
 	DRAGGING_OTHER
 } browser_drag_type;
 
@@ -75,6 +76,13 @@ struct browser_window {
 	/** Page being loaded, or 0. */
 	struct hlcache_handle *loading_content;
 
+	/** Page Favicon */
+	struct hlcache_handle *current_favicon;
+	/** handle for favicon which we started loading early */
+	struct hlcache_handle *loading_favicon;
+	/** favicon fetch already failed - prevents infinite error looping */
+	bool failed_favicon;
+
 	/** Window history structure. */
 	struct history *history;
 
@@ -99,7 +107,7 @@ struct browser_window {
 	bool history_add;
 
 	/** Fragment identifier for current_content. */
-	char *frag_id;
+	lwc_string *frag_id;
 
 	/** Current drag status. */
 	browser_drag_type drag_type;
@@ -190,6 +198,11 @@ struct browser_window {
 	/** Current context for free text search, or NULL if none */
 	struct search_context *cur_search;
 
+	/** current javascript context */
+	struct jscontext *jsctx;
+	/** current global javascript object */
+	struct jsobject *jsglobal;
+
 	/** cache of the currently displayed status text. */
 	char *status_text; /**< Current status bar text. */
 	int status_text_len; /**< Length of the ::status_text buffer. */
@@ -242,8 +255,35 @@ void browser_window_set_scale(struct browser_window *bw, float scale, bool all);
 void browser_window_get_contextual_content(struct browser_window *bw,
 		int x, int y, struct contextual_content *data);
 
-void browser_window_refresh_url_bar(struct browser_window *bw, const char *url,
-		const char *frag);
+/**
+ * Send a scroll request to a browser window at a particular point.  The
+ * 'deepest' scrollable object which can be scrolled in the requested
+ * direction at the given point will consume the scroll.
+ *
+ * \param bw	browser window to look inside
+ * \param x	x-coordinate of point of interest
+ * \param y	y-coordinate of point of interest
+ * \param scrx	number of px try to scroll something in x direction
+ * \param scry	number of px try to scroll something in y direction
+ * \return true iff scroll request has been consumed
+ */
+bool browser_window_scroll_at_point(struct browser_window *bw,
+		int x, int y, int scrx, int scry);
+
+/**
+ * Drop a file onto a browser window at a particular point.
+ *
+ * \param bw	browser window to look inside
+ * \param x	x-coordinate of point of interest
+ * \param y	y-coordinate of point of interest
+ * \param file	path to file to be dropped
+ * \return true iff file drop has been handled
+ */
+bool browser_window_drop_file_at_point(struct browser_window *bw,
+		int x, int y, char *file);
+
+void browser_window_refresh_url_bar(struct browser_window *bw, nsurl *url,
+		lwc_string *frag);
 
 void browser_window_mouse_click(struct browser_window *bw,
 		browser_mouse_state mouse, int x, int y);
@@ -374,9 +414,10 @@ void browser_window_set_scroll(struct browser_window *bw, int x, int y);
  *
  * \param  bw     browser window to set the type of the current drag for
  * \param  type   drag type
+ * \param  rect   area pointer may be confined to, during drag, or NULL
  */
 void browser_window_set_drag_type(struct browser_window *bw,
-		browser_drag_type type);
+		browser_drag_type type, const struct rect *rect);
 
 /*
  * Get the root level browser window
