@@ -66,23 +66,31 @@ void browser_window_scroll_callback(void *client_data,
 		} else {
 			struct rect rect;
 
-			rect.x0 = 0;
-			rect.y0 = 0;
-			rect.x1 = bw->width;
-			rect.y1 = bw->height;
+			rect.x0 = scrollbar_get_offset(bw->scroll_x);
+			rect.y0 = scrollbar_get_offset(bw->scroll_y);
+			rect.x1 = rect.x0 + bw->width;
+			rect.y1 = rect.y0 + bw->height;
 
 			browser_window_update_box(bw, &rect);
 		}
 		break;
 	case SCROLLBAR_MSG_SCROLL_START:
-		if (scrollbar_is_horizontal(scrollbar_data->scrollbar))
-			browser_window_set_drag_type(bw, DRAGGING_SCR_X);
-		else
-			browser_window_set_drag_type(bw, DRAGGING_SCR_Y);
+	{
+		struct rect rect = {
+			.x0 = scrollbar_data->x0,
+			.y0 = scrollbar_data->y0,
+			.x1 = scrollbar_data->x1,
+			.y1 = scrollbar_data->y1
+		};
 
+		if (scrollbar_is_horizontal(scrollbar_data->scrollbar))
+			browser_window_set_drag_type(bw, DRAGGING_SCR_X, &rect);
+		else
+			browser_window_set_drag_type(bw, DRAGGING_SCR_Y, &rect);
+	}
 		break;
 	case SCROLLBAR_MSG_SCROLL_FINISHED:
-		browser_window_set_drag_type(bw, DRAGGING_NONE);
+		browser_window_set_drag_type(bw, DRAGGING_NONE, NULL);
 
 		browser_window_set_pointer(bw, GUI_POINTER_DEFAULT);
 		break;
@@ -242,8 +250,10 @@ void browser_window_create_iframes(struct browser_window *bw,
 		window = &(bw->iframes[index++]);
 		if (cur->url) {
 			/* fetch iframe's content */
-			browser_window_go_unverifiable(window, cur->url,
-					content_get_url(bw->current_content),
+			browser_window_go_unverifiable(window,
+					nsurl_access(cur->url),
+					nsurl_access(hlcache_handle_get_url(
+							bw->current_content)),
 					false, bw->current_content);
 		}
 	}
@@ -375,8 +385,9 @@ void browser_window_create_frameset(struct browser_window *bw,
 
 			if (frame->url) {
 				browser_window_go_unverifiable(window,
-						frame->url,
-						content_get_url(parent),
+						nsurl_access(frame->url),
+						nsurl_access(hlcache_handle_get_url(
+								parent)),
 						true,
 						parent);
 			}
@@ -399,7 +410,7 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 	int row, row2, col, index;
 	struct browser_window *window;
 	float relative;
-	int size, extent;
+	int size, extent, applied;
 	int x, y;
 	int new_width, new_height;
 
@@ -459,14 +470,18 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 			avail_width -= widths[col][row];
 		}
 
-		/* try to distribute remainder to relative values in preference */
+		/* Redistribute to fit window */
 		if ((relative > 0) && (avail_width > 0)) {
+			/* Expand the relative sections to fill remainder */
 			for (col = 0; col < bw->cols; col++) {
 				index = (row * bw->cols) + col;
 				window = &bw->children[index];
 
-				if (window->frame_width.unit == FRAME_DIMENSION_RELATIVE) {
-					size = avail_width * window->frame_width.value / relative;
+				if (window->frame_width.unit ==
+						FRAME_DIMENSION_RELATIVE) {
+					size = avail_width * window->
+							frame_width.value /
+							relative;
 					avail_width -= size;
 					relative -= window->frame_width.value;
 					widths[col][row] += size;
@@ -474,13 +489,22 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 			}
 		} else if (bw_width != avail_width) {
 			/* proportionally distribute error */
-			extent = bw_width - avail_width;
+			extent = avail_width;
+			applied = 0;
 			for (col = 0; col < bw->cols; col++) {
 				if (col == bw->cols - 1) {
-					widths[col][row] = bw_width;
+					/* Last cell, use up remainder */
+					widths[col][row] += extent - applied;
+					widths[col][row] =
+							widths[col][row] < 0 ?
+							0 : widths[col][row];
 				} else {
-					size = bw_width * widths[col][row] / extent;
-					widths[col][row] = size;
+					/* Find size of cell adjustment */
+					size = (widths[col][row] * extent) /
+							(bw_width - extent);
+					/* Modify cell */
+					widths[col][row] += size;
+					applied += size;
 				}
 			}
 		}
@@ -529,14 +553,18 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 		if (avail_height == 0)
 			continue;
 
-		/* try to distribute remainder to relative values in preference */
+		/* Redistribute to fit window */
 		if ((relative > 0) && (avail_height > 0)) {
+			/* Expand the relative sections to fill remainder */
 			for (row = 0; row < bw->rows; row++) {
 				index = (row * bw->cols) + col;
 				window = &bw->children[index];
 
-				if (window->frame_height.unit == FRAME_DIMENSION_RELATIVE) {
-					size = avail_height * window->frame_height.value / relative;
+				if (window->frame_height.unit ==
+						FRAME_DIMENSION_RELATIVE) {
+					size = avail_height * window->
+							frame_height.value /
+							relative;
 					avail_height -= size;
 					relative -= window->frame_height.value;
 					heights[col][row] += size;
@@ -544,13 +572,22 @@ void browser_window_recalculate_frameset(struct browser_window *bw) {
 			}
 		} else if (bw_height != avail_height) {
 			/* proportionally distribute error */
-			extent = bw_height - avail_height;
+			extent = avail_height;
+			applied = 0;
 			for (row = 0; row < bw->rows; row++) {
 				if (row == bw->rows - 1) {
-					heights[col][row] = bw_height;
+					/* Last cell, use up remainder */
+					heights[col][row] += extent - applied;
+					heights[col][row] =
+							heights[col][row] < 0 ?
+							0 : heights[col][row];
 				} else {
-					size = bw_height * heights[col][row] / extent;
-					heights[col][row] = size;
+					/* Find size of cell adjustment */
+					size = (heights[col][row] * extent) /
+							(bw_height - extent);
+					/* Modify cell */
+					heights[col][row] += size;
+					applied += size;
 				}
 			}
 		}
@@ -624,7 +661,8 @@ void browser_window_resize_frame(struct browser_window *bw, int x, int y) {
 	else if (bw->drag_resize_right)
 		sibling = &parent->children[row * parent->cols + (col + 1)];
 	if (sibling)
-		change |= browser_window_resolve_frame_dimension(bw, sibling, x, y, true, false);
+		change |= browser_window_resolve_frame_dimension(bw, sibling,
+				x, y, true, false);
 
 	sibling = NULL;
 	if (bw->drag_resize_up)
@@ -632,14 +670,16 @@ void browser_window_resize_frame(struct browser_window *bw, int x, int y) {
 	else if (bw->drag_resize_down)
 		sibling = &parent->children[(row + 1) * parent->cols + col];
 	if (sibling)
-		change |= browser_window_resolve_frame_dimension(bw, sibling, x, y, false, true);
+		change |= browser_window_resolve_frame_dimension(bw, sibling,
+				x, y, false, true);
 
 	if (change)
 		browser_window_recalculate_frameset(parent);
 }
 
 
-bool browser_window_resolve_frame_dimension(struct browser_window *bw, struct browser_window *sibling,
+bool browser_window_resolve_frame_dimension(struct browser_window *bw,
+		struct browser_window *sibling,
 		int x, int y, bool width, bool height) {
 	int bw_dimension, sibling_dimension;
 	int bw_pixels, sibling_pixels;
@@ -755,8 +795,10 @@ bool browser_window_resolve_frame_dimension(struct browser_window *bw, struct br
 }
 
 
-bool browser_window_resize_frames(struct browser_window *bw, browser_mouse_state mouse, int x, int y,
-		gui_pointer_shape *pointer, const char **status, bool *action) {
+bool browser_window_resize_frames(struct browser_window *bw,
+		browser_mouse_state mouse, int x, int y,
+		gui_pointer_shape *pointer, const char **status,
+		bool *action) {
 	struct browser_window *parent;
 	bool left, right, up, down;
 	int i, resize_margin;
@@ -842,8 +884,13 @@ bool browser_window_resize_frames(struct browser_window *bw, browser_mouse_state
 			} else {
 				*pointer = GUI_POINTER_DOWN;
 			}
-			if (mouse & (BROWSER_MOUSE_DRAG_1 | BROWSER_MOUSE_DRAG_2)) {
-				browser_window_set_drag_type(bw, DRAGGING_FRAME);
+			if (mouse & (BROWSER_MOUSE_DRAG_1 |
+					BROWSER_MOUSE_DRAG_2)) {
+
+				/* TODO: Pass appropriate rectangle to allow
+				 *	 front end to clamp pointer range */
+				browser_window_set_drag_type(bw,
+						DRAGGING_FRAME, NULL);
 				bw->drag_start_x = x;
 				bw->drag_start_y = y;
 				bw->drag_resize_left = left;
@@ -851,15 +898,6 @@ bool browser_window_resize_frames(struct browser_window *bw, browser_mouse_state
 				bw->drag_resize_up = up;
 				bw->drag_resize_down = down;
 
-				/* TODO: Tell the front end the valid pointer
-				 *       movement range for the drag, so that
-				 *       they can clamp pointer.
-				 *
-				 *       Probably need a general function for
-				 *       this, to be used by all core-managed
-				 *       drag ops.
-				 */
-
 				*status = messages_get("FrameDrag");
 				*action = true;
 			}
@@ -869,14 +907,14 @@ bool browser_window_resize_frames(struct browser_window *bw, browser_mouse_state
 
 	if (bw->children) {
 		for (i = 0; i < (bw->cols * bw->rows); i++)
-			if (browser_window_resize_frames(&bw->children[i], mouse, x, y, pointer, status,
-					action))
+			if (browser_window_resize_frames(&bw->children[i],
+					mouse, x, y, pointer, status, action))
 				return true;
 	}
 	if (bw->iframes) {
 		for (i = 0; i < bw->iframe_count; i++)
-			if (browser_window_resize_frames(&bw->iframes[i], mouse, x, y, pointer, status,
-					action))
+			if (browser_window_resize_frames(&bw->iframes[i],
+					mouse, x, y, pointer, status, action))
 				return true;
 	}
 	return false;
