@@ -41,6 +41,12 @@ typedef enum {
 	GUI_SAVE_CLIPBOARD_CONTENTS
 } gui_save_type;
 
+typedef enum {
+	GDRAGGING_NONE,
+	GDRAGGING_SCROLLBAR,
+	GDRAGGING_OTHER
+} gui_drag_type;
+
 struct gui_window;
 struct gui_download_window;
 struct browser_window;
@@ -71,7 +77,6 @@ typedef enum { GUI_POINTER_DEFAULT, GUI_POINTER_POINT, GUI_POINTER_CARET,
 #include <stdio.h>
 bool nslog_ensure(FILE *fptr);
 
-void gui_multitask(void);
 void gui_poll(bool active);
 void gui_quit(void);
 
@@ -101,8 +106,10 @@ void gui_window_place_caret(struct gui_window *g, int x, int y, int height);
 void gui_window_remove_caret(struct gui_window *g);
 void gui_window_new_content(struct gui_window *g);
 bool gui_window_scroll_start(struct gui_window *g);
-bool gui_window_box_scroll_start(struct gui_window *g,
-		int x0, int y0, int x1, int y1);
+
+bool gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+		const struct rect *rect);
+
 void gui_window_save_link(struct gui_window *g, const char *url, 
 		const char *title);
 
@@ -137,6 +144,19 @@ void gui_cert_verify(const char *url, const struct ssl_cert_info *certs,
 		unsigned long num, nserror (*cb)(bool proceed, void *pw),
 		void *cbpw);
 
+/**
+ * Callback to translate resource to full url.
+ *
+ * Transforms a resource: path into a full URL. The returned URL
+ * is used as the target for a redirect. The caller takes ownership of
+ * the returned nsurl including unrefing it when finished with it.
+ *
+ * \param path The path of the resource to locate.
+ * \return A string containing the full URL of the target object or
+ *         NULL if no suitable resource can be found.
+ */
+nsurl* gui_get_resource_url(const char *path);
+
 /** css callback to obtain named system colours from a frontend. */
 css_error gui_system_colour(void *pw, lwc_string *name, css_color *color);
 
