@@ -34,7 +34,7 @@ struct redraw_context;
 struct history *history_create(void);
 struct history *history_clone(struct history *history);
 void history_add(struct history *history, struct hlcache_handle *content,
-		char *frag_id);
+		const char *frag_id);
 void history_update(struct history *history, struct hlcache_handle *content);
 void history_destroy(struct history *history);
 void history_back(struct browser_window *bw, struct history *history);
