@@ -223,11 +223,12 @@ static void hotlist_visited_internal(hlcache_handle *content, struct node *node)
 	const char *url;
 
 	if (content == NULL || 
-	    content_get_url(content) == NULL ||
+	    hlcache_handle_get_url(content) == NULL ||
 	    hotlist_tree == NULL)
 		return;
 
-	url = content_get_url(content);
+	/* TODO: do this with a nsurl instead */
+	url = nsurl_access(hlcache_handle_get_url(content));
 
 	for (; node; node = tree_node_get_next(node)) {
 		if (!tree_node_is_folder(node)) {
@@ -366,10 +367,12 @@ void hotlist_collapse_addresses(void)
 
 /**
  * Add a folder node.
+ *
+ * \param selected create the folder in the currently-selected node
  */
-void hotlist_add_folder(void)
+void hotlist_add_folder(bool selected)
 {
-	struct node *node;
+	struct node *node, *parent = NULL;
 	struct node_element *element;
 	char *title = strdup("Untitled");
 
@@ -379,7 +382,19 @@ void hotlist_add_folder(void)
 		return;
 	}
 	creating_node = true;
-	node = tree_create_folder_node(hotlist_tree, hotlist_tree_root, title,
+
+	if (selected == true) {
+		parent = tree_get_selected_node(tree_get_root(hotlist_tree));
+		if (parent && (tree_node_is_folder(parent) == false)) {
+			parent = tree_node_get_parent(parent);
+		}
+	}
+
+	if (parent == NULL) {
+		parent = tree_get_default_folder_node(hotlist_tree);
+	}
+
+	node = tree_create_folder_node(hotlist_tree, parent, title,
 				       true, false, false);
 	if (node == NULL) {
 		free(title);
@@ -393,12 +408,27 @@ void hotlist_add_folder(void)
 
 /**
  * Add an entry node.
+ *
+ * \param selected add the entry in the currently-selected node
  */
-void hotlist_add_entry(void)
+void hotlist_add_entry(bool selected)
 {
 	struct node *node;
+	struct node *parent = NULL;
 	creating_node = true;
-	node = tree_create_URL_node(hotlist_tree, hotlist_tree_root, "Address",
+
+	if (selected == true) {
+		parent = tree_get_selected_node(tree_get_root(hotlist_tree));
+		if (parent && (tree_node_is_folder(parent) == false)) {
+			parent = tree_node_get_parent(parent);
+		}
+	}
+
+	if (parent == NULL) {
+		parent = tree_get_default_folder_node(hotlist_tree);
+	}
+
+	node = tree_create_URL_node(hotlist_tree, parent, "Address",
 				    "Untitled", hotlist_node_callback, NULL);
 
 	if (node == NULL)
@@ -413,7 +443,7 @@ void hotlist_add_entry(void)
 void hotlist_add_page(const char *url)
 {
 	const struct url_data *data;
-	struct node *node;
+	struct node *node, *parent;
 
 	if (url == NULL)
 		return;
@@ -421,13 +451,14 @@ void hotlist_add_page(const char *url)
 	if (data == NULL)
 		return;
 
-	node = tree_create_URL_node(hotlist_tree, hotlist_tree_root, url, NULL,
+	parent = tree_get_default_folder_node(hotlist_tree);
+	node = tree_create_URL_node(hotlist_tree, parent, url, NULL,
 				    hotlist_node_callback, NULL);
 	tree_update_URL_node(hotlist_tree, node, url, data);
 }
 
 /**
- * Adds the currently viewed page to the hotlist at the given cooridinates
+ * Adds the currently viewed page to the hotlist at the given co-ordinates
  * \param url	url of the page
  * \param x	X cooridinate with respect to tree origin
  * \param y	Y cooridinate with respect to tree origin
@@ -461,3 +492,18 @@ void hotlist_launch_selected(bool tabs)
 {
 	tree_launch_selected(hotlist_tree, tabs);
 }
+
+/**
+ * Set the hotlist's default folder to the selected node.
+ *
+ * \param  clear  reset the default to tree root
+ */
+bool hotlist_set_default_folder(bool clear)
+{
+	if (clear == true) {
+		tree_clear_default_folder_node(hotlist_tree);
+		return true;
+	} else {
+		return tree_set_default_folder_node(hotlist_tree, NULL);
+	}
+}
