@@ -53,10 +53,10 @@ void hotlist_expand_addresses(void);
 void hotlist_collapse_all(void);
 void hotlist_collapse_directories(void);
 void hotlist_collapse_addresses(void);
-void hotlist_add_folder(void);
-void hotlist_add_entry(void);
+void hotlist_add_folder(bool selected);
+void hotlist_add_entry(bool selected);
 void hotlist_add_page(const char *url);
 void hotlist_add_page_xy(const char *url, int x, int y);
 void hotlist_launch_selected(bool tabs);
-		
+bool hotlist_set_default_folder(bool clear);
 #endif
