@@ -27,7 +27,6 @@
 #include <stdbool.h>
 #include <string.h>
 
-#include "content/hlcache.h"
 #include "desktop/gui.h"
 #include "desktop/mouse.h"
 #include "desktop/plotters.h"
@@ -36,6 +35,7 @@
 #include "render/box.h"
 #include "render/font.h"
 #include "render/form.h"
+#include "render/html_internal.h"
 #include "render/textplain.h"
 #include "utils/log.h"
 #include "utils/utf8.h"
@@ -87,22 +87,56 @@ static bool traverse_tree(struct box *box, unsigned start_idx, unsigned end_idx,
 		bool do_marker);
 static struct box *get_box(struct box *b, unsigned offset, size_t *pidx);
 
+
+/**
+ * Get the browser window containing the content a selection object belongs to.
+ *
+ * \param  s	selection object
+ * \return the browser window
+ */
+static struct browser_window * selection_get_browser_window(struct selection *s)
+{
+	if (s->is_html)
+		return html_get_browser_window(s->c);
+	else
+		return textplain_get_browser_window(s->c);
+}
+
+
 /**
  * Creates a new selection object associated with a browser window.
  *
- * \param  bw   browser window
+ * \return new selection context
  */
 
-struct selection *selection_create(struct browser_window *bw)
+struct selection *selection_create(struct content *c, bool is_html)
 {
 	struct selection *s = calloc(1, sizeof(struct selection));
 	if (s) {
-		s->bw = bw;
+		selection_prepare(s, c, is_html);
+	}
+
+	return s;
+}
+
+/**
+ * Prepare a newly created selection object for use.
+ *
+ * \param  s		selection object
+ * \param  c		content
+ * \param  is_html	true if content is html false if content is textplain
+ */
+
+void selection_prepare(struct selection *s, struct content *c, bool is_html)
+{
+	if (s) {
+		s->c = c;
+		s->is_html = is_html;
 		s->root = NULL;
 		s->drag_state = DRAG_NONE;
+		s->max_idx = 0;
 		selection_clear(s, false);
 	}
-	return s;
 }
 
 
@@ -116,7 +150,8 @@ struct selection *selection_create(struct browser_window *bw)
 
 void selection_destroy(struct selection *s)
 {
-	free(s);
+	if (s != NULL)
+		free(s);
 }
 
 
@@ -157,11 +192,9 @@ void selection_reinit(struct selection *s, struct box *root)
 	s->root = root;
 	if (root) {
 		s->max_idx = selection_label_subtree(root, root_idx);
-	}
-	else {
-		hlcache_handle *c = s->bw->current_content;
-		if (c && content_get_type(c) == CONTENT_TEXTPLAIN)
-			s->max_idx = textplain_size(c);
+	} else {
+		if (s->is_html == false)
+			s->max_idx = textplain_size(s->c);
 		else
 			s->max_idx = 0;
 	}
@@ -260,6 +293,12 @@ bool selection_click(struct selection *s, browser_mouse_state mouse,
 	browser_mouse_state modkeys =
 			(mouse & (BROWSER_MOUSE_MOD_1 | BROWSER_MOUSE_MOD_2));
 	int pos = -1;  /* 0 = inside selection, 1 = after it */
+	struct browser_window *top = selection_get_browser_window(s);
+
+	if (top == NULL)
+		return false; /* not our problem */
+
+	top = browser_window_get_root(top);
 
 	if (!SAME_SPACE(s, idx))
 		return false;	/* not our problem */
@@ -277,16 +316,20 @@ bool selection_click(struct selection *s, browser_mouse_state mouse,
 		((mouse & BROWSER_MOUSE_DRAG_1) ||
 		 (modkeys && (mouse & BROWSER_MOUSE_DRAG_2)))) {
 		/* drag-saving selection */
-		assert(s->bw);
-		gui_drag_save_selection(s, s->bw->window);
+
+		gui_drag_save_selection(s, top->window);
 	}
 	else if (!modkeys) {
-		if (pos && (mouse & BROWSER_MOUSE_PRESS_1))
+		if (pos && (mouse & BROWSER_MOUSE_PRESS_1)) {
 		/* Clear the selection if mouse is pressed outside the selection,
 		 * Otherwise clear on release (to allow for drags) */
+			browser_window_set_selection(top, s);
+
 			selection_clear(s, true);
-		else if (mouse & BROWSER_MOUSE_DRAG_1) {
+		} else if (mouse & BROWSER_MOUSE_DRAG_1) {
 			/* start new selection drag */
+			browser_window_set_selection(top, s);
+
 			selection_clear(s, true);
 			
 			selection_set_start(s, idx);
@@ -294,7 +337,7 @@ bool selection_click(struct selection *s, browser_mouse_state mouse,
 
 			s->drag_state = DRAG_END;
 
-			gui_start_selection(s->bw->window);
+			gui_start_selection(top->window);
 		}
 		else if (mouse & BROWSER_MOUSE_DRAG_2) {
 
@@ -302,6 +345,8 @@ bool selection_click(struct selection *s, browser_mouse_state mouse,
 			if (!selection_defined(s))
 				return false;	/* ignore Adjust drags */
 
+			browser_window_set_selection(top, s);
+
 			if (pos >= 0) {
 				selection_set_end(s, idx);
 
@@ -312,7 +357,8 @@ bool selection_click(struct selection *s, browser_mouse_state mouse,
 
 				s->drag_state = DRAG_START;
 			}
-			gui_start_selection(s->bw->window);
+
+			gui_start_selection(top->window);
 		}
 		/* Selection should be cleared when button is released but in
 		 * the RO interface click is the same as press */
@@ -361,6 +407,10 @@ void selection_track(struct selection *s, browser_mouse_state mouse,
 	if (!SAME_SPACE(s, idx))
 		return;
 
+	if (!mouse) {
+		s->drag_state = DRAG_NONE;
+	}
+
 	switch (s->drag_state) {
 
 		case DRAG_START:
@@ -564,7 +614,6 @@ bool traverse_tree(struct box *box, unsigned start_idx, unsigned end_idx,
 bool selection_traverse(struct selection *s, seln_traverse_handler handler,
 		void *handle)
 {
-	hlcache_handle *c;
 	save_text_whitespace before = WHITESPACE_NONE;
 	bool first = true;
 	const char *text;
@@ -573,16 +622,15 @@ bool selection_traverse(struct selection *s, seln_traverse_handler handler,
 	if (!selection_defined(s))
 		return true;	/* easy case, nothing to do */
 
-	if (s->root)
+	if (s->root) {
+		/* HTML */
 		return traverse_tree(s->root, s->start_idx, s->end_idx,
 				NUMBER_SPACE(s->max_idx), handler, handle,
 				&before, &first, false);
+	}
 
-	c = s->bw->current_content;
-	if (!c) return true;
-
-	text = textplain_get_raw_data(c, s->start_idx,
-							s->end_idx, &length);
+	/* Text */
+	text = textplain_get_raw_data(s->c, s->start_idx, s->end_idx, &length);
 
 	if (text && !handler(text, length, NULL, handle, NULL, 0))
 		return false;
@@ -667,18 +715,16 @@ void selection_redraw(struct selection *s, unsigned start_idx, unsigned end_idx)
 			return;
 	}
 	else {
-		hlcache_handle *c = s->bw->current_content;
-		if (c && content_get_type(c) == CONTENT_TEXTPLAIN && 
-				end_idx > start_idx) {
-			textplain_coords_from_range(c, start_idx,
+		if (s->is_html == false && end_idx > start_idx) {
+			textplain_coords_from_range(s->c, start_idx,
 							end_idx, &rdw.r);
 			rdw.inited = true;
 		}
 	}
 
 	if (rdw.inited)
-		browser_window_redraw_rect(s->bw, rdw.r.x0, rdw.r.y0,
-			rdw.r.x1 - rdw.r.x0, rdw.r.y1 - rdw.r.y0);
+		content__request_redraw(s->c, rdw.r.x0, rdw.r.y0,
+				rdw.r.x1 - rdw.r.x0, rdw.r.y1 - rdw.r.y0);
 }
 
 
@@ -694,6 +740,7 @@ void selection_clear(struct selection *s, bool redraw)
 {
 	int old_start, old_end;
 	bool was_defined;
+	struct browser_window *top = selection_get_browser_window(s);
 
 	assert(s);
 	was_defined = selection_defined(s);
@@ -704,7 +751,12 @@ void selection_clear(struct selection *s, bool redraw)
 	s->start_idx = 0;
 	s->end_idx = 0;
 
-	gui_clear_selection(s->bw->window);
+	if (!top)
+		return;
+
+	top = browser_window_get_root(top);
+
+	gui_clear_selection(top->window);
 
 	if (redraw && was_defined)
 		selection_redraw(s, old_start, old_end);
@@ -870,7 +922,8 @@ struct box *selection_get_end(struct selection *s, size_t *pidx)
  * \return true iff part of the given box lies within the selection
  */
 
-bool selection_highlighted(struct selection *s, unsigned start, unsigned end,
+bool selection_highlighted(const struct selection *s,
+		unsigned start, unsigned end,
 		unsigned *start_idx, unsigned *end_idx)
 {
 	/* caller should have checked first for efficiency */
@@ -958,14 +1011,11 @@ bool save_handler(const char *text, size_t length, struct box *box,
 
 bool selection_save_text(struct selection *s, const char *path)
 {
-	hlcache_handle *c = s->bw->current_content;
 	struct save_text_state sv = { NULL, 0, 0 };
 	utf8_convert_ret ret;
 	char *result;
 	FILE *out;
 
-	assert(c);
-
 	if (!selection_traverse(s, save_handler, &sv)) {
 		free(sv.block);
 		return false;
