@@ -66,7 +66,7 @@ struct line_info {
 };
 
 struct text_area {
-	
+
 	int scroll_x, scroll_y;		/**< scroll offsets of the textarea
 					 * content
 					 */
@@ -726,7 +726,7 @@ bool textarea_reflow(struct text_area *ta, unsigned int line)
  * \param y1	bottom Y coordinate of redraw area
  * \param ctx	current redraw context
  */
-void textarea_redraw(struct text_area *ta, int x, int y, 
+void textarea_redraw(struct text_area *ta, int x, int y,
 		const struct rect *clip, const struct redraw_context *ctx)
 {
 	const struct plotter_table *plot = ctx->plot;
@@ -961,6 +961,8 @@ bool textarea_keypress(struct text_area *ta, uint32_t key)
 				     		ta->selection_start,
 						ta->selection_end, ""))
 					return false;
+
+				caret = ta->selection_start;
 				ta->selection_start = ta->selection_end = -1;
 				redraw = true;
 			} else {
@@ -974,7 +976,6 @@ bool textarea_keypress(struct text_area *ta, uint32_t key)
 				}
 			}
 			break;
-			break;
 		case KEY_NL:
 			if (readonly)
 				break;
@@ -1124,6 +1125,7 @@ bool textarea_keypress(struct text_area *ta, uint32_t key)
 						ta->selection_end, ""))
 					return false;
 
+				caret = ta->selection_start;
 				ta->selection_start = ta->selection_end = -1;
 				redraw = true;
 			} else {
@@ -1444,3 +1446,18 @@ void textarea_get_dimensions(struct text_area *ta, int *width, int *height)
 	if (height != NULL)
 		*height = ta->vis_height;
 }
+
+/**
+ * Set the dimensions of a textarea, causing a reflow and
+ * emitting a redraw request.
+ *
+ * \param width 	the new width of the textarea
+ * \param height	the new height of the textarea
+ */
+void textarea_set_dimensions(struct text_area *ta, int width, int height)
+{
+	ta->vis_width = width;
+	ta->vis_height = height;
+	textarea_reflow(ta, 0);
+	ta->redraw_request(ta->data, 0, 0, ta->vis_width, ta->vis_height);
+}
