@@ -62,6 +62,13 @@ static plot_font_style_t plot_fstyle = {
 	.flags = FONTF_NONE
 };
 
+static plot_font_style_t plot_fstyle_def_folder = {
+	.family = PLOT_FONT_FAMILY_SANS_SERIF,
+	.size = TREE_TEXT_SIZE_PT * FONT_SIZE_SCALE,
+	.weight = 700,
+	.flags = FONTF_NONE
+};
+
 static plot_font_style_t plot_fstyle_selected = {
 	.family = PLOT_FONT_FAMILY_SANS_SERIF,
 	.size = TREE_TEXT_SIZE_PT * FONT_SIZE_SCALE,
@@ -69,6 +76,13 @@ static plot_font_style_t plot_fstyle_selected = {
 	.flags = FONTF_NONE
 };
 
+static plot_font_style_t plot_fstyle_selected_def_folder = {
+	.family = PLOT_FONT_FAMILY_SANS_SERIF,
+	.size = TREE_TEXT_SIZE_PT * FONT_SIZE_SCALE,
+	.weight = 700,
+	.flags = FONTF_NONE
+};
+
 /** plot style for treeview backgrounds. */
 static plot_style_t plot_style_fill_tree_background = {
 	.fill_type = PLOT_OP_TYPE_SOLID
@@ -118,6 +132,7 @@ struct node {
 	bool selected;			/**< Whether the node is selected */
 	bool expanded;			/**< Whether the node is expanded */
 	bool folder;			/**< Whether the node is a folder */
+	bool def_folder;		/**< Whether the node is the default folder */
 	bool retain_in_memory;		/**< Whether the node remains
 					   in memory after deletion */
 	bool deleted;			/**< Whether the node is currently
@@ -140,9 +155,6 @@ struct node {
 };
 
 struct tree {
-	/* These coordinates are only added to the coordinates passed to the
-	   plotters. This means they are invisible to the tree, what has to be
-	   taken into account i.e in keyboard/mouse event passing */
 	struct node *root;		/* Tree root element */
 	int width;			/* Tree width */
 	int height;			/* Tree height */
@@ -159,6 +171,7 @@ struct tree {
 	const struct treeview_table *callbacks;
 	void *client_data;		/* User assigned data for the
 					   callbacks */
+	struct node *def_folder;	/* Node to be used for additions by default */
 };
 
 void tree_set_icon_dir(char *icon_dir)
@@ -188,10 +201,14 @@ static void tree_setup_colours(void)
 	/* Text colour */
 	plot_fstyle.foreground = option_gui_colour_fg_1;
 	plot_fstyle.background = option_gui_colour_bg_1;
+	plot_fstyle_def_folder.foreground = option_gui_colour_fg_1;
+	plot_fstyle_def_folder.background = option_gui_colour_bg_1;
 
 	/* Selected text colour */
 	plot_fstyle_selected.foreground = option_gui_colour_fg_2;
 	plot_fstyle_selected.background = option_gui_colour_fg_1;
+	plot_fstyle_selected_def_folder.foreground = option_gui_colour_fg_2;
+	plot_fstyle_selected_def_folder.background = option_gui_colour_fg_1;
 }
 
 
@@ -275,9 +292,16 @@ static void tree_recalculate_node_element(struct tree *tree,
 	int width, height;
 	static char *cache_text = NULL;
 	static int cache_size = 0;
+	plot_font_style_t *fstyle;
+	static plot_font_style_t *cache_fstyle = NULL;
 
 	assert(element != NULL);
 
+	if (element->parent->def_folder)
+		fstyle = &plot_fstyle_def_folder;
+	else
+		fstyle = &plot_fstyle;
+
 	switch (element->type) {
 	case NODE_ELEMENT_TEXT_PLUS_ICON:
 	case NODE_ELEMENT_TEXT:
@@ -289,19 +313,21 @@ static void tree_recalculate_node_element(struct tree *tree,
 						&element->box.width, NULL);
 		} else {
 			if ((cache_text != NULL) &&
-				(strcmp(cache_text, element->text) == 0)) {
+				(strcmp(cache_text, element->text) == 0) &&
+				(cache_fstyle == fstyle)) {
 				element->box.width = cache_size;
 				#ifdef TREE_NOISY_DEBUG
 					LOG(("Tree font width cache hit"));
 				#endif
 			} else {
 				if(cache_text != NULL) free(cache_text);
-				nsfont.font_width(&plot_fstyle,
+				nsfont.font_width(fstyle,
 						  element->text,
 						  strlen(element->text),
 						  &cache_size);
 				element->box.width = cache_size;
 				cache_text = strdup(element->text);
+				cache_fstyle = fstyle;
 			}
 		}
 
@@ -1034,6 +1060,8 @@ static void tree_delete_node_internal(struct tree *tree, struct node *node,
 	parent = node->parent;
 	if (tree != NULL && parent == tree->root)
 		parent = NULL;
+	if ((tree != NULL) && (tree->def_folder == node))
+		tree->def_folder = NULL;
 	tree_delink_node(tree, node);
 	child = node->child;
 	node->child = NULL;
@@ -1101,7 +1129,8 @@ static void tree_delete_node_internal(struct tree *tree, struct node *node,
  */
 void tree_delete(struct tree *tree)
 {
-	tree_set_redraw(tree, false);
+	tree->redraw = false;
+
 	if (tree->root->child != NULL)
 		tree_delete_node_internal(tree, tree->root->child, true);
 
@@ -1133,11 +1162,20 @@ bool tree_get_redraw(struct tree *tree)
 void tree_delete_node(struct tree *tree, struct node *node, bool siblings)
 {
 	int y = node->box.y;
+	int height = tree->height;
+	int width = tree->width;
+	bool redraw_setting = tree->redraw;
+
+	tree->redraw = false;
+
 	tree_delete_node_internal(tree, node, siblings);
 	tree_recalculate_node_positions(tree, tree->root);
+
+	tree->redraw = redraw_setting;
+
 	if (tree->redraw)
-		tree->callbacks->redraw_request(0, y, tree->width, tree->height,
-						tree->client_data);
+		tree->callbacks->redraw_request(0, y,
+				width, height, tree->client_data);
 	tree_recalculate_size(tree);
 }
 
@@ -1393,6 +1431,18 @@ bool tree_node_is_folder(struct node *node)
 }
 
 
+/**
+ * Returns true if the node is the default folder for a tree
+ *
+ * \param node	the node to be checked
+ * \return	true if the node is a default folder, false otherwise
+ */
+bool tree_node_is_default(struct node *node)
+{
+	return node->def_folder;
+}
+
+
 /**
  * Update the text of a node element if it has changed.
  *
@@ -1544,6 +1594,84 @@ tree_drag_type tree_drag_status(struct tree *tree)
 }
 
 
+/**
+ * Get the default node of a tree for additions
+ *
+ * \param tree	the tree to get the default node of
+ * \return	the default node
+ */
+struct node *tree_get_default_folder_node(struct tree *tree)
+{
+	if (tree->def_folder != NULL) {
+		return tree->def_folder;
+	} else {
+		return tree_get_root(tree);
+	}
+}
+
+
+/**
+ * Set the default node of a tree to the selected node
+ *
+ * \param tree	the tree to set the default node of
+ * \param node  the node to set as default (NULL for selected node)
+ * \return	success
+ */
+bool tree_set_default_folder_node(struct tree *tree, struct node *node)
+{
+	struct node *sel_node;
+
+	if (node == NULL) {
+		sel_node = tree_get_selected_node(tree->root);
+	} else {
+		sel_node = node;
+	}
+
+	if((sel_node == NULL) ||
+		(tree_node_is_folder(sel_node) == false)) {
+		return false;
+	}
+
+	tree_clear_default_folder_node(tree);
+
+	tree->def_folder = sel_node;
+	sel_node->def_folder = true;
+	tree_handle_node_changed(tree, sel_node, true, false);
+
+	return true;
+}
+
+
+/**
+ * Clear the default node of a tree
+ *
+ * \param tree	the tree to clear the default node of
+ */
+void tree_clear_default_folder_node(struct tree *tree)
+{
+	struct node *def_node = NULL;
+	def_node = tree_get_default_folder_node(tree);
+
+	if (def_node != NULL) {
+		tree->def_folder = NULL;
+		def_node->def_folder = false;
+		tree_handle_node_changed(tree, def_node, true, false);
+	}
+}
+
+
+/**
+ * Returns the parent of a node
+ *
+ * \param node	the node to get the parent of
+ * \return	the node's parent
+ */
+struct node *tree_node_get_parent(struct node *node)
+{
+	return node->parent;
+}
+
+
 /**
  * Returns the first child of a node
  *
@@ -1623,6 +1751,7 @@ static void tree_draw_node_element(struct tree *tree,
 	struct bitmap *bitmap = NULL;
 	int x, y, width;
 	bool selected = false;
+	bool def_folder = false;
 	hlcache_handle *icon;
 	plot_font_style_t *fstyle;
 	const int icon_inset = (TREE_LINE_HEIGHT - TREE_ICON_SIZE) / 2;
@@ -1634,9 +1763,12 @@ static void tree_draw_node_element(struct tree *tree,
 	x = tree_x + element->box.x;
 	y = tree_y + element->box.y;
 	width = element->box.width;
-	if (&element->parent->data == element)
+	if (&element->parent->data == element) {
 		if (element->parent->selected)
 			selected = true;
+		if (element->parent->def_folder)
+			def_folder = true;
+	}
 
 	switch (element->type) {
 	case NODE_ELEMENT_TEXT_PLUS_ICON:
@@ -1693,11 +1825,18 @@ static void tree_draw_node_element(struct tree *tree,
 			return;
 
 		if (selected) {
-			fstyle = &plot_fstyle_selected;
+			if (def_folder == true)
+				fstyle = &plot_fstyle_selected_def_folder;
+			else
+				fstyle = &plot_fstyle_selected;
+
 			plot->rectangle(x, y, x + width,
 				       y + element->box.height,
 				       &plot_style_fill_tree_selected);
 		} else {
+		if (def_folder == true)
+			fstyle = &plot_fstyle_def_folder;
+		else
 			fstyle = &plot_fstyle;
 		}
 
@@ -1975,10 +2114,23 @@ struct node_element *tree_node_find_element(struct node *node,
 void tree_delete_selected_nodes(struct tree *tree, struct node *node)
 {
 	struct node *next;
+	int y = node->box.y;
+	int height = tree->height;
+	int width = tree->width;
+	bool redraw_setting = tree->redraw;
+
+	tree->redraw = false;
 
 	if (node == tree->root) {
 		if (node->child != NULL)
 			tree_delete_selected_nodes(tree, node->child);
+
+		tree->redraw = redraw_setting;
+
+		if (tree->redraw)
+			tree->callbacks->redraw_request(0, y,
+					width, height,
+					tree->client_data);
 		return;
 	}
 
@@ -1990,6 +2142,13 @@ void tree_delete_selected_nodes(struct tree *tree, struct node *node)
 			tree_delete_selected_nodes(tree, node->child);
 		node = next;
 	}
+
+	tree->redraw = redraw_setting;
+
+	if (tree->redraw)
+		tree->callbacks->redraw_request(0, y,
+				width, height,
+				tree->client_data);
 }
 
 
@@ -2193,6 +2352,29 @@ void tree_launch_selected(struct tree *tree, bool tabs)
 }
 
 
+/**
+ * Updates the node at position x,y to a selected state.
+ * The required areas of the tree are redrawn.
+ *
+ * \param tree	    the tree to update nodes for, may be NULL
+ * \param x	        x position in tree
+ * \param y	        y position in tree
+ * \param selected  the selection state to set
+ */
+void tree_set_node_selected_at(struct tree *tree, int x, int y, bool selected)
+{
+	bool expansion_toggle;
+	struct node *node;
+
+	node = tree_get_node_at(tree->root, x, y, &expansion_toggle);
+
+	if ((node == NULL) || (expansion_toggle == true))
+		return;
+
+	tree_set_node_selected(tree, node, false, selected);
+}
+
+
 /**
  * Handles a mouse action for a tree
  *
@@ -2807,6 +2989,7 @@ hlcache_handle *tree_load_icon(const char *name)
 	int len;
 	hlcache_handle *c;
 	nserror err;
+	nsurl *icon_nsurl;
 
 	/** @todo something like bitmap_from_disc is needed here */
 
@@ -2839,11 +3022,19 @@ hlcache_handle *tree_load_icon(const char *name)
 		icon_url = url;
 	}
 
+	err = nsurl_create(icon_url, &icon_nsurl);
+	if (err != NSERROR_OK) {
+		if (url != NULL)
+			free(url);
+		return NULL;
+	}
+
 	/* Fetch the icon */
-	err = hlcache_handle_retrieve(icon_url, 0, 0, 0,
+	err = hlcache_handle_retrieve(icon_nsurl, 0, 0, 0,
 				      tree_icon_callback, 0, 0, 
 				      CONTENT_IMAGE, &c);
 
+	nsurl_unref(icon_nsurl);
 
 	/* If we built the URL here, free it */
 	if (url != NULL)
