@@ -157,6 +157,7 @@ void tree_set_node_expanded(struct tree *tree, struct node *node, bool expanded,
 		bool folder, bool leaf);
 void tree_set_node_selected(struct tree *tree, struct node *node, bool all,
 		bool selected);
+void tree_set_node_selected_at(struct tree *tree, int x, int y, bool selected);
 void tree_set_node_sort_function(struct tree *tree, struct node *node,
 		int (*sort) (struct node *, struct node *));
 void tree_set_node_user_callback(struct node *node,
@@ -166,6 +167,7 @@ bool tree_get_redraw(struct tree *tree);
 bool tree_node_has_selection(struct node *node);
 bool tree_node_is_deleted(struct node *node);
 bool tree_node_is_folder(struct node *node);
+bool tree_node_is_default(struct node *node);
 void tree_update_node_element(struct tree *tree, struct node_element *element,
 		const char *text, void *bitmap);
 bool tree_update_element_text(struct tree *tree, struct node_element *element, char *text);
@@ -174,7 +176,12 @@ struct node *tree_get_root(struct tree *tree);
 bool tree_is_edited(struct tree *tree);
 tree_drag_type tree_drag_status(struct tree *tree);
 
+struct node *tree_get_default_folder_node(struct tree *tree);
+bool tree_set_default_folder_node(struct tree *tree, struct node *node);
+void tree_clear_default_folder_node(struct tree *tree);
+
 /* functions for traversing the tree */
+struct node *tree_node_get_parent(struct node *node);
 struct node *tree_node_get_child(struct node *node);
 struct node *tree_node_get_next(struct node *node);
 
