@@ -351,7 +351,8 @@ node_callback_resp tree_url_node_callback(void *user_data,
 {
 	struct tree *tree;
 	struct node_element *element;
-	url_func_result res;
+	nsurl *nsurl;
+	nserror error;
 	const char *text;
 	char *norm_text;
 	const struct url_data *data;
@@ -398,18 +399,20 @@ node_callback_resp tree_url_node_callback(void *user_data,
 		text = msg_data->data.text;
 
 		if (msg_data->flag == TREE_ELEMENT_URL) {
-			res = url_normalize(text, &norm_text);
-			if (res != URL_FUNC_OK) {
-				if (res == URL_FUNC_FAILED) {
-					warn_user("NoURLError", 0);
-					return NODE_CALLBACK_CONTINUE;
-				}
-				else {
-					warn_user("NoMemory", 0);
-					return NODE_CALLBACK_REJECT;
-				}
-
+			size_t len;
+			error = nsurl_create(text, &nsurl);
+			if (error != NSERROR_OK) {
+				warn_user("NoMemory", 0);
+				return NODE_CALLBACK_REJECT;
 			}
+			error = nsurl_get(nsurl, NSURL_WITH_FRAGMENT,
+					&norm_text, &len);
+			nsurl_unref(nsurl);
+			if (error != NSERROR_OK) {
+				warn_user("NoMemory", 0);
+				return NODE_CALLBACK_REJECT;
+			}
+
 			msg_data->data.text = norm_text;
 
 			data = urldb_get_url_data(norm_text);
@@ -484,18 +487,20 @@ static void tree_url_load_entry(xmlNode *li, struct tree *tree,
 		struct node *directory, tree_node_user_callback callback,
 		void *callback_data)
 {
-	char *url = NULL, *url1 = NULL;
+	char *url1 = NULL;
 	char *title = NULL;
 	struct node *entry;
 	xmlNode *xmlnode;
 	const struct url_data *data;
-	url_func_result res;
+	nsurl *url;
+	nserror error;
 
 	for (xmlnode = li->children; xmlnode; xmlnode = xmlnode->next) {
 		/* The li must contain an "a" element */
 		if (xmlnode->type == XML_ELEMENT_NODE &&
 		    strcmp((const char *)xmlnode->name, "a") == 0) {
-			url1 = (char *)xmlGetProp(xmlnode, (const xmlChar *) "href");
+			url1 = (char *)xmlGetProp(xmlnode,
+					(const xmlChar *) "href");
 			title = (char *)xmlNodeGetContent(xmlnode);
 		}
 	}
@@ -507,14 +512,13 @@ static void tree_url_load_entry(xmlNode *li, struct tree *tree,
 	}
 
 	/* We're loading external input.
-	 * This may be garbage, so attempt to normalise
+	 * This may be garbage, so attempt to normalise via nsurl
 	 */
-	res = url_normalize(url1, &url);
-	if (res != URL_FUNC_OK) {
+	error = nsurl_create(url1, &url);
+	if (error != NSERROR_OK) {
 		LOG(("Failed normalising '%s'", url1));
 
-		if (res == URL_FUNC_NOMEM)
-			warn_user("NoMemory", NULL);
+		warn_user("NoMemory", NULL);
 
 		xmlFree(url1);
 		xmlFree(title);
@@ -525,39 +529,39 @@ static void tree_url_load_entry(xmlNode *li, struct tree *tree,
 	/* No longer need this */
 	xmlFree(url1);
 
-	data = urldb_get_url_data(url);
+	data = urldb_get_url_data(nsurl_access(url));
 	if (data == NULL) {
 		/* No entry in database, so add one */
-		urldb_add_url(url);
+		urldb_add_url(nsurl_access(url));
 		/* now attempt to get url data */
-		data = urldb_get_url_data(url);
+		data = urldb_get_url_data(nsurl_access(url));
 	}
 	if (data == NULL) {
 		xmlFree(title);
-		free(url);
+		nsurl_unref(url);
 
 		return;
 	}
 
 	/* Make this URL persistent */
-	urldb_set_url_persistence(url, true);
+	urldb_set_url_persistence(nsurl_access(url), true);
 
 	/* Force the title in the hotlist */
-	urldb_set_url_title(url, title);
+	urldb_set_url_title(nsurl_access(url), title);
 
-	entry = tree_create_URL_node(tree, directory, url, title,
+	entry = tree_create_URL_node(tree, directory, nsurl_access(url), title,
 				     callback, callback_data);
 
  	if (entry == NULL) {
  		/** \todo why isn't this fatal? */
  		warn_user("NoMemory", 0);
  	} else {
-		tree_update_URL_node(tree, entry, url, data);
+		tree_update_URL_node(tree, entry, nsurl_access(url), data);
 	}
 
 
 	xmlFree(title);
-	free(url);
+	nsurl_unref(url);
 }
 
 /**
@@ -573,6 +577,7 @@ static void tree_url_load_directory(xmlNode *ul, struct tree *tree,
 	char *title;
 	struct node *dir;
 	xmlNode *xmlnode;
+	xmlChar *id;
 
 	assert(ul != NULL);
 	assert(directory != NULL);
@@ -592,6 +597,7 @@ static void tree_url_load_directory(xmlNode *ul, struct tree *tree,
 
 		} else if (strcmp((const char *)xmlnode->name, "h4") == 0) {
 			/* directory */
+			bool dir_is_default = false;
 			title = (char *) xmlNodeGetContent(xmlnode );
 			if (!title) {
 				warn_user("TreeLoadError", "(Empty <h4> "
@@ -610,6 +616,14 @@ static void tree_url_load_directory(xmlNode *ul, struct tree *tree,
 				warn_user("TreeLoadError", "(Expected "
 					  "<ul> not present.)");
 				return;
+			} else {
+				id = xmlGetProp(xmlnode,
+						(const xmlChar *) "id");
+				if (id != NULL) {
+					if(strcmp((const char *)id, "default") == 0)
+						dir_is_default = true;
+					xmlFree(id);
+				}
 			}
 
 			dir = tree_create_folder_node(tree, directory, title,
@@ -619,6 +633,10 @@ static void tree_url_load_directory(xmlNode *ul, struct tree *tree,
 				return;
 			}
 
+			if(dir_is_default == true) {
+				tree_set_default_folder_node(tree, dir);
+			}
+
 			if (callback != NULL)
 				tree_set_node_user_callback(dir, callback,
 							    callback_data);
@@ -735,6 +753,9 @@ static bool tree_url_save_directory(struct node *directory, xmlNode *node)
 	ul = xmlNewChild(node, NULL, (const xmlChar *)"ul", NULL);
 	if (ul == NULL)
 		return false;
+	if (tree_node_is_default(directory) == true)
+		xmlSetProp(ul, (const xmlChar *) "id",
+				(const xmlChar *) "default");
 
 	for (child = tree_node_get_child(directory); child;
 	     child = tree_node_get_next(child)) {
@@ -764,12 +785,6 @@ static bool tree_url_save_directory(struct node *directory, xmlNode *node)
 }
 
 
-
-
-
-
-
-
 /**
  * Perform a save to a specified file in the form of a html page
  *
