@@ -49,7 +49,6 @@
 #include "framebuffer/gui.h"
 #include "framebuffer/fbtk.h"
 #include "framebuffer/framebuffer.h"
-#include "framebuffer/bitmap.h"
 #include "framebuffer/schedule.h"
 #include "framebuffer/findfile.h"
 #include "framebuffer/image_data.h"
@@ -62,10 +61,6 @@
 
 #define NSFB_TOOLBAR_DEFAULT_LAYOUT "blfsrut"
 
-char *default_stylesheet_url;
-char *quirks_stylesheet_url;
-char *adblock_stylesheet_url;
-
 fbtk_widget_t *fbtk;
 
 struct gui_window *input_window = NULL;
@@ -226,7 +221,7 @@ fb_pan(fbtk_widget_t *widget,
 		dstbox.y1 = dstbox.y0 + height + bwidget->pany;
 
 		/* move part that remains visible up */
-		nsfb_plot_copy(nsfb, &srcbox, &dstbox);
+		nsfb_plot_copy(nsfb, &srcbox, nsfb, &dstbox);
 
 		/* redraw newly exposed area */
 		bwidget->scrolly += bwidget->pany;
@@ -246,7 +241,7 @@ fb_pan(fbtk_widget_t *widget,
 		dstbox.y1 = dstbox.y0 + height - bwidget->pany;
 
 		/* move part that remains visible down */
-		nsfb_plot_copy(nsfb, &srcbox, &dstbox);
+		nsfb_plot_copy(nsfb, &srcbox, nsfb, &dstbox);
 
 		/* redraw newly exposed area */
 		bwidget->scrolly += bwidget->pany;
@@ -266,7 +261,7 @@ fb_pan(fbtk_widget_t *widget,
 		dstbox.y1 = dstbox.y0 + height;
 
 		/* move part that remains visible left */
-		nsfb_plot_copy(nsfb, &srcbox, &dstbox);
+		nsfb_plot_copy(nsfb, &srcbox, nsfb, &dstbox);
 
 		/* redraw newly exposed area */
 		bwidget->scrollx += bwidget->panx;
@@ -286,7 +281,7 @@ fb_pan(fbtk_widget_t *widget,
 		dstbox.y1 = dstbox.y0 + height;
 
 		/* move part that remains visible right */
-		nsfb_plot_copy(nsfb, &srcbox, &dstbox);
+		nsfb_plot_copy(nsfb, &srcbox, nsfb, &dstbox);
 
 		/* redraw newly exposed area */
 		bwidget->scrollx += bwidget->panx;
@@ -309,6 +304,7 @@ fb_redraw(fbtk_widget_t *widget,
 	struct rect clip;
 	struct redraw_context ctx = {
 		.interactive = true,
+		.background_images = true,
 		.plot = &fb_plotters
 	};
 
@@ -446,16 +442,6 @@ gui_init(int argc, char** argv)
 
 	option_core_select_menu = true;
 
-	/* set up stylesheet urls */
-	default_stylesheet_url = strdup("resource:default.css");
-	LOG(("Using '%s' as Default CSS URL", default_stylesheet_url));
-
-	quirks_stylesheet_url = strdup("resource:quirks.css");
-	LOG(("Using '%s' as quirks CSS URL", quirks_stylesheet_url));
-
-	adblock_stylesheet_url = strdup("resource:adblock.css");
-	LOG(("Using '%s' as AdBlock CSS URL", adblock_stylesheet_url));
-
 	if (option_cookie_file == NULL) {
 		option_cookie_file = strdup("~/.netsurf/Cookies");
 		LOG(("Using '%s' as Cookies file", option_cookie_file));
@@ -528,13 +514,6 @@ main(int argc, char** argv)
 }
 
 
-void
-gui_multitask(void)
-{
-	/* LOG(("gui_multitask")); */
-}
-
-
 void
 gui_poll(bool active)
 {
@@ -607,12 +586,22 @@ fb_browser_window_click(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 
 		case NSFB_KEY_MOUSE_4:
 			/* scroll up */
-			widget_scroll_y(gw, -100, false);
+			scale = gw->bw->scale;
+			if (browser_window_scroll_at_point(gw->bw,
+					(cbi->x + bwidget->scrollx) / scale,
+					(cbi->y + bwidget->scrolly) / scale,
+					0, -100) == false)
+				widget_scroll_y(gw, -100, false);
 			break;
 
 		case NSFB_KEY_MOUSE_5:
 			/* scroll down */
-			widget_scroll_y(gw, 100, false);
+			scale = gw->bw->scale;
+			if (browser_window_scroll_at_point(gw->bw,
+					(cbi->x + bwidget->scrollx) / scale,
+					(cbi->y + bwidget->scrolly) / scale,
+					0, 100) == false)
+				widget_scroll_y(gw, 100, false);
 			break;
 
 		default:
@@ -671,7 +660,6 @@ static int
 fb_browser_window_input(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 {
 	struct gui_window *gw = cbi->context;
-	int res = 0;
 	static uint8_t modifier = 0;
 	int ucs4 = -1;
 
@@ -723,7 +711,7 @@ fb_browser_window_input(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 			ucs4 = fbtk_keycode_to_ucs4(cbi->event->value.keycode,
 						    modifier);
 			if (ucs4 != -1)
-				res = browser_window_key_press(gw->bw, ucs4);
+				browser_window_key_press(gw->bw, ucs4);
 			break;
 		}
 		break;
@@ -1413,7 +1401,7 @@ static void
 throbber_advance(void *pw)
 {
 	struct gui_window *g = pw;
-	struct bitmap *image;
+	struct fbtk_bitmap *image;
 
 	switch (g->throbber_index) {
 	case 0:
@@ -1505,8 +1493,8 @@ gui_window_scroll_start(struct gui_window *g)
 }
 
 bool
-gui_window_box_scroll_start(struct gui_window *g,
-			    int x0, int y0, int x1, int y1)
+gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+                      const struct rect *rect)
 {
 	return true;
 }
