@@ -45,7 +45,6 @@
 #include "framebuffer/gui.h"
 #include "framebuffer/fbtk.h"
 #include "framebuffer/framebuffer.h"
-#include "framebuffer/bitmap.h"
 #include "framebuffer/schedule.h"
 #include "framebuffer/findfile.h"
 #include "framebuffer/image_data.h"
@@ -63,6 +62,7 @@ localhistory_redraw(fbtk_widget_t *widget, fbtk_callback_info *cbi)
 
 	struct redraw_context ctx = {
 		.interactive = true,
+		.background_images = true,
 		.plot = &fb_plotters
 	};
 
