@@ -70,7 +70,6 @@ DECLARE(checkHoverURLs);
 DECLARE(checkDisplayRecentURLs);
 DECLARE(comboLanguage);
 DECLARE(checkSendReferer);
-DECLARE(checkShowSingleTab);
 
 DECLARE(comboProxyType);
 DECLARE(entryProxyHost);
@@ -103,8 +102,12 @@ DECLARE(spinDiscCacheAge);
 DECLARE(checkClearDownloads);
 DECLARE(checkRequestOverwrite);
 DECLARE(fileChooserDownloads);
+/* Tabs */
+DECLARE(checkShowSingleTab);
 DECLARE(checkFocusNew);
 DECLARE(checkNewBlank);
+DECLARE(comboTabPosition);
+
 DECLARE(checkUrlSearch);
 DECLARE(comboSearch);
 DECLARE(combotheme);
@@ -139,7 +142,7 @@ DECLARE(setDefaultExportOptions);
 
 GtkDialog* nsgtk_options_init(struct browser_window *bw, GtkWindow *parent)
 {
-	gladeFile = glade_xml_new(glade_options_file_location, NULL, NULL);
+	gladeFile = glade_xml_new(glade_file_location->options, NULL, NULL);
 	if (gladeFile == NULL)
 		return NULL;
 	
@@ -199,6 +202,8 @@ GtkDialog* nsgtk_options_init(struct browser_window *bw, GtkWindow *parent)
 	
 	CONNECT(comboButtonType, "changed");
 
+	CONNECT(comboTabPosition, "changed");
+
 	CONNECT(spinMemoryCacheSize, "value-changed");
 	CONNECT(spinDiscCacheAge, "value-changed");
 	
@@ -396,6 +401,8 @@ void nsgtk_options_load(void)
 	
 	SET_COMBO(comboButtonType, option_button_type -1);
 
+	SET_COMBO(comboTabPosition, option_position_tab);
+
 	SET_SPIN(spinMemoryCacheSize, option_memory_cache_size >> 20);
 	SET_SPIN(spinDiscCacheAge, option_disc_cache_age);
 	
@@ -572,7 +579,8 @@ ENTRY_CHANGED(entryHomePageURL, option_homepage_url)
 END_HANDLER
 
 BUTTON_CLICKED(setCurrentPage)
-	const gchar *url = content_get_url(current_browser->current_content);
+	const gchar *url = nsurl_access(hlcache_handle_get_url(
+			current_browser->current_content));
 	gtk_entry_set_text(GTK_ENTRY(entryHomePageURL), url);
 	option_homepage_url = 
 		strdup(gtk_entry_get_text(GTK_ENTRY(entryHomePageURL)));
@@ -758,6 +766,19 @@ COMBO_CHANGED(comboButtonType, option_button_type)
 	}
 END_HANDLER
 
+COMBO_CHANGED(comboTabPosition, option_position_tab)
+	nsgtk_scaffolding *current = scaf_list;
+	option_button_type++;
+	/* value of 0 is reserved for 'unset' */
+	while (current)	{
+		nsgtk_scaffolding_reset_offset(current);
+
+		nsgtk_reflow_all_windows();
+
+		current = nsgtk_scaffolding_iterate(current);
+	}
+END_HANDLER
+
 SPIN_CHANGED(spinMemoryCacheSize, option_memory_cache_size)
 	option_memory_cache_size <<= 20;
 END_HANDLER
