@@ -81,27 +81,15 @@
 #include "utils/utf8.h"
 #include "utils/utils.h"
 
-char *default_stylesheet_url;
-char *quirks_stylesheet_url;
-char *adblock_stylesheet_url;
 char *options_file_location;
 char *toolbar_indices_file_location;
 char *res_dir_location;
 char *print_options_file_location;
 char *languages_file_location;
 
-char *glade_netsurf_file_location;
-char *glade_password_file_location;
-char *glade_warning_file_location;
-char *glade_login_file_location;
-char *glade_ssl_file_location;
-char *glade_toolbar_file_location;
-char *glade_options_file_location;
+GdkPixbuf *favicon_pixbuf; /* favicon default pixbuf */
 
-static char *glade_downloads_file_location;
-static char *glade_history_file_location;
-static char *glade_hotlist_file_location;
-static char *glade_cookies_file_location;
+struct glade_file_location_s *glade_file_location;
 
 static GtkWindow *nsgtk_warning_window;
 GtkWidget *widWarning;
@@ -226,18 +214,23 @@ nsgtk_init_glade(char **respath)
 
 	glade_init();
 
-	glade_netsurf_file_location = nsgtk_new_glade(respath, "netsurf", NULL);
-	glade_password_file_location = nsgtk_new_glade(respath, "password", NULL);
-	glade_login_file_location = nsgtk_new_glade(respath, "login", NULL);
-	glade_ssl_file_location = nsgtk_new_glade(respath, "ssl", NULL);
-	glade_toolbar_file_location = nsgtk_new_glade(respath, "toolbar", NULL);
-	glade_downloads_file_location = nsgtk_new_glade(respath, "downloads", NULL);
-	glade_history_file_location = nsgtk_new_glade(respath, "history", NULL);
-	glade_options_file_location = nsgtk_new_glade(respath, "options", NULL);
-	glade_hotlist_file_location = nsgtk_new_glade(respath, "hotlist", NULL);
-	glade_cookies_file_location = nsgtk_new_glade(respath, "cookies", NULL);
+	glade_file_location = calloc(1, sizeof(struct glade_file_location_s));
+	if (glade_file_location == NULL) {
+		die("Unable to allocate glade file locations");
+	}
 
-	glade_warning_file_location = nsgtk_new_glade(respath, "warning", &gladeWarning);
+	glade_file_location->netsurf = nsgtk_new_glade(respath, "netsurf", NULL);
+	glade_file_location->password = nsgtk_new_glade(respath, "password", NULL);
+	glade_file_location->login = nsgtk_new_glade(respath, "login", NULL);
+	glade_file_location->ssl = nsgtk_new_glade(respath, "ssl", NULL);
+	glade_file_location->toolbar = nsgtk_new_glade(respath, "toolbar", NULL);
+	glade_file_location->downloads = nsgtk_new_glade(respath, "downloads", NULL);
+	glade_file_location->history = nsgtk_new_glade(respath, "history", NULL);
+	glade_file_location->options = nsgtk_new_glade(respath, "options", NULL);
+	glade_file_location->hotlist = nsgtk_new_glade(respath, "hotlist", NULL);
+	glade_file_location->cookies = nsgtk_new_glade(respath, "cookies", NULL);
+
+	glade_file_location->warning = nsgtk_new_glade(respath, "warning", &gladeWarning);
 	nsgtk_warning_window = GTK_WINDOW(glade_xml_get_widget(gladeWarning, "wndWarning"));
 	widWarning = glade_xml_get_widget(gladeWarning, "labelWarning");
 }
@@ -326,10 +319,27 @@ static void check_options(char **respath)
 
 }
 
-char* gui_get_resource_url(const char *filename)
+nsurl *gui_get_resource_url(const char *path)
 {
 	char buf[PATH_MAX];
-	return path_to_url(filepath_sfind(respaths, buf, filename));
+	char *raw;
+	nsurl *url = NULL;
+
+	/* default.css -> gtkdefault.css */
+	if (strcmp(path, "default.css") == 0)
+		path = "gtkdefault.css";
+
+	/* favicon.ico -> favicon.png */
+	if (strcmp(path, "favicon.ico") == 0)
+		path = "favicon.png";	
+
+	raw = path_to_url(filepath_sfind(respaths, buf, path));
+	if (raw != NULL) {
+		nsurl_create(raw, &url);
+		free(raw);
+	}
+
+	return url;
 }
 
 
@@ -339,26 +349,29 @@ char* gui_get_resource_url(const char *filename)
 static void gui_init(int argc, char** argv, char **respath)
 {
 	char buf[PATH_MAX];
-	struct browser_window *bw;
 	const char *addr = NETSURF_HOMEPAGE;
 	char *resource_filename;
 
 	/* check user options */
 	check_options(respath);
 
+	/* find the languages file */	
+	languages_file_location = filepath_find(respath, "languages");
+	if ((languages_file_location == NULL) || 
+	    (strlen(languages_file_location) < 10)) {
+		die("Unable to find resources.\n");		
+	}
+
 	/* Obtain resources path location. 
 	 *
 	 * Uses the directory the languages file was found in,
-	 * previously the location of the glade files was used,
-	 * however these may be translated which breaks things
-	 * relying on res_dir_location.
-	 */	
-	resource_filename = filepath_find(respath, "languages");
-	resource_filename[strlen(resource_filename) - 9] = 0;
-	res_dir_location = resource_filename;
-
-	/* languages file */
-	languages_file_location = filepath_find(respath, "languages");
+	 * @todo find and slaughter all references to this!
+	 */
+	res_dir_location = calloc(1, strlen(languages_file_location) - 8);
+	memcpy(res_dir_location, 
+	       languages_file_location, 
+	       strlen(languages_file_location) - 9);
+	LOG(("Using '%s' for resource path", res_dir_location));
 
 	/* initialise the glade templates */
 	nsgtk_init_glade(respath);
@@ -378,6 +391,17 @@ static void gui_init(int argc, char** argv, char **respath)
 	search_default_ico_location = filepath_find(respath, "default.ico");
 	LOG(("Using '%s' as default search ico", search_default_ico_location));
 
+	/* Default favicon */
+	resource_filename = filepath_find(respath, "favicon.png");
+	if (resource_filename != NULL) {
+		favicon_pixbuf = gdk_pixbuf_new_from_file(resource_filename, NULL);
+		free(resource_filename);
+		if (favicon_pixbuf == NULL) {
+			favicon_pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, 16,16);
+			
+		}
+	}
+
 	/* Toolbar inicies file */
 	toolbar_indices_file_location = filepath_find(respath, "toolbarIndices");
 	LOG(("Using '%s' as custom toolbar settings file", toolbar_indices_file_location));
@@ -392,16 +416,6 @@ static void gui_init(int argc, char** argv, char **respath)
 	filepath_sfinddef(respath, buf, "mime.types", "/etc/");
 	gtk_fetch_filetype_init(buf);
 
-	/* set up stylesheet urls */
-	default_stylesheet_url = strdup("resource:gtkdefault.css");
-	LOG(("Using '%s' as Default CSS URL", default_stylesheet_url));
-
-	quirks_stylesheet_url = strdup("resource:quirks.css");
-	LOG(("Using '%s' as quirks CSS URL", quirks_stylesheet_url));
-
-	adblock_stylesheet_url = strdup("resource:adblock.css");
-	LOG(("Using '%s' as AdBlock CSS URL", adblock_stylesheet_url));
-
 	urldb_load(option_url_file);
 	urldb_load_cookies(option_cookie_file);
 
@@ -414,28 +428,29 @@ static void gui_init(int argc, char** argv, char **respath)
 					    gdk_screen_get_default()));
 	LOG(("Set CSS DPI to %f", FIXTOFLT(nscss_screen_dpi)));
 
-	if (nsgtk_history_init(glade_history_file_location) == false)
+	if (nsgtk_history_init(glade_file_location->history) == false)
 		die("Unable to initialise history window.\n");
 
-	if (nsgtk_download_init(glade_downloads_file_location) == false)
+	if (nsgtk_download_init(glade_file_location->downloads) == false)
 		die("Unable to initialise download window.\n");
 
-	if (nsgtk_cookies_init(glade_cookies_file_location) == false)
+	if (nsgtk_cookies_init(glade_file_location->cookies) == false)
 		die("Unable to initialise cookies window.\n");
 
-	if (nsgtk_hotlist_init(glade_hotlist_file_location) == false)
+	if (nsgtk_hotlist_init(glade_file_location->hotlist) == false)
 		die("Unable to initialise hotlist window.\n");
 
 	sslcert_init(tree_content_icon_name);
 
-        if (option_homepage_url != NULL && option_homepage_url[0] != '\0')
+        if ((option_homepage_url != NULL) && 
+	    (option_homepage_url[0] != '\0'))
                 addr = option_homepage_url;
 
 	if (2 <= argc)
 		addr = argv[1];
 
         /* Last step of initialization. Opens the main browser window. */
-	bw = browser_window_create(addr, 0, 0, true, false);
+	browser_window_create(addr, 0, 0, true, false);
 }
 
 
@@ -457,10 +472,12 @@ static void nsgtk_check_homedir(void)
 	snprintf(buf, PATH_MAX, "%s/.netsurf", hdir);
 	if (access(buf, F_OK) != 0) {
 		LOG(("You don't have a ~/.netsurf - creating one for you."));
-		if (mkdir(buf, 0777) == -1) {
+		if (mkdir(buf, S_IRWXU) == -1) {
 			LOG(("Unable to create %s", buf));
 			die("NetSurf requires ~/.netsurf to exist, but it cannot be created.\n");
 		}
+	} else {
+		chmod(buf, S_IRWXU);
 	}
 }
 
@@ -493,7 +510,11 @@ int main(int argc, char** argv)
 	gui_init(argc, argv, respaths);
 
 	netsurf_main_loop();
-
+	
+	/* Ensure all scaffoldings are destroyed before we go into exit */
+	while (scaf_list != NULL)
+		nsgtk_scaffolding_destroy(scaf_list);
+	
 	netsurf_exit();
 
 	return 0;
@@ -563,13 +584,6 @@ void gui_poll(bool active)
 }
 
 
-void gui_multitask(void)
-{
-	while (gtk_events_pending())
-		gtk_main_iteration();
-}
-
-
 void gui_quit(void)
 {
 	nsgtk_download_destroy();
@@ -579,9 +593,6 @@ void gui_quit(void)
 	nsgtk_history_destroy();
 	nsgtk_hotlist_destroy();
 	sslcert_cleanup();
-	free(default_stylesheet_url);
-	free(quirks_stylesheet_url);
-	free(adblock_stylesheet_url);
 	free(option_cookie_file);
 	free(option_cookie_jar);
 	free(print_options_file_location);
@@ -595,10 +606,6 @@ void gui_quit(void)
 
 
 
-
-
-
-
 static void nsgtk_select_menu_clicked(GtkCheckMenuItem *checkmenuitem,
 					gpointer user_data)
 {
@@ -685,7 +692,7 @@ void gui_cert_verify(const char *url, const struct ssl_cert_info *certs,
 {	
 	static struct nsgtk_treeview *ssl_window;	
 	struct sslcert_session_data *data;
-	GladeXML *x = glade_xml_new(glade_ssl_file_location, NULL, NULL);
+	GladeXML *x = glade_xml_new(glade_file_location->ssl, NULL, NULL);
 	GtkButton *accept, *reject;
 	void **session = calloc(sizeof(void *), 3);
 	GtkWindow *window;
