@@ -32,10 +32,7 @@ extern const struct plotter_table nsgtk_plotters;
 /* make sure this is NULL if no redraw is in progress */
 extern GtkWidget *current_widget;
 extern GdkDrawable *current_drawable;
-extern GdkGC *current_gc;
-#ifdef CAIRO_VERSION
 extern cairo_t *current_cr;
-#endif
 
 void nsgtk_set_colour(colour c);
 void nsgtk_plot_caret(int x, int y, int h);
