@@ -74,8 +74,8 @@
 #include "gtk/treeview.h"
 #include "gtk/window.h"
 #include "gtk/options.h"
-#include "gtk/sexy_icon_entry.h"
 #include "gtk/compat.h"
+#include "gtk/gdk.h"
 #include "image/ico.h"
 #include "render/box.h"
 #include "render/font.h"
@@ -88,9 +88,6 @@
 
 #include "utils/log.h"
 
-/** Connect a GTK signal handler to a widget */
-#define SIG_CONNECT(obj, sig, callback, ptr) \
-	g_signal_connect(G_OBJECT(obj), (sig), G_CALLBACK(callback), (ptr))
 
 /** Obtain a GTK widget handle from glade xml object */
 #define GET_WIDGET(x) glade_xml_get_widget(g->xml, (x))
@@ -112,7 +109,7 @@ static gboolean nsgtk_on_##q##_activate(struct gtk_scaffolding *g)
 
 /** Macro to define a handler for menu events. */
 #define MENUHANDLER(q)\
-static gboolean nsgtk_on_##q##_activate(GtkMenuItem *widget, gpointer data)
+static gboolean nsgtk_on_##q##_activate_menu(GtkMenuItem *widget, gpointer data)
 
 /** Macro to define a handler for button events. */
 #define BUTTONHANDLER(q)\
@@ -134,9 +131,7 @@ struct gtk_scaffolding {
 	GtkToolbar			*tool_bar;
 	struct nsgtk_button_connect	*buttons[PLACEHOLDER_BUTTON];
 	GtkImage			*throbber;
-	GtkImage			*icoFav;
 	struct gtk_search		*search;
-	GtkImage			*webSearchIco;
 	GtkWidget			*webSearchEntry;
 	GtkPaned			*status_pane;
 
@@ -316,6 +311,8 @@ void nsgtk_scaffolding_destroy(nsgtk_scaffolding *g)
 	if (g->being_destroyed) return;
 	g->being_destroyed = 1;
 	nsgtk_window_close(g);
+	/* We're now unlinked, so let's finally destroy ourselves */
+	nsgtk_window_destroy_browser(g->top_level);
 }
 
 /**
@@ -335,9 +332,9 @@ static void nsgtk_window_update_back_forward(struct gtk_scaffolding *g)
 
 	/* update the url bar, particularly necessary when tabbing */
 	if (bw->current_content != NULL &&
-			content_get_url(bw->current_content) != NULL)
+			hlcache_handle_get_url(bw->current_content) != NULL)
 		browser_window_refresh_url_bar(bw,
-				content_get_url(bw->current_content),
+				hlcache_handle_get_url(bw->current_content),
 				bw->frag_id);
 
 	/* update the local history window, as well as queuing a redraw
@@ -497,7 +494,7 @@ static gboolean nsgtk_window_tool_bar_clicked(GtkToolbar *toolbar,
 /**
  * Update the menus when the number of tabs changes.
  */
-static void nsgtk_window_tabs_num_changed(GtkNotebook *notebook,
+static void nsgtk_window_tabs_add(GtkNotebook *notebook,
 		GtkWidget *page, guint page_num, struct gtk_scaffolding *g)
 {
 	gboolean visible = gtk_notebook_get_show_tabs(g->notebook);
@@ -509,6 +506,26 @@ static void nsgtk_window_tabs_num_changed(GtkNotebook *notebook,
 	nsgtk_scaffolding_set_sensitivity(g);
 }
 
+/**
+ * Update the menus when the number of tabs changes.
+ */
+static void nsgtk_window_tabs_remove(GtkNotebook *notebook,
+		GtkWidget *page, guint page_num, struct gtk_scaffolding *g)
+{
+
+	if (gtk_notebook_get_n_pages(notebook) == 0) {
+		nsgtk_scaffolding_destroy(g);
+	} else {
+	gboolean visible = gtk_notebook_get_show_tabs(g->notebook);
+	g_object_set(g->menu_bar->view_submenu->tabs_menuitem, "visible", visible, NULL);
+	g_object_set(g->menu_popup->view_submenu->tabs_menuitem, "visible", visible, NULL);
+	g->buttons[NEXTTAB_BUTTON]->sensitivity = visible;
+	g->buttons[PREVTAB_BUTTON]->sensitivity = visible;
+	g->buttons[CLOSETAB_BUTTON]->sensitivity = visible;
+	nsgtk_scaffolding_set_sensitivity(g);
+	}
+}
+
 /**
  * Handle opening a file path.
  */
@@ -619,8 +636,8 @@ MULTIHANDLER(savepage)
 	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(fc), filter);
 	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(fc), filter);
 
-	res = url_nice(content_get_url(nsgtk_get_browser_window(
-			g->top_level)->current_content), &path, false);
+	res = url_nice(nsurl_access(hlcache_handle_get_url(nsgtk_get_browser_window(
+			g->top_level)->current_content)), &path, false);
 	if (res != URL_FUNC_OK) {
 		path = strdup(messages_get("SaveText"));
 		if (path == NULL) {
@@ -680,7 +697,8 @@ MULTIHANDLER(pdf)
 
 	LOG(("Print preview (generating PDF)  started."));
 
-	res = url_nice(content_get_url(bw->current_content), &url_name, true);
+	res = url_nice(nsurl_access(hlcache_handle_get_url(bw->current_content)),
+			&url_name, true);
 	if (res != URL_FUNC_OK) {
 		warn_user(messages_get(res == URL_FUNC_NOMEM ? "NoMemory"
 							     : "URIError"), 0);
@@ -754,8 +772,8 @@ MULTIHANDLER(plaintext)
 	char *filename;
 	url_func_result res;
 
-	res = url_nice(content_get_url(nsgtk_get_browser_window(
-			g->top_level)->current_content), &filename, false);
+	res = url_nice(nsurl_access(hlcache_handle_get_url(nsgtk_get_browser_window(
+			g->top_level)->current_content)), &filename, false);
 	if (res != URL_FUNC_OK) {
 		filename = strdup(messages_get("SaveText"));
 		if (filename == NULL) {
@@ -1405,9 +1423,9 @@ MULTIHANDLER(addbookmarks)
 	struct browser_window *bw = nsgtk_get_browser_window(g->top_level);
 
 	if (bw == NULL || bw->current_content == NULL ||
-			content_get_url(bw->current_content) == NULL)
+			hlcache_handle_get_url(bw->current_content) == NULL)
 		return TRUE;
-	hotlist_add_page(content_get_url(bw->current_content));
+	hotlist_add_page(nsurl_access(hlcache_handle_get_url(bw->current_content)));
 	return TRUE;
 }
 
@@ -1509,13 +1527,13 @@ static gboolean nsgtk_history_expose_event(GtkWidget *widget,
 
 	struct redraw_context ctx = {
 		.interactive = true,
+		.background_images = true,
 		.plot = &nsgtk_plotters
 	};
 
 	current_widget = widget;
-	current_drawable = widget->window;
 
-	current_cr = gdk_cairo_create(current_drawable);
+	current_cr = gdk_cairo_create(widget->window);
 
 	clip.x0 = event->area.x;
 	clip.y0 = event->area.y;
@@ -1567,8 +1585,8 @@ static void nsgtk_attach_menu_handlers(struct gtk_scaffolding *g)
 		}
 	}
 #define CONNECT_CHECK(q)\
-	g_signal_connect(g->menu_bar->view_submenu->toolbars_submenu->q##_menuitem, "toggled", G_CALLBACK(nsgtk_on_##q##_activate), g);\
-	g_signal_connect(g->menu_popup->view_submenu->toolbars_submenu->q##_menuitem, "toggled", G_CALLBACK(nsgtk_on_##q##_activate), g)
+	g_signal_connect(g->menu_bar->view_submenu->toolbars_submenu->q##_menuitem, "toggled", G_CALLBACK(nsgtk_on_##q##_activate_menu), g);\
+	g_signal_connect(g->menu_popup->view_submenu->toolbars_submenu->q##_menuitem, "toggled", G_CALLBACK(nsgtk_on_##q##_activate_menu), g)
 	CONNECT_CHECK(menubar);
 	CONNECT_CHECK(toolbar);
 #undef CONNECT_CHECK
@@ -1590,30 +1608,33 @@ static bool nsgtk_new_scaffolding_popup(struct gtk_scaffolding *g, GtkAccelGroup
 	if (nmenu == NULL)
 		return false;
 
+/** Connect a GTK signal handler to a widget */
+#define SIG_CONNECT(obj, sig, callback, ptr) \
+	g_signal_connect(G_OBJECT(obj), (sig), G_CALLBACK(callback), (ptr))
+
 	SIG_CONNECT(nmenu->popup_menu, "hide",
 		    nsgtk_window_popup_menu_hidden, g);
 
-	SIG_CONNECT(nmenu->savelink_menuitem, "activate",
-		    nsgtk_on_savelink_activate, g);
+	g_signal_connect(nmenu->savelink_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_savelink_activate_menu), g);
 
-	SIG_CONNECT(nmenu->opentab_menuitem, "activate",
-		    nsgtk_on_link_opentab_activate, g);
+	g_signal_connect(nmenu->opentab_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_link_opentab_activate_menu), g);
 
-	SIG_CONNECT(nmenu->openwin_menuitem, "activate",
-		    nsgtk_on_link_openwin_activate, g);
+	g_signal_connect(nmenu->openwin_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_link_openwin_activate_menu), g);
 
-	SIG_CONNECT(nmenu->cut_menuitem, "activate",
-		    nsgtk_on_cut_activate, g);
+	g_signal_connect(nmenu->cut_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_cut_activate_menu), g);
 
-	SIG_CONNECT(nmenu->copy_menuitem, "activate",
-		    nsgtk_on_copy_activate, g);
+	g_signal_connect(nmenu->copy_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_copy_activate_menu), g);
 
-	SIG_CONNECT(nmenu->paste_menuitem, "activate",
-		    nsgtk_on_paste_activate, g);
-
-	SIG_CONNECT(nmenu->customize_menuitem, "activate",
-		    nsgtk_on_customize_activate, g);
+	g_signal_connect(nmenu->paste_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_paste_activate_menu), g);
 
+	g_signal_connect(nmenu->customize_menuitem, "activate", 
+			 G_CALLBACK(nsgtk_on_customize_activate_menu), g);
 
 	/* set initial popup menu visibility */
 	popup_menu_hide(nmenu, true, false, false, false, true);
@@ -1644,7 +1665,7 @@ nsgtk_scaffolding *nsgtk_new_scaffolding(struct gui_window *toplevel)
 	/* load the window template from the glade xml file, and extract
 	 * widget references from it for later use.
 	 */
-	g->xml = glade_xml_new(glade_netsurf_file_location,
+	g->xml = glade_xml_new(glade_file_location->netsurf,
 			"wndBrowser", NULL);
 	glade_xml_signal_autoconnect(g->xml);
 	g->window = GTK_WINDOW(GET_WIDGET("wndBrowser"));
@@ -1776,7 +1797,7 @@ nsgtk_scaffolding *nsgtk_new_scaffolding(struct gui_window *toplevel)
 
 	gtk_toolbar_set_show_arrow(g->tool_bar, TRUE);
 	gtk_widget_show_all(GTK_WIDGET(g->tool_bar));
-	nsgtk_tab_init(GTK_WIDGET(g->notebook));
+	nsgtk_tab_init(g->notebook);
 
 	gtk_widget_set_size_request(GTK_WIDGET(
 			g->buttons[HISTORY_BUTTON]->button), 20, -1);
@@ -1843,9 +1864,9 @@ nsgtk_scaffolding *nsgtk_new_scaffolding(struct gui_window *toplevel)
 			gtk_widget_hide_on_delete, NULL);
 
 	g_signal_connect_after(g->notebook, "page-added",
-			G_CALLBACK(nsgtk_window_tabs_num_changed), g);
+			G_CALLBACK(nsgtk_window_tabs_add), g);
 	g_signal_connect_after(g->notebook, "page-removed",
-			G_CALLBACK(nsgtk_window_tabs_num_changed), g);
+			G_CALLBACK(nsgtk_window_tabs_remove), g);
 
 	/* connect signals to handlers. */
 	CONNECT(g->window, "delete-event", nsgtk_window_delete_event, g);
@@ -1934,13 +1955,9 @@ void gui_window_set_title(struct gui_window *_g, const char *title)
 	nsgtk_tab_set_title(_g, title);
 
 	if (g->top_level == _g) {
-		if (title == NULL || title[0] == '\0')
-		{
+		if (title == NULL || title[0] == '\0') {
 			gtk_window_set_title(g->window, "NetSurf");
-
-		}
-		else
-		{
+		} else {
 			strcpy(nt, title);
 			strcat(nt, suffix);
 			gtk_window_set_title(g->window, nt);
@@ -1989,88 +2006,59 @@ void gui_window_stop_throbber(struct gui_window* _g)
 	gtk_image_set_from_pixbuf(g->throbber, nsgtk_throbber->framedata[0]);
 }
 
+
 /**
  * set favicon
  */
-void gui_window_set_icon(struct gui_window *_g, hlcache_handle *icon)
+void
+nsgtk_scaffolding_set_icon(struct gui_window *gw)
 {
-	struct gtk_scaffolding *g = nsgtk_get_scaffold(_g);
-	struct bitmap *icon_bitmap;
-	GtkImage *iconImage = NULL;
+	struct gtk_scaffolding *sc = nsgtk_get_scaffold(gw);
+	GdkPixbuf *icon_pixbuf = nsgtk_get_icon(gw);
 
-	if (g->top_level != _g)
+	/* check icon needs to be shown */
+	if ((icon_pixbuf == NULL) || 
+	    (sc->top_level != gw)) {
 		return;
-
-	icon_bitmap = (icon != NULL) ? content_get_bitmap(icon) : NULL;
-
-	if (icon_bitmap != NULL) {
-		GdkPixbuf *pb = gtk_bitmap_get_primary(icon_bitmap);
-		if (pb != NULL && gdk_pixbuf_get_width(pb) > 0 &&
-			gdk_pixbuf_get_height(pb) > 0) {
-			pb = gdk_pixbuf_scale_simple(pb, 16, 16, GDK_INTERP_HYPER);
-			iconImage = GTK_IMAGE(gtk_image_new_from_pixbuf(pb));
-		}
-	}
-	if (iconImage == NULL) {
-		/** \todo Does pb need cleaning up? */
-		char imagepath[strlen(res_dir_location) +
-				SLEN("favicon.png") + 1];
-		sprintf(imagepath, "%sfavicon.png", res_dir_location);
-		iconImage = GTK_IMAGE(gtk_image_new_from_file(imagepath));
 	}
 
-	if (iconImage == NULL)
-		return;
+	nsgtk_entry_set_icon_from_pixbuf(sc->url_bar, 
+					 GTK_ENTRY_ICON_PRIMARY, 
+					 icon_pixbuf);
 
-	if (g->icoFav != NULL)
-		g_object_unref(g->icoFav);
-	g->icoFav = iconImage;
-
-	sexy_icon_entry_set_icon(SEXY_ICON_ENTRY(g->url_bar),
-			SEXY_ICON_ENTRY_PRIMARY, GTK_IMAGE(g->icoFav));
-	gtk_widget_show_all(GTK_WIDGET(g->buttons[URL_BAR_ITEM]->button));
+	gtk_widget_show_all(GTK_WIDGET(sc->buttons[URL_BAR_ITEM]->button));
 }
 
 void gui_window_set_search_ico(hlcache_handle *ico)
 {
-	GdkPixbuf *pbico;
-	GtkImage *searchico;
-	struct bitmap *ico_bitmap;
+	struct bitmap *srch_bitmap;
 	nsgtk_scaffolding *current;
+	GdkPixbuf *srch_pixbuf;
 
-	if (ico == NULL && (ico = search_web_ico()) == NULL)
+	if ((ico == NULL) && 
+	    (ico = search_web_ico()) == NULL) {
 		return;
+	}
 
-	ico_bitmap = content_get_bitmap(ico);
-	if (ico_bitmap == NULL)
+	srch_bitmap = content_get_bitmap(ico);
+	if (srch_bitmap == NULL) {
 		return;
+	}
 
-	pbico = gtk_bitmap_get_primary(ico_bitmap);
-	if (pbico != NULL && gdk_pixbuf_get_width(pbico) > 0 &&
-			gdk_pixbuf_get_height(pbico) > 0) {
-		pbico = gdk_pixbuf_scale_simple(pbico, 20, 20,
-				GDK_INTERP_HYPER);
-		searchico = GTK_IMAGE(gtk_image_new_from_pixbuf(pbico));
-	} else {
-		/** \todo Does pbico need cleaning up? */
+	srch_pixbuf = nsgdk_pixbuf_get_from_surface(srch_bitmap->surface, 16, 16);
+
+	if (srch_pixbuf == NULL) {
 		return;
 	}
 
 	/* add ico to each window's toolbar */
 	for (current = scaf_list; current != NULL; current = current->next) {
-		if (searchico != NULL) {
-			/** \todo Are we leaking webSearchIco here? */
-			current->webSearchIco = searchico;
-			sexy_icon_entry_set_icon(SEXY_ICON_ENTRY(
-					current->webSearchEntry),
-					SEXY_ICON_ENTRY_PRIMARY,
-					current->webSearchIco);
-		}
-		if (pbico != NULL)
-			searchico = GTK_IMAGE(gtk_image_new_from_pixbuf(pbico));
-		else
-			searchico = NULL;
+		nsgtk_entry_set_icon_from_pixbuf(current->webSearchEntry, 
+						 GTK_ENTRY_ICON_PRIMARY,
+						 srch_pixbuf);
 	}
+
+	g_object_unref(srch_pixbuf);
 }
 
 bool nsgtk_scaffolding_is_busy(nsgtk_scaffolding *g)
@@ -2141,8 +2129,6 @@ void nsgtk_scaffolding_update_url_bar_ref(nsgtk_scaffolding *g)
 {
 	g->url_bar = GTK_WIDGET(gtk_bin_get_child(GTK_BIN(
 			nsgtk_scaffolding_button(g, URL_BAR_ITEM)->button)));
-	g->icoFav = sexy_icon_entry_get_icon(SEXY_ICON_ENTRY(g->url_bar),
-			SEXY_ICON_ENTRY_PRIMARY);
 
 	gtk_entry_set_completion(GTK_ENTRY(g->url_bar),
 			g->url_bar_completion);
@@ -2157,8 +2143,6 @@ void nsgtk_scaffolding_update_websearch_ref(nsgtk_scaffolding *g)
 {
 	g->webSearchEntry = gtk_bin_get_child(GTK_BIN(
 			g->buttons[WEBSEARCH_ITEM]->button));
-	g->webSearchIco = sexy_icon_entry_get_icon(SEXY_ICON_ENTRY(
-			g->webSearchEntry), SEXY_ICON_ENTRY_PRIMARY);
 }
 
 void nsgtk_scaffolding_set_websearch(nsgtk_scaffolding *g, const char *content)
@@ -2237,15 +2221,25 @@ struct gui_window *nsgtk_scaffolding_top_level(nsgtk_scaffolding *g)
 	return g->top_level;
 }
 
-void nsgtk_scaffolding_set_top_level (struct gui_window *gw)
+/* set the current active top level gui window */
+void nsgtk_scaffolding_set_top_level(struct gui_window *gw)
 {
-	nsgtk_get_scaffold(gw)->top_level = gw;
-	struct browser_window *bw = nsgtk_get_browser_window(gw);
+	struct browser_window *bw;
+	nsgtk_scaffolding *sc;
+
+	assert(gw != NULL);
+
+	bw = nsgtk_get_browser_window(gw);
 
 	assert(bw != NULL);
 
+	sc = nsgtk_get_scaffold(gw);
+	assert(sc != NULL);
+
+	sc->top_level = gw;
+
 	/* Synchronise the history (will also update the URL bar) */
-	nsgtk_window_update_back_forward(nsgtk_get_scaffold(gw));
+	nsgtk_window_update_back_forward(sc);
 
 	/* clear effects of potential searches */
 	browser_window_search_destroy_context(bw);
@@ -2253,14 +2247,11 @@ void nsgtk_scaffolding_set_top_level (struct gui_window *gw)
 	nsgtk_search_set_forward_state(true, bw);
 	nsgtk_search_set_back_state(true, bw);
 
-	/* Ensure the window's title bar as well as favicon are updated */
-	if (bw->current_content != NULL) {
-		gui_window_set_title(gw,
-				content_get_title(bw->current_content));
+	nsgtk_scaffolding_set_icon(gw);
 
-		if (content_get_type(bw->current_content) == CONTENT_HTML)
-			gui_window_set_icon(gw,
-					html_get_favicon(bw->current_content));
+	/* Ensure the window's title bar is updated */
+	if (bw->current_content != NULL) {
+		gui_window_set_title(gw, content_get_title(bw->current_content));
 	}
 }
 
