@@ -39,6 +39,8 @@
 #include "gtk/plotters.h"
 #include "gtk/schedule.h"
 #include "gtk/tabs.h"
+#include "gtk/bitmap.h"
+#include "gtk/gdk.h"
 #include "utils/log.h"
 #include "utils/utils.h"
 
@@ -46,7 +48,7 @@ struct gui_window {
 	/** The gtk scaffold object containing menu, buttons, url bar, [tabs],
 	 * drawing area, etc that may contain one or more gui_windows.
 	 */
-	nsgtk_scaffolding	*scaffold;
+	nsgtk_scaffolding *scaffold;
 
 	/** The 'content' window that is rendered in the gui_window */
 	struct browser_window	*bw;
@@ -74,9 +76,6 @@ struct gui_window {
 	/** display widget for this page or frame */
 	GtkLayout *layout;
 
-	/** frames only; top level of gtk structure of gui_window */
-	GtkScrolledWindow *scrolledwindow;
-
 	/** handle to the the visible tab */
 	GtkWidget *tab;
 
@@ -89,6 +88,9 @@ struct gui_window {
 	/** to allow disactivation / resume of normal window behaviour */
 	gulong signalhandler[NSGTK_WINDOW_SIGNAL_COUNT];
 
+	/** The icon this window should have */
+	GdkPixbuf *icon;
+
 	/** list for cleanup */
 	struct gui_window *next, *prev;
 };
@@ -101,6 +103,11 @@ nsgtk_scaffolding *nsgtk_get_scaffold(struct gui_window *g)
 	return g->scaffold;
 }
 
+GdkPixbuf *nsgtk_get_icon(struct gui_window *gw)
+{
+	return gw->icon;
+}
+
 struct browser_window *nsgtk_get_browser_window(struct gui_window *g)
 {
 	return g->bw;
@@ -145,6 +152,7 @@ static gboolean nsgtk_window_expose_event(GtkWidget *widget,
 	struct rect clip;
 	struct redraw_context ctx = {
 		.interactive = true,
+		.background_images = true,
 		.plot = &nsgtk_plotters
 	};
 
@@ -156,10 +164,8 @@ static gboolean nsgtk_window_expose_event(GtkWidget *widget,
 	assert(z);
 	assert(GTK_WIDGET(g->layout) == widget);
 
-
 	current_widget = (GtkWidget *)g->layout;
-	current_drawable = g->layout->bin_window;
-	current_cr = gdk_cairo_create(current_drawable);
+	current_cr = gdk_cairo_create(g->layout->bin_window);
 
 	clip.x0 = event->area.x;
 	clip.y0 = event->area.y;
@@ -168,8 +174,9 @@ static gboolean nsgtk_window_expose_event(GtkWidget *widget,
 
 	browser_window_redraw(g->bw, 0, 0, &clip, &ctx);
 
-	if (g->careth != 0)
+	if (g->careth != 0) {
 		nsgtk_plot_caret(g->caretx, g->carety, g->careth);
+	}
 
 	current_widget = NULL;
 	cairo_destroy(current_cr);
@@ -322,6 +329,13 @@ static gboolean nsgtk_window_scroll_event(GtkWidget *widget,
 
 	switch (event->direction) {
 	case GDK_SCROLL_LEFT:
+		if (browser_window_scroll_at_point(g->bw,
+				event->x / g->bw->scale,
+				event->y / g->bw->scale,
+				-100, 0) == true)
+			/* Core handled it */
+			return TRUE;
+
 		scroll = hscroll;
 		value = gtk_adjustment_get_value(scroll) -
 			(scroll->step_increment * 2);
@@ -330,6 +344,13 @@ static gboolean nsgtk_window_scroll_event(GtkWidget *widget,
 		break;
 
 	case GDK_SCROLL_UP:
+		if (browser_window_scroll_at_point(g->bw,
+				event->x / g->bw->scale,
+				event->y / g->bw->scale,
+				0, -100) == true)
+			/* Core handled it */
+			return TRUE;
+
 		scroll = vscroll;
 		value = gtk_adjustment_get_value(scroll) -
 			(scroll->step_increment * 2);
@@ -338,6 +359,13 @@ static gboolean nsgtk_window_scroll_event(GtkWidget *widget,
 		break;
 
 	case GDK_SCROLL_RIGHT:
+		if (browser_window_scroll_at_point(g->bw,
+				event->x / g->bw->scale,
+				event->y / g->bw->scale,
+				100, 0) == true)
+			/* Core handled it */
+			return TRUE;
+
 		scroll = hscroll;
 		value = gtk_adjustment_get_value(scroll) +
 			(scroll->step_increment * 2);
@@ -346,12 +374,20 @@ static gboolean nsgtk_window_scroll_event(GtkWidget *widget,
 		break;
 
 	case GDK_SCROLL_DOWN:
+		if (browser_window_scroll_at_point(g->bw,
+				event->x / g->bw->scale,
+				event->y / g->bw->scale,
+				0, 100) == true)
+			/* Core handled it */
+			return TRUE;
+
 		scroll = vscroll;
 		value = gtk_adjustment_get_value(scroll) +
 			(scroll->step_increment * 2);
 		if (value > scroll->upper - alloc->height)
 			value = scroll->upper - alloc->height;
 		break;
+
 	default:
 		return TRUE;
 	}
@@ -484,7 +520,6 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 					     bool new_tab)
 {
 	struct gui_window *g; /**< what we're creating to return */
-	GtkPolicyType scrollpolicy;
 
 	g = calloc(1, sizeof(*g));
 	if (!g) {
@@ -522,7 +557,8 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 	/* Construct our primary elements */
 
 	/* top-level document (not a frame) => create a new tab */
-	GladeXML *xml = glade_xml_new(glade_netsurf_file_location, "tabContents", NULL);
+	GladeXML *xml = glade_xml_new(glade_file_location->netsurf, 
+				"tabContents", NULL);
 	if (!xml) {
 		warn_user("MiscError", "Failed to create tab contents");
 		free(g);
@@ -560,23 +596,6 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 
 	g_object_unref(xml);
 
-	switch(bw->scrolling) {
-	case SCROLLING_NO:
-		scrollpolicy = GTK_POLICY_NEVER;
-		break;
-	case SCROLLING_YES:
-		scrollpolicy = GTK_POLICY_ALWAYS;
-		break;
-	case SCROLLING_AUTO:
-	default:
-		scrollpolicy = GTK_POLICY_AUTOMATIC;
-		break;
-	}
-
-	if (g->scrolledwindow)
-		gtk_scrolled_window_set_policy(g->scrolledwindow,
-				scrollpolicy, scrollpolicy);
-
 	/* Attach ourselves to the list (push_top) */
 	if (window_list)
 		window_list->prev = g;
@@ -629,8 +648,8 @@ struct gui_window *gui_create_browser_window(struct browser_window *bw,
 void nsgtk_reflow_all_windows(void)
 {
 	for (struct gui_window *g = window_list; g; g = g->next) {
-		nsgtk_tab_options_changed(GTK_WIDGET(
-				nsgtk_scaffolding_notebook(g->scaffold)));
+		nsgtk_tab_options_changed(
+				nsgtk_scaffolding_notebook(g->scaffold));
 		g->bw->reformat_pending = true;
 	}
 
@@ -681,22 +700,39 @@ void gui_window_destroy(struct gui_window *g)
 	LOG(("	   Scaffolding: %p", g->scaffold));
 	LOG(("	   Window name: %s", g->bw->name));
 
-	if (g->scrolledwindow == NULL) {
-		/* tab => remove tab */
-		gtk_widget_destroy(gtk_widget_get_parent(GTK_WIDGET(g->layout)));
-		/* if it was the last tab, destroy scaffold too */
-		gint numbertabs = gtk_notebook_get_n_pages(nsgtk_scaffolding_notebook(g->scaffold));
-		if (numbertabs == 0)
-			nsgtk_scaffolding_destroy(g->scaffold);
-	} else {
-		/* frame within a document => destroy frame only */
-		gtk_widget_destroy(GTK_WIDGET(g->scrolledwindow));
-	}
-
-	free(g);
-
+	/* tab => remove tab */
+	gtk_widget_destroy(gtk_widget_get_parent(GTK_WIDGET(g->layout)));
 }
 
+/**
+ * set favicon
+ */
+void gui_window_set_icon(struct gui_window *gw, hlcache_handle *icon)
+{
+	struct bitmap *icon_bitmap = NULL;
+
+	/* free any existing icon */
+	if (gw->icon != NULL) {
+		g_object_unref(gw->icon);
+		gw->icon = NULL;
+	}
+
+	if (icon != NULL) {
+		icon_bitmap = content_get_bitmap(icon);
+		if (icon_bitmap != NULL) {
+			LOG(("Using %p bitmap", icon_bitmap));
+			gw->icon = nsgdk_pixbuf_get_from_surface(icon_bitmap->surface, 16, 16);
+		} 
+	} 
+
+	if (gw->icon == NULL) {
+		LOG(("Using default favicon"));
+		g_object_ref(favicon_pixbuf);
+		gw->icon = favicon_pixbuf;
+	}
+
+	nsgtk_scaffolding_set_icon(gw);
+}
 
 static void nsgtk_redraw_caret(struct gui_window *g)
 {
@@ -958,8 +994,8 @@ bool gui_window_scroll_start(struct gui_window *g)
 	return true;
 }
 
-bool gui_window_box_scroll_start(struct gui_window *g,
-		    int x0, int y0, int x1, int y1)
+bool gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+		const struct rect *rect)
 {
 	return true;
 }
