@@ -99,6 +99,7 @@ static bool nssprite_convert(struct content *c)
 
 	const char *data;
 	unsigned long size;
+	char title[100];
 
 	data = content__get_source_data(c, &size);
 
@@ -119,37 +120,40 @@ static bool nssprite_convert(struct content *c)
 		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
 		return false;
 	}
-	unsigned char* imagebuf = bitmap_get_buffer(nssprite->bitmap);
+	uint32_t* imagebuf = (uint32_t *)bitmap_get_buffer(nssprite->bitmap);
 	if (!imagebuf) {
 		msg_data.error = messages_get("NoMemory");
 		content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
 		return false;
 	}
-	unsigned int row_width = bitmap_get_rowstride(nssprite->bitmap);
-
-	memcpy(imagebuf, sprite->image, row_width * sprite->height); // TODO: avoid copying entire image buffer
+	unsigned char *spritebuf = (unsigned char *)sprite->image;
 
 	/* reverse byte order of each word */
 	for (uint32_t y = 0; y < sprite->height; y++) {
 		for (uint32_t x = 0; x < sprite->width; x++) {
 			int offset = 4 * (y * sprite->width + x);
-			uint32_t r = imagebuf[offset+3];
-			uint32_t g = imagebuf[offset+2];
-			uint32_t b = imagebuf[offset+1];
-			uint32_t a = imagebuf[offset];
-			imagebuf[offset] = r;
-			imagebuf[offset+1] = g;
-			imagebuf[offset+2] = b;
-			imagebuf[offset+3] = a;
+
+			*imagebuf = (spritebuf[offset] << 24) |
+					(spritebuf[offset + 1] << 16) |
+					(spritebuf[offset + 2] << 8) |
+					(spritebuf[offset + 3]);
+
+			imagebuf++;
 		}
 	}
 
 	c->width = sprite->width;
 	c->height = sprite->height;
+
+	snprintf(title, sizeof(title), messages_get("SpriteTitle"), 
+		 c->width, c->height, size);
+	content__set_title(c, title);
+
 	bitmap_modified(nssprite->bitmap);
 
 	content_set_ready(c);
 	content_set_done(c);
+	content_set_status(c, ""); /* Done: update status bar */
 
 	return true;
 }
