@@ -23,25 +23,6 @@
 #ifndef _NETSURF_NS_SPRITE_H_
 #define _NETSURF_NS_SPRITE_H_
 
-#include "utils/config.h"
-#ifdef WITH_NSSPRITE
-
-#include <stdbool.h>
-
-struct content;
-struct rect;
-
-struct content_nssprite_data {
-	struct rosprite_area* sprite_area;
-};
-
-bool nssprite_convert(struct content *c);
-void nssprite_destroy(struct content *c);
-bool nssprite_redraw(struct content *c, int x, int y,
-		int width, int height, const struct rect *clip,
-		float scale, colour background_colour);
-bool nssprite_clone(const struct content *old, struct content *new_content);
-
-#endif /* WITH_NSSPRITE */
+nserror nssprite_init(void);
 
 #endif
