@@ -498,7 +498,9 @@ png_cache_convert_error:
 
 	free((png_bytep *) row_pointers);
 
-	return (struct bitmap *) bitmap;
+	bitmap_modified((struct bitmap *)bitmap);
+
+	return (struct bitmap *)bitmap;
 }
 
 static bool nspng_convert(struct content *c)
@@ -591,7 +593,8 @@ static const content_handler nspng_content_handler = {
 };
 
 static const char *nspng_types[] = {
-	"image/png"
+	"image/png",
+	"image/x-png"
 };
 
 CONTENT_FACTORY_REGISTER_TYPES(nspng, nspng_types, nspng_content_handler);
