@@ -370,11 +370,10 @@ gui_window_remove_caret(struct gui_window *g)
 }
 
 bool
-gui_window_box_scroll_start(struct gui_window *g,
-                            int x0, int y0, int x1, int y1)
+gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+                      const struct rect *rect)
 {
-  fprintf(stdout, "WINDOW SCROLL_START WIN %u X0 %d Y0 %d X1 %d Y1 %d\n",
-          g->win_num, x0, y0, x1, y1);
+  fprintf(stdout, "WINDOW SCROLL_START WIN %u TYPE %i\n", g->win_num, type);
   return false;
 }
 
@@ -404,6 +403,7 @@ monkey_window_handle_new(int argc, char **argv)
   if (argc > 3)
     return;
   bw = browser_window_create((argc == 3) ? argv[2] : NULL, NULL, NULL, true, false);
+  (void) bw;
 }
 
 static void
@@ -448,11 +448,12 @@ monkey_window_handle_redraw(int argc, char **argv)
   struct rect clip;
   struct redraw_context ctx = {
     .interactive = true,
+    .background_images = true,
     .plot = &monkey_plotters
   };
   
   if (argc != 3 && argc != 7) {
-    fprintf(stdout, "ERROR WINDOW REDRAW ARGS BAD");
+    fprintf(stdout, "ERROR WINDOW REDRAW ARGS BAD\n");
     return;
   }
 
@@ -476,7 +477,9 @@ monkey_window_handle_redraw(int argc, char **argv)
   }
   
   LOG(("Issue redraw"));
+  fprintf(stdout, "WINDOW REDRAW WIN %d START\n", atoi(argv[2]));
   browser_window_redraw(gw->bw, gw->scrollx, gw->scrolly, &clip, &ctx);  
+  fprintf(stdout, "WINDOW REDRAW WIN %d STOP\n", atoi(argv[2]));
 }
 
 static void
@@ -484,7 +487,7 @@ monkey_window_handle_reload(int argc, char **argv)
 {
   struct gui_window *gw;
   if (argc != 3 && argc != 4) {
-    fprintf(stdout, "ERROR WINDOW RELOAD ARGS BAD");
+    fprintf(stdout, "ERROR WINDOW RELOAD ARGS BAD\n");
   }
   
   gw = monkey_find_window_by_num(atoi(argv[2]));
@@ -513,6 +516,8 @@ monkey_window_handle_command(int argc, char **argv)
     monkey_window_handle_redraw(argc, argv);
   } else if (strcmp(argv[1], "RELOAD") == 0) {
     monkey_window_handle_reload(argc, argv);
+  } else {
+    fprintf(stdout, "ERROR WINDOW COMMAND UNKNOWN %s\n", argv[1]);
   }
   
 }
