@@ -72,11 +72,10 @@ void *box_style_alloc(void *ptr, size_t len, void *pw)
 /**
  * Destructor for box nodes which own styles
  *
- * @param b The box being destroyed.
- * @return 0 to allow talloc to continue destroying the tree.
+ * \param b The box being destroyed.
+ * \return 0 to allow talloc to continue destroying the tree.
  */
-static int
-free_box_style(struct box *b)
+static int box_talloc_destructor(struct box *b)
 {
 	if ((b->flags & STYLE_OWNED) && b->style != NULL) {
 		css_computed_style_destroy(b->style);
@@ -87,6 +86,12 @@ free_box_style(struct box *b)
 		css_select_results_destroy(b->styles);
 		b->styles = NULL;
 	}
+
+	if (b->href != NULL)
+		nsurl_unref(b->href);
+
+	if (b->id != NULL)
+		lwc_string_unref(b->id);
 	
 	return 0;
 }
@@ -97,7 +102,7 @@ free_box_style(struct box *b)
  * \param  styles       selection results for the box, or NULL
  * \param  style        computed style for the box (not copied), or 0
  * \param  style_owned  whether style is owned by this box
- * \param  href         href for the box (not copied), or 0
+ * \param  href         href for the box (copied), or 0
  * \param  target       target for the box (not copied), or 0
  * \param  title        title for the box (not copied), or 0
  * \param  id           id for the box (not copied), or 0
@@ -109,8 +114,8 @@ free_box_style(struct box *b)
  */
 
 struct box * box_create(css_select_results *styles, css_computed_style *style,
-		bool style_owned, char *href, const char *target, char *title,
-		char *id, void *context)
+		bool style_owned, nsurl *href, const char *target, 
+		const char *title, lwc_string *id, void *context)
 {
 	unsigned int i;
 	struct box *box;
@@ -119,10 +124,9 @@ struct box * box_create(css_select_results *styles, css_computed_style *style,
 	if (!box) {
 		return 0;
 	}
-	
-	if (style_owned == true || styles != NULL)
-		talloc_set_destructor(box, free_box_style);
-	
+
+	talloc_set_destructor(box, box_talloc_destructor);
+
 	box->type = BOX_INLINE;
 	box->flags = 0;
 	box->flags = style_owned ? (box->flags | STYLE_OWNED) : box->flags;
@@ -142,7 +146,7 @@ struct box * box_create(css_select_results *styles, css_computed_style *style,
 	box->text = NULL;
 	box->length = 0;
 	box->space = 0;
-	box->href = href;
+	box->href = (href == NULL) ? NULL : nsurl_ref(href);
 	box->target = target;
 	box->title = title;
 	box->columns = 1;
@@ -849,11 +853,14 @@ struct box *box_pick_text_box(struct html_content *html,
  * \return  the box or 0 if not found
  */
 
-struct box *box_find_by_id(struct box *box, const char *id)
+struct box *box_find_by_id(struct box *box, lwc_string *id)
 {
 	struct box *a, *b;
+	bool m;
 
-	if (box->id != NULL && strcmp(id, box->id) == 0)
+	if (box->id != NULL &&
+			lwc_string_isequal(id, box->id, &m) == lwc_error_ok &&
+			m == true)
 		return box;
 
 	for (a = box->children; a; a = a->next) {
@@ -935,7 +942,7 @@ void box_dump(FILE *stream, struct box *box, unsigned int depth)
 		fprintf(stream, "space ");
 	if (box->object) {
 		fprintf(stream, "(object '%s') ", 
-				content_get_url(box->object));
+				nsurl_access(hlcache_handle_get_url(box->object)));
 	}
 	if (box->iframe) {
 		fprintf(stream, "(iframe) ");
@@ -945,13 +952,13 @@ void box_dump(FILE *stream, struct box *box, unsigned int depth)
 	if (box->style)
 		nscss_dump_computed_style(stream, box->style);
 	if (box->href)
-		fprintf(stream, " -> '%s'", box->href);
+		fprintf(stream, " -> '%s'", nsurl_access(box->href));
 	if (box->target)
 		fprintf(stream, " |%s|", box->target);
 	if (box->title)
 		fprintf(stream, " [%s]", box->title);
 	if (box->id)
-		fprintf(stream, " <%s>", box->id);
+		fprintf(stream, " <%s>", lwc_string_data(box->id));
 	if (box->type == BOX_INLINE || box->type == BOX_INLINE_END)
 		fprintf(stream, " inline_end %p", box->inline_end);
 	if (box->float_children)
