@@ -104,6 +104,7 @@ struct html_content;
 #define UNKNOWN_WIDTH INT_MAX
 #define UNKNOWN_MAX_WIDTH INT_MAX
 
+typedef void (*box_construct_complete_cb)(struct html_content *c, bool success);
 
 /** Type of a struct box. */
 typedef enum {
@@ -128,7 +129,8 @@ typedef enum {
 	MAKE_HEIGHT = 1 << 7,	/* box causes its own height */
 	NEED_MIN    = 1 << 8,	/* minimum width is required for layout */
 	REPLACE_DIM = 1 << 9,	/* replaced element has given dimensions */
-	IFRAME      = 1 << 10	/* box contains an iframe */
+	IFRAME      = 1 << 10,	/* box contains an iframe */
+	CONVERT_CHILDREN = 1 << 11  /* wanted children converting */
 } box_flags;
 
 /* Sides of a box */
@@ -211,9 +213,9 @@ struct box {
 	/** Width of space after current text (depends on font and size). */
 	int space;
 
-	char *href;   /**< Link, or 0. */
+	nsurl *href;   /**< Link, or 0. */
 	const char *target;  /**< Link target, or 0. */
-	char *title;  /**< Title, or 0. */
+	const char *title;  /**< Title, or 0. */
 
 	unsigned int columns;  /**< Number of columns for TABLE / TABLE_CELL. */
 	unsigned int rows;     /**< Number of rows for TABLE only. */
@@ -249,7 +251,7 @@ struct box {
 	struct form_control* gadget;
 
 	char *usemap; /** (Image)map to use with this object, or 0 if none */
-	char *id; /**<  value of id attribute (or name for anchors) */
+	lwc_string *id; /**<  value of id attribute (or name for anchors) */
 
 	/** Background image for this box, or 0 if none */
 	struct hlcache_handle *background;
@@ -282,11 +284,11 @@ struct column {
 
 /** Parameters for <object> and similar elements. */
 struct object_params {
-	char *data;
+	nsurl *data;
 	char *type;
 	char *codetype;
-	char *codebase;
-	char *classid;
+	nsurl *codebase;
+	nsurl *classid;
 	struct object_param *params;
 };
 
@@ -310,14 +312,13 @@ extern const char *TARGET_BLANK;
 
 void *box_style_alloc(void *ptr, size_t len, void *pw);
 struct box * box_create(css_select_results *styles, css_computed_style *style,
-		bool style_owned, char *href, const char *target, char *title,
-		char *id, void *context);
+		bool style_owned, nsurl *href, const char *target, 
+		const char *title, lwc_string *id, void *context);
 void box_add_child(struct box *parent, struct box *child);
 void box_insert_sibling(struct box *box, struct box *new_box);
 void box_unlink_and_free(struct box *box);
 void box_free(struct box *box);
 void box_free_box(struct box *box);
-void box_free_object_params(struct object_params *op);
 void box_bounds(struct box *box, struct rect *r);
 void box_coords(struct box *box, int *x, int *y);
 struct box *box_at_point(struct box *box, const int x, const int y,
@@ -326,17 +327,18 @@ struct box *box_object_at_point(struct hlcache_handle *h, int x, int y);
 struct box *box_href_at_point(struct hlcache_handle *h, int x, int y);
 struct box *box_pick_text_box(struct html_content *html,
 		int x, int y, int dir, int *dx, int *dy);
-struct box *box_find_by_id(struct box *box, const char *id);
+struct box *box_find_by_id(struct box *box, lwc_string *id);
 bool box_visible(struct box *box);
 void box_dump(FILE *stream, struct box *box, unsigned int depth);
-bool box_extract_link(const char *rel, const char *base, char **result);
+bool box_extract_link(const char *rel, nsurl *base, nsurl **result);
 
 bool box_handle_scrollbars(struct content *c, struct box *box,
 		bool bottom, bool right);
 bool box_vscrollbar_present(const struct box *box);
 bool box_hscrollbar_present(const struct box *box);
 
-bool xml_to_box(xmlNode *n, struct html_content *c);
+bool xml_to_box(xmlNode *n, struct html_content *c, 
+		box_construct_complete_cb cb);
 
 bool box_normalise_block(struct box *block, struct html_content *c);
 
