@@ -30,7 +30,6 @@
 #include "render/box.h"
 #include "render/html_internal.h"
 #include "render/table.h"
-#include "desktop/gui.h"
 #include "utils/log.h"
 #include "utils/talloc.h"
 
@@ -114,8 +113,6 @@ bool box_normalise_block(struct box *block, html_content *c)
 	assert(block->type == BOX_BLOCK || block->type == BOX_INLINE_BLOCK ||
 			block->type == BOX_TABLE_CELL);
 
-	gui_multitask();
-
 	for (child = block->children; child != NULL; child = next_child) {
 #ifdef BOX_NORMALISE_DEBUG
 		LOG(("child %p, child->type = %d", child, child->type));
@@ -157,6 +154,7 @@ bool box_normalise_block(struct box *block, html_content *c)
 			ctx.ctx = c->select_ctx;
 			ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
 			ctx.base_url = c->base_url;
+			ctx.universal = c->universal;
 
 			style = nscss_get_blank_style(&ctx, block->style,
 					box_style_alloc, NULL);
@@ -258,6 +256,7 @@ bool box_normalise_table(struct box *table, html_content * c)
 			ctx.ctx = c->select_ctx;
 			ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
 			ctx.base_url = c->base_url;
+			ctx.universal = c->universal;
 
 			style = nscss_get_blank_style(&ctx, table->style,
 					box_style_alloc, NULL);
@@ -344,6 +343,7 @@ bool box_normalise_table(struct box *table, html_content * c)
 		ctx.ctx = c->select_ctx;
 		ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
 		ctx.base_url = c->base_url;
+		ctx.universal = c->universal;
 
 		style = nscss_get_blank_style(&ctx, table->style, 
 				box_style_alloc, NULL);
@@ -479,6 +479,7 @@ bool box_normalise_table_spans(struct box *table, struct span_info *spans,
 					ctx.quirks = (c->quirks == 
 						BINDING_QUIRKS_MODE_FULL);
 					ctx.base_url = c->base_url;
+					ctx.universal = c->universal;
 
 					style = nscss_get_blank_style(&ctx, 
 							table_row->style,
@@ -588,6 +589,7 @@ bool box_normalise_table_row_group(struct box *row_group,
 			ctx.ctx = c->select_ctx;
 			ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
 			ctx.base_url = c->base_url;
+			ctx.universal = c->universal;
 
 			style = nscss_get_blank_style(&ctx, row_group->style,
 					box_style_alloc, NULL);
@@ -662,6 +664,7 @@ bool box_normalise_table_row_group(struct box *row_group,
 		ctx.ctx = c->select_ctx;
 		ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
 		ctx.base_url = c->base_url;
+		ctx.universal = c->universal;
 
 		style = nscss_get_blank_style(&ctx, row_group->style, 
 				box_style_alloc, NULL);
@@ -731,6 +734,7 @@ bool box_normalise_table_row(struct box *row,
 			ctx.ctx = c->select_ctx;
 			ctx.quirks = (c->quirks == BINDING_QUIRKS_MODE_FULL);
 			ctx.base_url = c->base_url;
+			ctx.universal = c->universal;
 
 			style = nscss_get_blank_style(&ctx, row->style, 
 					box_style_alloc, NULL);
