@@ -1269,7 +1269,6 @@ void form_select_menu_scroll_callback(void *client_data,
 	struct form_control *control = client_data;
 	struct form_select_menu *menu = control->data.select.menu;
 	html_content *html = (html_content *)menu->c;
-	struct browser_window *root_bw;
 	
 	switch (scrollbar_data->msg) {
 		case SCROLLBAR_MSG_REDRAW:
@@ -1287,17 +1286,25 @@ void form_select_menu_scroll_callback(void *client_data,
      					menu->height);
 			break;
 		case SCROLLBAR_MSG_SCROLL_START:
-			browser_window_set_drag_type(html->bw, DRAGGING_OTHER);
+		{
+			struct rect rect = {
+				.x0 = scrollbar_data->x0,
+				.y0 = scrollbar_data->y0,
+				.x1 = scrollbar_data->x1,
+				.y1 = scrollbar_data->y1
+			};
+
+			browser_window_set_drag_type(html->bw,
+					DRAGGING_CONTENT_SCROLLBAR, &rect);
 
 			menu->scroll_capture = true;
-
-			root_bw = browser_window_get_root(html->bw);
-			gui_window_box_scroll_start(root_bw->window,
-					scrollbar_data->x0, scrollbar_data->y0,
-     					scrollbar_data->x1, scrollbar_data->y1);
+		}
 			break;
 		case SCROLLBAR_MSG_SCROLL_FINISHED:
 			menu->scroll_capture = false;
+
+			browser_window_set_drag_type(html->bw,
+					DRAGGING_NONE, NULL);
 			break;
 		default:
 			break;
@@ -1500,7 +1507,8 @@ void form_submit(hlcache_handle *h, struct browser_window *target,
 
 		url_destroy_components(&components);
 
-		browser_window_go(target, url, content_get_url(h), true);
+		browser_window_go(target, url, nsurl_access(hlcache_handle_get_url(h)),
+				true);
 		break;
 
 	case method_POST_URLENC:
@@ -1512,13 +1520,13 @@ void form_submit(hlcache_handle *h, struct browser_window *target,
 		}
 
 		browser_window_go_post(target, form->action, data, 0,
-				true,  content_get_url(h),
+				true,  nsurl_access(hlcache_handle_get_url(h)),
 				false, true, 0);
 		break;
 
 	case method_POST_MULTIPART:
 		browser_window_go_post(target, form->action, 0,
-				success, true, content_get_url(h),
+				success, true, nsurl_access(hlcache_handle_get_url(h)),
 				false, true, 0);
 		break;
 	}
