@@ -32,9 +32,9 @@
 #include "content/fetch.h"
 #include "content/hlcache.h"
 #include "desktop/browser.h"
-#include "desktop/gui.h"
 #include "desktop/options.h"
 #include "desktop/selection.h"
+#include "desktop/scrollbar.h"
 #include "image/bitmap.h"
 #include "render/box.h"
 #include "render/font.h"
@@ -49,6 +49,7 @@
 #include "utils/schedule.h"
 #include "utils/talloc.h"
 #include "utils/url.h"
+#include "utils/utf8.h"
 #include "utils/utils.h"
 
 #define CHUNK 4096
@@ -79,11 +80,16 @@ static void html_close(struct content *c);
 struct selection *html_get_selection(struct content *c);
 static void html_get_contextual_content(struct content *c,
 		int x, int y, struct contextual_content *data);
+static bool html_scroll_at_point(struct content *c,
+		int x, int y, int scrx, int scry);
+static bool html_drop_file_at_point(struct content *c,
+		int x, int y, char *file);
 struct search_context *html_get_search(struct content *c);
 static nserror html_clone(const struct content *old, struct content **newc);
 static content_type html_content_type(void);
 
 static void html_finish_conversion(html_content *c);
+static void html_box_convert_done(html_content *c, bool success);
 static nserror html_convert_css_callback(hlcache_handle *css,
 		const hlcache_event *event, void *pw);
 static bool html_meta_refresh(html_content *c, xmlNode *head);
@@ -93,7 +99,7 @@ static bool html_process_style_element(html_content *c, unsigned int *index,
 		xmlNode *style);
 static void html_inline_style_done(struct content_css_data *css, void *pw);
 static bool html_replace_object(struct content_html_object *object,
-		const char *url);
+		nsurl *url);
 static nserror html_object_callback(hlcache_handle *object,
 		const hlcache_event *event, void *pw);
 static void html_object_done(struct box *box, hlcache_handle *object,
@@ -124,6 +130,8 @@ static const content_handler html_content_handler = {
 	.close = html_close,
 	.get_selection = html_get_selection,
 	.get_contextual_content = html_get_contextual_content,
+	.scroll_at_point = html_scroll_at_point,
+	.drop_file_at_point = html_drop_file_at_point,
 	.clone = html_clone,
 	.type = html_content_type,
 	.no_share = true,
@@ -149,6 +157,10 @@ static const char *html_types[] = {
 
 static lwc_string *html_charset;
 
+static nsurl *html_default_stylesheet_url;
+static nsurl *html_adblock_stylesheet_url;
+static nsurl *html_quirks_stylesheet_url;
+
 nserror html_init(void)
 {
 	uint32_t i;
@@ -161,6 +173,21 @@ nserror html_init(void)
 		goto error;
 	}
 
+	error = nsurl_create("resource:default.css", 
+			&html_default_stylesheet_url);
+	if (error != NSERROR_OK)
+		goto error;
+
+	error = nsurl_create("resource:adblock.css",
+			&html_adblock_stylesheet_url);
+	if (error != NSERROR_OK)
+		goto error;
+
+	error = nsurl_create("resource:quirks.css",
+			&html_quirks_stylesheet_url);
+	if (error != NSERROR_OK)
+		goto error;
+
 	for (i = 0; i < NOF_ELEMENTS(html_types); i++) {
 		error = content_factory_register_handler(html_types[i],
 				&html_content_handler);
@@ -178,6 +205,21 @@ error:
 
 void html_fini(void)
 {
+	if (html_quirks_stylesheet_url != NULL) {
+		nsurl_unref(html_quirks_stylesheet_url);
+		html_quirks_stylesheet_url = NULL;
+	}
+
+	if (html_adblock_stylesheet_url != NULL) {
+		nsurl_unref(html_adblock_stylesheet_url);
+		html_adblock_stylesheet_url = NULL;
+	}
+
+	if (html_default_stylesheet_url != NULL) {
+		nsurl_unref(html_default_stylesheet_url);
+		html_default_stylesheet_url = NULL;
+	}
+
 	if (html_charset != NULL) {
 		lwc_string_unref(html_charset);
 		html_charset = NULL;
@@ -232,7 +274,7 @@ nserror html_create_html_data(html_content *c, const http_parameter *params)
 	c->document = NULL;
 	c->quirks = BINDING_QUIRKS_MODE_NONE;
 	c->encoding = NULL;
-	c->base_url = (char *) content__get_url(&c->base);
+	c->base_url = nsurl_ref(content_get_url(&c->base));
 	c->base_target = NULL;
 	c->aborted = false;
 	c->layout = NULL;
@@ -240,6 +282,7 @@ nserror html_create_html_data(html_content *c, const http_parameter *params)
 	c->stylesheet_count = 0;
 	c->stylesheets = NULL;
 	c->select_ctx = NULL;
+	c->universal = NULL;
 	c->num_objects = 0;
 	c->object_list = NULL;
 	c->forms = NULL;
@@ -252,6 +295,11 @@ nserror html_create_html_data(html_content *c, const http_parameter *params)
 	c->font_func = &nsfont;
 	c->scrollbar = NULL;
 
+	if (lwc_intern_string("*", SLEN("*"), &c->universal) != lwc_error_ok) {
+		error = BINDING_NOMEM;
+		goto error;
+	}
+
 	selection_prepare(&c->sel, (struct content *)c, true);
 
 	nerror = http_parameter_list_find_item(params, html_charset, &charset);
@@ -295,42 +343,32 @@ error:
 
 	content_broadcast(&c->base, CONTENT_MSG_ERROR, msg_data);
 
+	if (c->universal != NULL) {
+		lwc_string_unref(c->universal);
+		c->universal = NULL;
+	}
+
+	if (c->base_url != NULL) {
+		nsurl_unref(c->base_url);
+		c->base_url = NULL;
+	}
+
 	return nerror;
 }
 
 
 /**
  * Process data for CONTENT_HTML.
- *
- * The data is parsed in chunks of size CHUNK, multitasking in between.
  */
 
 bool html_process_data(struct content *c, const char *data, unsigned int size)
 {
 	html_content *html = (html_content *) c;
-	unsigned long x;
 	binding_error err;
 	const char *encoding;
 
-	for (x = 0; x + CHUNK <= size; x += CHUNK) {
-		err = binding_parse_chunk(html->parser_binding,
-				(const uint8_t *) data + x, CHUNK);
-		if (err == BINDING_ENCODINGCHANGE) {
-			goto encoding_change;
-		} else if (err != BINDING_OK) {
-			union content_msg_data msg_data;
-
-			msg_data.error = messages_get("NoMemory");
-			content_broadcast(c, CONTENT_MSG_ERROR, msg_data);
-
-			return false;
-		}
-
-		gui_multitask();
-	}
-
 	err = binding_parse_chunk(html->parser_binding,
-			(const uint8_t *) data + x, (size - x));
+			(const uint8_t *) data, size);
 	if (err == BINDING_ENCODINGCHANGE) {
 		goto encoding_change;
 	} else if (err != BINDING_OK) {
@@ -404,8 +442,8 @@ encoding_change:
 
 		source_data = content__get_source_data(c, &source_size);
 
-		/* Recurse to reprocess all that data.  This is safe because
-		 * the encoding is now specified at parser-start which means
+		/* Recurse to reprocess all the data.  This is safe because
+		 * the encoding is now specified at parser start which means
 		 * it cannot be changed again. */
 		return html_process_data(c, source_data, source_size);
 	}
@@ -546,10 +584,11 @@ bool html_convert(struct content *c)
 		/* Make all actions absolute */
 		if (f->action == NULL || f->action[0] == '\0') {
 			/* HTML5 4.10.22.3 step 11 */
-			res = url_join(content__get_url(c), 
-					htmlc->base_url, &action);
+			res = url_join(nsurl_access(content_get_url(c)), 
+					nsurl_access(htmlc->base_url), &action);
 		} else {
-			res = url_join(f->action, htmlc->base_url, &action);
+			res = url_join(f->action, nsurl_access(htmlc->base_url),
+					&action);
 		}
 
 		if (res != URL_FUNC_OK) {
@@ -653,16 +692,46 @@ void html_finish_conversion(html_content *c)
 	}
 
 	/* convert xml tree to box tree */
-	LOG(("XML to box"));
+	LOG(("XML to box (%p)", c));
 	content_set_status(&c->base, messages_get("Processing"));
 	content_broadcast(&c->base, CONTENT_MSG_STATUS, msg_data);
-	if (xml_to_box(html, c) == false) {
+	if (xml_to_box(html, c, html_box_convert_done) == false) {
 		html_destroy_objects(c);
 		msg_data.error = messages_get("NoMemory");
 		content_broadcast(&c->base, CONTENT_MSG_ERROR, msg_data);
 		content_set_error(&c->base);
 		return;
 	}
+}
+
+/**
+ * Perform post-box-creation conversion of a document
+ *
+ * \param c        HTML content to complete conversion of
+ * \param success  Whether box tree construction was successful
+ */
+void html_box_convert_done(html_content *c, bool success)
+{
+	union content_msg_data msg_data;
+	xmlNode *html;
+
+	LOG(("Done XML to box (%p)", c));
+
+	/* Clean up and report error if unsuccessful or aborted */
+	if (success == false || c->aborted) {
+		html_destroy_objects(c);
+		if (success == false)
+			msg_data.error = messages_get("NoMemory");
+		else
+			msg_data.error = messages_get("Stopped");
+		content_broadcast(&c->base, CONTENT_MSG_ERROR, msg_data);
+		content_set_error(&c->base);
+		return;
+	}
+
+	html = xmlDocGetRootElement(c->document);
+	assert(html != NULL);
+
 #if ALWAYS_DUMP_BOX
 	box_dump(stderr, c->layout->children, 0);
 #endif
@@ -695,6 +764,86 @@ void html_finish_conversion(html_content *c)
 }
 
 
+/** process link node */
+static bool html_process_link(html_content *c, xmlNode *node)
+{
+	struct content_rfc5988_link link;
+	char *xmlstr;
+	nserror error;
+	lwc_string *rel;
+	nsurl *href;
+
+	/* check that the relation exists - w3c spec says must be present */
+	xmlstr = (char *)xmlGetProp(node, (const xmlChar *)"rel");
+	if (xmlstr == NULL) {
+		return false;
+	}
+	if (lwc_intern_string(xmlstr, strlen(xmlstr), &rel) != lwc_error_ok) {
+		xmlFree(xmlstr);
+		return false;
+	}
+	xmlFree(xmlstr);
+		
+	/* check that the href exists - w3c spec says must be present */
+	xmlstr = (char *)xmlGetProp(node, (const xmlChar *) "href");
+	if (xmlstr == NULL) {
+		return false;
+	}
+	error = nsurl_join(c->base_url, xmlstr, &href);
+	xmlFree(xmlstr);
+	if (error != NSERROR_OK) {
+		lwc_string_unref(rel);		
+		return false;
+	}
+
+	memset(&link, 0, sizeof(struct content_rfc5988_link));
+
+	link.rel = rel;
+	link.href = href;
+
+	/* look for optional properties -- we don't care if internment fails */
+	xmlstr = (char *)xmlGetProp(node, (const xmlChar *) "hreflang");
+	if (xmlstr != NULL) {
+		lwc_intern_string(xmlstr, strlen(xmlstr), &link.hreflang);
+		xmlFree(xmlstr);
+	}
+
+	xmlstr = (char *) xmlGetProp(node, (const xmlChar *) "type");
+	if (xmlstr != NULL) {
+		lwc_intern_string(xmlstr, strlen(xmlstr), &link.type);
+		xmlFree(xmlstr);
+	}
+
+	xmlstr = (char *) xmlGetProp(node, (const xmlChar *) "media");
+	if (xmlstr != NULL) {
+		lwc_intern_string(xmlstr, strlen(xmlstr), &link.media);
+		xmlFree(xmlstr);
+	}
+
+	xmlstr = (char *) xmlGetProp(node, (const xmlChar *) "sizes");
+	if (xmlstr != NULL) {
+		lwc_intern_string(xmlstr, strlen(xmlstr), &link.sizes);
+		xmlFree(xmlstr);
+	}
+
+	/* add to content */
+	content__add_rfc5988_link(&c->base, &link);
+
+	if (link.sizes != NULL)
+		lwc_string_unref(link.sizes);
+	if (link.media != NULL)
+		lwc_string_unref(link.media);
+	if (link.type != NULL)
+		lwc_string_unref(link.type);
+	if (link.hreflang != NULL)
+		lwc_string_unref(link.hreflang);
+
+	nsurl_unref(link.href);
+	lwc_string_unref(link.rel);
+
+	return true;
+}
+
 /**
  * Process elements in <head>.
  *
@@ -735,12 +884,13 @@ bool html_head(html_content *c, xmlNode *head)
 			char *href = (char *) xmlGetProp(node,
 					(const xmlChar *) "href");
 			if (href) {
-				char *url;
-				url_func_result res;
-				res = url_normalize(href, &url);
-				if (res == URL_FUNC_OK) {
-					c->base_url = talloc_strdup(c, url);
-					free(url);
+				nsurl *url;
+				nserror error;
+				error = nsurl_create(href, &url);
+				if (error == NSERROR_OK) {
+					if (c->base_url != NULL)
+						nsurl_unref(c->base_url);
+					c->base_url = url;
 				}
 				xmlFree(href);
 			}
@@ -764,6 +914,8 @@ bool html_head(html_content *c, xmlNode *head)
 				}
 				xmlFree(s);
 			}
+		} else if (strcmp((const char *) node->name, "link") == 0) {
+			html_process_link(c, node);
 		}
 	}
 	return true;
@@ -786,15 +938,16 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 	xmlChar *equiv, *content;
 	union content_msg_data msg_data;
 	char *url, *end, *refresh = NULL, quote = 0;
-	url_func_result res;
+	nsurl *nsurl;
+	nserror error;
 
-	for (n = head == 0 ? 0 : head->children; n; n = n->next) {
+	for (n = head == NULL ? NULL : head->children; n; n = n->next) {
 		if (n->type != XML_ELEMENT_NODE)
 			continue;
 
 		/* Recurse into noscript elements */
 		if (strcmp((const char *) n->name, "noscript") == 0) {
-			if (!html_meta_refresh(c, n)) {
+			if (html_meta_refresh(c, n) == false) {
 				/* Some error occurred */
 				return false;
 			} else if (c->base.refresh) {
@@ -803,15 +956,15 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 			}
 		}
 
-		if (strcmp((const char *) n->name, "meta")) {
+		if (strcmp((const char *) n->name, "meta") != 0) {
 			continue;
 		}
 
 		equiv = xmlGetProp(n, (const xmlChar *) "http-equiv");
-		if (!equiv)
+		if (equiv == NULL)
 			continue;
 
-		if (strcasecmp((const char *) equiv, "refresh")) {
+		if (strcasecmp((const char *) equiv, "refresh") != 0) {
 			xmlFree(equiv);
 			continue;
 		}
@@ -819,7 +972,7 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 		xmlFree(equiv);
 
 		content = xmlGetProp(n, (const xmlChar *) "content");
-		if (!content)
+		if (content == NULL)
 			continue;
 
 		end = (char *) content + strlen((const char *) content);
@@ -835,8 +988,21 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 		 * nonascii := [#x80-#xD7FF#xE000-#xFFFD#x10000-#x10FFFF]
 		 */
 
-		/* *LWS intpart */
-		msg_data.delay = (int)strtol((char *) content, &url, 10);
+		url = (char *) content;
+
+		/* *LWS */
+		while (url < end && isspace(*url)) {
+			url++;
+		}
+
+		/* intpart */
+		if (url == end || (*url < '0' || '9' < *url)) {
+			/* Empty content, or invalid timeval */
+			xmlFree(content);
+			continue;
+		}
+
+		msg_data.delay = (int) strtol(url, &url, 10);
 		/* a very small delay and self-referencing URL can cause a loop
 		 * that grinds machines to a halt. To prevent this we set a
 		 * minimum refresh delay of 1s. */
@@ -867,14 +1033,8 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 			/* Just delay specified, so refresh current page */
 			xmlFree(content);
 
-			c->base.refresh = talloc_strdup(c, 
-					content__get_url(&c->base));
-			if (!c->base.refresh) {
-				msg_data.error = messages_get("NoMemory");
-				content_broadcast(&c->base,
-					CONTENT_MSG_ERROR, msg_data);
-				return false;
-			}
+			c->base.refresh = nsurl_ref(
+					content_get_url(&c->base));
 
 			content_broadcast(&c->base, CONTENT_MSG_REFRESH, 
 					msg_data);
@@ -887,10 +1047,12 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 				url += 3;
 			} else {
 				/* Unexpected input, ignore this header */
+				xmlFree(content);
 				continue;
 			}
 		} else {
 			/* Insufficient input, ignore this header */
+			xmlFree(content);
 			continue;
 		}
 
@@ -905,10 +1067,12 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 				url++;
 			} else {
 				/* Unexpected input, ignore this header */
+				xmlFree(content);
 				continue;
 			}
 		} else {
 			/* Insufficient input, ignore this header */
+			xmlFree(content);
 			continue;
 		}
 
@@ -940,30 +1104,20 @@ bool html_meta_refresh(html_content *c, xmlNode *head)
 		if (url < end)
 			*url = '\0';
 
-		res = url_join(refresh, c->base_url, &refresh);
+		error = nsurl_join(c->base_url, refresh, &nsurl);
+		if (error != NSERROR_OK) {
+			xmlFree(content);
+
+			msg_data.error = messages_get("NoMemory");
+			content_broadcast(&c->base, CONTENT_MSG_ERROR, 
+					msg_data);
+
+			return false;
+		}
 
 		xmlFree(content);
 
-		if (res == URL_FUNC_NOMEM) {
-			msg_data.error = messages_get("NoMemory");
-			content_broadcast(&c->base, CONTENT_MSG_ERROR, 
-					msg_data);
-			return false;
-		} else if (res == URL_FUNC_FAILED) {
-			/* This isn't fatal so carry on looking */
-			continue;
-		}
-
-		c->base.refresh = talloc_strdup(c, refresh);
-
-		free(refresh);
-
-		if (!c->base.refresh) {
-			msg_data.error = messages_get("NoMemory");
-			content_broadcast(&c->base, CONTENT_MSG_ERROR, 
-					msg_data);
-			return false;
-		}
+		c->base.refresh = nsurl;
 
 		content_broadcast(&c->base, CONTENT_MSG_REFRESH, msg_data);
 	}
@@ -986,13 +1140,13 @@ bool html_find_stylesheets(html_content *c, xmlNode *html)
 {
 	content_type accept = CONTENT_CSS;
 	xmlNode *node;
-	char *rel, *type, *media, *href, *url, *url2;
+	char *rel, *type, *media, *href;
 	unsigned int i = STYLESHEET_START;
 	union content_msg_data msg_data;
-	url_func_result res;
 	struct html_stylesheet *stylesheets;
 	hlcache_child_context child;
 	nserror ns_error;
+	nsurl *joined;
 
 	child.charset = c->encoding;
 	child.quirks = c->base.quirks;
@@ -1014,8 +1168,8 @@ bool html_find_stylesheets(html_content *c, xmlNode *html)
 
 	c->base.active = 0;
 
-	ns_error = hlcache_handle_retrieve(default_stylesheet_url, 0,
-			content__get_url(&c->base), NULL,
+	ns_error = hlcache_handle_retrieve(html_default_stylesheet_url, 0,
+			content_get_url(&c->base), NULL,
 			html_convert_css_callback, c, &child, accept,
 			&c->stylesheets[STYLESHEET_BASE].data.external);
 	if (ns_error != NSERROR_OK)
@@ -1024,8 +1178,8 @@ bool html_find_stylesheets(html_content *c, xmlNode *html)
 	c->base.active++;
 
 	if (c->quirks == BINDING_QUIRKS_MODE_FULL) {
-		ns_error = hlcache_handle_retrieve(quirks_stylesheet_url, 0,
-				content__get_url(&c->base), NULL,
+		ns_error = hlcache_handle_retrieve(html_quirks_stylesheet_url, 
+				0, content_get_url(&c->base), NULL,
 				html_convert_css_callback, c, &child, accept,
 				&c->stylesheets[STYLESHEET_QUIRKS].
 						data.external);
@@ -1036,8 +1190,8 @@ bool html_find_stylesheets(html_content *c, xmlNode *html)
 	}
 
 	if (option_block_ads) {
-		ns_error = hlcache_handle_retrieve(adblock_stylesheet_url, 0,
-				content__get_url(&c->base), NULL,
+		ns_error = hlcache_handle_retrieve(html_adblock_stylesheet_url,
+				0, content_get_url(&c->base), NULL,
 				html_convert_css_callback, c, &child, accept,
 				&c->stylesheets[STYLESHEET_ADBLOCK].
 						data.external);
@@ -1114,42 +1268,35 @@ bool html_find_stylesheets(html_content *c, xmlNode *html)
 			 * those with a title attribute) should be loaded
 			 * (see HTML4 14.3) */
 
-			res = url_join(href, c->base_url, &url);
-			xmlFree(href);
-			if (res != URL_FUNC_OK)
-				continue;
-
-			LOG(("linked stylesheet %i '%s'", i, url));
-
-			res = url_normalize(url, &url2);
-
-			free(url);
-
-			if (res != URL_FUNC_OK) {
-				if (res == URL_FUNC_NOMEM)
-					goto no_memory;
-				continue;
+			ns_error = nsurl_join(c->base_url, href, &joined);
+			if (ns_error != NSERROR_OK) {
+				xmlFree(href);
+				goto no_memory;
 			}
+			xmlFree(href);
+
+			LOG(("linked stylesheet %i '%s'", i,
+					nsurl_access(joined)));
 
 			/* start fetch */
 			stylesheets = talloc_realloc(c,
 					c->stylesheets,
 					struct html_stylesheet, i + 1);
 			if (stylesheets == NULL) {
-				free(url2);
+				nsurl_unref(joined);
 				goto no_memory;
 			}
 
 			c->stylesheets = stylesheets;
 			c->stylesheet_count++;
 			c->stylesheets[i].type = HTML_STYLESHEET_EXTERNAL;
-			ns_error = hlcache_handle_retrieve(url2, 0,
-					content__get_url(&c->base), NULL,
+			ns_error = hlcache_handle_retrieve(joined, 0,
+					content_get_url(&c->base), NULL,
 					html_convert_css_callback, c, &child,
 					accept,
 					&c->stylesheets[i].data.external);
 
-			free(url2);
+			nsurl_unref(joined);
 
 			if (ns_error != NSERROR_OK)
 				goto no_memory;
@@ -1233,7 +1380,7 @@ bool html_process_style_element(html_content *c, unsigned int *index,
 	}
 
 	error = nscss_create_css_data(sheet,
-		c->base_url, NULL, c->quirks,
+		nsurl_access(c->base_url), NULL, c->quirks,
 		html_inline_style_done, c);
 	if (error != NSERROR_OK) {
 		c->stylesheet_count--;
@@ -1324,13 +1471,15 @@ nserror html_convert_css_callback(hlcache_handle *css,
 		break;
 
 	case CONTENT_MSG_DONE:
-		LOG(("got stylesheet '%s'", content_get_url(css)));
+		LOG(("got stylesheet '%s'",
+				nsurl_access(hlcache_handle_get_url(css))));
 		parent->base.active--;
 		break;
 
 	case CONTENT_MSG_ERROR:
-		LOG(("stylesheet %s failed: %s", 
-				content_get_url(css), event->data.error));
+		LOG(("stylesheet %s failed: %s",
+				nsurl_access(hlcache_handle_get_url(css)),
+				event->data.error));
 		hlcache_handle_release(css);
 		s->data.external = NULL;
 		parent->base.active--;
@@ -1367,15 +1516,13 @@ nserror html_convert_css_callback(hlcache_handle *css,
  * \return  true on success, false on memory exhaustion
  */
 
-bool html_fetch_object(html_content *c, const char *url, struct box *box,
+bool html_fetch_object(html_content *c, nsurl *url, struct box *box,
 		content_type permitted_types,
 		int available_width, int available_height,
 		bool background)
 {
 	struct content_html_object *object;
 	hlcache_child_context child;
-	char *url2;
-	url_func_result res;
 	nserror error;
 
 	/* If we've already been aborted, don't bother attempting the fetch */
@@ -1385,16 +1532,8 @@ bool html_fetch_object(html_content *c, const char *url, struct box *box,
 	child.charset = c->encoding;
 	child.quirks = c->base.quirks;
 
-	/* Normalize the URL */
-	res = url_normalize(url, &url2);
-	if (res != URL_FUNC_OK) {
-		LOG(("failed to normalize url '%s'", url));
-		return res != URL_FUNC_NOMEM;
-	}
-
 	object = talloc(c, struct content_html_object);
 	if (object == NULL) {
-		free(url2);
 		return false;
 	}
 
@@ -1405,30 +1544,26 @@ bool html_fetch_object(html_content *c, const char *url, struct box *box,
 	object->permitted_types = permitted_types;
 	object->background = background;
  
-	error = hlcache_handle_retrieve(url2, HLCACHE_RETRIEVE_SNIFF_TYPE, 
-			content__get_url(&c->base), NULL, 
+	error = hlcache_handle_retrieve(url, 
+			HLCACHE_RETRIEVE_SNIFF_TYPE, 
+			content_get_url(&c->base), NULL, 
 			html_object_callback, object, &child, 
-			permitted_types, &object->content);
-
-	/* No longer need normalized url */
-	free(url2);
-
-        if (error != NSERROR_OK) {
+			object->permitted_types, &object->content);
+       	if (error != NSERROR_OK) {
 		talloc_free(object);
 		return error != NSERROR_NOMEM;
 	}
 
-	/* add to object list */
+	/* add to content object list */
 	object->next = c->object_list;
 	c->object_list = object;
 
 	c->num_objects++;
 	c->base.active++;
 
-	return error != NSERROR_NOMEM;
+	return true;
 }
 
-
 /**
  * Start a fetch for an object required by a page, replacing an existing object.
  *
@@ -1437,13 +1572,11 @@ bool html_fetch_object(html_content *c, const char *url, struct box *box,
  * \return  true on success, false on memory exhaustion
  */
 
-bool html_replace_object(struct content_html_object *object, const char *url)
+bool html_replace_object(struct content_html_object *object, nsurl *url)
 {
 	html_content *c;
 	hlcache_child_context child;
 	html_content *page;
-	char *url2;
-	url_func_result res;
 	nserror error;
 
 	assert(object != NULL);
@@ -1464,19 +1597,13 @@ bool html_replace_object(struct content_html_object *object, const char *url)
 		object->box->object = NULL;
 	}
 
-	res = url_normalize(url, &url2);
-	if (res != URL_FUNC_OK)
-		return res != URL_FUNC_NOMEM;
-
 	/* initialise fetch */
-	error = hlcache_handle_retrieve(url2, HLCACHE_RETRIEVE_SNIFF_TYPE, 
-			content__get_url(&c->base), NULL, 
+	error = hlcache_handle_retrieve(url, HLCACHE_RETRIEVE_SNIFF_TYPE, 
+			content_get_url(&c->base), NULL, 
 			html_object_callback, object, &child,
 			object->permitted_types,
 			&object->content);
 
-	free(url2);
-
 	if (error != NSERROR_OK)
 		return false;
 
@@ -1501,14 +1628,13 @@ nserror html_object_callback(hlcache_handle *object,
 	int x, y;
 	struct box *box;
 
-	assert(c->base.status == CONTENT_STATUS_READY ||
-			c->base.status == CONTENT_STATUS_DONE);
+	assert(c->base.status != CONTENT_STATUS_ERROR);
 
 	box = o->box;
 
 	switch (event->type) {
 	case CONTENT_MSG_LOADING:
-		if (c->bw != NULL)
+		if (c->base.status != CONTENT_STATUS_LOADING && c->bw != NULL)
 			content_open(object,
 					c->bw, &c->base,
 					box,
@@ -1530,7 +1656,8 @@ nserror html_object_callback(hlcache_handle *object,
 		c->base.active--;
 		html_object_done(box, object, o->background);
 
-		if (box->flags & REPLACE_DIM) {
+		if (c->base.status != CONTENT_STATUS_LOADING &&
+				box->flags & REPLACE_DIM) {
 			union content_msg_data data;
 
 			if (!box_visible(box))
@@ -1570,37 +1697,37 @@ nserror html_object_callback(hlcache_handle *object,
 		break;
 
 	case CONTENT_MSG_REDRAW:
-	{
-		union content_msg_data data = event->data;
+		if (c->base.status != CONTENT_STATUS_LOADING) {
+			union content_msg_data data = event->data;
 
-		if (!box_visible(box))
-			break;
+			if (!box_visible(box))
+				break;
 
-		box_coords(box, &x, &y);
+			box_coords(box, &x, &y);
 
-		if (hlcache_handle_get_content(object) == 
-				event->data.redraw.object) {
-			data.redraw.x = data.redraw.x *
+			if (hlcache_handle_get_content(object) == 
+					event->data.redraw.object) {
+				data.redraw.x = data.redraw.x *
 					box->width / content_get_width(object);
-			data.redraw.y = data.redraw.y *
+				data.redraw.y = data.redraw.y *
 					box->height / 
 					content_get_height(object);
-			data.redraw.width = data.redraw.width *
+				data.redraw.width = data.redraw.width *
 					box->width / content_get_width(object);
-			data.redraw.height = data.redraw.height *
+				data.redraw.height = data.redraw.height *
 					box->height / 
 					content_get_height(object);
-			data.redraw.object_width = box->width;
-			data.redraw.object_height = box->height;
+				data.redraw.object_width = box->width;
+				data.redraw.object_height = box->height;
+			}
+
+			data.redraw.x += x + box->padding[LEFT];
+			data.redraw.y += y + box->padding[TOP];
+			data.redraw.object_x += x + box->padding[LEFT];
+			data.redraw.object_y += y + box->padding[TOP];
+
+			content_broadcast(&c->base, CONTENT_MSG_REDRAW, data);
 		}
-
-		data.redraw.x += x + box->padding[LEFT];
-		data.redraw.y += y + box->padding[TOP];
-		data.redraw.object_x += x + box->padding[LEFT];
-		data.redraw.object_y += y + box->padding[TOP];
-
-		content_broadcast(&c->base, CONTENT_MSG_REDRAW, data);
-	}
 		break;
 
 	case CONTENT_MSG_REFRESH:
@@ -1612,6 +1739,10 @@ nserror html_object_callback(hlcache_handle *object,
 
 		break;
 
+	case CONTENT_MSG_LINK:
+		/* Don't care about favicons */
+		break;
+
 	default:
 		assert(0);
 	}
@@ -1702,7 +1833,7 @@ void html_object_failed(struct box *box, html_content *content,
 void html_object_refresh(void *p)
 {
 	struct content_html_object *object = p;
-	const char *refresh_url;
+	nsurl *refresh_url;
 
 	assert(content_get_type(object->content) == CONTENT_HTML);
 
@@ -1731,26 +1862,6 @@ void html_stop(struct content *c)
 
 	switch (c->status) {
 	case CONTENT_STATUS_LOADING:
-		/* Clean up objects if we were stopped after queuing up some 
-		 * fetches within xml_to_box (i.e. gui_multitask is somewhere 
-		 * in our call stack) */
-		for (object = htmlc->object_list; object != NULL; 
-				object = object->next) {
-			if (object->content == NULL)
-				continue;
-
-			/* If there's a content already associated with our 
-			 * handle, something's gone very wrong */
-			assert(content_get_status(object->content) == 
-					CONTENT_STATUS_ERROR);
-			
-			hlcache_handle_abort(object->content);
-			hlcache_handle_release(object->content);
-			object->content = NULL;
-
-			c->active--;
-		}
-
 		/* Still loading; simply flag that we've been aborted
 		 * html_convert/html_finish_conversion will do the rest */
 		htmlc->aborted = true;
@@ -1894,11 +2005,17 @@ void html_destroy(struct content *c)
 
 	imagemap_destroy(html);
 
+	if (c->refresh)
+		nsurl_unref(c->refresh);
+
+	if (html->base_url)
+		nsurl_unref(html->base_url);
+
 	if (html->parser_binding != NULL)
 		binding_destroy_tree(html->parser_binding);
 
 	if (html->document != NULL)
-		xmlFreeDoc(html->document);
+		binding_destroy_document(html->document);
 
 	/* Free base target */
 	if (html->base_target != NULL) {
@@ -1925,6 +2042,11 @@ void html_destroy(struct content *c)
 		html->select_ctx = NULL;
 	}
 
+	if (html->universal != NULL) {
+		lwc_string_unref(html->universal);
+		html->universal = NULL;
+	}
+
 	/* Free stylesheets */
 	for (i = 0; i != html->stylesheet_count; i++) {
 		if (html->stylesheets[i].type == HTML_STYLESHEET_EXTERNAL &&
@@ -1981,7 +2103,7 @@ void html_destroy_frameset(struct content_html_frames *frameset)
 				frameset->children[i].name = NULL;
 			}
 			if (frameset->children[i].url) {
-				talloc_free(frameset->children[i].url);
+				nsurl_unref(frameset->children[i].url);
 				frameset->children[i].url = NULL;
 			}
 		  	if (frameset->children[i].children)
@@ -2000,8 +2122,10 @@ void html_destroy_iframe(struct content_html_iframe *iframe)
 		next = iframe->next;
 		if (iframe->name)
 			talloc_free(iframe->name);
-		if (iframe->url)
-			talloc_free(iframe->url);
+		if (iframe->url) {
+			nsurl_unref(iframe->url);
+			iframe->url = NULL;
+		}
 		talloc_free(iframe);
 	}
 }
@@ -2142,17 +2266,229 @@ void html_get_contextual_content(struct content *c,
 			data->object = box->object;
 
 		if (box->href)
-			data->link_url = box->href;
+			data->link_url = nsurl_access(box->href);
 
 		if (box->usemap) {
 			const char *target = NULL;
-			data->link_url = imagemap_get(html, box->usemap,
-					box_x, box_y, x, y, &target);
+			data->link_url = nsurl_access(imagemap_get(html,
+					box->usemap, box_x, box_y, x, y,
+					&target));
 		}
 	}
 }
 
 
+/**
+ * Scroll deepest thing within the content which can be scrolled at given point
+ *
+ * \param c	html content to look inside
+ * \param x	x-coordinate of point of interest
+ * \param y	y-coordinate of point of interest
+ * \param scrx	x-coordinate of point of interest
+ * \param scry	y-coordinate of point of interest
+ * \return true iff scroll was consumed by something in the content
+ */
+bool html_scroll_at_point(struct content *c, int x, int y, int scrx, int scry)
+{
+	html_content *html = (html_content *) c;
+
+	struct box *box = html->layout;
+	struct box *next;
+	int box_x = 0, box_y = 0;
+	hlcache_handle *containing_content = NULL;
+	bool handled_scroll = false;
+
+	/* TODO: invert order; visit deepest box first */
+
+	while ((next = box_at_point(box, x, y, &box_x, &box_y,
+			&containing_content)) != NULL) {
+		box = next;
+
+		if (box->style && css_computed_visibility(box->style) == 
+				CSS_VISIBILITY_HIDDEN)
+			continue;
+
+		/* Pass into iframe */
+		if (box->iframe && browser_window_scroll_at_point(box->iframe,
+				x - box_x, y - box_y, scrx, scry) == true)
+			return true;
+
+		/* Handle box scrollbars */
+		if (box->scroll_y && scrollbar_scroll(box->scroll_y, scry))
+			handled_scroll = true;
+
+		if (box->scroll_x && scrollbar_scroll(box->scroll_x, scrx))
+			handled_scroll = true;
+
+		if (handled_scroll == true)
+			return true;
+	}
+
+	return false;
+}
+
+
+/**
+ * Drop a file onto a content at a particular point.
+ *
+ * \param c	html content to look inside
+ * \param x	x-coordinate of point of interest
+ * \param y	y-coordinate of point of interest
+ * \param file	path to file to be dropped
+ * \return true iff file drop has been handled
+ */
+bool html_drop_file_at_point(struct content *c, int x, int y, char *file)
+{
+	html_content *html = (html_content *) c;
+
+	struct box *box = html->layout;
+	struct box *next;
+	struct box *file_box = NULL;
+	struct box *text_box = NULL;
+	int box_x = 0, box_y = 0;
+	hlcache_handle *containing_content = NULL;
+
+	/* Scan box tree for boxes that can handle drop */
+	while ((next = box_at_point(box, x, y, &box_x, &box_y,
+			&containing_content)) != NULL) {
+		box = next;
+
+		if (box->style && css_computed_visibility(box->style) ==
+				CSS_VISIBILITY_HIDDEN)
+			continue;
+
+		if (box->iframe)
+			return browser_window_drop_file_at_point(box->iframe,
+					x - box_x, y - box_y, file);
+
+		if (box->gadget) {
+			switch (box->gadget->type) {
+				case GADGET_FILE:
+					file_box = box;
+				break;
+
+				case GADGET_TEXTBOX:
+				case GADGET_TEXTAREA:
+				case GADGET_PASSWORD:
+					text_box = box;
+					break;
+
+				default:	/* appease compiler */
+					break;
+			}
+		}
+	}
+
+	if (!file_box && !text_box)
+		/* No box capable of handling drop */
+		return false;
+
+	/* Handle the drop */
+	if (file_box) {
+		/* File dropped on file input */
+		utf8_convert_ret ret;
+		char *utf8_fn;
+
+		ret = utf8_from_local_encoding(file, 0,
+				&utf8_fn);
+		if (ret != UTF8_CONVERT_OK) {
+			/* A bad encoding should never happen */
+			assert(ret != UTF8_CONVERT_BADENC);
+			LOG(("utf8_from_local_encoding failed"));
+			/* Load was for us - just no memory */
+			return true;
+		}
+
+		/* Found: update form input */
+		free(file_box->gadget->value);
+		file_box->gadget->value = utf8_fn;
+
+		/* Redraw box. */
+		if (containing_content == NULL)
+			html__redraw_a_box(c, file_box);
+		else
+			html_redraw_a_box(containing_content, file_box);
+
+	} else if (html->bw != NULL) {
+		/* File dropped on text input */
+
+		size_t file_len;
+		FILE *fp = NULL;
+		char *buffer;
+		char *utf8_buff;
+		utf8_convert_ret ret;
+		unsigned int size;
+		struct browser_window *bw;
+
+		/* Open file */
+		fp = fopen(file, "rb");
+		if (fp == NULL) {
+			/* Couldn't open file, but drop was for us */
+			return true;
+		}
+
+		/* Get filesize */
+		fseek(fp, 0, SEEK_END);
+		file_len = ftell(fp);
+		fseek(fp, 0, SEEK_SET);
+
+		/* Allocate buffer for file data */
+		buffer = malloc(file_len + 1);
+		if (buffer == NULL) {
+			/* No memory, but drop was for us */
+			fclose(fp);
+			return true;
+		}
+
+		/* Stick file into buffer */
+		if (file_len != fread(buffer, 1, file_len, fp)) {
+			/* Failed, but drop was for us */
+			free(buffer);
+			fclose(fp);
+			return true;
+		}
+
+		/* Done with file */
+		fclose(fp);
+
+		/* Ensure buffer's string termination */
+		buffer[file_len] = '\0';
+
+		/* TODO: Sniff for text? */
+
+		/* Convert to UTF-8 */
+		ret = utf8_from_local_encoding(buffer, file_len, &utf8_buff);
+		if (ret != UTF8_CONVERT_OK) {
+			/* bad encoding shouldn't happen */
+			assert(ret != UTF8_CONVERT_BADENC);
+			LOG(("utf8_from_local_encoding failed"));
+			free(buffer);
+			warn_user("NoMemory", NULL);
+			return true;
+		}
+
+		/* Done with buffer */
+		free(buffer);
+
+		/* Get new length */
+		size = strlen(utf8_buff);
+
+		/* Simulate a click over the input box, to place caret */
+		browser_window_mouse_click(html->bw,
+				BROWSER_MOUSE_PRESS_1, x, y);
+
+		bw = browser_window_get_root(html->bw);
+
+		/* Paste the file as text */
+		browser_window_paste_text(bw, utf8_buff, size, true);
+
+		free(utf8_buff);
+	}
+
+	return true;
+}
+
+
 /**
  * Set an HTML content's search context
  *
@@ -2335,7 +2671,7 @@ struct content_html_iframe *html_get_iframe(hlcache_handle *h)
  * \param h  Content to retrieve base target from
  * \return Pointer to URL
  */
-const char *html_get_base_url(hlcache_handle *h)
+nsurl *html_get_base_url(hlcache_handle *h)
 {
 	html_content *c = (html_content *) hlcache_handle_get_content(h);
 
@@ -2397,22 +2733,6 @@ struct content_html_object *html_get_objects(hlcache_handle *h, unsigned int *n)
 	return c->object_list;
 }
 
-/**
- * Retrieve favicon associated with an HTML document
- *
- * \param h  HTML document to retrieve favicon from
- * \return Pointer to favicon, or NULL if none
- */
-hlcache_handle *html_get_favicon(hlcache_handle *h)
-{
-	html_content *c = (html_content *) hlcache_handle_get_content(h);
-
-	assert(c != NULL);
-
-	return NULL;
-}
-
-
 /**
  * Retrieve layout coordinates of box with given id
  *
@@ -2422,7 +2742,7 @@ hlcache_handle *html_get_favicon(hlcache_handle *h)
  * \param y        Updated to global y coord iff id found
  * \return  true iff id found
  */
-bool html_get_id_offset(hlcache_handle *h, const char *frag_id, int *x, int *y)
+bool html_get_id_offset(hlcache_handle *h, lwc_string *frag_id, int *x, int *y)
 {
 	struct box *pos;
 	struct box *layout;
