@@ -101,7 +101,7 @@ struct content_html_frames {
 	int margin_height;	/** frame margin height */
 
 	char *name;	/** frame name (for targetting) */
-	char *url;	/** frame url */
+	nsurl *url;	/** frame url */
 
 	bool no_resize;	/** frame is not resizable */
 	frame_scrolling scrolling;	/** scrolling characteristics */
@@ -119,7 +119,7 @@ struct content_html_iframe {
 	int margin_height;	/** frame margin height */
 
 	char *name;	/** frame name (for targetting) */
-	char *url;	/** frame url */
+	nsurl *url;	/** frame url */
 
 	frame_scrolling scrolling;	/** scrolling characteristics */
 	bool border;	/** frame has a border */
@@ -134,10 +134,6 @@ struct content_html_iframe {
 #define STYLESHEET_ADBLOCK	2	/* adblocking stylesheet */
 #define STYLESHEET_START	3	/* start of document stylesheets */
 
-extern char *default_stylesheet_url;
-extern char *adblock_stylesheet_url;
-extern char *quirks_stylesheet_url;
-
 /** Render padding and margin box outlines in html_redraw(). */
 extern bool html_redraw_debug;
 
@@ -167,14 +163,13 @@ const char *html_get_encoding(struct hlcache_handle *h);
 binding_encoding_source html_get_encoding_source(struct hlcache_handle *h);
 struct content_html_frames *html_get_frameset(struct hlcache_handle *h);
 struct content_html_iframe *html_get_iframe(struct hlcache_handle *h);
-const char *html_get_base_url(struct hlcache_handle *h);
+nsurl *html_get_base_url(struct hlcache_handle *h);
 const char *html_get_base_target(struct hlcache_handle *h);
 struct html_stylesheet *html_get_stylesheets(struct hlcache_handle *h, 
 		unsigned int *n);
 struct content_html_object *html_get_objects(struct hlcache_handle *h, 
 		unsigned int *n);
-struct hlcache_handle *html_get_favicon(struct hlcache_handle *h);
-bool html_get_id_offset(struct hlcache_handle *h, const char *frag_id,
+bool html_get_id_offset(struct hlcache_handle *h, lwc_string *frag_id,
 		int *x, int *y);
 
 #endif
