@@ -108,7 +108,7 @@ void html_mouse_track(struct content *c, struct browser_window *bw,
 		if (idx != 0)
 			selection_track(&html->sel, mouse, idx);
 
-		browser_window_set_drag_type(bw, DRAGGING_NONE);
+		browser_window_set_drag_type(bw, DRAGGING_NONE, NULL);
 	}
 
 	switch (bw->drag_type) {
@@ -167,8 +167,8 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 {
 	html_content *html = (html_content *) c;
 	enum { ACTION_NONE, ACTION_SUBMIT, ACTION_GO } action = ACTION_NONE;
-	char *title = 0;
-	const char *url = 0;
+	const char *title = 0;
+	nsurl *url = 0;
 	const char *target = 0;
 	char status_buffer[200];
 	const char *status = 0;
@@ -223,8 +223,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 		return;
 	}
 
-	if (bw->drag_type != DRAGGING_NONE && !mouse &&
-			html->scrollbar != NULL) {
+	if (!mouse && html->scrollbar != NULL) {
 		/* drag end: scrollbar */
 		html_overflow_scroll_drag_end(html->scrollbar, mouse, x, y);
 	}
@@ -255,7 +254,7 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	}
 
 	/* Content related drags handled by now */
-	browser_window_set_drag_type(bw, DRAGGING_NONE);
+	browser_window_set_drag_type(bw, DRAGGING_NONE, NULL);
 
 	/* search the box tree for a link, imagemap, form control, or
 	 * box with scrollbars */
@@ -543,21 +542,23 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 	} else if (url) {
 		if (title) {
 			snprintf(status_buffer, sizeof status_buffer, "%s: %s",
-					url, title);
+					nsurl_access(url), title);
 			status = status_buffer;
 		} else
-			status = url;
+			status = nsurl_access(url);
 
 		pointer = get_pointer_shape(bw, url_box, imagemap);
 
 		if (mouse & BROWSER_MOUSE_CLICK_1 &&
 				mouse & BROWSER_MOUSE_MOD_1) {
 			/* force download of link */
-			browser_window_go_post(bw, url, 0, 0, false,
-					content_get_url(h), true, true, 0);
+			browser_window_go_post(bw, nsurl_access(url), 0, 0,
+					false, nsurl_access(hlcache_handle_get_url(h)),
+					true, true, 0);
 		} else if (mouse & BROWSER_MOUSE_CLICK_2 &&
 				mouse & BROWSER_MOUSE_MOD_1) {
-				gui_window_save_link(bw->window, url, title);
+				gui_window_save_link(bw->window,
+						nsurl_access(url), title);
 		} else if (mouse & (BROWSER_MOUSE_CLICK_1 |
 				BROWSER_MOUSE_CLICK_2))
 			action = ACTION_GO;
@@ -689,7 +690,8 @@ void html_mouse_action(struct content *c, struct browser_window *bw,
 		break;
 	case ACTION_GO:
 		browser_window_go(browser_window_find_target(bw, target, mouse),
-				url, content_get_url(h), true);
+				nsurl_access(url),
+				nsurl_access(hlcache_handle_get_url(h)), true);
 		break;
 	case ACTION_NONE:
 		break;
@@ -815,8 +817,6 @@ void html_overflow_scroll_callback(void *client_data,
 	html_content *html = (html_content *)data->c;
 	struct box *box = data->box;
 	int x, y, box_x, box_y, diff_x, diff_y;
-	struct browser_window *root_bw;
-	
 	
 	switch(scrollbar_data->msg) {
 		case SCROLLBAR_MSG_REDRAW:
@@ -846,17 +846,24 @@ void html_overflow_scroll_callback(void *client_data,
 			html_redraw_a_box(html->bw->current_content, box);
 			break;
 		case SCROLLBAR_MSG_SCROLL_START:
-			browser_window_set_drag_type(html->bw, DRAGGING_OTHER);
+		{
+			struct rect rect = {
+				.x0 = scrollbar_data->x0,
+				.y0 = scrollbar_data->y0,
+				.x1 = scrollbar_data->x1,
+				.y1 = scrollbar_data->y1
+			};
+			browser_window_set_drag_type(html->bw,
+					DRAGGING_CONTENT_SCROLLBAR, &rect);
 
 			html->scrollbar = scrollbar_data->scrollbar;
-
-			root_bw = browser_window_get_root(html->bw);
-			gui_window_box_scroll_start(root_bw->window,
-					scrollbar_data->x0, scrollbar_data->y0,
-     					scrollbar_data->x1, scrollbar_data->y1);
+		}
 			break;
 		case SCROLLBAR_MSG_SCROLL_FINISHED:
 			html->scrollbar = NULL;
+
+			browser_window_set_drag_type(html->bw,
+					DRAGGING_NONE, NULL);
 			
 			browser_window_set_pointer(html->bw,
 					GUI_POINTER_DEFAULT);
