@@ -2035,7 +2035,7 @@ bool html_redraw_background(int x, int y, struct box *box, float scale,
 		.fill_colour = *background_colour,
 	};
 
-	if (html_redraw_printing && option_remove_backgrounds)
+	if (ctx->background_images == false)
 		return true;
 
 	plot_content = (background->background != NULL);
@@ -2173,6 +2173,7 @@ bool html_redraw_background(int x, int y, struct box *box, float scale,
 			width = content_get_width(background->background);
 			height = content_get_height(background->background);
 
+			/* ensure clip area only as large as required */
 			if (!repeat_x) {
 				if (r.x0 < x)
 					r.x0 = x;
