@@ -33,6 +33,16 @@
 #include "utils/config.h"
 #include "utils/log.h"
 #include "utils/talloc.h"
+#include "utils/utils.h"
+
+/**
+ * Private data attached to each DOM node
+ */
+typedef struct hubbub_private {
+	binding_private base;
+
+	uint32_t refcnt;
+} hubbub_private;
 
 typedef struct hubbub_ctx {
 	hubbub_parser *parser;
@@ -69,6 +79,9 @@ static struct {
 	{ "xmlns", "http://www.w3.org/2000/xmlns/" }
 };
 
+static hubbub_private *create_private(uint32_t refcnt);
+static hubbub_private *copy_private(const hubbub_private *p, uint32_t refcnt);
+static void destroy_private(hubbub_private *p);
 static inline char *c_string_from_hubbub_string(hubbub_ctx *ctx, 
 		const hubbub_string *str);
 static void create_namespaces(hubbub_ctx *ctx, xmlNode *root);
@@ -168,7 +181,13 @@ binding_error binding_create_tree(void *arena, const char *charset, void **ctx)
 		free(c);
 		return BINDING_NOMEM;
 	}
-	c->document->_private = (void *) 0;
+	c->document->_private = create_private(0);
+	if (c->document->_private == NULL) {
+		xmlFreeDoc(c->document);
+		hubbub_parser_destroy(c->parser);
+		free(c);
+		return BINDING_NOMEM;
+	}
 
 	for (i = 0; i < sizeof(c->namespaces) / sizeof(c->namespaces[0]); i++) {
 		c->namespaces[i] = NULL;
@@ -200,7 +219,7 @@ binding_error binding_destroy_tree(void *ctx)
 		hubbub_parser_destroy(c->parser);
 
 	if (c->owns_doc)
-		xmlFreeDoc(c->document);
+		binding_destroy_document(c->document);
 
 	c->parser = NULL;
 	c->encoding = NULL;
@@ -290,8 +309,93 @@ struct form_control *binding_get_control_for_node(void *ctx, xmlNodePtr node)
 	return ctl;
 }
 
+void binding_destroy_document(xmlDocPtr doc)
+{
+	xmlNode *n = (xmlNode *) doc;
+
+	while (n != NULL) {
+		destroy_private(n->_private);
+
+		if (n->children != NULL) {
+			n = n->children;
+		} else if (n->next != NULL) {
+			n = n->next;
+		} else {
+			while (n->parent != NULL && n->parent->next == NULL)
+				n = n->parent;
+
+			if (n->parent != NULL)
+				n = n->parent->next;
+			else
+				n = NULL;
+		}
+	}
+
+	xmlFreeDoc(doc);
+}
+
 /*****************************************************************************/
 
+hubbub_private *create_private(uint32_t refcnt)
+{
+	hubbub_private *pvt = calloc(1, sizeof(*pvt));
+
+	if (pvt != NULL)
+		pvt->refcnt = refcnt;
+
+	return pvt;
+}
+
+hubbub_private *copy_private(const hubbub_private *p, uint32_t refcnt)
+{
+	hubbub_private *pvt = calloc(1, sizeof(*pvt));
+
+	if (pvt != NULL) {
+		pvt->refcnt = refcnt;
+
+		if (p->base.nclasses > 0) {
+			pvt->base.classes = 
+				malloc(p->base.nclasses * sizeof(lwc_string *));
+			if (pvt->base.classes == NULL) {
+				free(pvt);
+				return NULL;
+			}
+
+			while (pvt->base.nclasses < p->base.nclasses) {
+				pvt->base.classes[pvt->base.nclasses] =
+					lwc_string_ref(p->base.classes[
+							pvt->base.nclasses]);
+				pvt->base.nclasses++;
+			}
+		}
+
+		if (p->base.localname != NULL)
+			pvt->base.localname = lwc_string_ref(p->base.localname);
+
+		if (p->base.id != NULL)
+			pvt->base.id = lwc_string_ref(p->base.id);
+	}	
+
+	return pvt;
+}
+
+void destroy_private(hubbub_private *p)
+{
+	if (p->base.localname != NULL)
+		lwc_string_unref(p->base.localname);
+
+	if (p->base.id != NULL)
+		lwc_string_unref(p->base.id);
+
+	while (p->base.nclasses > 0)
+		lwc_string_unref(p->base.classes[--p->base.nclasses]);
+
+	if (p->base.classes != NULL)
+		free(p->base.classes);
+
+	free(p);
+}
+
 char *c_string_from_hubbub_string(hubbub_ctx *ctx, const hubbub_string *str)
 {
 	return strndup((const char *) str->ptr, (int) str->len);
@@ -328,7 +432,12 @@ hubbub_error create_comment(void *ctx, const hubbub_string *data, void **result)
 		free(content);
 		return HUBBUB_NOMEM;
 	}
-	n->_private = (void *) (uintptr_t) 1;
+	n->_private = create_private(1);
+	if (n->_private == NULL) {
+		xmlFreeNode(n);
+		free(content);
+		return HUBBUB_NOMEM;
+	}
 
 	free(content);
 
@@ -374,7 +483,14 @@ hubbub_error create_doctype(void *ctx, const hubbub_doctype *doctype,
 		free(name);
 		return HUBBUB_NOMEM;
 	}
-	n->_private = (void *) (uintptr_t) 1;
+	n->_private = create_private(1);
+	if (n->_private == NULL) {
+		xmlFreeDtd(n);
+		free(system);
+		free(public);
+		free(name);
+		return HUBBUB_NOMEM;
+	}
 
 	*result = (void *) n;
 
@@ -388,18 +504,20 @@ hubbub_error create_doctype(void *ctx, const hubbub_doctype *doctype,
 hubbub_error create_element(void *ctx, const hubbub_tag *tag, void **result)
 {
 	hubbub_ctx *c = (hubbub_ctx *) ctx;
-	char *name;
+	lwc_string *iname;
 	xmlNodePtr n;
 
-	name = c_string_from_hubbub_string(c, &tag->name);
-	if (name == NULL)
+	if (lwc_intern_string((const char *) tag->name.ptr, tag->name.len, 
+			&iname) != lwc_error_ok) {
 		return HUBBUB_NOMEM;
+	}
 
 	if (c->namespaces[0] != NULL) {
 		n = xmlNewDocNode(c->document, c->namespaces[tag->ns - 1], 
-				BAD_CAST name, NULL);
+				BAD_CAST lwc_string_data(iname), NULL);
 	} else {
-		n = xmlNewDocNode(c->document, NULL, BAD_CAST name, NULL);
+		n = xmlNewDocNode(c->document, NULL, 
+				BAD_CAST lwc_string_data(iname), NULL);
 
 		/* We're creating the root node of the document. Therefore,
 		 * create the namespaces and set this node's namespace */
@@ -410,25 +528,33 @@ hubbub_error create_element(void *ctx, const hubbub_tag *tag, void **result)
 		}
 	}
 	if (n == NULL) {
-		free(name);
+		lwc_string_unref(iname);
+		return HUBBUB_NOMEM;
+	}
+	n->_private = create_private(1);
+	if (n->_private == NULL) {
+		xmlFreeNode(n);
+		lwc_string_unref(iname);
 		return HUBBUB_NOMEM;
 	}
-	n->_private = (void *) (uintptr_t) 1;
 
 	if (tag->n_attributes > 0 && add_attributes(ctx, (void *) n, 
 			tag->attributes, tag->n_attributes) != HUBBUB_OK) {
+		destroy_private(n->_private);
 		xmlFreeNode(n);
-		free(name);
+		lwc_string_unref(iname);
 		return HUBBUB_NOMEM;
 	}
 
-	if (strcasecmp(name, "form") == 0) {
+	if (lwc_string_length(iname) == SLEN("form") &&
+			strcasecmp(lwc_string_data(iname), "form") == 0) {
 		struct form *form = parse_form_element(n, c->encoding);
 
 		/* Memory exhaustion */
 		if (form == NULL) {
+			destroy_private(n->_private);
 			xmlFreeNode(n);
-			free(name);
+			lwc_string_unref(iname);
 			return HUBBUB_NOMEM;
 		}
 
@@ -437,9 +563,9 @@ hubbub_error create_element(void *ctx, const hubbub_tag *tag, void **result)
 		c->forms = form;
 	}
 
-	*result = (void *) n;
+	((binding_private *) n->_private)->localname = iname;
 
-	free(name);
+	*result = (void *) n;
 
 	return HUBBUB_OK;
 }
@@ -453,7 +579,11 @@ hubbub_error create_text(void *ctx, const hubbub_string *data, void **result)
 	if (n == NULL) {
 		return HUBBUB_NOMEM;
 	}
-	n->_private = (void *) (uintptr_t) 1;
+	n->_private = create_private(1);
+	if (n->_private == NULL) {
+		xmlFreeNode(n);
+		return HUBBUB_NOMEM;
+	}
 
 	*result = (void *) n;
 
@@ -463,17 +593,18 @@ hubbub_error create_text(void *ctx, const hubbub_string *data, void **result)
 hubbub_error ref_node(void *ctx, void *node)
 {
 	hubbub_ctx *c = (hubbub_ctx *) ctx;
+	hubbub_private *pvt;
 
 	if (node == c->document) {
 		xmlDoc *n = (xmlDoc *) node;
-		uintptr_t count = (uintptr_t) n->_private;
+		pvt = n->_private;
 
-		n->_private = (void *) (++count);
+		pvt->refcnt++;
 	} else {
 		xmlNode *n = (xmlNode *) node;
-		uintptr_t count = (uintptr_t) n->_private;
+		pvt = n->_private;
 
-		n->_private = (void *) (++count);
+		pvt->refcnt++;
 	}
 
 	return HUBBUB_OK;
@@ -482,23 +613,25 @@ hubbub_error ref_node(void *ctx, void *node)
 hubbub_error unref_node(void *ctx, void *node)
 {
 	hubbub_ctx *c = (hubbub_ctx *) ctx;
+	hubbub_private *pvt;
 
 	if (node == c->document) {
 		xmlDoc *n = (xmlDoc *) node;
-		uintptr_t count = (uintptr_t) n->_private;
+		pvt = n->_private;
 
-		assert(count != 0 && "Node has refcount of zero");
+		assert(pvt->refcnt != 0 && "Node has refcount of zero");
 
-		n->_private = (void *) (--count);
+		pvt->refcnt--;
 	} else {
 		xmlNode *n = (xmlNode *) node;
-		uintptr_t count = (uintptr_t) n->_private;
+		pvt = n->_private;
 
-		assert(count != 0 && "Node has refcount of zero");
+		assert(pvt->refcnt != 0 && "Node has refcount of zero");
 
-		n->_private = (void *) (--count);
+		pvt->refcnt--;
 
-		if (count == 0 && n->parent == NULL) {
+		if (pvt->refcnt == 0 && n->parent == NULL) {
+			destroy_private(pvt);
 			xmlFreeNode(n);
 		}
 	}
@@ -585,15 +718,88 @@ hubbub_error remove_child(void *ctx, void *parent, void *child, void **result)
 hubbub_error clone_node(void *ctx, void *node, bool deep, void **result)
 {
 	xmlNode *n = (xmlNode *) node;
+	xmlNode *clonedtree;
 
-	*result = xmlCopyNode(n, deep ? 1 : 2);
-
-	if (*result == NULL)
+	/* Shallow clone node */
+	clonedtree = xmlCopyNode(n, 2);
+	if (clonedtree == NULL)
 		return HUBBUB_NOMEM;
 
-	((xmlNode *)(*result))->_private = (void *) (uintptr_t) 1;
+	clonedtree->_private = copy_private(n->_private, 1);
+	if (clonedtree->_private == NULL) {
+		xmlFreeNode(clonedtree);
+		return HUBBUB_NOMEM;
+	}
+
+	/* Iteratively clone children too, if required */
+	if (deep && n->children != NULL) {
+		xmlNode *parent = clonedtree, *copy;
+
+		n = n->children;
+
+		while (n != node) {
+			copy = xmlCopyNode(n, 2);
+			if (copy == NULL)
+				goto error;
+
+			copy->_private = copy_private(n->_private, 0);
+			if (copy->_private == NULL) {
+				xmlFreeNode(copy);
+				goto error;
+			}
+
+			xmlAddChild(parent, copy);
+
+			if (n->children != NULL) {
+				parent = copy;
+				n = n->children;
+			} else if (n->next != NULL) {
+				n = n->next;
+			} else {
+				while (n->parent != node && 
+						n->parent->next == NULL) {
+					parent = parent->parent;
+					n = n->parent;
+				}
+
+				if (n->parent != node) {
+					parent = parent->parent;
+					n = n->parent->next;
+				} else
+					n = node;
+			}
+		}
+	}
+
+	*result = clonedtree;
 
 	return HUBBUB_OK;
+
+error:
+	n = clonedtree;
+
+	while (n != NULL) {
+		destroy_private(n->_private);
+
+		if (n->children != NULL) {
+			n = n->children;
+		} else if (n->next != NULL) {
+			n = n->next;
+		} else {
+			while (n->parent != NULL && n->parent->next == NULL) {
+				n = n->parent;
+			}
+
+			if (n->parent != NULL)
+				n = n->parent->next;
+			else
+				n = NULL;
+		}
+	}
+
+	xmlFreeNode(clonedtree);
+
+	return HUBBUB_NOMEM;
 }
 
 hubbub_error reparent_children(void *ctx, void *node, void *new_parent)
@@ -705,24 +911,81 @@ hubbub_error form_associate(void *ctx, void *form, void *node)
 	return HUBBUB_OK;
 }
 
+static hubbub_error parse_class_attr(lwc_string *value, 
+		lwc_string ***classes, uint32_t *nclasses)
+{
+	const char *pv;
+	lwc_string **cls = NULL;
+	uint32_t count = 0;
+
+	/* Count number of classes */
+	for (pv = lwc_string_data(value); *pv != '\0'; ) {
+		if (*pv != ' ') {
+			while (*pv != ' ' && *pv != '\0')
+				pv++;
+			count++;
+		} else {
+			while (*pv == ' ')
+				pv++;
+		}
+	}
+
+	/* If there are some, unpack them */
+	if (count > 0) {
+		cls = malloc(count * sizeof(lwc_string *));
+		if (cls == NULL)
+			return HUBBUB_NOMEM;
+
+		for (pv = lwc_string_data(value), count = 0; *pv != '\0'; ) {
+			if (*pv != ' ') {
+				const char *s = pv;
+				while (*pv != ' ' && *pv != '\0')
+					pv++;
+				if (lwc_intern_string(s, pv - s, 
+						&cls[count]) != lwc_error_ok)
+					goto error;
+				count++;
+			} else {
+				while (*pv == ' ')
+					pv++;
+			}
+		}
+	}
+
+	*classes = cls;
+	*nclasses = count;
+
+	return HUBBUB_OK;
+error:
+	while (count > 0)
+		lwc_string_unref(cls[--count]);
+
+	free(cls);
+		
+	return HUBBUB_NOMEM;
+}
+
 hubbub_error add_attributes(void *ctx, void *node, 
 		const hubbub_attribute *attributes, uint32_t n_attributes)
 {
 	hubbub_ctx *c = (hubbub_ctx *) ctx;
 	xmlNode *n = (xmlNode *) node;
+	binding_private *p = n->_private;
 	uint32_t attr;
 
 	for (attr = 0; attr < n_attributes; attr++) {
 		xmlAttr *prop;
-		char *name, *value;
+		lwc_string *name, *value;
 
-		name = c_string_from_hubbub_string(c, &attributes[attr].name);
-		if (name == NULL)
+		if (lwc_intern_string((const char *) attributes[attr].name.ptr,
+				attributes[attr].name.len, 
+				&name) != lwc_error_ok)
 			return HUBBUB_NOMEM;
 
-		value = c_string_from_hubbub_string(c, &attributes[attr].value);
-		if (value == NULL) {
-			free(name);
+		if (lwc_intern_string((const char *) attributes[attr].value.ptr,
+				attributes[attr].value.len, 
+				&value) != lwc_error_ok) {
+			lwc_string_unref(name);
 			return HUBBUB_NOMEM;
 		}
 
@@ -730,18 +993,38 @@ hubbub_error add_attributes(void *ctx, void *node,
 				c->namespaces[0] != NULL) {
 			prop = xmlNewNsProp(n, 
 					c->namespaces[attributes[attr].ns - 1],
-					BAD_CAST name, BAD_CAST value);
+					BAD_CAST lwc_string_data(name), 
+					BAD_CAST lwc_string_data(value));
 		} else {
-			prop = xmlNewProp(n, BAD_CAST name, BAD_CAST value);
-		}
-		if (prop == NULL) {
-			free(value);
-			free(name);
-			return HUBBUB_NOMEM;
+			prop = xmlNewProp(n, BAD_CAST lwc_string_data(name), 
+					BAD_CAST lwc_string_data(value));
 		}
 
-		free(value);
-		free(name);
+		/* Handle @id / @class */
+		if (p->id == NULL && lwc_string_length(name) == SLEN("id") &&
+				strcasecmp(lwc_string_data(name), "id") == 0) {
+			p->id = lwc_string_ref(value);
+		} else if (p->nclasses == 0 && 
+				lwc_string_length(name) == SLEN("class") && 
+				strcasecmp(lwc_string_data(name), 
+						"class") == 0) {
+			hubbub_error error;
+
+			error = parse_class_attr(value, &p->classes, 
+					&p->nclasses);
+			if (error != HUBBUB_OK) {
+				lwc_string_unref(value);
+				lwc_string_unref(name);
+				return error;
+			}
+		}
+
+		lwc_string_unref(value);
+		lwc_string_unref(name);
+
+		if (prop == NULL) {
+			return HUBBUB_NOMEM;
+		}
 	}
 
 	return HUBBUB_OK;
