@@ -68,9 +68,9 @@ extern bool print_active, print_text_black;
 typedef enum { GUI_DRAG_NONE, GUI_DRAG_SELECTION, GUI_DRAG_DOWNLOAD_SAVE,
 		GUI_DRAG_SAVE, GUI_DRAG_SCROLL, GUI_DRAG_STATUS_RESIZE,
 		GUI_DRAG_TREEVIEW, GUI_DRAG_BUTTONBAR,
-		GUI_DRAG_FRAME } gui_drag_type;
+		GUI_DRAG_FRAME } ro_gui_drag_type;
 
-extern gui_drag_type gui_current_drag_type;
+extern ro_gui_drag_type gui_current_drag_type;
 
 /** desktop font, size and style being used */
 extern char ro_gui_desktop_font_family[];
@@ -100,7 +100,6 @@ struct gui_window {
 
 	/** Options. */
 	struct {
-		bool background_images;	/**< Display background images. */
 		bool buffer_animations;	/**< Use screen buffering for animations. */
 		bool buffer_everything;	/**< Use screen buffering for everything. */
 	} option;
@@ -133,7 +132,6 @@ bool ro_gui_download_prequit(void);
 void ro_gui_401login_init(void);
 
 /* in window.c */
-void ro_gui_scroll_request(wimp_scroll *scroll);
 bool ro_gui_window_dataload(struct gui_window *g, wimp_message *message);
 void ro_gui_window_mouse_at(struct gui_window *g, wimp_pointer *pointer);
 void ro_gui_window_iconise(struct gui_window *g,
@@ -181,9 +179,6 @@ extern bool sched_active;
 extern os_t sched_time;
 bool schedule_run(void);
 
-/* in debugwin.c */
-void ro_gui_debugwin_open(void);
-
 /* in search.c */
 void ro_gui_search_init(void);
 void ro_gui_search_prepare(struct browser_window *g);
