@@ -28,6 +28,7 @@
 #include <stdlib.h>
 #include <time.h>
 #include "oslib/osfile.h"
+#include "oslib/osmodule.h"
 #include "oslib/wimp.h"
 #include "content/content.h"
 #include "content/hlcache.h"
@@ -37,6 +38,7 @@
 #include "riscos/dialog.h"
 #include "riscos/hotlist.h"
 #include "riscos/menus.h"
+#include "riscos/message.h"
 #include "riscos/options.h"
 #include "riscos/save.h"
 #include "riscos/toolbar.h"
@@ -45,6 +47,7 @@
 #include "riscos/wimp_event.h"
 #include "utils/log.h"
 #include "utils/messages.h"
+#include "utils/schedule.h"
 #include "utils/utils.h"
 #include "utils/url.h"
 
@@ -57,6 +60,8 @@ static void ro_gui_hotlist_menu_warning(wimp_w w, wimp_i i, wimp_menu *menu,
 static bool ro_gui_hotlist_menu_select(wimp_w w, wimp_i i, wimp_menu *menu,
 		wimp_selection *selection, menu_action action);
 static void ro_gui_hotlist_toolbar_click(button_bar_action action);
+static void ro_gui_hotlist_addurl_bounce(wimp_message *message);
+static void ro_gui_hotlist_scheduled_callback(void *p);
 
 struct ro_treeview_callbacks ro_hotlist_treeview_callbacks = {
 	ro_gui_hotlist_toolbar_click,
@@ -64,13 +69,29 @@ struct ro_treeview_callbacks ro_hotlist_treeview_callbacks = {
 	ro_gui_hotlist_toolbar_save_buttons
 };
 
+/* Hotlist Protocol Message Blocks, which are currently not in OSLib. */
+
+struct ro_hotlist_message_hotlist_addurl {
+	wimp_MESSAGE_HEADER_MEMBERS	/**< The standard message header. */
+	char		*url;		/**< Pointer to the URL in RMA.   */
+	char		*title;		/**< Pointer to the title in RMA. */
+	char		appname[32];	/**< The application name.        */
+};
+
+struct ro_hotlist_message_hotlist_changed {
+	wimp_MESSAGE_HEADER_MEMBERS	/**< The standard message header. */
+};
+
+static char	*hotlist_url = NULL;    /**< URL area claimed from RMA.   */
+static char	*hotlist_title = NULL;	/**< Title area claimed from RMA. */
+
 /* The RISC OS hotlist window, toolbar and treeview data. */
 
 static struct ro_hotlist {
-	wimp_w		window;		/*< The hotlist RO window handle. */
-	struct toolbar	*toolbar;	/*< The hotlist toolbar handle.   */
-	ro_treeview	*tv;		/*< The hotlist treeview handle.  */
-	wimp_menu	*menu;		/*< The hotlist window menu.      */
+	wimp_w		window;		/**< The hotlist RO window handle. */
+	struct toolbar	*toolbar;	/**< The hotlist toolbar handle.   */
+	ro_treeview	*tv;		/**< The hotlist treeview handle.  */
+	wimp_menu	*menu;		/**< The hotlist window menu.      */
 } hotlist_window;
 
 /**
@@ -176,6 +197,23 @@ void ro_gui_hotlist_postinitialise(void)
 
 void ro_gui_hotlist_open(void)
 {
+	os_error	*error;
+	char		command[2048];
+
+	if (option_external_hotlists && option_external_hotlist_app != NULL &&
+			*option_external_hotlist_app != '\0') {
+		snprintf(command, sizeof(command), "Filer_Run %s",
+				option_external_hotlist_app);
+		error = xos_cli(command);
+
+		if (error == NULL)
+			return;
+
+		LOG(("xos_cli: 0x%x: %s", error->errnum, error->errmess));
+		warn_user("Failed to launch external hotlist: %s",
+				error->errmess);
+	}
+
 	tree_set_redraw(ro_treeview_get_tree(hotlist_window.tv), true);
 
 	ro_gui_hotlist_toolbar_update_buttons();
@@ -221,7 +259,7 @@ void ro_gui_hotlist_toolbar_click(button_bar_action action)
 		break;
 
 	case TOOLBAR_BUTTON_CREATE:
-		hotlist_add_folder();
+		hotlist_add_folder(true);
 		break;
 
 	default:
@@ -338,10 +376,10 @@ bool ro_gui_hotlist_menu_select(wimp_w w, wimp_i i, wimp_menu *menu,
 		ro_gui_dialog_open_persistent(w, dialog_saveas, true);
 		return true;
 	case TREE_NEW_FOLDER:
-		hotlist_add_folder();
+		hotlist_add_folder(true);
 		return true;
 	case TREE_NEW_LINK:
-		hotlist_add_entry();
+		hotlist_add_entry(true);
 		return true;
 	case TREE_EXPAND_ALL:
 		hotlist_expand_all();
@@ -420,6 +458,134 @@ bool ro_gui_hotlist_check_menu(wimp_menu *menu)
 		return false;
 }
 
+
+/**
+ * Add a URL to the hotlist.  This will be passed on to the core hotlist, then
+ * Message_HotlistAddURL will broadcast to any bookmark applications via the
+ * Hotlist Protocol.
+ *
+ * \param *url			The URL to be added.
+ */
+
+void ro_gui_hotlist_add_page(const char *url)
+{
+	const struct url_data				*data;
+	wimp_message					message;
+	struct ro_hotlist_message_hotlist_addurl	*add_url =
+			(struct ro_hotlist_message_hotlist_addurl *) &message;
+
+	if (url == NULL)
+		return;
+
+	/* If we're not using external hotlists, add the page to NetSurf's
+	 * own hotlist and return...
+	 */
+
+	if (!option_external_hotlists) {
+		hotlist_add_page(url);
+		return;
+	}
+
+	/* ...otherwise try broadcasting the details to any other
+	 * interested parties.  If no-one answers, we'll fall back to
+	 * NetSurf's hotlist anyway when the message bounces.
+	 */
+
+	ro_gui_hotlist_add_cleanup();
+
+	LOG(("Sending Hotlist AddURL to potential hotlist clients."));
+
+	data = urldb_get_url_data(url);
+	if (data == NULL)
+		return;
+
+	hotlist_url = osmodule_alloc(strlen(url) + 1);
+	hotlist_title = osmodule_alloc(strlen(data->title) + 1);
+
+	if (hotlist_url == NULL || hotlist_title == NULL) {
+		ro_gui_hotlist_add_cleanup();
+		return;
+	}
+
+	strcpy(hotlist_url, url);
+	strcpy(hotlist_title, data->title);
+
+	add_url->size = 60;
+	add_url->your_ref = 0;
+	add_url->action = message_HOTLIST_ADD_URL;
+	add_url->url = hotlist_url;
+	add_url->title = hotlist_title;
+	strcpy(add_url->appname, "NetSurf");
+
+	if (!ro_message_send_message(wimp_USER_MESSAGE_RECORDED, &message, 0,
+			ro_gui_hotlist_addurl_bounce))
+		ro_gui_hotlist_add_cleanup();
+
+	/* Listen for the next Null poll, as an indication that the
+	 * message didn't bounce.
+	 */
+
+	schedule(0, ro_gui_hotlist_scheduled_callback, NULL);
+}
+
+
+/**
+ * Handle bounced Message_HotlistAddURL, so that RMA storage can be freed.
+ *
+ * \param *message		The bounced message content.
+ */
+
+static void ro_gui_hotlist_addurl_bounce(wimp_message *message)
+{
+	LOG(("Hotlist AddURL Bounced"));
+
+	if (hotlist_url != NULL)
+		hotlist_add_page(hotlist_url);
+
+	ro_gui_hotlist_add_cleanup();
+
+	/* There's no longer any need to listen for the next Null poll. */
+
+	schedule_remove(ro_gui_hotlist_scheduled_callback, NULL);
+}
+
+
+/**
+ * Callback to schedule for the next available Null poll, by which point
+ * a hotlist client will have claimed the Message_HotlistAddURL and any
+ * details in RMA can safely be discarded.
+ *
+ * \param *p			Unused data pointer.
+ */
+
+static void ro_gui_hotlist_scheduled_callback(void *p)
+{
+	LOG(("Hotlist AddURL was claimed by something."));
+
+	ro_gui_hotlist_add_cleanup();
+}
+
+
+/**
+ * Clean up RMA storage used by the Message_HotlistAddURL protocol.
+ */
+
+void ro_gui_hotlist_add_cleanup(void)
+{
+	LOG(("Clean up RMA"));
+
+	if (hotlist_url != NULL) {
+		osmodule_free(hotlist_url);
+		hotlist_url = NULL;
+	}
+
+	if (hotlist_title != NULL) {
+		osmodule_free(hotlist_title);
+		hotlist_title = NULL;
+	}
+}
+
+
 #if 0
 /**
  * Handle URL dropped on hotlist
