@@ -247,7 +247,6 @@ void ro_gui_search_add_recent(const char *search, void *p)
 
 bool ro_gui_search_prepare_menu(void)
 {
-	os_error *error;
 	int i;
 	int suggestions = 0;
 
@@ -268,14 +267,6 @@ bool ro_gui_search_prepare_menu(void)
 	recent_search_menu->entries[suggestions - 1].menu_flags |=
 			wimp_MENU_LAST;
 
-	if ((current_menu_open) && (current_menu == recent_search_menu)) {
-		error = xwimp_create_menu(current_menu, 0, 0);
-		if (error) {
-			LOG(("xwimp_create_menu: 0x%x: %s",
-					error->errnum, error->errmess));
-			warn_user("MenuError", error->errmess);
-		}
-	}
 	return true;
 }
 
