@@ -72,11 +72,6 @@ static const char windowclassname_main[] = "nswsmainwindow";
 
 static struct nsws_pointers nsws_pointer;
 
-void gui_multitask(void)
-{
-/*	LOG(("gui_multitask")); */
-}
-
 void gui_poll(bool active)
 {
 	MSG Msg; /* message from system */
@@ -1723,8 +1718,8 @@ bool gui_window_scroll_start(struct gui_window *w)
 	return true;
 }
 
-bool gui_window_box_scroll_start(struct gui_window *w,
-				 int x0, int y0, int x1, int y1)
+bool gui_window_drag_start(struct gui_window *g, gui_drag_type type,
+		const struct rect *rect)
 {
 	return true;
 }
